// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   templates/CSSPropertyMetadata.cpp.tmpl
// and input files:
//   ../../../../src/3rdparty/chromium/third_party/WebKit/Source/core/css/CSSProperties.json5


#include "core/css/CSSPropertyMetadata.h"

#include "platform/RuntimeEnabledFeatures.h"
#include <bitset>

namespace blink {

bool CSSPropertyMetadata::IsDescriptor(CSSPropertyID unresolvedProperty) {
  switch (unresolvedProperty) {
    case CSSPropertyInvalid:
      NOTREACHED();
      return false;
    case CSSPropertyFontFamily:
    case CSSPropertyFontFeatureSettings:
    case CSSPropertyFontStretch:
    case CSSPropertyFontStyle:
    case CSSPropertyFontWeight:
    case CSSPropertyZoom:
    case CSSPropertyFontDisplay:
    case CSSPropertyFontVariant:
    case CSSPropertyHeight:
    case CSSPropertyMaxHeight:
    case CSSPropertyMaxWidth:
    case CSSPropertyMaxZoom:
    case CSSPropertyMinHeight:
    case CSSPropertyMinWidth:
    case CSSPropertyMinZoom:
    case CSSPropertyOrientation:
    case CSSPropertySrc:
    case CSSPropertyUnicodeRange:
    case CSSPropertyUserZoom:
    case CSSPropertyWidth:
    case CSSPropertyAliasWebkitFontFeatureSettings:
      return true;
    default:
      return false;
  }
}

bool CSSPropertyMetadata::IsProperty(CSSPropertyID unresolvedProperty) {
  switch (unresolvedProperty) {
    case CSSPropertyInvalid:
      NOTREACHED();
      return false;
    case CSSPropertyAnimationDelay:
    case CSSPropertyAnimationDirection:
    case CSSPropertyAnimationDuration:
    case CSSPropertyAnimationFillMode:
    case CSSPropertyAnimationIterationCount:
    case CSSPropertyAnimationName:
    case CSSPropertyAnimationPlayState:
    case CSSPropertyAnimationTimingFunction:
    case CSSPropertyTransitionDelay:
    case CSSPropertyTransitionDuration:
    case CSSPropertyTransitionProperty:
    case CSSPropertyTransitionTimingFunction:
    case CSSPropertyColor:
    case CSSPropertyDirection:
    case CSSPropertyFontFamily:
    case CSSPropertyFontFeatureSettings:
    case CSSPropertyFontKerning:
    case CSSPropertyFontSize:
    case CSSPropertyFontSizeAdjust:
    case CSSPropertyFontStretch:
    case CSSPropertyFontStyle:
    case CSSPropertyFontVariantCaps:
    case CSSPropertyFontVariantLigatures:
    case CSSPropertyFontVariantNumeric:
    case CSSPropertyFontVariationSettings:
    case CSSPropertyFontWeight:
    case CSSPropertyTextOrientation:
    case CSSPropertyTextRendering:
    case CSSPropertyWebkitFontSmoothing:
    case CSSPropertyWebkitLocale:
    case CSSPropertyWebkitTextOrientation:
    case CSSPropertyWebkitWritingMode:
    case CSSPropertyWritingMode:
    case CSSPropertyZoom:
    case CSSPropertyAlignContent:
    case CSSPropertyAlignItems:
    case CSSPropertyAlignSelf:
    case CSSPropertyAlignmentBaseline:
    case CSSPropertyAll:
    case CSSPropertyAnimation:
    case CSSPropertyBackdropFilter:
    case CSSPropertyBackfaceVisibility:
    case CSSPropertyBackground:
    case CSSPropertyBackgroundAttachment:
    case CSSPropertyBackgroundBlendMode:
    case CSSPropertyBackgroundClip:
    case CSSPropertyBackgroundColor:
    case CSSPropertyBackgroundImage:
    case CSSPropertyBackgroundOrigin:
    case CSSPropertyBackgroundPosition:
    case CSSPropertyBackgroundPositionX:
    case CSSPropertyBackgroundPositionY:
    case CSSPropertyBackgroundRepeat:
    case CSSPropertyBackgroundRepeatX:
    case CSSPropertyBackgroundRepeatY:
    case CSSPropertyBackgroundSize:
    case CSSPropertyBaselineShift:
    case CSSPropertyBlockSize:
    case CSSPropertyBorder:
    case CSSPropertyBorderBottom:
    case CSSPropertyBorderBottomColor:
    case CSSPropertyBorderBottomLeftRadius:
    case CSSPropertyBorderBottomRightRadius:
    case CSSPropertyBorderBottomStyle:
    case CSSPropertyBorderBottomWidth:
    case CSSPropertyBorderCollapse:
    case CSSPropertyBorderColor:
    case CSSPropertyBorderImage:
    case CSSPropertyBorderImageOutset:
    case CSSPropertyBorderImageRepeat:
    case CSSPropertyBorderImageSlice:
    case CSSPropertyBorderImageSource:
    case CSSPropertyBorderImageWidth:
    case CSSPropertyBorderLeft:
    case CSSPropertyBorderLeftColor:
    case CSSPropertyBorderLeftStyle:
    case CSSPropertyBorderLeftWidth:
    case CSSPropertyBorderRadius:
    case CSSPropertyBorderRight:
    case CSSPropertyBorderRightColor:
    case CSSPropertyBorderRightStyle:
    case CSSPropertyBorderRightWidth:
    case CSSPropertyBorderSpacing:
    case CSSPropertyBorderStyle:
    case CSSPropertyBorderTop:
    case CSSPropertyBorderTopColor:
    case CSSPropertyBorderTopLeftRadius:
    case CSSPropertyBorderTopRightRadius:
    case CSSPropertyBorderTopStyle:
    case CSSPropertyBorderTopWidth:
    case CSSPropertyBorderWidth:
    case CSSPropertyBottom:
    case CSSPropertyBoxShadow:
    case CSSPropertyBoxSizing:
    case CSSPropertyBreakAfter:
    case CSSPropertyBreakBefore:
    case CSSPropertyBreakInside:
    case CSSPropertyBufferedRendering:
    case CSSPropertyCaptionSide:
    case CSSPropertyCaretColor:
    case CSSPropertyClear:
    case CSSPropertyClip:
    case CSSPropertyClipPath:
    case CSSPropertyClipRule:
    case CSSPropertyColorInterpolation:
    case CSSPropertyColorInterpolationFilters:
    case CSSPropertyColorRendering:
    case CSSPropertyColumnCount:
    case CSSPropertyColumnFill:
    case CSSPropertyColumnGap:
    case CSSPropertyColumnRule:
    case CSSPropertyColumnRuleColor:
    case CSSPropertyColumnRuleStyle:
    case CSSPropertyColumnRuleWidth:
    case CSSPropertyColumnSpan:
    case CSSPropertyColumnWidth:
    case CSSPropertyColumns:
    case CSSPropertyContain:
    case CSSPropertyContent:
    case CSSPropertyCounterIncrement:
    case CSSPropertyCounterReset:
    case CSSPropertyCursor:
    case CSSPropertyCx:
    case CSSPropertyCy:
    case CSSPropertyD:
    case CSSPropertyDisplay:
    case CSSPropertyDominantBaseline:
    case CSSPropertyEmptyCells:
    case CSSPropertyFill:
    case CSSPropertyFillOpacity:
    case CSSPropertyFillRule:
    case CSSPropertyFilter:
    case CSSPropertyFlex:
    case CSSPropertyFlexBasis:
    case CSSPropertyFlexDirection:
    case CSSPropertyFlexFlow:
    case CSSPropertyFlexGrow:
    case CSSPropertyFlexShrink:
    case CSSPropertyFlexWrap:
    case CSSPropertyFloat:
    case CSSPropertyFloodColor:
    case CSSPropertyFloodOpacity:
    case CSSPropertyFont:
    case CSSPropertyFontVariant:
    case CSSPropertyGrid:
    case CSSPropertyGridArea:
    case CSSPropertyGridAutoColumns:
    case CSSPropertyGridAutoFlow:
    case CSSPropertyGridAutoRows:
    case CSSPropertyGridColumn:
    case CSSPropertyGridColumnEnd:
    case CSSPropertyGridColumnGap:
    case CSSPropertyGridColumnStart:
    case CSSPropertyGridGap:
    case CSSPropertyGridRow:
    case CSSPropertyGridRowEnd:
    case CSSPropertyGridRowGap:
    case CSSPropertyGridRowStart:
    case CSSPropertyGridTemplate:
    case CSSPropertyGridTemplateAreas:
    case CSSPropertyGridTemplateColumns:
    case CSSPropertyGridTemplateRows:
    case CSSPropertyHeight:
    case CSSPropertyHyphens:
    case CSSPropertyImageOrientation:
    case CSSPropertyImageRendering:
    case CSSPropertyInlineSize:
    case CSSPropertyIsolation:
    case CSSPropertyJustifyContent:
    case CSSPropertyJustifyItems:
    case CSSPropertyJustifySelf:
    case CSSPropertyLeft:
    case CSSPropertyLetterSpacing:
    case CSSPropertyLightingColor:
    case CSSPropertyLineBreak:
    case CSSPropertyLineHeight:
    case CSSPropertyLineHeightStep:
    case CSSPropertyListStyle:
    case CSSPropertyListStyleImage:
    case CSSPropertyListStylePosition:
    case CSSPropertyListStyleType:
    case CSSPropertyMargin:
    case CSSPropertyMarginBottom:
    case CSSPropertyMarginLeft:
    case CSSPropertyMarginRight:
    case CSSPropertyMarginTop:
    case CSSPropertyMarker:
    case CSSPropertyMarkerEnd:
    case CSSPropertyMarkerMid:
    case CSSPropertyMarkerStart:
    case CSSPropertyMask:
    case CSSPropertyMaskSourceType:
    case CSSPropertyMaskType:
    case CSSPropertyMaxBlockSize:
    case CSSPropertyMaxHeight:
    case CSSPropertyMaxInlineSize:
    case CSSPropertyMaxWidth:
    case CSSPropertyMinBlockSize:
    case CSSPropertyMinHeight:
    case CSSPropertyMinInlineSize:
    case CSSPropertyMinWidth:
    case CSSPropertyMixBlendMode:
    case CSSPropertyObjectFit:
    case CSSPropertyObjectPosition:
    case CSSPropertyOffset:
    case CSSPropertyOffsetAnchor:
    case CSSPropertyOffsetDistance:
    case CSSPropertyOffsetPath:
    case CSSPropertyOffsetPosition:
    case CSSPropertyOffsetRotate:
    case CSSPropertyOpacity:
    case CSSPropertyOrder:
    case CSSPropertyOrphans:
    case CSSPropertyOutline:
    case CSSPropertyOutlineColor:
    case CSSPropertyOutlineOffset:
    case CSSPropertyOutlineStyle:
    case CSSPropertyOutlineWidth:
    case CSSPropertyOverflow:
    case CSSPropertyOverflowAnchor:
    case CSSPropertyOverflowWrap:
    case CSSPropertyOverflowX:
    case CSSPropertyOverflowY:
    case CSSPropertyPadding:
    case CSSPropertyPaddingBottom:
    case CSSPropertyPaddingLeft:
    case CSSPropertyPaddingRight:
    case CSSPropertyPaddingTop:
    case CSSPropertyPage:
    case CSSPropertyPageBreakAfter:
    case CSSPropertyPageBreakBefore:
    case CSSPropertyPageBreakInside:
    case CSSPropertyPaintOrder:
    case CSSPropertyPerspective:
    case CSSPropertyPerspectiveOrigin:
    case CSSPropertyPlaceContent:
    case CSSPropertyPlaceItems:
    case CSSPropertyPlaceSelf:
    case CSSPropertyPointerEvents:
    case CSSPropertyPosition:
    case CSSPropertyQuotes:
    case CSSPropertyR:
    case CSSPropertyResize:
    case CSSPropertyRight:
    case CSSPropertyRotate:
    case CSSPropertyRx:
    case CSSPropertyRy:
    case CSSPropertyScale:
    case CSSPropertyScrollBehavior:
    case CSSPropertyScrollBoundaryBehavior:
    case CSSPropertyScrollBoundaryBehaviorX:
    case CSSPropertyScrollBoundaryBehaviorY:
    case CSSPropertyScrollPadding:
    case CSSPropertyScrollPaddingBlock:
    case CSSPropertyScrollPaddingBlockEnd:
    case CSSPropertyScrollPaddingBlockStart:
    case CSSPropertyScrollPaddingBottom:
    case CSSPropertyScrollPaddingInline:
    case CSSPropertyScrollPaddingInlineEnd:
    case CSSPropertyScrollPaddingInlineStart:
    case CSSPropertyScrollPaddingLeft:
    case CSSPropertyScrollPaddingRight:
    case CSSPropertyScrollPaddingTop:
    case CSSPropertyScrollSnapAlign:
    case CSSPropertyScrollSnapMargin:
    case CSSPropertyScrollSnapMarginBlock:
    case CSSPropertyScrollSnapMarginBlockEnd:
    case CSSPropertyScrollSnapMarginBlockStart:
    case CSSPropertyScrollSnapMarginBottom:
    case CSSPropertyScrollSnapMarginInline:
    case CSSPropertyScrollSnapMarginInlineEnd:
    case CSSPropertyScrollSnapMarginInlineStart:
    case CSSPropertyScrollSnapMarginLeft:
    case CSSPropertyScrollSnapMarginRight:
    case CSSPropertyScrollSnapMarginTop:
    case CSSPropertyScrollSnapStop:
    case CSSPropertyScrollSnapType:
    case CSSPropertyShapeImageThreshold:
    case CSSPropertyShapeMargin:
    case CSSPropertyShapeOutside:
    case CSSPropertyShapeRendering:
    case CSSPropertySize:
    case CSSPropertySpeak:
    case CSSPropertyStopColor:
    case CSSPropertyStopOpacity:
    case CSSPropertyStroke:
    case CSSPropertyStrokeDasharray:
    case CSSPropertyStrokeDashoffset:
    case CSSPropertyStrokeLinecap:
    case CSSPropertyStrokeLinejoin:
    case CSSPropertyStrokeMiterlimit:
    case CSSPropertyStrokeOpacity:
    case CSSPropertyStrokeWidth:
    case CSSPropertyTabSize:
    case CSSPropertyTableLayout:
    case CSSPropertyTextAlign:
    case CSSPropertyTextAlignLast:
    case CSSPropertyTextAnchor:
    case CSSPropertyTextCombineUpright:
    case CSSPropertyTextDecoration:
    case CSSPropertyTextDecorationColor:
    case CSSPropertyTextDecorationLine:
    case CSSPropertyTextDecorationSkip:
    case CSSPropertyTextDecorationStyle:
    case CSSPropertyTextIndent:
    case CSSPropertyTextJustify:
    case CSSPropertyTextOverflow:
    case CSSPropertyTextShadow:
    case CSSPropertyTextSizeAdjust:
    case CSSPropertyTextTransform:
    case CSSPropertyTextUnderlinePosition:
    case CSSPropertyTop:
    case CSSPropertyTouchAction:
    case CSSPropertyTransform:
    case CSSPropertyTransformBox:
    case CSSPropertyTransformOrigin:
    case CSSPropertyTransformStyle:
    case CSSPropertyTransition:
    case CSSPropertyTranslate:
    case CSSPropertyUnicodeBidi:
    case CSSPropertyUserSelect:
    case CSSPropertyVectorEffect:
    case CSSPropertyVerticalAlign:
    case CSSPropertyVisibility:
    case CSSPropertyWebkitAppRegion:
    case CSSPropertyWebkitAppearance:
    case CSSPropertyWebkitBackgroundClip:
    case CSSPropertyWebkitBackgroundOrigin:
    case CSSPropertyWebkitBorderAfter:
    case CSSPropertyWebkitBorderAfterColor:
    case CSSPropertyWebkitBorderAfterStyle:
    case CSSPropertyWebkitBorderAfterWidth:
    case CSSPropertyWebkitBorderBefore:
    case CSSPropertyWebkitBorderBeforeColor:
    case CSSPropertyWebkitBorderBeforeStyle:
    case CSSPropertyWebkitBorderBeforeWidth:
    case CSSPropertyWebkitBorderEnd:
    case CSSPropertyWebkitBorderEndColor:
    case CSSPropertyWebkitBorderEndStyle:
    case CSSPropertyWebkitBorderEndWidth:
    case CSSPropertyWebkitBorderHorizontalSpacing:
    case CSSPropertyWebkitBorderImage:
    case CSSPropertyWebkitBorderStart:
    case CSSPropertyWebkitBorderStartColor:
    case CSSPropertyWebkitBorderStartStyle:
    case CSSPropertyWebkitBorderStartWidth:
    case CSSPropertyWebkitBorderVerticalSpacing:
    case CSSPropertyWebkitBoxAlign:
    case CSSPropertyWebkitBoxDecorationBreak:
    case CSSPropertyWebkitBoxDirection:
    case CSSPropertyWebkitBoxFlex:
    case CSSPropertyWebkitBoxFlexGroup:
    case CSSPropertyWebkitBoxLines:
    case CSSPropertyWebkitBoxOrdinalGroup:
    case CSSPropertyWebkitBoxOrient:
    case CSSPropertyWebkitBoxPack:
    case CSSPropertyWebkitBoxReflect:
    case CSSPropertyWebkitColumnBreakAfter:
    case CSSPropertyWebkitColumnBreakBefore:
    case CSSPropertyWebkitColumnBreakInside:
    case CSSPropertyWebkitFontSizeDelta:
    case CSSPropertyWebkitHighlight:
    case CSSPropertyWebkitHyphenateCharacter:
    case CSSPropertyWebkitLineBreak:
    case CSSPropertyWebkitLineClamp:
    case CSSPropertyWebkitLogicalHeight:
    case CSSPropertyWebkitLogicalWidth:
    case CSSPropertyWebkitMarginAfter:
    case CSSPropertyWebkitMarginAfterCollapse:
    case CSSPropertyWebkitMarginBefore:
    case CSSPropertyWebkitMarginBeforeCollapse:
    case CSSPropertyWebkitMarginBottomCollapse:
    case CSSPropertyWebkitMarginCollapse:
    case CSSPropertyWebkitMarginEnd:
    case CSSPropertyWebkitMarginStart:
    case CSSPropertyWebkitMarginTopCollapse:
    case CSSPropertyWebkitMask:
    case CSSPropertyWebkitMaskBoxImage:
    case CSSPropertyWebkitMaskBoxImageOutset:
    case CSSPropertyWebkitMaskBoxImageRepeat:
    case CSSPropertyWebkitMaskBoxImageSlice:
    case CSSPropertyWebkitMaskBoxImageSource:
    case CSSPropertyWebkitMaskBoxImageWidth:
    case CSSPropertyWebkitMaskClip:
    case CSSPropertyWebkitMaskComposite:
    case CSSPropertyWebkitMaskImage:
    case CSSPropertyWebkitMaskOrigin:
    case CSSPropertyWebkitMaskPosition:
    case CSSPropertyWebkitMaskPositionX:
    case CSSPropertyWebkitMaskPositionY:
    case CSSPropertyWebkitMaskRepeat:
    case CSSPropertyWebkitMaskRepeatX:
    case CSSPropertyWebkitMaskRepeatY:
    case CSSPropertyWebkitMaskSize:
    case CSSPropertyWebkitMaxLogicalHeight:
    case CSSPropertyWebkitMaxLogicalWidth:
    case CSSPropertyWebkitMinLogicalHeight:
    case CSSPropertyWebkitMinLogicalWidth:
    case CSSPropertyWebkitPaddingAfter:
    case CSSPropertyWebkitPaddingBefore:
    case CSSPropertyWebkitPaddingEnd:
    case CSSPropertyWebkitPaddingStart:
    case CSSPropertyWebkitPerspectiveOriginX:
    case CSSPropertyWebkitPerspectiveOriginY:
    case CSSPropertyWebkitPrintColorAdjust:
    case CSSPropertyWebkitRtlOrdering:
    case CSSPropertyWebkitRubyPosition:
    case CSSPropertyWebkitTapHighlightColor:
    case CSSPropertyWebkitTextCombine:
    case CSSPropertyWebkitTextDecorationsInEffect:
    case CSSPropertyWebkitTextEmphasis:
    case CSSPropertyWebkitTextEmphasisColor:
    case CSSPropertyWebkitTextEmphasisPosition:
    case CSSPropertyWebkitTextEmphasisStyle:
    case CSSPropertyWebkitTextFillColor:
    case CSSPropertyWebkitTextSecurity:
    case CSSPropertyWebkitTextStroke:
    case CSSPropertyWebkitTextStrokeColor:
    case CSSPropertyWebkitTextStrokeWidth:
    case CSSPropertyWebkitTransformOriginX:
    case CSSPropertyWebkitTransformOriginY:
    case CSSPropertyWebkitTransformOriginZ:
    case CSSPropertyWebkitUserDrag:
    case CSSPropertyWebkitUserModify:
    case CSSPropertyWhiteSpace:
    case CSSPropertyWidows:
    case CSSPropertyWidth:
    case CSSPropertyWillChange:
    case CSSPropertyWordBreak:
    case CSSPropertyWordSpacing:
    case CSSPropertyWordWrap:
    case CSSPropertyX:
    case CSSPropertyY:
    case CSSPropertyZIndex:
    case CSSPropertyAliasEpubCaptionSide:
    case CSSPropertyAliasEpubTextCombine:
    case CSSPropertyAliasEpubTextEmphasis:
    case CSSPropertyAliasEpubTextEmphasisColor:
    case CSSPropertyAliasEpubTextEmphasisStyle:
    case CSSPropertyAliasEpubTextOrientation:
    case CSSPropertyAliasEpubTextTransform:
    case CSSPropertyAliasEpubWordBreak:
    case CSSPropertyAliasEpubWritingMode:
    case CSSPropertyAliasWebkitAlignContent:
    case CSSPropertyAliasWebkitAlignItems:
    case CSSPropertyAliasWebkitAlignSelf:
    case CSSPropertyAliasWebkitAnimation:
    case CSSPropertyAliasWebkitAnimationDelay:
    case CSSPropertyAliasWebkitAnimationDirection:
    case CSSPropertyAliasWebkitAnimationDuration:
    case CSSPropertyAliasWebkitAnimationFillMode:
    case CSSPropertyAliasWebkitAnimationIterationCount:
    case CSSPropertyAliasWebkitAnimationName:
    case CSSPropertyAliasWebkitAnimationPlayState:
    case CSSPropertyAliasWebkitAnimationTimingFunction:
    case CSSPropertyAliasWebkitBackfaceVisibility:
    case CSSPropertyAliasWebkitBackgroundSize:
    case CSSPropertyAliasWebkitBorderBottomLeftRadius:
    case CSSPropertyAliasWebkitBorderBottomRightRadius:
    case CSSPropertyAliasWebkitBorderRadius:
    case CSSPropertyAliasWebkitBorderTopLeftRadius:
    case CSSPropertyAliasWebkitBorderTopRightRadius:
    case CSSPropertyAliasWebkitBoxShadow:
    case CSSPropertyAliasWebkitBoxSizing:
    case CSSPropertyAliasWebkitClipPath:
    case CSSPropertyAliasWebkitColumnCount:
    case CSSPropertyAliasWebkitColumnGap:
    case CSSPropertyAliasWebkitColumnRule:
    case CSSPropertyAliasWebkitColumnRuleColor:
    case CSSPropertyAliasWebkitColumnRuleStyle:
    case CSSPropertyAliasWebkitColumnRuleWidth:
    case CSSPropertyAliasWebkitColumnSpan:
    case CSSPropertyAliasWebkitColumnWidth:
    case CSSPropertyAliasWebkitColumns:
    case CSSPropertyAliasWebkitFilter:
    case CSSPropertyAliasWebkitFlex:
    case CSSPropertyAliasWebkitFlexBasis:
    case CSSPropertyAliasWebkitFlexDirection:
    case CSSPropertyAliasWebkitFlexFlow:
    case CSSPropertyAliasWebkitFlexGrow:
    case CSSPropertyAliasWebkitFlexShrink:
    case CSSPropertyAliasWebkitFlexWrap:
    case CSSPropertyAliasWebkitFontFeatureSettings:
    case CSSPropertyAliasWebkitJustifyContent:
    case CSSPropertyAliasWebkitOpacity:
    case CSSPropertyAliasWebkitOrder:
    case CSSPropertyAliasWebkitPerspective:
    case CSSPropertyAliasWebkitPerspectiveOrigin:
    case CSSPropertyAliasWebkitShapeImageThreshold:
    case CSSPropertyAliasWebkitShapeMargin:
    case CSSPropertyAliasWebkitShapeOutside:
    case CSSPropertyAliasWebkitTextSizeAdjust:
    case CSSPropertyAliasWebkitTransform:
    case CSSPropertyAliasWebkitTransformOrigin:
    case CSSPropertyAliasWebkitTransformStyle:
    case CSSPropertyAliasWebkitTransition:
    case CSSPropertyAliasWebkitTransitionDelay:
    case CSSPropertyAliasWebkitTransitionDuration:
    case CSSPropertyAliasWebkitTransitionProperty:
    case CSSPropertyAliasWebkitTransitionTimingFunction:
    case CSSPropertyAliasWebkitUserSelect:
    case CSSPropertyVariable:
    case CSSPropertyApplyAtRule:
      return true;
    default:
      return false;
  }
}

bool CSSPropertyMetadata::IsInterpolableProperty(CSSPropertyID unresolvedProperty) {
  switch (unresolvedProperty) {
    case CSSPropertyInvalid:
      NOTREACHED();
      return false;
    case CSSPropertyColor:
    case CSSPropertyFontSize:
    case CSSPropertyFontSizeAdjust:
    case CSSPropertyFontVariationSettings:
    case CSSPropertyFontWeight:
    case CSSPropertyBackdropFilter:
    case CSSPropertyBackgroundColor:
    case CSSPropertyBackgroundImage:
    case CSSPropertyBackgroundPositionX:
    case CSSPropertyBackgroundPositionY:
    case CSSPropertyBackgroundSize:
    case CSSPropertyBaselineShift:
    case CSSPropertyBorderBottomColor:
    case CSSPropertyBorderBottomLeftRadius:
    case CSSPropertyBorderBottomRightRadius:
    case CSSPropertyBorderBottomWidth:
    case CSSPropertyBorderImageOutset:
    case CSSPropertyBorderImageSlice:
    case CSSPropertyBorderImageSource:
    case CSSPropertyBorderImageWidth:
    case CSSPropertyBorderLeftColor:
    case CSSPropertyBorderLeftWidth:
    case CSSPropertyBorderRightColor:
    case CSSPropertyBorderRightWidth:
    case CSSPropertyBorderTopColor:
    case CSSPropertyBorderTopLeftRadius:
    case CSSPropertyBorderTopRightRadius:
    case CSSPropertyBorderTopWidth:
    case CSSPropertyBottom:
    case CSSPropertyBoxShadow:
    case CSSPropertyCaretColor:
    case CSSPropertyClip:
    case CSSPropertyClipPath:
    case CSSPropertyColumnCount:
    case CSSPropertyColumnGap:
    case CSSPropertyColumnRuleColor:
    case CSSPropertyColumnRuleWidth:
    case CSSPropertyColumnWidth:
    case CSSPropertyCx:
    case CSSPropertyCy:
    case CSSPropertyD:
    case CSSPropertyFill:
    case CSSPropertyFillOpacity:
    case CSSPropertyFilter:
    case CSSPropertyFlexBasis:
    case CSSPropertyFlexGrow:
    case CSSPropertyFlexShrink:
    case CSSPropertyFloodColor:
    case CSSPropertyFloodOpacity:
    case CSSPropertyHeight:
    case CSSPropertyLeft:
    case CSSPropertyLetterSpacing:
    case CSSPropertyLightingColor:
    case CSSPropertyLineHeight:
    case CSSPropertyListStyleImage:
    case CSSPropertyMarginBottom:
    case CSSPropertyMarginLeft:
    case CSSPropertyMarginRight:
    case CSSPropertyMarginTop:
    case CSSPropertyMaxHeight:
    case CSSPropertyMaxWidth:
    case CSSPropertyMinHeight:
    case CSSPropertyMinWidth:
    case CSSPropertyObjectPosition:
    case CSSPropertyOffsetAnchor:
    case CSSPropertyOffsetDistance:
    case CSSPropertyOffsetPosition:
    case CSSPropertyOffsetRotate:
    case CSSPropertyOpacity:
    case CSSPropertyOrder:
    case CSSPropertyOrphans:
    case CSSPropertyOutlineColor:
    case CSSPropertyOutlineOffset:
    case CSSPropertyOutlineWidth:
    case CSSPropertyPaddingBottom:
    case CSSPropertyPaddingLeft:
    case CSSPropertyPaddingRight:
    case CSSPropertyPaddingTop:
    case CSSPropertyPerspective:
    case CSSPropertyPerspectiveOrigin:
    case CSSPropertyR:
    case CSSPropertyRight:
    case CSSPropertyRotate:
    case CSSPropertyRx:
    case CSSPropertyRy:
    case CSSPropertyScale:
    case CSSPropertyShapeImageThreshold:
    case CSSPropertyShapeMargin:
    case CSSPropertyShapeOutside:
    case CSSPropertyStopColor:
    case CSSPropertyStopOpacity:
    case CSSPropertyStroke:
    case CSSPropertyStrokeDasharray:
    case CSSPropertyStrokeDashoffset:
    case CSSPropertyStrokeMiterlimit:
    case CSSPropertyStrokeOpacity:
    case CSSPropertyStrokeWidth:
    case CSSPropertyTextDecorationColor:
    case CSSPropertyTextIndent:
    case CSSPropertyTextShadow:
    case CSSPropertyTop:
    case CSSPropertyTransform:
    case CSSPropertyTransformOrigin:
    case CSSPropertyTranslate:
    case CSSPropertyVerticalAlign:
    case CSSPropertyVisibility:
    case CSSPropertyWebkitBorderHorizontalSpacing:
    case CSSPropertyWebkitBorderVerticalSpacing:
    case CSSPropertyWebkitMaskBoxImageOutset:
    case CSSPropertyWebkitMaskBoxImageSlice:
    case CSSPropertyWebkitMaskBoxImageSource:
    case CSSPropertyWebkitMaskBoxImageWidth:
    case CSSPropertyWebkitMaskImage:
    case CSSPropertyWebkitMaskPositionX:
    case CSSPropertyWebkitMaskPositionY:
    case CSSPropertyWebkitMaskSize:
    case CSSPropertyWebkitPerspectiveOriginX:
    case CSSPropertyWebkitPerspectiveOriginY:
    case CSSPropertyWebkitTextStrokeColor:
    case CSSPropertyWebkitTransformOriginX:
    case CSSPropertyWebkitTransformOriginY:
    case CSSPropertyWebkitTransformOriginZ:
    case CSSPropertyWidows:
    case CSSPropertyWidth:
    case CSSPropertyWordSpacing:
    case CSSPropertyX:
    case CSSPropertyY:
    case CSSPropertyZIndex:
    case CSSPropertyAliasWebkitBackgroundSize:
    case CSSPropertyAliasWebkitBorderBottomLeftRadius:
    case CSSPropertyAliasWebkitBorderBottomRightRadius:
    case CSSPropertyAliasWebkitBorderTopLeftRadius:
    case CSSPropertyAliasWebkitBorderTopRightRadius:
    case CSSPropertyAliasWebkitBoxShadow:
    case CSSPropertyAliasWebkitClipPath:
    case CSSPropertyAliasWebkitColumnCount:
    case CSSPropertyAliasWebkitColumnGap:
    case CSSPropertyAliasWebkitColumnRuleColor:
    case CSSPropertyAliasWebkitColumnRuleWidth:
    case CSSPropertyAliasWebkitColumnWidth:
    case CSSPropertyAliasWebkitFilter:
    case CSSPropertyAliasWebkitFlexBasis:
    case CSSPropertyAliasWebkitFlexGrow:
    case CSSPropertyAliasWebkitFlexShrink:
    case CSSPropertyAliasWebkitOpacity:
    case CSSPropertyAliasWebkitOrder:
    case CSSPropertyAliasWebkitPerspective:
    case CSSPropertyAliasWebkitPerspectiveOrigin:
    case CSSPropertyAliasWebkitShapeImageThreshold:
    case CSSPropertyAliasWebkitShapeMargin:
    case CSSPropertyAliasWebkitShapeOutside:
    case CSSPropertyAliasWebkitTransform:
    case CSSPropertyAliasWebkitTransformOrigin:
      return true;
    default:
      return false;
  }
}

bool CSSPropertyMetadata::IsInheritedProperty(CSSPropertyID unresolvedProperty) {
  switch (unresolvedProperty) {
    case CSSPropertyInvalid:
      NOTREACHED();
      return false;
    case CSSPropertyColor:
    case CSSPropertyDirection:
    case CSSPropertyFontFamily:
    case CSSPropertyFontFeatureSettings:
    case CSSPropertyFontKerning:
    case CSSPropertyFontSize:
    case CSSPropertyFontSizeAdjust:
    case CSSPropertyFontStretch:
    case CSSPropertyFontStyle:
    case CSSPropertyFontVariantCaps:
    case CSSPropertyFontVariantLigatures:
    case CSSPropertyFontVariantNumeric:
    case CSSPropertyFontVariationSettings:
    case CSSPropertyFontWeight:
    case CSSPropertyTextOrientation:
    case CSSPropertyTextRendering:
    case CSSPropertyWebkitFontSmoothing:
    case CSSPropertyWebkitLocale:
    case CSSPropertyWebkitTextOrientation:
    case CSSPropertyWebkitWritingMode:
    case CSSPropertyWritingMode:
    case CSSPropertyBorderCollapse:
    case CSSPropertyCaptionSide:
    case CSSPropertyCaretColor:
    case CSSPropertyClipRule:
    case CSSPropertyColorInterpolation:
    case CSSPropertyColorInterpolationFilters:
    case CSSPropertyColorRendering:
    case CSSPropertyCursor:
    case CSSPropertyDominantBaseline:
    case CSSPropertyEmptyCells:
    case CSSPropertyFill:
    case CSSPropertyFillOpacity:
    case CSSPropertyFillRule:
    case CSSPropertyHyphens:
    case CSSPropertyImageOrientation:
    case CSSPropertyImageRendering:
    case CSSPropertyLetterSpacing:
    case CSSPropertyLineBreak:
    case CSSPropertyLineHeight:
    case CSSPropertyLineHeightStep:
    case CSSPropertyListStyleImage:
    case CSSPropertyListStylePosition:
    case CSSPropertyListStyleType:
    case CSSPropertyMarkerEnd:
    case CSSPropertyMarkerMid:
    case CSSPropertyMarkerStart:
    case CSSPropertyOrphans:
    case CSSPropertyOverflowWrap:
    case CSSPropertyPaintOrder:
    case CSSPropertyPointerEvents:
    case CSSPropertyQuotes:
    case CSSPropertyShapeRendering:
    case CSSPropertySpeak:
    case CSSPropertyStroke:
    case CSSPropertyStrokeDasharray:
    case CSSPropertyStrokeDashoffset:
    case CSSPropertyStrokeLinecap:
    case CSSPropertyStrokeLinejoin:
    case CSSPropertyStrokeMiterlimit:
    case CSSPropertyStrokeOpacity:
    case CSSPropertyStrokeWidth:
    case CSSPropertyTabSize:
    case CSSPropertyTextAlign:
    case CSSPropertyTextAlignLast:
    case CSSPropertyTextAnchor:
    case CSSPropertyTextCombineUpright:
    case CSSPropertyTextDecorationSkip:
    case CSSPropertyTextIndent:
    case CSSPropertyTextJustify:
    case CSSPropertyTextShadow:
    case CSSPropertyTextSizeAdjust:
    case CSSPropertyTextTransform:
    case CSSPropertyTextUnderlinePosition:
    case CSSPropertyUserSelect:
    case CSSPropertyVisibility:
    case CSSPropertyWebkitBorderHorizontalSpacing:
    case CSSPropertyWebkitBorderVerticalSpacing:
    case CSSPropertyWebkitBoxDirection:
    case CSSPropertyWebkitHighlight:
    case CSSPropertyWebkitHyphenateCharacter:
    case CSSPropertyWebkitLineBreak:
    case CSSPropertyWebkitPrintColorAdjust:
    case CSSPropertyWebkitRtlOrdering:
    case CSSPropertyWebkitRubyPosition:
    case CSSPropertyWebkitTapHighlightColor:
    case CSSPropertyWebkitTextCombine:
    case CSSPropertyWebkitTextDecorationsInEffect:
    case CSSPropertyWebkitTextEmphasisColor:
    case CSSPropertyWebkitTextEmphasisPosition:
    case CSSPropertyWebkitTextEmphasisStyle:
    case CSSPropertyWebkitTextFillColor:
    case CSSPropertyWebkitTextSecurity:
    case CSSPropertyWebkitTextStrokeColor:
    case CSSPropertyWebkitTextStrokeWidth:
    case CSSPropertyWebkitUserModify:
    case CSSPropertyWhiteSpace:
    case CSSPropertyWidows:
    case CSSPropertyWordBreak:
    case CSSPropertyWordSpacing:
    case CSSPropertyWordWrap:
    case CSSPropertyAliasEpubCaptionSide:
    case CSSPropertyAliasEpubTextCombine:
    case CSSPropertyAliasEpubTextEmphasisColor:
    case CSSPropertyAliasEpubTextEmphasisStyle:
    case CSSPropertyAliasEpubTextOrientation:
    case CSSPropertyAliasEpubTextTransform:
    case CSSPropertyAliasEpubWordBreak:
    case CSSPropertyAliasEpubWritingMode:
    case CSSPropertyAliasWebkitFontFeatureSettings:
    case CSSPropertyAliasWebkitTextSizeAdjust:
    case CSSPropertyAliasWebkitUserSelect:
    case CSSPropertyVariable:
      return true;
    default:
      return false;
  }
}

bool CSSPropertyMetadata::PropertySupportsPercentage(CSSPropertyID unresolvedProperty) {
  switch (unresolvedProperty) {
    case CSSPropertyInvalid:
      NOTREACHED();
      return false;
    case CSSPropertyBottom:
    case CSSPropertyHeight:
    case CSSPropertyLeft:
    case CSSPropertyRight:
    case CSSPropertyScrollPaddingBlockEnd:
    case CSSPropertyScrollPaddingBlockStart:
    case CSSPropertyScrollPaddingBottom:
    case CSSPropertyScrollPaddingInlineEnd:
    case CSSPropertyScrollPaddingInlineStart:
    case CSSPropertyScrollPaddingLeft:
    case CSSPropertyScrollPaddingRight:
    case CSSPropertyScrollPaddingTop:
    case CSSPropertyTop:
    case CSSPropertyWidth:
      return true;
    default:
      return false;
  }
}

char CSSPropertyMetadata::RepetitionSeparator(CSSPropertyID unresolvedProperty) {
  switch (unresolvedProperty) {
  case CSSPropertyAnimationDirection:
    return ',';
  case CSSPropertyAnimationIterationCount:
    return ',';
  case CSSPropertyContent:
    return ',';
  case CSSPropertyAliasWebkitAnimationDirection:
    return ',';
  case CSSPropertyAliasWebkitAnimationIterationCount:
    return ',';
  default:
    return 0;
  }
}

bool CSSPropertyMetadata::PropertyIsRepeated(CSSPropertyID unresolvedProperty) {
  return RepetitionSeparator(unresolvedProperty) != 0;
}

bool CSSPropertyMetadata::IsEnabledProperty(CSSPropertyID unresolvedProperty) {
  CSSPropertyID property = resolveCSSPropertyID(unresolvedProperty);
  static std::bitset<numCSSProperties>* enabledProperties = []() {
    std::bitset<numCSSProperties>* props = new std::bitset<numCSSProperties>();
    props->set(); // All bits sets to 1.
    if (!RuntimeEnabledFeatures::CSSFontSizeAdjustEnabled())
      props->reset(CSSPropertyFontSizeAdjust - 3);
    if (!RuntimeEnabledFeatures::CSSVariableFontsEnabled())
      props->reset(CSSPropertyFontVariationSettings - 3);
    if (!RuntimeEnabledFeatures::CSSBackdropFilterEnabled())
      props->reset(CSSPropertyBackdropFilter - 3);
    if (!RuntimeEnabledFeatures::CSSContainmentEnabled())
      props->reset(CSSPropertyContain - 3);
    if (!RuntimeEnabledFeatures::CSSFontDisplayEnabled())
      props->reset(CSSPropertyFontDisplay - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGrid - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridArea - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridAutoColumns - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridAutoFlow - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridAutoRows - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridColumn - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridColumnEnd - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridColumnGap - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridColumnStart - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridGap - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridRow - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridRowEnd - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridRowGap - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridRowStart - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridTemplate - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridTemplateAreas - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridTemplateColumns - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyGridTemplateRows - 3);
    if (!RuntimeEnabledFeatures::CSSHyphensEnabled())
      props->reset(CSSPropertyHyphens - 3);
    if (!RuntimeEnabledFeatures::ImageOrientationEnabled())
      props->reset(CSSPropertyImageOrientation - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyJustifyItems - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyJustifySelf - 3);
    if (!RuntimeEnabledFeatures::CSSSnapSizeEnabled())
      props->reset(CSSPropertyLineHeightStep - 3);
    if (!RuntimeEnabledFeatures::CSSMaskSourceTypeEnabled())
      props->reset(CSSPropertyMaskSourceType - 3);
    if (!RuntimeEnabledFeatures::CSSOffsetPositionAnchorEnabled())
      props->reset(CSSPropertyOffsetAnchor - 3);
    if (!RuntimeEnabledFeatures::CSSOffsetPositionAnchorEnabled())
      props->reset(CSSPropertyOffsetPosition - 3);
    if (!RuntimeEnabledFeatures::ScrollAnchoringEnabled())
      props->reset(CSSPropertyOverflowAnchor - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyPlaceContent - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyPlaceItems - 3);
    if (!RuntimeEnabledFeatures::CSSGridLayoutEnabled())
      props->reset(CSSPropertyPlaceSelf - 3);
    if (!RuntimeEnabledFeatures::CSSIndependentTransformPropertiesEnabled())
      props->reset(CSSPropertyRotate - 3);
    if (!RuntimeEnabledFeatures::CSSIndependentTransformPropertiesEnabled())
      props->reset(CSSPropertyScale - 3);
    if (!RuntimeEnabledFeatures::CSSOMSmoothScrollEnabled())
      props->reset(CSSPropertyScrollBehavior - 3);
    if (!RuntimeEnabledFeatures::CSSScrollBoundaryBehaviorEnabled())
      props->reset(CSSPropertyScrollBoundaryBehavior - 3);
    if (!RuntimeEnabledFeatures::CSSScrollBoundaryBehaviorEnabled())
      props->reset(CSSPropertyScrollBoundaryBehaviorX - 3);
    if (!RuntimeEnabledFeatures::CSSScrollBoundaryBehaviorEnabled())
      props->reset(CSSPropertyScrollBoundaryBehaviorY - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPadding - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingBlock - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingBlockEnd - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingBlockStart - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingBottom - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingInline - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingInlineEnd - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingInlineStart - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingLeft - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingRight - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollPaddingTop - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapAlign - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMargin - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginBlock - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginBlockEnd - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginBlockStart - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginBottom - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginInline - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginInlineEnd - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginInlineStart - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginLeft - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginRight - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapMarginTop - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapStop - 3);
    if (!RuntimeEnabledFeatures::CSSScrollSnapPointsEnabled())
      props->reset(CSSPropertyScrollSnapType - 3);
    if (!RuntimeEnabledFeatures::CSS3TextDecorationsEnabled())
      props->reset(CSSPropertyTextDecorationColor - 3);
    if (!RuntimeEnabledFeatures::CSS3TextDecorationsEnabled())
      props->reset(CSSPropertyTextDecorationLine - 3);
    if (!RuntimeEnabledFeatures::CSS3TextDecorationsEnabled())
      props->reset(CSSPropertyTextDecorationSkip - 3);
    if (!RuntimeEnabledFeatures::CSS3TextDecorationsEnabled())
      props->reset(CSSPropertyTextDecorationStyle - 3);
    if (!RuntimeEnabledFeatures::CSS3TextEnabled())
      props->reset(CSSPropertyTextJustify - 3);
    if (!RuntimeEnabledFeatures::CSS3TextDecorationsEnabled())
      props->reset(CSSPropertyTextUnderlinePosition - 3);
    if (!RuntimeEnabledFeatures::CSSTransformBoxEnabled())
      props->reset(CSSPropertyTransformBox - 3);
    if (!RuntimeEnabledFeatures::CSSIndependentTransformPropertiesEnabled())
      props->reset(CSSPropertyTranslate - 3);
    return props;
  }();

  if (unresolvedProperty >= 3)
    return enabledProperties->test(property - 3);

  if (unresolvedProperty == CSSPropertyVariable)
    return true;
  DCHECK_EQ(unresolvedProperty, CSSPropertyApplyAtRule);
  return RuntimeEnabledFeatures::CSSApplyAtRulesEnabled();
}

void CSSPropertyMetadata::FilterEnabledCSSPropertiesIntoVector(
    const CSSPropertyID* properties,
    size_t propertyCount,
    Vector<CSSPropertyID>& outVector) {
  for (unsigned i = 0; i < propertyCount; i++) {
    CSSPropertyID property = properties[i];
    if (IsEnabledProperty(property))
      outVector.push_back(property);
  }
}

} // namespace blink
