// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   templates/ElementTypeHelpers.h.tmpl
// and input files:
//   ../../../../src/3rdparty/chromium/third_party/WebKit/Source/core/svg/SVGTagNames.json5


#ifndef SVGElementTypeHelpers_h
#define SVGElementTypeHelpers_h

#include "core/svg/SVGElement.h"
#include "SVGNames.h"
#include "platform/RuntimeEnabledFeatures.h"

namespace blink {
// Type checking.
class SVGAElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGAElement(const SVGAElement&);
void isSVGAElement(const SVGAElement*);

inline bool isSVGAElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::aTag);
}
inline bool isSVGAElement(const SVGElement* element) {
  return element && isSVGAElement(*element);
}
inline bool isSVGAElement(const Node& node) {
  return node.IsSVGElement() && isSVGAElement(ToSVGElement(node));
}
inline bool isSVGAElement(const Node* node) {
  return node && isSVGAElement(*node);
}
template <>
inline bool IsElementOfType<const SVGAElement>(const Node& node) {
  return isSVGAElement(node);
}
template <>
inline bool IsElementOfType<const SVGAElement>(const SVGElement& element) {
  return isSVGAElement(element);
}

class SVGAnimateMotionElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGAnimateMotionElement(const SVGAnimateMotionElement&);
void isSVGAnimateMotionElement(const SVGAnimateMotionElement*);

inline bool isSVGAnimateMotionElement(const SVGElement& element) {
  if (!RuntimeEnabledFeatures::SMILEnabled())
    return false;
  return element.HasTagName(SVGNames::animateMotionTag);
}
inline bool isSVGAnimateMotionElement(const SVGElement* element) {
  return element && isSVGAnimateMotionElement(*element);
}
inline bool isSVGAnimateMotionElement(const Node& node) {
  return node.IsSVGElement() && isSVGAnimateMotionElement(ToSVGElement(node));
}
inline bool isSVGAnimateMotionElement(const Node* node) {
  return node && isSVGAnimateMotionElement(*node);
}
template <>
inline bool IsElementOfType<const SVGAnimateMotionElement>(const Node& node) {
  return isSVGAnimateMotionElement(node);
}
template <>
inline bool IsElementOfType<const SVGAnimateMotionElement>(const SVGElement& element) {
  return isSVGAnimateMotionElement(element);
}

class SVGAnimateTransformElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGAnimateTransformElement(const SVGAnimateTransformElement&);
void isSVGAnimateTransformElement(const SVGAnimateTransformElement*);

inline bool isSVGAnimateTransformElement(const SVGElement& element) {
  if (!RuntimeEnabledFeatures::SMILEnabled())
    return false;
  return element.HasTagName(SVGNames::animateTransformTag);
}
inline bool isSVGAnimateTransformElement(const SVGElement* element) {
  return element && isSVGAnimateTransformElement(*element);
}
inline bool isSVGAnimateTransformElement(const Node& node) {
  return node.IsSVGElement() && isSVGAnimateTransformElement(ToSVGElement(node));
}
inline bool isSVGAnimateTransformElement(const Node* node) {
  return node && isSVGAnimateTransformElement(*node);
}
template <>
inline bool IsElementOfType<const SVGAnimateTransformElement>(const Node& node) {
  return isSVGAnimateTransformElement(node);
}
template <>
inline bool IsElementOfType<const SVGAnimateTransformElement>(const SVGElement& element) {
  return isSVGAnimateTransformElement(element);
}

class SVGCircleElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGCircleElement(const SVGCircleElement&);
void isSVGCircleElement(const SVGCircleElement*);

inline bool isSVGCircleElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::circleTag);
}
inline bool isSVGCircleElement(const SVGElement* element) {
  return element && isSVGCircleElement(*element);
}
inline bool isSVGCircleElement(const Node& node) {
  return node.IsSVGElement() && isSVGCircleElement(ToSVGElement(node));
}
inline bool isSVGCircleElement(const Node* node) {
  return node && isSVGCircleElement(*node);
}
template <>
inline bool IsElementOfType<const SVGCircleElement>(const Node& node) {
  return isSVGCircleElement(node);
}
template <>
inline bool IsElementOfType<const SVGCircleElement>(const SVGElement& element) {
  return isSVGCircleElement(element);
}

class SVGClipPathElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGClipPathElement(const SVGClipPathElement&);
void isSVGClipPathElement(const SVGClipPathElement*);

inline bool isSVGClipPathElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::clipPathTag);
}
inline bool isSVGClipPathElement(const SVGElement* element) {
  return element && isSVGClipPathElement(*element);
}
inline bool isSVGClipPathElement(const Node& node) {
  return node.IsSVGElement() && isSVGClipPathElement(ToSVGElement(node));
}
inline bool isSVGClipPathElement(const Node* node) {
  return node && isSVGClipPathElement(*node);
}
template <>
inline bool IsElementOfType<const SVGClipPathElement>(const Node& node) {
  return isSVGClipPathElement(node);
}
template <>
inline bool IsElementOfType<const SVGClipPathElement>(const SVGElement& element) {
  return isSVGClipPathElement(element);
}

class SVGDefsElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGDefsElement(const SVGDefsElement&);
void isSVGDefsElement(const SVGDefsElement*);

inline bool isSVGDefsElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::defsTag);
}
inline bool isSVGDefsElement(const SVGElement* element) {
  return element && isSVGDefsElement(*element);
}
inline bool isSVGDefsElement(const Node& node) {
  return node.IsSVGElement() && isSVGDefsElement(ToSVGElement(node));
}
inline bool isSVGDefsElement(const Node* node) {
  return node && isSVGDefsElement(*node);
}
template <>
inline bool IsElementOfType<const SVGDefsElement>(const Node& node) {
  return isSVGDefsElement(node);
}
template <>
inline bool IsElementOfType<const SVGDefsElement>(const SVGElement& element) {
  return isSVGDefsElement(element);
}

class SVGDescElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGDescElement(const SVGDescElement&);
void isSVGDescElement(const SVGDescElement*);

inline bool isSVGDescElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::descTag);
}
inline bool isSVGDescElement(const SVGElement* element) {
  return element && isSVGDescElement(*element);
}
inline bool isSVGDescElement(const Node& node) {
  return node.IsSVGElement() && isSVGDescElement(ToSVGElement(node));
}
inline bool isSVGDescElement(const Node* node) {
  return node && isSVGDescElement(*node);
}
template <>
inline bool IsElementOfType<const SVGDescElement>(const Node& node) {
  return isSVGDescElement(node);
}
template <>
inline bool IsElementOfType<const SVGDescElement>(const SVGElement& element) {
  return isSVGDescElement(element);
}

class SVGDiscardElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGDiscardElement(const SVGDiscardElement&);
void isSVGDiscardElement(const SVGDiscardElement*);

inline bool isSVGDiscardElement(const SVGElement& element) {
  if (!RuntimeEnabledFeatures::SMILEnabled())
    return false;
  return element.HasTagName(SVGNames::discardTag);
}
inline bool isSVGDiscardElement(const SVGElement* element) {
  return element && isSVGDiscardElement(*element);
}
inline bool isSVGDiscardElement(const Node& node) {
  return node.IsSVGElement() && isSVGDiscardElement(ToSVGElement(node));
}
inline bool isSVGDiscardElement(const Node* node) {
  return node && isSVGDiscardElement(*node);
}
template <>
inline bool IsElementOfType<const SVGDiscardElement>(const Node& node) {
  return isSVGDiscardElement(node);
}
template <>
inline bool IsElementOfType<const SVGDiscardElement>(const SVGElement& element) {
  return isSVGDiscardElement(element);
}

class SVGEllipseElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGEllipseElement(const SVGEllipseElement&);
void isSVGEllipseElement(const SVGEllipseElement*);

inline bool isSVGEllipseElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::ellipseTag);
}
inline bool isSVGEllipseElement(const SVGElement* element) {
  return element && isSVGEllipseElement(*element);
}
inline bool isSVGEllipseElement(const Node& node) {
  return node.IsSVGElement() && isSVGEllipseElement(ToSVGElement(node));
}
inline bool isSVGEllipseElement(const Node* node) {
  return node && isSVGEllipseElement(*node);
}
template <>
inline bool IsElementOfType<const SVGEllipseElement>(const Node& node) {
  return isSVGEllipseElement(node);
}
template <>
inline bool IsElementOfType<const SVGEllipseElement>(const SVGElement& element) {
  return isSVGEllipseElement(element);
}

class SVGFEBlendElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEBlendElement(const SVGFEBlendElement&);
void isSVGFEBlendElement(const SVGFEBlendElement*);

inline bool isSVGFEBlendElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feBlendTag);
}
inline bool isSVGFEBlendElement(const SVGElement* element) {
  return element && isSVGFEBlendElement(*element);
}
inline bool isSVGFEBlendElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEBlendElement(ToSVGElement(node));
}
inline bool isSVGFEBlendElement(const Node* node) {
  return node && isSVGFEBlendElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEBlendElement>(const Node& node) {
  return isSVGFEBlendElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEBlendElement>(const SVGElement& element) {
  return isSVGFEBlendElement(element);
}

class SVGFEColorMatrixElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEColorMatrixElement(const SVGFEColorMatrixElement&);
void isSVGFEColorMatrixElement(const SVGFEColorMatrixElement*);

inline bool isSVGFEColorMatrixElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feColorMatrixTag);
}
inline bool isSVGFEColorMatrixElement(const SVGElement* element) {
  return element && isSVGFEColorMatrixElement(*element);
}
inline bool isSVGFEColorMatrixElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEColorMatrixElement(ToSVGElement(node));
}
inline bool isSVGFEColorMatrixElement(const Node* node) {
  return node && isSVGFEColorMatrixElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEColorMatrixElement>(const Node& node) {
  return isSVGFEColorMatrixElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEColorMatrixElement>(const SVGElement& element) {
  return isSVGFEColorMatrixElement(element);
}

class SVGFEComponentTransferElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEComponentTransferElement(const SVGFEComponentTransferElement&);
void isSVGFEComponentTransferElement(const SVGFEComponentTransferElement*);

inline bool isSVGFEComponentTransferElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feComponentTransferTag);
}
inline bool isSVGFEComponentTransferElement(const SVGElement* element) {
  return element && isSVGFEComponentTransferElement(*element);
}
inline bool isSVGFEComponentTransferElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEComponentTransferElement(ToSVGElement(node));
}
inline bool isSVGFEComponentTransferElement(const Node* node) {
  return node && isSVGFEComponentTransferElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEComponentTransferElement>(const Node& node) {
  return isSVGFEComponentTransferElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEComponentTransferElement>(const SVGElement& element) {
  return isSVGFEComponentTransferElement(element);
}

class SVGFECompositeElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFECompositeElement(const SVGFECompositeElement&);
void isSVGFECompositeElement(const SVGFECompositeElement*);

inline bool isSVGFECompositeElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feCompositeTag);
}
inline bool isSVGFECompositeElement(const SVGElement* element) {
  return element && isSVGFECompositeElement(*element);
}
inline bool isSVGFECompositeElement(const Node& node) {
  return node.IsSVGElement() && isSVGFECompositeElement(ToSVGElement(node));
}
inline bool isSVGFECompositeElement(const Node* node) {
  return node && isSVGFECompositeElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFECompositeElement>(const Node& node) {
  return isSVGFECompositeElement(node);
}
template <>
inline bool IsElementOfType<const SVGFECompositeElement>(const SVGElement& element) {
  return isSVGFECompositeElement(element);
}

class SVGFEConvolveMatrixElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEConvolveMatrixElement(const SVGFEConvolveMatrixElement&);
void isSVGFEConvolveMatrixElement(const SVGFEConvolveMatrixElement*);

inline bool isSVGFEConvolveMatrixElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feConvolveMatrixTag);
}
inline bool isSVGFEConvolveMatrixElement(const SVGElement* element) {
  return element && isSVGFEConvolveMatrixElement(*element);
}
inline bool isSVGFEConvolveMatrixElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEConvolveMatrixElement(ToSVGElement(node));
}
inline bool isSVGFEConvolveMatrixElement(const Node* node) {
  return node && isSVGFEConvolveMatrixElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEConvolveMatrixElement>(const Node& node) {
  return isSVGFEConvolveMatrixElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEConvolveMatrixElement>(const SVGElement& element) {
  return isSVGFEConvolveMatrixElement(element);
}

class SVGFEDiffuseLightingElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEDiffuseLightingElement(const SVGFEDiffuseLightingElement&);
void isSVGFEDiffuseLightingElement(const SVGFEDiffuseLightingElement*);

inline bool isSVGFEDiffuseLightingElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feDiffuseLightingTag);
}
inline bool isSVGFEDiffuseLightingElement(const SVGElement* element) {
  return element && isSVGFEDiffuseLightingElement(*element);
}
inline bool isSVGFEDiffuseLightingElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEDiffuseLightingElement(ToSVGElement(node));
}
inline bool isSVGFEDiffuseLightingElement(const Node* node) {
  return node && isSVGFEDiffuseLightingElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEDiffuseLightingElement>(const Node& node) {
  return isSVGFEDiffuseLightingElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEDiffuseLightingElement>(const SVGElement& element) {
  return isSVGFEDiffuseLightingElement(element);
}

class SVGFEDisplacementMapElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEDisplacementMapElement(const SVGFEDisplacementMapElement&);
void isSVGFEDisplacementMapElement(const SVGFEDisplacementMapElement*);

inline bool isSVGFEDisplacementMapElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feDisplacementMapTag);
}
inline bool isSVGFEDisplacementMapElement(const SVGElement* element) {
  return element && isSVGFEDisplacementMapElement(*element);
}
inline bool isSVGFEDisplacementMapElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEDisplacementMapElement(ToSVGElement(node));
}
inline bool isSVGFEDisplacementMapElement(const Node* node) {
  return node && isSVGFEDisplacementMapElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEDisplacementMapElement>(const Node& node) {
  return isSVGFEDisplacementMapElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEDisplacementMapElement>(const SVGElement& element) {
  return isSVGFEDisplacementMapElement(element);
}

class SVGFEDistantLightElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEDistantLightElement(const SVGFEDistantLightElement&);
void isSVGFEDistantLightElement(const SVGFEDistantLightElement*);

inline bool isSVGFEDistantLightElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feDistantLightTag);
}
inline bool isSVGFEDistantLightElement(const SVGElement* element) {
  return element && isSVGFEDistantLightElement(*element);
}
inline bool isSVGFEDistantLightElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEDistantLightElement(ToSVGElement(node));
}
inline bool isSVGFEDistantLightElement(const Node* node) {
  return node && isSVGFEDistantLightElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEDistantLightElement>(const Node& node) {
  return isSVGFEDistantLightElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEDistantLightElement>(const SVGElement& element) {
  return isSVGFEDistantLightElement(element);
}

class SVGFEDropShadowElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEDropShadowElement(const SVGFEDropShadowElement&);
void isSVGFEDropShadowElement(const SVGFEDropShadowElement*);

inline bool isSVGFEDropShadowElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feDropShadowTag);
}
inline bool isSVGFEDropShadowElement(const SVGElement* element) {
  return element && isSVGFEDropShadowElement(*element);
}
inline bool isSVGFEDropShadowElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEDropShadowElement(ToSVGElement(node));
}
inline bool isSVGFEDropShadowElement(const Node* node) {
  return node && isSVGFEDropShadowElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEDropShadowElement>(const Node& node) {
  return isSVGFEDropShadowElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEDropShadowElement>(const SVGElement& element) {
  return isSVGFEDropShadowElement(element);
}

class SVGFEFloodElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEFloodElement(const SVGFEFloodElement&);
void isSVGFEFloodElement(const SVGFEFloodElement*);

inline bool isSVGFEFloodElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feFloodTag);
}
inline bool isSVGFEFloodElement(const SVGElement* element) {
  return element && isSVGFEFloodElement(*element);
}
inline bool isSVGFEFloodElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEFloodElement(ToSVGElement(node));
}
inline bool isSVGFEFloodElement(const Node* node) {
  return node && isSVGFEFloodElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFloodElement>(const Node& node) {
  return isSVGFEFloodElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFloodElement>(const SVGElement& element) {
  return isSVGFEFloodElement(element);
}

class SVGFEFuncAElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEFuncAElement(const SVGFEFuncAElement&);
void isSVGFEFuncAElement(const SVGFEFuncAElement*);

inline bool isSVGFEFuncAElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feFuncATag);
}
inline bool isSVGFEFuncAElement(const SVGElement* element) {
  return element && isSVGFEFuncAElement(*element);
}
inline bool isSVGFEFuncAElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEFuncAElement(ToSVGElement(node));
}
inline bool isSVGFEFuncAElement(const Node* node) {
  return node && isSVGFEFuncAElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncAElement>(const Node& node) {
  return isSVGFEFuncAElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncAElement>(const SVGElement& element) {
  return isSVGFEFuncAElement(element);
}

class SVGFEFuncBElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEFuncBElement(const SVGFEFuncBElement&);
void isSVGFEFuncBElement(const SVGFEFuncBElement*);

inline bool isSVGFEFuncBElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feFuncBTag);
}
inline bool isSVGFEFuncBElement(const SVGElement* element) {
  return element && isSVGFEFuncBElement(*element);
}
inline bool isSVGFEFuncBElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEFuncBElement(ToSVGElement(node));
}
inline bool isSVGFEFuncBElement(const Node* node) {
  return node && isSVGFEFuncBElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncBElement>(const Node& node) {
  return isSVGFEFuncBElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncBElement>(const SVGElement& element) {
  return isSVGFEFuncBElement(element);
}

class SVGFEFuncGElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEFuncGElement(const SVGFEFuncGElement&);
void isSVGFEFuncGElement(const SVGFEFuncGElement*);

inline bool isSVGFEFuncGElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feFuncGTag);
}
inline bool isSVGFEFuncGElement(const SVGElement* element) {
  return element && isSVGFEFuncGElement(*element);
}
inline bool isSVGFEFuncGElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEFuncGElement(ToSVGElement(node));
}
inline bool isSVGFEFuncGElement(const Node* node) {
  return node && isSVGFEFuncGElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncGElement>(const Node& node) {
  return isSVGFEFuncGElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncGElement>(const SVGElement& element) {
  return isSVGFEFuncGElement(element);
}

class SVGFEFuncRElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEFuncRElement(const SVGFEFuncRElement&);
void isSVGFEFuncRElement(const SVGFEFuncRElement*);

inline bool isSVGFEFuncRElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feFuncRTag);
}
inline bool isSVGFEFuncRElement(const SVGElement* element) {
  return element && isSVGFEFuncRElement(*element);
}
inline bool isSVGFEFuncRElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEFuncRElement(ToSVGElement(node));
}
inline bool isSVGFEFuncRElement(const Node* node) {
  return node && isSVGFEFuncRElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncRElement>(const Node& node) {
  return isSVGFEFuncRElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncRElement>(const SVGElement& element) {
  return isSVGFEFuncRElement(element);
}

class SVGFEGaussianBlurElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEGaussianBlurElement(const SVGFEGaussianBlurElement&);
void isSVGFEGaussianBlurElement(const SVGFEGaussianBlurElement*);

inline bool isSVGFEGaussianBlurElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feGaussianBlurTag);
}
inline bool isSVGFEGaussianBlurElement(const SVGElement* element) {
  return element && isSVGFEGaussianBlurElement(*element);
}
inline bool isSVGFEGaussianBlurElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEGaussianBlurElement(ToSVGElement(node));
}
inline bool isSVGFEGaussianBlurElement(const Node* node) {
  return node && isSVGFEGaussianBlurElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEGaussianBlurElement>(const Node& node) {
  return isSVGFEGaussianBlurElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEGaussianBlurElement>(const SVGElement& element) {
  return isSVGFEGaussianBlurElement(element);
}

class SVGFEImageElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEImageElement(const SVGFEImageElement&);
void isSVGFEImageElement(const SVGFEImageElement*);

inline bool isSVGFEImageElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feImageTag);
}
inline bool isSVGFEImageElement(const SVGElement* element) {
  return element && isSVGFEImageElement(*element);
}
inline bool isSVGFEImageElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEImageElement(ToSVGElement(node));
}
inline bool isSVGFEImageElement(const Node* node) {
  return node && isSVGFEImageElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEImageElement>(const Node& node) {
  return isSVGFEImageElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEImageElement>(const SVGElement& element) {
  return isSVGFEImageElement(element);
}

class SVGFEMergeElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEMergeElement(const SVGFEMergeElement&);
void isSVGFEMergeElement(const SVGFEMergeElement*);

inline bool isSVGFEMergeElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feMergeTag);
}
inline bool isSVGFEMergeElement(const SVGElement* element) {
  return element && isSVGFEMergeElement(*element);
}
inline bool isSVGFEMergeElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEMergeElement(ToSVGElement(node));
}
inline bool isSVGFEMergeElement(const Node* node) {
  return node && isSVGFEMergeElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEMergeElement>(const Node& node) {
  return isSVGFEMergeElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEMergeElement>(const SVGElement& element) {
  return isSVGFEMergeElement(element);
}

class SVGFEMergeNodeElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEMergeNodeElement(const SVGFEMergeNodeElement&);
void isSVGFEMergeNodeElement(const SVGFEMergeNodeElement*);

inline bool isSVGFEMergeNodeElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feMergeNodeTag);
}
inline bool isSVGFEMergeNodeElement(const SVGElement* element) {
  return element && isSVGFEMergeNodeElement(*element);
}
inline bool isSVGFEMergeNodeElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEMergeNodeElement(ToSVGElement(node));
}
inline bool isSVGFEMergeNodeElement(const Node* node) {
  return node && isSVGFEMergeNodeElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEMergeNodeElement>(const Node& node) {
  return isSVGFEMergeNodeElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEMergeNodeElement>(const SVGElement& element) {
  return isSVGFEMergeNodeElement(element);
}

class SVGFEMorphologyElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEMorphologyElement(const SVGFEMorphologyElement&);
void isSVGFEMorphologyElement(const SVGFEMorphologyElement*);

inline bool isSVGFEMorphologyElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feMorphologyTag);
}
inline bool isSVGFEMorphologyElement(const SVGElement* element) {
  return element && isSVGFEMorphologyElement(*element);
}
inline bool isSVGFEMorphologyElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEMorphologyElement(ToSVGElement(node));
}
inline bool isSVGFEMorphologyElement(const Node* node) {
  return node && isSVGFEMorphologyElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEMorphologyElement>(const Node& node) {
  return isSVGFEMorphologyElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEMorphologyElement>(const SVGElement& element) {
  return isSVGFEMorphologyElement(element);
}

class SVGFEOffsetElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEOffsetElement(const SVGFEOffsetElement&);
void isSVGFEOffsetElement(const SVGFEOffsetElement*);

inline bool isSVGFEOffsetElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feOffsetTag);
}
inline bool isSVGFEOffsetElement(const SVGElement* element) {
  return element && isSVGFEOffsetElement(*element);
}
inline bool isSVGFEOffsetElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEOffsetElement(ToSVGElement(node));
}
inline bool isSVGFEOffsetElement(const Node* node) {
  return node && isSVGFEOffsetElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEOffsetElement>(const Node& node) {
  return isSVGFEOffsetElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEOffsetElement>(const SVGElement& element) {
  return isSVGFEOffsetElement(element);
}

class SVGFEPointLightElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFEPointLightElement(const SVGFEPointLightElement&);
void isSVGFEPointLightElement(const SVGFEPointLightElement*);

inline bool isSVGFEPointLightElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::fePointLightTag);
}
inline bool isSVGFEPointLightElement(const SVGElement* element) {
  return element && isSVGFEPointLightElement(*element);
}
inline bool isSVGFEPointLightElement(const Node& node) {
  return node.IsSVGElement() && isSVGFEPointLightElement(ToSVGElement(node));
}
inline bool isSVGFEPointLightElement(const Node* node) {
  return node && isSVGFEPointLightElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEPointLightElement>(const Node& node) {
  return isSVGFEPointLightElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEPointLightElement>(const SVGElement& element) {
  return isSVGFEPointLightElement(element);
}

class SVGFESpecularLightingElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFESpecularLightingElement(const SVGFESpecularLightingElement&);
void isSVGFESpecularLightingElement(const SVGFESpecularLightingElement*);

inline bool isSVGFESpecularLightingElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feSpecularLightingTag);
}
inline bool isSVGFESpecularLightingElement(const SVGElement* element) {
  return element && isSVGFESpecularLightingElement(*element);
}
inline bool isSVGFESpecularLightingElement(const Node& node) {
  return node.IsSVGElement() && isSVGFESpecularLightingElement(ToSVGElement(node));
}
inline bool isSVGFESpecularLightingElement(const Node* node) {
  return node && isSVGFESpecularLightingElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFESpecularLightingElement>(const Node& node) {
  return isSVGFESpecularLightingElement(node);
}
template <>
inline bool IsElementOfType<const SVGFESpecularLightingElement>(const SVGElement& element) {
  return isSVGFESpecularLightingElement(element);
}

class SVGFESpotLightElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFESpotLightElement(const SVGFESpotLightElement&);
void isSVGFESpotLightElement(const SVGFESpotLightElement*);

inline bool isSVGFESpotLightElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feSpotLightTag);
}
inline bool isSVGFESpotLightElement(const SVGElement* element) {
  return element && isSVGFESpotLightElement(*element);
}
inline bool isSVGFESpotLightElement(const Node& node) {
  return node.IsSVGElement() && isSVGFESpotLightElement(ToSVGElement(node));
}
inline bool isSVGFESpotLightElement(const Node* node) {
  return node && isSVGFESpotLightElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFESpotLightElement>(const Node& node) {
  return isSVGFESpotLightElement(node);
}
template <>
inline bool IsElementOfType<const SVGFESpotLightElement>(const SVGElement& element) {
  return isSVGFESpotLightElement(element);
}

class SVGFETileElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFETileElement(const SVGFETileElement&);
void isSVGFETileElement(const SVGFETileElement*);

inline bool isSVGFETileElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feTileTag);
}
inline bool isSVGFETileElement(const SVGElement* element) {
  return element && isSVGFETileElement(*element);
}
inline bool isSVGFETileElement(const Node& node) {
  return node.IsSVGElement() && isSVGFETileElement(ToSVGElement(node));
}
inline bool isSVGFETileElement(const Node* node) {
  return node && isSVGFETileElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFETileElement>(const Node& node) {
  return isSVGFETileElement(node);
}
template <>
inline bool IsElementOfType<const SVGFETileElement>(const SVGElement& element) {
  return isSVGFETileElement(element);
}

class SVGFETurbulenceElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFETurbulenceElement(const SVGFETurbulenceElement&);
void isSVGFETurbulenceElement(const SVGFETurbulenceElement*);

inline bool isSVGFETurbulenceElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::feTurbulenceTag);
}
inline bool isSVGFETurbulenceElement(const SVGElement* element) {
  return element && isSVGFETurbulenceElement(*element);
}
inline bool isSVGFETurbulenceElement(const Node& node) {
  return node.IsSVGElement() && isSVGFETurbulenceElement(ToSVGElement(node));
}
inline bool isSVGFETurbulenceElement(const Node* node) {
  return node && isSVGFETurbulenceElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFETurbulenceElement>(const Node& node) {
  return isSVGFETurbulenceElement(node);
}
template <>
inline bool IsElementOfType<const SVGFETurbulenceElement>(const SVGElement& element) {
  return isSVGFETurbulenceElement(element);
}

class SVGFilterElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGFilterElement(const SVGFilterElement&);
void isSVGFilterElement(const SVGFilterElement*);

inline bool isSVGFilterElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::filterTag);
}
inline bool isSVGFilterElement(const SVGElement* element) {
  return element && isSVGFilterElement(*element);
}
inline bool isSVGFilterElement(const Node& node) {
  return node.IsSVGElement() && isSVGFilterElement(ToSVGElement(node));
}
inline bool isSVGFilterElement(const Node* node) {
  return node && isSVGFilterElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFilterElement>(const Node& node) {
  return isSVGFilterElement(node);
}
template <>
inline bool IsElementOfType<const SVGFilterElement>(const SVGElement& element) {
  return isSVGFilterElement(element);
}

class SVGForeignObjectElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGForeignObjectElement(const SVGForeignObjectElement&);
void isSVGForeignObjectElement(const SVGForeignObjectElement*);

inline bool isSVGForeignObjectElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::foreignObjectTag);
}
inline bool isSVGForeignObjectElement(const SVGElement* element) {
  return element && isSVGForeignObjectElement(*element);
}
inline bool isSVGForeignObjectElement(const Node& node) {
  return node.IsSVGElement() && isSVGForeignObjectElement(ToSVGElement(node));
}
inline bool isSVGForeignObjectElement(const Node* node) {
  return node && isSVGForeignObjectElement(*node);
}
template <>
inline bool IsElementOfType<const SVGForeignObjectElement>(const Node& node) {
  return isSVGForeignObjectElement(node);
}
template <>
inline bool IsElementOfType<const SVGForeignObjectElement>(const SVGElement& element) {
  return isSVGForeignObjectElement(element);
}

class SVGGElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGGElement(const SVGGElement&);
void isSVGGElement(const SVGGElement*);

inline bool isSVGGElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::gTag);
}
inline bool isSVGGElement(const SVGElement* element) {
  return element && isSVGGElement(*element);
}
inline bool isSVGGElement(const Node& node) {
  return node.IsSVGElement() && isSVGGElement(ToSVGElement(node));
}
inline bool isSVGGElement(const Node* node) {
  return node && isSVGGElement(*node);
}
template <>
inline bool IsElementOfType<const SVGGElement>(const Node& node) {
  return isSVGGElement(node);
}
template <>
inline bool IsElementOfType<const SVGGElement>(const SVGElement& element) {
  return isSVGGElement(element);
}

class SVGImageElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGImageElement(const SVGImageElement&);
void isSVGImageElement(const SVGImageElement*);

inline bool isSVGImageElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::imageTag);
}
inline bool isSVGImageElement(const SVGElement* element) {
  return element && isSVGImageElement(*element);
}
inline bool isSVGImageElement(const Node& node) {
  return node.IsSVGElement() && isSVGImageElement(ToSVGElement(node));
}
inline bool isSVGImageElement(const Node* node) {
  return node && isSVGImageElement(*node);
}
template <>
inline bool IsElementOfType<const SVGImageElement>(const Node& node) {
  return isSVGImageElement(node);
}
template <>
inline bool IsElementOfType<const SVGImageElement>(const SVGElement& element) {
  return isSVGImageElement(element);
}

class SVGLineElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGLineElement(const SVGLineElement&);
void isSVGLineElement(const SVGLineElement*);

inline bool isSVGLineElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::lineTag);
}
inline bool isSVGLineElement(const SVGElement* element) {
  return element && isSVGLineElement(*element);
}
inline bool isSVGLineElement(const Node& node) {
  return node.IsSVGElement() && isSVGLineElement(ToSVGElement(node));
}
inline bool isSVGLineElement(const Node* node) {
  return node && isSVGLineElement(*node);
}
template <>
inline bool IsElementOfType<const SVGLineElement>(const Node& node) {
  return isSVGLineElement(node);
}
template <>
inline bool IsElementOfType<const SVGLineElement>(const SVGElement& element) {
  return isSVGLineElement(element);
}

class SVGLinearGradientElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGLinearGradientElement(const SVGLinearGradientElement&);
void isSVGLinearGradientElement(const SVGLinearGradientElement*);

inline bool isSVGLinearGradientElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::linearGradientTag);
}
inline bool isSVGLinearGradientElement(const SVGElement* element) {
  return element && isSVGLinearGradientElement(*element);
}
inline bool isSVGLinearGradientElement(const Node& node) {
  return node.IsSVGElement() && isSVGLinearGradientElement(ToSVGElement(node));
}
inline bool isSVGLinearGradientElement(const Node* node) {
  return node && isSVGLinearGradientElement(*node);
}
template <>
inline bool IsElementOfType<const SVGLinearGradientElement>(const Node& node) {
  return isSVGLinearGradientElement(node);
}
template <>
inline bool IsElementOfType<const SVGLinearGradientElement>(const SVGElement& element) {
  return isSVGLinearGradientElement(element);
}

class SVGMPathElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGMPathElement(const SVGMPathElement&);
void isSVGMPathElement(const SVGMPathElement*);

inline bool isSVGMPathElement(const SVGElement& element) {
  if (!RuntimeEnabledFeatures::SMILEnabled())
    return false;
  return element.HasTagName(SVGNames::mpathTag);
}
inline bool isSVGMPathElement(const SVGElement* element) {
  return element && isSVGMPathElement(*element);
}
inline bool isSVGMPathElement(const Node& node) {
  return node.IsSVGElement() && isSVGMPathElement(ToSVGElement(node));
}
inline bool isSVGMPathElement(const Node* node) {
  return node && isSVGMPathElement(*node);
}
template <>
inline bool IsElementOfType<const SVGMPathElement>(const Node& node) {
  return isSVGMPathElement(node);
}
template <>
inline bool IsElementOfType<const SVGMPathElement>(const SVGElement& element) {
  return isSVGMPathElement(element);
}

class SVGMarkerElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGMarkerElement(const SVGMarkerElement&);
void isSVGMarkerElement(const SVGMarkerElement*);

inline bool isSVGMarkerElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::markerTag);
}
inline bool isSVGMarkerElement(const SVGElement* element) {
  return element && isSVGMarkerElement(*element);
}
inline bool isSVGMarkerElement(const Node& node) {
  return node.IsSVGElement() && isSVGMarkerElement(ToSVGElement(node));
}
inline bool isSVGMarkerElement(const Node* node) {
  return node && isSVGMarkerElement(*node);
}
template <>
inline bool IsElementOfType<const SVGMarkerElement>(const Node& node) {
  return isSVGMarkerElement(node);
}
template <>
inline bool IsElementOfType<const SVGMarkerElement>(const SVGElement& element) {
  return isSVGMarkerElement(element);
}

class SVGMaskElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGMaskElement(const SVGMaskElement&);
void isSVGMaskElement(const SVGMaskElement*);

inline bool isSVGMaskElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::maskTag);
}
inline bool isSVGMaskElement(const SVGElement* element) {
  return element && isSVGMaskElement(*element);
}
inline bool isSVGMaskElement(const Node& node) {
  return node.IsSVGElement() && isSVGMaskElement(ToSVGElement(node));
}
inline bool isSVGMaskElement(const Node* node) {
  return node && isSVGMaskElement(*node);
}
template <>
inline bool IsElementOfType<const SVGMaskElement>(const Node& node) {
  return isSVGMaskElement(node);
}
template <>
inline bool IsElementOfType<const SVGMaskElement>(const SVGElement& element) {
  return isSVGMaskElement(element);
}

class SVGMetadataElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGMetadataElement(const SVGMetadataElement&);
void isSVGMetadataElement(const SVGMetadataElement*);

inline bool isSVGMetadataElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::metadataTag);
}
inline bool isSVGMetadataElement(const SVGElement* element) {
  return element && isSVGMetadataElement(*element);
}
inline bool isSVGMetadataElement(const Node& node) {
  return node.IsSVGElement() && isSVGMetadataElement(ToSVGElement(node));
}
inline bool isSVGMetadataElement(const Node* node) {
  return node && isSVGMetadataElement(*node);
}
template <>
inline bool IsElementOfType<const SVGMetadataElement>(const Node& node) {
  return isSVGMetadataElement(node);
}
template <>
inline bool IsElementOfType<const SVGMetadataElement>(const SVGElement& element) {
  return isSVGMetadataElement(element);
}

class SVGPathElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGPathElement(const SVGPathElement&);
void isSVGPathElement(const SVGPathElement*);

inline bool isSVGPathElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::pathTag);
}
inline bool isSVGPathElement(const SVGElement* element) {
  return element && isSVGPathElement(*element);
}
inline bool isSVGPathElement(const Node& node) {
  return node.IsSVGElement() && isSVGPathElement(ToSVGElement(node));
}
inline bool isSVGPathElement(const Node* node) {
  return node && isSVGPathElement(*node);
}
template <>
inline bool IsElementOfType<const SVGPathElement>(const Node& node) {
  return isSVGPathElement(node);
}
template <>
inline bool IsElementOfType<const SVGPathElement>(const SVGElement& element) {
  return isSVGPathElement(element);
}

class SVGPatternElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGPatternElement(const SVGPatternElement&);
void isSVGPatternElement(const SVGPatternElement*);

inline bool isSVGPatternElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::patternTag);
}
inline bool isSVGPatternElement(const SVGElement* element) {
  return element && isSVGPatternElement(*element);
}
inline bool isSVGPatternElement(const Node& node) {
  return node.IsSVGElement() && isSVGPatternElement(ToSVGElement(node));
}
inline bool isSVGPatternElement(const Node* node) {
  return node && isSVGPatternElement(*node);
}
template <>
inline bool IsElementOfType<const SVGPatternElement>(const Node& node) {
  return isSVGPatternElement(node);
}
template <>
inline bool IsElementOfType<const SVGPatternElement>(const SVGElement& element) {
  return isSVGPatternElement(element);
}

class SVGPolygonElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGPolygonElement(const SVGPolygonElement&);
void isSVGPolygonElement(const SVGPolygonElement*);

inline bool isSVGPolygonElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::polygonTag);
}
inline bool isSVGPolygonElement(const SVGElement* element) {
  return element && isSVGPolygonElement(*element);
}
inline bool isSVGPolygonElement(const Node& node) {
  return node.IsSVGElement() && isSVGPolygonElement(ToSVGElement(node));
}
inline bool isSVGPolygonElement(const Node* node) {
  return node && isSVGPolygonElement(*node);
}
template <>
inline bool IsElementOfType<const SVGPolygonElement>(const Node& node) {
  return isSVGPolygonElement(node);
}
template <>
inline bool IsElementOfType<const SVGPolygonElement>(const SVGElement& element) {
  return isSVGPolygonElement(element);
}

class SVGPolylineElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGPolylineElement(const SVGPolylineElement&);
void isSVGPolylineElement(const SVGPolylineElement*);

inline bool isSVGPolylineElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::polylineTag);
}
inline bool isSVGPolylineElement(const SVGElement* element) {
  return element && isSVGPolylineElement(*element);
}
inline bool isSVGPolylineElement(const Node& node) {
  return node.IsSVGElement() && isSVGPolylineElement(ToSVGElement(node));
}
inline bool isSVGPolylineElement(const Node* node) {
  return node && isSVGPolylineElement(*node);
}
template <>
inline bool IsElementOfType<const SVGPolylineElement>(const Node& node) {
  return isSVGPolylineElement(node);
}
template <>
inline bool IsElementOfType<const SVGPolylineElement>(const SVGElement& element) {
  return isSVGPolylineElement(element);
}

class SVGRadialGradientElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGRadialGradientElement(const SVGRadialGradientElement&);
void isSVGRadialGradientElement(const SVGRadialGradientElement*);

inline bool isSVGRadialGradientElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::radialGradientTag);
}
inline bool isSVGRadialGradientElement(const SVGElement* element) {
  return element && isSVGRadialGradientElement(*element);
}
inline bool isSVGRadialGradientElement(const Node& node) {
  return node.IsSVGElement() && isSVGRadialGradientElement(ToSVGElement(node));
}
inline bool isSVGRadialGradientElement(const Node* node) {
  return node && isSVGRadialGradientElement(*node);
}
template <>
inline bool IsElementOfType<const SVGRadialGradientElement>(const Node& node) {
  return isSVGRadialGradientElement(node);
}
template <>
inline bool IsElementOfType<const SVGRadialGradientElement>(const SVGElement& element) {
  return isSVGRadialGradientElement(element);
}

class SVGRectElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGRectElement(const SVGRectElement&);
void isSVGRectElement(const SVGRectElement*);

inline bool isSVGRectElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::rectTag);
}
inline bool isSVGRectElement(const SVGElement* element) {
  return element && isSVGRectElement(*element);
}
inline bool isSVGRectElement(const Node& node) {
  return node.IsSVGElement() && isSVGRectElement(ToSVGElement(node));
}
inline bool isSVGRectElement(const Node* node) {
  return node && isSVGRectElement(*node);
}
template <>
inline bool IsElementOfType<const SVGRectElement>(const Node& node) {
  return isSVGRectElement(node);
}
template <>
inline bool IsElementOfType<const SVGRectElement>(const SVGElement& element) {
  return isSVGRectElement(element);
}

class SVGSVGElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGSVGElement(const SVGSVGElement&);
void isSVGSVGElement(const SVGSVGElement*);

inline bool isSVGSVGElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::svgTag);
}
inline bool isSVGSVGElement(const SVGElement* element) {
  return element && isSVGSVGElement(*element);
}
inline bool isSVGSVGElement(const Node& node) {
  return node.IsSVGElement() && isSVGSVGElement(ToSVGElement(node));
}
inline bool isSVGSVGElement(const Node* node) {
  return node && isSVGSVGElement(*node);
}
template <>
inline bool IsElementOfType<const SVGSVGElement>(const Node& node) {
  return isSVGSVGElement(node);
}
template <>
inline bool IsElementOfType<const SVGSVGElement>(const SVGElement& element) {
  return isSVGSVGElement(element);
}

class SVGSetElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGSetElement(const SVGSetElement&);
void isSVGSetElement(const SVGSetElement*);

inline bool isSVGSetElement(const SVGElement& element) {
  if (!RuntimeEnabledFeatures::SMILEnabled())
    return false;
  return element.HasTagName(SVGNames::setTag);
}
inline bool isSVGSetElement(const SVGElement* element) {
  return element && isSVGSetElement(*element);
}
inline bool isSVGSetElement(const Node& node) {
  return node.IsSVGElement() && isSVGSetElement(ToSVGElement(node));
}
inline bool isSVGSetElement(const Node* node) {
  return node && isSVGSetElement(*node);
}
template <>
inline bool IsElementOfType<const SVGSetElement>(const Node& node) {
  return isSVGSetElement(node);
}
template <>
inline bool IsElementOfType<const SVGSetElement>(const SVGElement& element) {
  return isSVGSetElement(element);
}

class SVGStopElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGStopElement(const SVGStopElement&);
void isSVGStopElement(const SVGStopElement*);

inline bool isSVGStopElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::stopTag);
}
inline bool isSVGStopElement(const SVGElement* element) {
  return element && isSVGStopElement(*element);
}
inline bool isSVGStopElement(const Node& node) {
  return node.IsSVGElement() && isSVGStopElement(ToSVGElement(node));
}
inline bool isSVGStopElement(const Node* node) {
  return node && isSVGStopElement(*node);
}
template <>
inline bool IsElementOfType<const SVGStopElement>(const Node& node) {
  return isSVGStopElement(node);
}
template <>
inline bool IsElementOfType<const SVGStopElement>(const SVGElement& element) {
  return isSVGStopElement(element);
}

class SVGSwitchElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGSwitchElement(const SVGSwitchElement&);
void isSVGSwitchElement(const SVGSwitchElement*);

inline bool isSVGSwitchElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::switchTag);
}
inline bool isSVGSwitchElement(const SVGElement* element) {
  return element && isSVGSwitchElement(*element);
}
inline bool isSVGSwitchElement(const Node& node) {
  return node.IsSVGElement() && isSVGSwitchElement(ToSVGElement(node));
}
inline bool isSVGSwitchElement(const Node* node) {
  return node && isSVGSwitchElement(*node);
}
template <>
inline bool IsElementOfType<const SVGSwitchElement>(const Node& node) {
  return isSVGSwitchElement(node);
}
template <>
inline bool IsElementOfType<const SVGSwitchElement>(const SVGElement& element) {
  return isSVGSwitchElement(element);
}

class SVGSymbolElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGSymbolElement(const SVGSymbolElement&);
void isSVGSymbolElement(const SVGSymbolElement*);

inline bool isSVGSymbolElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::symbolTag);
}
inline bool isSVGSymbolElement(const SVGElement* element) {
  return element && isSVGSymbolElement(*element);
}
inline bool isSVGSymbolElement(const Node& node) {
  return node.IsSVGElement() && isSVGSymbolElement(ToSVGElement(node));
}
inline bool isSVGSymbolElement(const Node* node) {
  return node && isSVGSymbolElement(*node);
}
template <>
inline bool IsElementOfType<const SVGSymbolElement>(const Node& node) {
  return isSVGSymbolElement(node);
}
template <>
inline bool IsElementOfType<const SVGSymbolElement>(const SVGElement& element) {
  return isSVGSymbolElement(element);
}

class SVGTSpanElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGTSpanElement(const SVGTSpanElement&);
void isSVGTSpanElement(const SVGTSpanElement*);

inline bool isSVGTSpanElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::tspanTag);
}
inline bool isSVGTSpanElement(const SVGElement* element) {
  return element && isSVGTSpanElement(*element);
}
inline bool isSVGTSpanElement(const Node& node) {
  return node.IsSVGElement() && isSVGTSpanElement(ToSVGElement(node));
}
inline bool isSVGTSpanElement(const Node* node) {
  return node && isSVGTSpanElement(*node);
}
template <>
inline bool IsElementOfType<const SVGTSpanElement>(const Node& node) {
  return isSVGTSpanElement(node);
}
template <>
inline bool IsElementOfType<const SVGTSpanElement>(const SVGElement& element) {
  return isSVGTSpanElement(element);
}

class SVGTextElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGTextElement(const SVGTextElement&);
void isSVGTextElement(const SVGTextElement*);

inline bool isSVGTextElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::textTag);
}
inline bool isSVGTextElement(const SVGElement* element) {
  return element && isSVGTextElement(*element);
}
inline bool isSVGTextElement(const Node& node) {
  return node.IsSVGElement() && isSVGTextElement(ToSVGElement(node));
}
inline bool isSVGTextElement(const Node* node) {
  return node && isSVGTextElement(*node);
}
template <>
inline bool IsElementOfType<const SVGTextElement>(const Node& node) {
  return isSVGTextElement(node);
}
template <>
inline bool IsElementOfType<const SVGTextElement>(const SVGElement& element) {
  return isSVGTextElement(element);
}

class SVGTextPathElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGTextPathElement(const SVGTextPathElement&);
void isSVGTextPathElement(const SVGTextPathElement*);

inline bool isSVGTextPathElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::textPathTag);
}
inline bool isSVGTextPathElement(const SVGElement* element) {
  return element && isSVGTextPathElement(*element);
}
inline bool isSVGTextPathElement(const Node& node) {
  return node.IsSVGElement() && isSVGTextPathElement(ToSVGElement(node));
}
inline bool isSVGTextPathElement(const Node* node) {
  return node && isSVGTextPathElement(*node);
}
template <>
inline bool IsElementOfType<const SVGTextPathElement>(const Node& node) {
  return isSVGTextPathElement(node);
}
template <>
inline bool IsElementOfType<const SVGTextPathElement>(const SVGElement& element) {
  return isSVGTextPathElement(element);
}

class SVGTitleElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGTitleElement(const SVGTitleElement&);
void isSVGTitleElement(const SVGTitleElement*);

inline bool isSVGTitleElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::titleTag);
}
inline bool isSVGTitleElement(const SVGElement* element) {
  return element && isSVGTitleElement(*element);
}
inline bool isSVGTitleElement(const Node& node) {
  return node.IsSVGElement() && isSVGTitleElement(ToSVGElement(node));
}
inline bool isSVGTitleElement(const Node* node) {
  return node && isSVGTitleElement(*node);
}
template <>
inline bool IsElementOfType<const SVGTitleElement>(const Node& node) {
  return isSVGTitleElement(node);
}
template <>
inline bool IsElementOfType<const SVGTitleElement>(const SVGElement& element) {
  return isSVGTitleElement(element);
}

class SVGUseElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGUseElement(const SVGUseElement&);
void isSVGUseElement(const SVGUseElement*);

inline bool isSVGUseElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::useTag);
}
inline bool isSVGUseElement(const SVGElement* element) {
  return element && isSVGUseElement(*element);
}
inline bool isSVGUseElement(const Node& node) {
  return node.IsSVGElement() && isSVGUseElement(ToSVGElement(node));
}
inline bool isSVGUseElement(const Node* node) {
  return node && isSVGUseElement(*node);
}
template <>
inline bool IsElementOfType<const SVGUseElement>(const Node& node) {
  return isSVGUseElement(node);
}
template <>
inline bool IsElementOfType<const SVGUseElement>(const SVGElement& element) {
  return isSVGUseElement(element);
}

class SVGViewElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGViewElement(const SVGViewElement&);
void isSVGViewElement(const SVGViewElement*);

inline bool isSVGViewElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::viewTag);
}
inline bool isSVGViewElement(const SVGElement* element) {
  return element && isSVGViewElement(*element);
}
inline bool isSVGViewElement(const Node& node) {
  return node.IsSVGElement() && isSVGViewElement(ToSVGElement(node));
}
inline bool isSVGViewElement(const Node* node) {
  return node && isSVGViewElement(*node);
}
template <>
inline bool IsElementOfType<const SVGViewElement>(const Node& node) {
  return isSVGViewElement(node);
}
template <>
inline bool IsElementOfType<const SVGViewElement>(const SVGElement& element) {
  return isSVGViewElement(element);
}

class SVGScriptElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGScriptElement(const SVGScriptElement&);
void isSVGScriptElement(const SVGScriptElement*);

inline bool isSVGScriptElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::scriptTag);
}
inline bool isSVGScriptElement(const SVGElement* element) {
  return element && isSVGScriptElement(*element);
}
inline bool isSVGScriptElement(const Node& node) {
  return node.IsSVGElement() && isSVGScriptElement(ToSVGElement(node));
}
inline bool isSVGScriptElement(const Node* node) {
  return node && isSVGScriptElement(*node);
}
template <>
inline bool IsElementOfType<const SVGScriptElement>(const Node& node) {
  return isSVGScriptElement(node);
}
template <>
inline bool IsElementOfType<const SVGScriptElement>(const SVGElement& element) {
  return isSVGScriptElement(element);
}

class SVGStyleElement;
// Catch unnecessary runtime check of type known at compile time.
void isSVGStyleElement(const SVGStyleElement&);
void isSVGStyleElement(const SVGStyleElement*);

inline bool isSVGStyleElement(const SVGElement& element) {
  return element.HasTagName(SVGNames::styleTag);
}
inline bool isSVGStyleElement(const SVGElement* element) {
  return element && isSVGStyleElement(*element);
}
inline bool isSVGStyleElement(const Node& node) {
  return node.IsSVGElement() && isSVGStyleElement(ToSVGElement(node));
}
inline bool isSVGStyleElement(const Node* node) {
  return node && isSVGStyleElement(*node);
}
template <>
inline bool IsElementOfType<const SVGStyleElement>(const Node& node) {
  return isSVGStyleElement(node);
}
template <>
inline bool IsElementOfType<const SVGStyleElement>(const SVGElement& element) {
  return isSVGStyleElement(element);
}

// Using macros because the types are forward-declared and we don't want to use
// reinterpret_cast in the casting functions above. reinterpret_cast would be
// unsafe due to multiple inheritence.

#define toSVGAElement(x) blink::ToElement<blink::SVGAElement>(x)
#define toSVGAElementOrDie(x) blink::ToElementOrDie<blink::SVGAElement>(x)
#define toSVGAnimateMotionElement(x) blink::ToElement<blink::SVGAnimateMotionElement>(x)
#define toSVGAnimateMotionElementOrDie(x) blink::ToElementOrDie<blink::SVGAnimateMotionElement>(x)
#define toSVGAnimateTransformElement(x) blink::ToElement<blink::SVGAnimateTransformElement>(x)
#define toSVGAnimateTransformElementOrDie(x) blink::ToElementOrDie<blink::SVGAnimateTransformElement>(x)
#define toSVGCircleElement(x) blink::ToElement<blink::SVGCircleElement>(x)
#define toSVGCircleElementOrDie(x) blink::ToElementOrDie<blink::SVGCircleElement>(x)
#define toSVGClipPathElement(x) blink::ToElement<blink::SVGClipPathElement>(x)
#define toSVGClipPathElementOrDie(x) blink::ToElementOrDie<blink::SVGClipPathElement>(x)
#define toSVGDefsElement(x) blink::ToElement<blink::SVGDefsElement>(x)
#define toSVGDefsElementOrDie(x) blink::ToElementOrDie<blink::SVGDefsElement>(x)
#define toSVGDescElement(x) blink::ToElement<blink::SVGDescElement>(x)
#define toSVGDescElementOrDie(x) blink::ToElementOrDie<blink::SVGDescElement>(x)
#define toSVGDiscardElement(x) blink::ToElement<blink::SVGDiscardElement>(x)
#define toSVGDiscardElementOrDie(x) blink::ToElementOrDie<blink::SVGDiscardElement>(x)
#define toSVGEllipseElement(x) blink::ToElement<blink::SVGEllipseElement>(x)
#define toSVGEllipseElementOrDie(x) blink::ToElementOrDie<blink::SVGEllipseElement>(x)
#define toSVGFEBlendElement(x) blink::ToElement<blink::SVGFEBlendElement>(x)
#define toSVGFEBlendElementOrDie(x) blink::ToElementOrDie<blink::SVGFEBlendElement>(x)
#define toSVGFEColorMatrixElement(x) blink::ToElement<blink::SVGFEColorMatrixElement>(x)
#define toSVGFEColorMatrixElementOrDie(x) blink::ToElementOrDie<blink::SVGFEColorMatrixElement>(x)
#define toSVGFEComponentTransferElement(x) blink::ToElement<blink::SVGFEComponentTransferElement>(x)
#define toSVGFEComponentTransferElementOrDie(x) blink::ToElementOrDie<blink::SVGFEComponentTransferElement>(x)
#define toSVGFECompositeElement(x) blink::ToElement<blink::SVGFECompositeElement>(x)
#define toSVGFECompositeElementOrDie(x) blink::ToElementOrDie<blink::SVGFECompositeElement>(x)
#define toSVGFEConvolveMatrixElement(x) blink::ToElement<blink::SVGFEConvolveMatrixElement>(x)
#define toSVGFEConvolveMatrixElementOrDie(x) blink::ToElementOrDie<blink::SVGFEConvolveMatrixElement>(x)
#define toSVGFEDiffuseLightingElement(x) blink::ToElement<blink::SVGFEDiffuseLightingElement>(x)
#define toSVGFEDiffuseLightingElementOrDie(x) blink::ToElementOrDie<blink::SVGFEDiffuseLightingElement>(x)
#define toSVGFEDisplacementMapElement(x) blink::ToElement<blink::SVGFEDisplacementMapElement>(x)
#define toSVGFEDisplacementMapElementOrDie(x) blink::ToElementOrDie<blink::SVGFEDisplacementMapElement>(x)
#define toSVGFEDistantLightElement(x) blink::ToElement<blink::SVGFEDistantLightElement>(x)
#define toSVGFEDistantLightElementOrDie(x) blink::ToElementOrDie<blink::SVGFEDistantLightElement>(x)
#define toSVGFEDropShadowElement(x) blink::ToElement<blink::SVGFEDropShadowElement>(x)
#define toSVGFEDropShadowElementOrDie(x) blink::ToElementOrDie<blink::SVGFEDropShadowElement>(x)
#define toSVGFEFloodElement(x) blink::ToElement<blink::SVGFEFloodElement>(x)
#define toSVGFEFloodElementOrDie(x) blink::ToElementOrDie<blink::SVGFEFloodElement>(x)
#define toSVGFEFuncAElement(x) blink::ToElement<blink::SVGFEFuncAElement>(x)
#define toSVGFEFuncAElementOrDie(x) blink::ToElementOrDie<blink::SVGFEFuncAElement>(x)
#define toSVGFEFuncBElement(x) blink::ToElement<blink::SVGFEFuncBElement>(x)
#define toSVGFEFuncBElementOrDie(x) blink::ToElementOrDie<blink::SVGFEFuncBElement>(x)
#define toSVGFEFuncGElement(x) blink::ToElement<blink::SVGFEFuncGElement>(x)
#define toSVGFEFuncGElementOrDie(x) blink::ToElementOrDie<blink::SVGFEFuncGElement>(x)
#define toSVGFEFuncRElement(x) blink::ToElement<blink::SVGFEFuncRElement>(x)
#define toSVGFEFuncRElementOrDie(x) blink::ToElementOrDie<blink::SVGFEFuncRElement>(x)
#define toSVGFEGaussianBlurElement(x) blink::ToElement<blink::SVGFEGaussianBlurElement>(x)
#define toSVGFEGaussianBlurElementOrDie(x) blink::ToElementOrDie<blink::SVGFEGaussianBlurElement>(x)
#define toSVGFEImageElement(x) blink::ToElement<blink::SVGFEImageElement>(x)
#define toSVGFEImageElementOrDie(x) blink::ToElementOrDie<blink::SVGFEImageElement>(x)
#define toSVGFEMergeElement(x) blink::ToElement<blink::SVGFEMergeElement>(x)
#define toSVGFEMergeElementOrDie(x) blink::ToElementOrDie<blink::SVGFEMergeElement>(x)
#define toSVGFEMergeNodeElement(x) blink::ToElement<blink::SVGFEMergeNodeElement>(x)
#define toSVGFEMergeNodeElementOrDie(x) blink::ToElementOrDie<blink::SVGFEMergeNodeElement>(x)
#define toSVGFEMorphologyElement(x) blink::ToElement<blink::SVGFEMorphologyElement>(x)
#define toSVGFEMorphologyElementOrDie(x) blink::ToElementOrDie<blink::SVGFEMorphologyElement>(x)
#define toSVGFEOffsetElement(x) blink::ToElement<blink::SVGFEOffsetElement>(x)
#define toSVGFEOffsetElementOrDie(x) blink::ToElementOrDie<blink::SVGFEOffsetElement>(x)
#define toSVGFEPointLightElement(x) blink::ToElement<blink::SVGFEPointLightElement>(x)
#define toSVGFEPointLightElementOrDie(x) blink::ToElementOrDie<blink::SVGFEPointLightElement>(x)
#define toSVGFESpecularLightingElement(x) blink::ToElement<blink::SVGFESpecularLightingElement>(x)
#define toSVGFESpecularLightingElementOrDie(x) blink::ToElementOrDie<blink::SVGFESpecularLightingElement>(x)
#define toSVGFESpotLightElement(x) blink::ToElement<blink::SVGFESpotLightElement>(x)
#define toSVGFESpotLightElementOrDie(x) blink::ToElementOrDie<blink::SVGFESpotLightElement>(x)
#define toSVGFETileElement(x) blink::ToElement<blink::SVGFETileElement>(x)
#define toSVGFETileElementOrDie(x) blink::ToElementOrDie<blink::SVGFETileElement>(x)
#define toSVGFETurbulenceElement(x) blink::ToElement<blink::SVGFETurbulenceElement>(x)
#define toSVGFETurbulenceElementOrDie(x) blink::ToElementOrDie<blink::SVGFETurbulenceElement>(x)
#define toSVGFilterElement(x) blink::ToElement<blink::SVGFilterElement>(x)
#define toSVGFilterElementOrDie(x) blink::ToElementOrDie<blink::SVGFilterElement>(x)
#define toSVGForeignObjectElement(x) blink::ToElement<blink::SVGForeignObjectElement>(x)
#define toSVGForeignObjectElementOrDie(x) blink::ToElementOrDie<blink::SVGForeignObjectElement>(x)
#define toSVGGElement(x) blink::ToElement<blink::SVGGElement>(x)
#define toSVGGElementOrDie(x) blink::ToElementOrDie<blink::SVGGElement>(x)
#define toSVGImageElement(x) blink::ToElement<blink::SVGImageElement>(x)
#define toSVGImageElementOrDie(x) blink::ToElementOrDie<blink::SVGImageElement>(x)
#define toSVGLineElement(x) blink::ToElement<blink::SVGLineElement>(x)
#define toSVGLineElementOrDie(x) blink::ToElementOrDie<blink::SVGLineElement>(x)
#define toSVGLinearGradientElement(x) blink::ToElement<blink::SVGLinearGradientElement>(x)
#define toSVGLinearGradientElementOrDie(x) blink::ToElementOrDie<blink::SVGLinearGradientElement>(x)
#define toSVGMPathElement(x) blink::ToElement<blink::SVGMPathElement>(x)
#define toSVGMPathElementOrDie(x) blink::ToElementOrDie<blink::SVGMPathElement>(x)
#define toSVGMarkerElement(x) blink::ToElement<blink::SVGMarkerElement>(x)
#define toSVGMarkerElementOrDie(x) blink::ToElementOrDie<blink::SVGMarkerElement>(x)
#define toSVGMaskElement(x) blink::ToElement<blink::SVGMaskElement>(x)
#define toSVGMaskElementOrDie(x) blink::ToElementOrDie<blink::SVGMaskElement>(x)
#define toSVGMetadataElement(x) blink::ToElement<blink::SVGMetadataElement>(x)
#define toSVGMetadataElementOrDie(x) blink::ToElementOrDie<blink::SVGMetadataElement>(x)
#define toSVGPathElement(x) blink::ToElement<blink::SVGPathElement>(x)
#define toSVGPathElementOrDie(x) blink::ToElementOrDie<blink::SVGPathElement>(x)
#define toSVGPatternElement(x) blink::ToElement<blink::SVGPatternElement>(x)
#define toSVGPatternElementOrDie(x) blink::ToElementOrDie<blink::SVGPatternElement>(x)
#define toSVGPolygonElement(x) blink::ToElement<blink::SVGPolygonElement>(x)
#define toSVGPolygonElementOrDie(x) blink::ToElementOrDie<blink::SVGPolygonElement>(x)
#define toSVGPolylineElement(x) blink::ToElement<blink::SVGPolylineElement>(x)
#define toSVGPolylineElementOrDie(x) blink::ToElementOrDie<blink::SVGPolylineElement>(x)
#define toSVGRadialGradientElement(x) blink::ToElement<blink::SVGRadialGradientElement>(x)
#define toSVGRadialGradientElementOrDie(x) blink::ToElementOrDie<blink::SVGRadialGradientElement>(x)
#define toSVGRectElement(x) blink::ToElement<blink::SVGRectElement>(x)
#define toSVGRectElementOrDie(x) blink::ToElementOrDie<blink::SVGRectElement>(x)
#define toSVGSVGElement(x) blink::ToElement<blink::SVGSVGElement>(x)
#define toSVGSVGElementOrDie(x) blink::ToElementOrDie<blink::SVGSVGElement>(x)
#define toSVGSetElement(x) blink::ToElement<blink::SVGSetElement>(x)
#define toSVGSetElementOrDie(x) blink::ToElementOrDie<blink::SVGSetElement>(x)
#define toSVGStopElement(x) blink::ToElement<blink::SVGStopElement>(x)
#define toSVGStopElementOrDie(x) blink::ToElementOrDie<blink::SVGStopElement>(x)
#define toSVGSwitchElement(x) blink::ToElement<blink::SVGSwitchElement>(x)
#define toSVGSwitchElementOrDie(x) blink::ToElementOrDie<blink::SVGSwitchElement>(x)
#define toSVGSymbolElement(x) blink::ToElement<blink::SVGSymbolElement>(x)
#define toSVGSymbolElementOrDie(x) blink::ToElementOrDie<blink::SVGSymbolElement>(x)
#define toSVGTSpanElement(x) blink::ToElement<blink::SVGTSpanElement>(x)
#define toSVGTSpanElementOrDie(x) blink::ToElementOrDie<blink::SVGTSpanElement>(x)
#define toSVGTextElement(x) blink::ToElement<blink::SVGTextElement>(x)
#define toSVGTextElementOrDie(x) blink::ToElementOrDie<blink::SVGTextElement>(x)
#define toSVGTextPathElement(x) blink::ToElement<blink::SVGTextPathElement>(x)
#define toSVGTextPathElementOrDie(x) blink::ToElementOrDie<blink::SVGTextPathElement>(x)
#define toSVGTitleElement(x) blink::ToElement<blink::SVGTitleElement>(x)
#define toSVGTitleElementOrDie(x) blink::ToElementOrDie<blink::SVGTitleElement>(x)
#define toSVGUseElement(x) blink::ToElement<blink::SVGUseElement>(x)
#define toSVGUseElementOrDie(x) blink::ToElementOrDie<blink::SVGUseElement>(x)
#define toSVGViewElement(x) blink::ToElement<blink::SVGViewElement>(x)
#define toSVGViewElementOrDie(x) blink::ToElementOrDie<blink::SVGViewElement>(x)
#define toSVGScriptElement(x) blink::ToElement<blink::SVGScriptElement>(x)
#define toSVGScriptElementOrDie(x) blink::ToElementOrDie<blink::SVGScriptElement>(x)
#define toSVGStyleElement(x) blink::ToElement<blink::SVGStyleElement>(x)
#define toSVGStyleElementOrDie(x) blink::ToElementOrDie<blink::SVGStyleElement>(x)

} // namespace blink

#endif
