// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_BEGIN_FRAME_ARGS_MOJOM_BLINK_H_
#define CC_IPC_BEGIN_FRAME_ARGS_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/begin_frame_args.mojom-shared.h"
#include "mojo/common/time.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"
#include "cc/output/begin_frame_args.h"


namespace WTF {
struct cc_mojom_internal_BeginFrameArgsType_DataHashFn {
  static unsigned GetHash(const ::cc::mojom::BeginFrameArgsType& value) {
    using utype = std::underlying_type<::cc::mojom::BeginFrameArgsType>::type;
    return DefaultHash<utype>::Hash().GetHash(static_cast<utype>(value));
  }
  static bool Equal(const ::cc::mojom::BeginFrameArgsType& left, const ::cc::mojom::BeginFrameArgsType& right) {
    return left == right;
  }
  static const bool safe_to_compare_to_empty_or_deleted = true;
};

template <>
struct DefaultHash<::cc::mojom::BeginFrameArgsType> {
  using Hash = cc_mojom_internal_BeginFrameArgsType_DataHashFn;
};

template <>
struct HashTraits<::cc::mojom::BeginFrameArgsType>
    : public GenericHashTraits<::cc::mojom::BeginFrameArgsType> {
  static_assert(true,
                "-1000000 is a reserved enum value");
  static_assert(true,
                "-1000001 is a reserved enum value");
  static const bool hasIsEmptyValueFunction = true;
  static bool IsEmptyValue(const ::cc::mojom::BeginFrameArgsType& value) {
    return value == static_cast<::cc::mojom::BeginFrameArgsType>(-1000000);
  }
  static void ConstructDeletedValue(::cc::mojom::BeginFrameArgsType& slot, bool) {
    slot = static_cast<::cc::mojom::BeginFrameArgsType>(-1000001);
  }
  static bool IsDeletedValue(const ::cc::mojom::BeginFrameArgsType& value) {
    return value == static_cast<::cc::mojom::BeginFrameArgsType>(-1000001);
  }
};
}  // namespace WTF


namespace cc {
namespace mojom {
namespace blink {
using BeginFrameArgsType = BeginFrameArgsType;  // Alias for definition in the parent namespace.
class BeginFrameArgs;
using BeginFrameArgsPtr = mojo::StructPtr<BeginFrameArgs>;

class BeginFrameAck;
using BeginFrameAckPtr = mojo::InlinedStructPtr<BeginFrameAck>;




class  BeginFrameAck {
 public:
  using DataView = BeginFrameAckDataView;
  using Data_ = internal::BeginFrameAck_Data;

  template <typename... Args>
  static BeginFrameAckPtr New(Args&&... args) {
    return BeginFrameAckPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static BeginFrameAckPtr From(const U& u) {
    return mojo::TypeConverter<BeginFrameAckPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BeginFrameAck>::Convert(*this);
  }


  BeginFrameAck();

  BeginFrameAck(
      uint64_t sequence_number,
      uint32_t source_id);

  ~BeginFrameAck();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BeginFrameAckPtr>
  BeginFrameAckPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BeginFrameAck>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        BeginFrameAck::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        BeginFrameAck::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  uint64_t sequence_number;
  uint32_t source_id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class  BeginFrameArgs {
 public:
  using DataView = BeginFrameArgsDataView;
  using Data_ = internal::BeginFrameArgs_Data;

  template <typename... Args>
  static BeginFrameArgsPtr New(Args&&... args) {
    return BeginFrameArgsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static BeginFrameArgsPtr From(const U& u) {
    return mojo::TypeConverter<BeginFrameArgsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BeginFrameArgs>::Convert(*this);
  }


  BeginFrameArgs();

  BeginFrameArgs(
      ::mojo::common::mojom::blink::TimeTicksPtr frame_time,
      ::mojo::common::mojom::blink::TimeTicksPtr deadline,
      WTF::TimeDelta interval,
      uint64_t sequence_number,
      uint32_t source_id,
      BeginFrameArgsType type,
      bool on_critical_path);

  ~BeginFrameArgs();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BeginFrameArgsPtr>
  BeginFrameArgsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BeginFrameArgs>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        BeginFrameArgs::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        BeginFrameArgs::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  ::mojo::common::mojom::blink::TimeTicksPtr frame_time;
  ::mojo::common::mojom::blink::TimeTicksPtr deadline;
  WTF::TimeDelta interval;
  uint64_t sequence_number;
  uint32_t source_id;
  BeginFrameArgsType type;
  bool on_critical_path;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(BeginFrameArgs);
};



template <typename StructPtrType>
BeginFrameArgsPtr BeginFrameArgs::Clone() const {
  return New(
      mojo::Clone(frame_time),
      mojo::Clone(deadline),
      mojo::Clone(interval),
      mojo::Clone(sequence_number),
      mojo::Clone(source_id),
      mojo::Clone(type),
      mojo::Clone(on_critical_path)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BeginFrameArgs>::value>::type*>
bool BeginFrameArgs::Equals(const T& other) const {
  if (!mojo::Equals(this->frame_time, other.frame_time))
    return false;
  if (!mojo::Equals(this->deadline, other.deadline))
    return false;
  if (!mojo::Equals(this->interval, other.interval))
    return false;
  if (!mojo::Equals(this->sequence_number, other.sequence_number))
    return false;
  if (!mojo::Equals(this->source_id, other.source_id))
    return false;
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->on_critical_path, other.on_critical_path))
    return false;
  return true;
}
template <typename StructPtrType>
BeginFrameAckPtr BeginFrameAck::Clone() const {
  return New(
      mojo::Clone(sequence_number),
      mojo::Clone(source_id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BeginFrameAck>::value>::type*>
bool BeginFrameAck::Equals(const T& other) const {
  if (!mojo::Equals(this->sequence_number, other.sequence_number))
    return false;
  if (!mojo::Equals(this->source_id, other.source_id))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::blink::BeginFrameArgs::DataView,
                                         ::cc::mojom::blink::BeginFrameArgsPtr> {
  static bool IsNull(const ::cc::mojom::blink::BeginFrameArgsPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::blink::BeginFrameArgsPtr* output) { output->reset(); }

  static const decltype(::cc::mojom::blink::BeginFrameArgs::frame_time)& frame_time(
      const ::cc::mojom::blink::BeginFrameArgsPtr& input) {
    return input->frame_time;
  }

  static const decltype(::cc::mojom::blink::BeginFrameArgs::deadline)& deadline(
      const ::cc::mojom::blink::BeginFrameArgsPtr& input) {
    return input->deadline;
  }

  static const decltype(::cc::mojom::blink::BeginFrameArgs::interval)& interval(
      const ::cc::mojom::blink::BeginFrameArgsPtr& input) {
    return input->interval;
  }

  static decltype(::cc::mojom::blink::BeginFrameArgs::sequence_number) sequence_number(
      const ::cc::mojom::blink::BeginFrameArgsPtr& input) {
    return input->sequence_number;
  }

  static decltype(::cc::mojom::blink::BeginFrameArgs::source_id) source_id(
      const ::cc::mojom::blink::BeginFrameArgsPtr& input) {
    return input->source_id;
  }

  static decltype(::cc::mojom::blink::BeginFrameArgs::type) type(
      const ::cc::mojom::blink::BeginFrameArgsPtr& input) {
    return input->type;
  }

  static decltype(::cc::mojom::blink::BeginFrameArgs::on_critical_path) on_critical_path(
      const ::cc::mojom::blink::BeginFrameArgsPtr& input) {
    return input->on_critical_path;
  }

  static bool Read(::cc::mojom::blink::BeginFrameArgs::DataView input, ::cc::mojom::blink::BeginFrameArgsPtr* output);
};


template <>
struct  StructTraits<::cc::mojom::blink::BeginFrameAck::DataView,
                                         ::cc::mojom::blink::BeginFrameAckPtr> {
  static bool IsNull(const ::cc::mojom::blink::BeginFrameAckPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::blink::BeginFrameAckPtr* output) { output->reset(); }

  static decltype(::cc::mojom::blink::BeginFrameAck::sequence_number) sequence_number(
      const ::cc::mojom::blink::BeginFrameAckPtr& input) {
    return input->sequence_number;
  }

  static decltype(::cc::mojom::blink::BeginFrameAck::source_id) source_id(
      const ::cc::mojom::blink::BeginFrameAckPtr& input) {
    return input->source_id;
  }

  static bool Read(::cc::mojom::blink::BeginFrameAck::DataView input, ::cc::mojom::blink::BeginFrameAckPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_BEGIN_FRAME_ARGS_MOJOM_BLINK_H_