// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_COPY_OUTPUT_REQUEST_MOJOM_SHARED_H_
#define CC_IPC_COPY_OUTPUT_REQUEST_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "cc/ipc/copy_output_request.mojom-shared-internal.h"
#include "cc/ipc/copy_output_result.mojom-shared.h"
#include "cc/ipc/texture_mailbox.mojom-shared.h"
#include "mojo/common/unguessable_token.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"


namespace cc {
namespace mojom {
class CopyOutputRequestDataView;



}  // namespace mojom
}  // namespace cc

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::cc::mojom::CopyOutputRequestDataView> {
  using Data = ::cc::mojom::internal::CopyOutputRequest_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace cc {
namespace mojom {
// Interface base classes. They are used for type safety check.
class CopyOutputResultSenderInterfaceBase {};

using CopyOutputResultSenderPtrDataView =
    mojo::InterfacePtrDataView<CopyOutputResultSenderInterfaceBase>;
using CopyOutputResultSenderRequestDataView =
    mojo::InterfaceRequestDataView<CopyOutputResultSenderInterfaceBase>;
using CopyOutputResultSenderAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<CopyOutputResultSenderInterfaceBase>;
using CopyOutputResultSenderAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<CopyOutputResultSenderInterfaceBase>;
class CopyOutputRequestDataView {
 public:
  CopyOutputRequestDataView() {}

  CopyOutputRequestDataView(
      internal::CopyOutputRequest_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSourceDataView(
      ::mojo::common::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSource(UserType* output) {
    auto* pointer = data_->source.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
  bool force_bitmap_result() const {
    return data_->force_bitmap_result;
  }
  inline void GetAreaDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadArea(UserType* output) {
    auto* pointer = data_->area.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  inline void GetTextureMailboxDataView(
      ::cc::mojom::TextureMailboxDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTextureMailbox(UserType* output) {
    auto* pointer = data_->texture_mailbox.Get();
    return mojo::internal::Deserialize<::cc::mojom::TextureMailboxDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeResultSender() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::cc::mojom::CopyOutputResultSenderPtrDataView>(
            &data_->result_sender, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::CopyOutputRequest_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CopyOutputResultSender_SendResult_ParamsDataView {
 public:
  CopyOutputResultSender_SendResult_ParamsDataView() {}

  CopyOutputResultSender_SendResult_ParamsDataView(
      internal::CopyOutputResultSender_SendResult_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResultDataView(
      ::cc::mojom::CopyOutputResultDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) {
    auto* pointer = data_->result.Get();
    return mojo::internal::Deserialize<::cc::mojom::CopyOutputResultDataView>(
        pointer, output, context_);
  }
 private:
  internal::CopyOutputResultSender_SendResult_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace cc

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::cc::mojom::CopyOutputRequestDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::cc::mojom::CopyOutputRequestDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::cc::mojom::internal::CopyOutputRequest_Data);
    decltype(CallWithContext(Traits::source, input, custom_context)) in_source = CallWithContext(Traits::source, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::UnguessableTokenDataView>(
        in_source, context);
    decltype(CallWithContext(Traits::area, input, custom_context)) in_area = CallWithContext(Traits::area, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectDataView>(
        in_area, context);
    decltype(CallWithContext(Traits::texture_mailbox, input, custom_context)) in_texture_mailbox = CallWithContext(Traits::texture_mailbox, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::cc::mojom::TextureMailboxDataView>(
        in_texture_mailbox, context);
    decltype(CallWithContext(Traits::result_sender, input, custom_context)) in_result_sender = CallWithContext(Traits::result_sender, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::cc::mojom::CopyOutputResultSenderPtrDataView>(
        in_result_sender, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::cc::mojom::internal::CopyOutputRequest_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::cc::mojom::internal::CopyOutputRequest_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::source, input, custom_context)) in_source = CallWithContext(Traits::source, input, custom_context);
    typename decltype(result->source)::BaseType* source_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::UnguessableTokenDataView>(
        in_source, buffer, &source_ptr, context);
    result->source.Set(source_ptr);
    result->force_bitmap_result = CallWithContext(Traits::force_bitmap_result, input, custom_context);
    decltype(CallWithContext(Traits::area, input, custom_context)) in_area = CallWithContext(Traits::area, input, custom_context);
    typename decltype(result->area)::BaseType* area_ptr;
    mojo::internal::Serialize<::gfx::mojom::RectDataView>(
        in_area, buffer, &area_ptr, context);
    result->area.Set(area_ptr);
    decltype(CallWithContext(Traits::texture_mailbox, input, custom_context)) in_texture_mailbox = CallWithContext(Traits::texture_mailbox, input, custom_context);
    typename decltype(result->texture_mailbox)::BaseType* texture_mailbox_ptr;
    mojo::internal::Serialize<::cc::mojom::TextureMailboxDataView>(
        in_texture_mailbox, buffer, &texture_mailbox_ptr, context);
    result->texture_mailbox.Set(texture_mailbox_ptr);
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::result_sender, input, custom_context))>::type
        in_result_sender;
    mojo::internal::Serialize<::cc::mojom::CopyOutputResultSenderPtrDataView>(
        in_result_sender, &result->result_sender, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->result_sender),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid result_sender in CopyOutputRequest struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::cc::mojom::internal::CopyOutputRequest_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::cc::mojom::CopyOutputRequestDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace cc {
namespace mojom {

inline void CopyOutputRequestDataView::GetSourceDataView(
    ::mojo::common::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->source.Get();
  *output = ::mojo::common::mojom::UnguessableTokenDataView(pointer, context_);
}
inline void CopyOutputRequestDataView::GetAreaDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->area.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}
inline void CopyOutputRequestDataView::GetTextureMailboxDataView(
    ::cc::mojom::TextureMailboxDataView* output) {
  auto pointer = data_->texture_mailbox.Get();
  *output = ::cc::mojom::TextureMailboxDataView(pointer, context_);
}


inline void CopyOutputResultSender_SendResult_ParamsDataView::GetResultDataView(
    ::cc::mojom::CopyOutputResultDataView* output) {
  auto pointer = data_->result.Get();
  *output = ::cc::mojom::CopyOutputResultDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace cc

#endif  // CC_IPC_COPY_OUTPUT_REQUEST_MOJOM_SHARED_H_
