// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_COPY_OUTPUT_RESULT_MOJOM_H_
#define CC_IPC_COPY_OUTPUT_RESULT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/copy_output_result.mojom-shared.h"
#include "cc/ipc/texture_mailbox.mojom.h"
#include "cc/ipc/texture_mailbox_releaser.mojom.h"
#include "skia/public/interfaces/bitmap.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>
#include "cc/output/copy_output_result.h"


namespace cc {
namespace mojom {
class CopyOutputResult;
using CopyOutputResultPtr = mojo::StructPtr<CopyOutputResult>;






class  CopyOutputResult {
 public:
  using DataView = CopyOutputResultDataView;
  using Data_ = internal::CopyOutputResult_Data;

  template <typename... Args>
  static CopyOutputResultPtr New(Args&&... args) {
    return CopyOutputResultPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CopyOutputResultPtr From(const U& u) {
    return mojo::TypeConverter<CopyOutputResultPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CopyOutputResult>::Convert(*this);
  }


  CopyOutputResult();

  CopyOutputResult(
      const gfx::Size& size,
      const SkBitmap& bitmap,
      const viz::TextureMailbox& texture_mailbox,
      ::cc::mojom::TextureMailboxReleaserPtr releaser);

  ~CopyOutputResult();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CopyOutputResultPtr>
  CopyOutputResultPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CopyOutputResult>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CopyOutputResult::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CopyOutputResult::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  gfx::Size size;
  SkBitmap bitmap;
  viz::TextureMailbox texture_mailbox;
  ::cc::mojom::TextureMailboxReleaserPtr releaser;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(CopyOutputResult);
};


template <typename StructPtrType>
CopyOutputResultPtr CopyOutputResult::Clone() const {
  return New(
      mojo::Clone(size),
      mojo::Clone(bitmap),
      mojo::Clone(texture_mailbox),
      mojo::Clone(releaser)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CopyOutputResult>::value>::type*>
bool CopyOutputResult::Equals(const T& other) const {
  if (!mojo::Equals(this->size, other.size))
    return false;
  if (!mojo::Equals(this->bitmap, other.bitmap))
    return false;
  if (!mojo::Equals(this->texture_mailbox, other.texture_mailbox))
    return false;
  if (!mojo::Equals(this->releaser, other.releaser))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::CopyOutputResult::DataView,
                                         ::cc::mojom::CopyOutputResultPtr> {
  static bool IsNull(const ::cc::mojom::CopyOutputResultPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::CopyOutputResultPtr* output) { output->reset(); }

  static const decltype(::cc::mojom::CopyOutputResult::size)& size(
      const ::cc::mojom::CopyOutputResultPtr& input) {
    return input->size;
  }

  static const decltype(::cc::mojom::CopyOutputResult::bitmap)& bitmap(
      const ::cc::mojom::CopyOutputResultPtr& input) {
    return input->bitmap;
  }

  static const decltype(::cc::mojom::CopyOutputResult::texture_mailbox)& texture_mailbox(
      const ::cc::mojom::CopyOutputResultPtr& input) {
    return input->texture_mailbox;
  }

  static  decltype(::cc::mojom::CopyOutputResult::releaser)& releaser(
       ::cc::mojom::CopyOutputResultPtr& input) {
    return input->releaser;
  }

  static bool Read(::cc::mojom::CopyOutputResult::DataView input, ::cc::mojom::CopyOutputResultPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_COPY_OUTPUT_RESULT_MOJOM_H_