// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_FILTER_OPERATION_MOJOM_BLINK_H_
#define CC_IPC_FILTER_OPERATION_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/filter_operation.mojom-shared.h"
#include "skia/public/interfaces/image_filter.mojom-blink.h"
#include "skia/public/interfaces/blur_image_filter_tile_mode.mojom-blink.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


namespace WTF {
struct cc_mojom_internal_FilterType_DataHashFn {
  static unsigned GetHash(const ::cc::mojom::FilterType& value) {
    using utype = std::underlying_type<::cc::mojom::FilterType>::type;
    return DefaultHash<utype>::Hash().GetHash(static_cast<utype>(value));
  }
  static bool Equal(const ::cc::mojom::FilterType& left, const ::cc::mojom::FilterType& right) {
    return left == right;
  }
  static const bool safe_to_compare_to_empty_or_deleted = true;
};

template <>
struct DefaultHash<::cc::mojom::FilterType> {
  using Hash = cc_mojom_internal_FilterType_DataHashFn;
};

template <>
struct HashTraits<::cc::mojom::FilterType>
    : public GenericHashTraits<::cc::mojom::FilterType> {
  static_assert(true,
                "-1000000 is a reserved enum value");
  static_assert(true,
                "-1000001 is a reserved enum value");
  static const bool hasIsEmptyValueFunction = true;
  static bool IsEmptyValue(const ::cc::mojom::FilterType& value) {
    return value == static_cast<::cc::mojom::FilterType>(-1000000);
  }
  static void ConstructDeletedValue(::cc::mojom::FilterType& slot, bool) {
    slot = static_cast<::cc::mojom::FilterType>(-1000001);
  }
  static bool IsDeletedValue(const ::cc::mojom::FilterType& value) {
    return value == static_cast<::cc::mojom::FilterType>(-1000001);
  }
};
}  // namespace WTF


namespace cc {
namespace mojom {
namespace blink {
using FilterType = FilterType;  // Alias for definition in the parent namespace.
class FilterOperation;
using FilterOperationPtr = mojo::StructPtr<FilterOperation>;






class  FilterOperation {
 public:
  using DataView = FilterOperationDataView;
  using Data_ = internal::FilterOperation_Data;

  template <typename... Args>
  static FilterOperationPtr New(Args&&... args) {
    return FilterOperationPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static FilterOperationPtr From(const U& u) {
    return mojo::TypeConverter<FilterOperationPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, FilterOperation>::Convert(*this);
  }


  FilterOperation();

  FilterOperation(
      FilterType type,
      float amount,
      ::gfx::mojom::blink::PointPtr drop_shadow_offset,
      uint32_t drop_shadow_color,
      ::skia::mojom::blink::ImageFilterPtr image_filter,
      const WTF::Optional<WTF::Vector<float>>& matrix,
      int32_t zoom_inset,
      ::skia::mojom::blink::BlurTileMode blur_tile_mode);

  ~FilterOperation();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = FilterOperationPtr>
  FilterOperationPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, FilterOperation>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        FilterOperation::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        FilterOperation::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  FilterType type;
  float amount;
  ::gfx::mojom::blink::PointPtr drop_shadow_offset;
  uint32_t drop_shadow_color;
  ::skia::mojom::blink::ImageFilterPtr image_filter;
  WTF::Optional<WTF::Vector<float>> matrix;
  int32_t zoom_inset;
  ::skia::mojom::blink::BlurTileMode blur_tile_mode;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(FilterOperation);
};


template <typename StructPtrType>
FilterOperationPtr FilterOperation::Clone() const {
  return New(
      mojo::Clone(type),
      mojo::Clone(amount),
      mojo::Clone(drop_shadow_offset),
      mojo::Clone(drop_shadow_color),
      mojo::Clone(image_filter),
      mojo::Clone(matrix),
      mojo::Clone(zoom_inset),
      mojo::Clone(blur_tile_mode)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, FilterOperation>::value>::type*>
bool FilterOperation::Equals(const T& other) const {
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->amount, other.amount))
    return false;
  if (!mojo::Equals(this->drop_shadow_offset, other.drop_shadow_offset))
    return false;
  if (!mojo::Equals(this->drop_shadow_color, other.drop_shadow_color))
    return false;
  if (!mojo::Equals(this->image_filter, other.image_filter))
    return false;
  if (!mojo::Equals(this->matrix, other.matrix))
    return false;
  if (!mojo::Equals(this->zoom_inset, other.zoom_inset))
    return false;
  if (!mojo::Equals(this->blur_tile_mode, other.blur_tile_mode))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::blink::FilterOperation::DataView,
                                         ::cc::mojom::blink::FilterOperationPtr> {
  static bool IsNull(const ::cc::mojom::blink::FilterOperationPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::blink::FilterOperationPtr* output) { output->reset(); }

  static decltype(::cc::mojom::blink::FilterOperation::type) type(
      const ::cc::mojom::blink::FilterOperationPtr& input) {
    return input->type;
  }

  static decltype(::cc::mojom::blink::FilterOperation::amount) amount(
      const ::cc::mojom::blink::FilterOperationPtr& input) {
    return input->amount;
  }

  static const decltype(::cc::mojom::blink::FilterOperation::drop_shadow_offset)& drop_shadow_offset(
      const ::cc::mojom::blink::FilterOperationPtr& input) {
    return input->drop_shadow_offset;
  }

  static decltype(::cc::mojom::blink::FilterOperation::drop_shadow_color) drop_shadow_color(
      const ::cc::mojom::blink::FilterOperationPtr& input) {
    return input->drop_shadow_color;
  }

  static const decltype(::cc::mojom::blink::FilterOperation::image_filter)& image_filter(
      const ::cc::mojom::blink::FilterOperationPtr& input) {
    return input->image_filter;
  }

  static const decltype(::cc::mojom::blink::FilterOperation::matrix)& matrix(
      const ::cc::mojom::blink::FilterOperationPtr& input) {
    return input->matrix;
  }

  static decltype(::cc::mojom::blink::FilterOperation::zoom_inset) zoom_inset(
      const ::cc::mojom::blink::FilterOperationPtr& input) {
    return input->zoom_inset;
  }

  static decltype(::cc::mojom::blink::FilterOperation::blur_tile_mode) blur_tile_mode(
      const ::cc::mojom::blink::FilterOperationPtr& input) {
    return input->blur_tile_mode;
  }

  static bool Read(::cc::mojom::blink::FilterOperation::DataView input, ::cc::mojom::blink::FilterOperationPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_FILTER_OPERATION_MOJOM_BLINK_H_