// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_FRAME_SINK_MANAGER_MOJOM_H_
#define CC_IPC_FRAME_SINK_MANAGER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/frame_sink_manager.mojom-shared.h"
#include "cc/ipc/frame_sink_id.mojom.h"
#include "cc/ipc/local_surface_id.mojom.h"
#include "cc/ipc/compositor_frame_sink.mojom.h"
#include "cc/ipc/surface_id.mojom.h"
#include "cc/ipc/surface_info.mojom.h"
#include "gpu/ipc/common/surface_handle.mojom.h"
#include "mojo/common/time.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/color_space.mojom.h"
#include <string>
#include <vector>


namespace cc {
namespace mojom {
class DisplayPrivate;
using DisplayPrivatePtr = mojo::InterfacePtr<DisplayPrivate>;
using DisplayPrivatePtrInfo = mojo::InterfacePtrInfo<DisplayPrivate>;
using ThreadSafeDisplayPrivatePtr =
    mojo::ThreadSafeInterfacePtr<DisplayPrivate>;
using DisplayPrivateRequest = mojo::InterfaceRequest<DisplayPrivate>;
using DisplayPrivateAssociatedPtr =
    mojo::AssociatedInterfacePtr<DisplayPrivate>;
using ThreadSafeDisplayPrivateAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<DisplayPrivate>;
using DisplayPrivateAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<DisplayPrivate>;
using DisplayPrivateAssociatedRequest =
    mojo::AssociatedInterfaceRequest<DisplayPrivate>;

class FrameSinkManager;
using FrameSinkManagerPtr = mojo::InterfacePtr<FrameSinkManager>;
using FrameSinkManagerPtrInfo = mojo::InterfacePtrInfo<FrameSinkManager>;
using ThreadSafeFrameSinkManagerPtr =
    mojo::ThreadSafeInterfacePtr<FrameSinkManager>;
using FrameSinkManagerRequest = mojo::InterfaceRequest<FrameSinkManager>;
using FrameSinkManagerAssociatedPtr =
    mojo::AssociatedInterfacePtr<FrameSinkManager>;
using ThreadSafeFrameSinkManagerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<FrameSinkManager>;
using FrameSinkManagerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<FrameSinkManager>;
using FrameSinkManagerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<FrameSinkManager>;

class FrameSinkManagerClient;
using FrameSinkManagerClientPtr = mojo::InterfacePtr<FrameSinkManagerClient>;
using FrameSinkManagerClientPtrInfo = mojo::InterfacePtrInfo<FrameSinkManagerClient>;
using ThreadSafeFrameSinkManagerClientPtr =
    mojo::ThreadSafeInterfacePtr<FrameSinkManagerClient>;
using FrameSinkManagerClientRequest = mojo::InterfaceRequest<FrameSinkManagerClient>;
using FrameSinkManagerClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<FrameSinkManagerClient>;
using ThreadSafeFrameSinkManagerClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<FrameSinkManagerClient>;
using FrameSinkManagerClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<FrameSinkManagerClient>;
using FrameSinkManagerClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<FrameSinkManagerClient>;


class DisplayPrivateProxy;

template <typename ImplRefTraits>
class DisplayPrivateStub;

class DisplayPrivateRequestValidator;

class  DisplayPrivate
    : public DisplayPrivateInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = DisplayPrivateProxy;

  template <typename ImplRefTraits>
  using Stub_ = DisplayPrivateStub<ImplRefTraits>;

  using RequestValidator_ = DisplayPrivateRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetDisplayVisibleMinVersion = 0,
    kResizeDisplayMinVersion = 0,
    kSetDisplayColorSpaceMinVersion = 0,
    kSetOutputIsSecureMinVersion = 0,
    kSetLocalSurfaceIdMinVersion = 0,
  };
  virtual ~DisplayPrivate() {}

  virtual void SetDisplayVisible(bool visible) = 0;

  virtual void ResizeDisplay(const gfx::Size& size_in_pixel) = 0;

  virtual void SetDisplayColorSpace(const gfx::ColorSpace& color_space) = 0;

  virtual void SetOutputIsSecure(bool secure) = 0;

  virtual void SetLocalSurfaceId(const viz::LocalSurfaceId& local_surface_id, float scale_factor) = 0;
};

class FrameSinkManagerProxy;

template <typename ImplRefTraits>
class FrameSinkManagerStub;

class FrameSinkManagerRequestValidator;

class  FrameSinkManager
    : public FrameSinkManagerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = true;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = FrameSinkManagerProxy;

  template <typename ImplRefTraits>
  using Stub_ = FrameSinkManagerStub<ImplRefTraits>;

  using RequestValidator_ = FrameSinkManagerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateRootCompositorFrameSinkMinVersion = 0,
    kCreateCompositorFrameSinkMinVersion = 0,
    kRegisterFrameSinkHierarchyMinVersion = 0,
    kUnregisterFrameSinkHierarchyMinVersion = 0,
    kDropTemporaryReferenceMinVersion = 0,
  };
  virtual ~FrameSinkManager() {}

  virtual void CreateRootCompositorFrameSink(const viz::FrameSinkId& frame_sink_id, ::gpu::SurfaceHandle widget, ::cc::mojom::CompositorFrameSinkAssociatedRequest compositor_frame_sink, ::cc::mojom::CompositorFrameSinkPrivateRequest compositor_frame_sink_private, ::cc::mojom::CompositorFrameSinkClientPtr compositor_frame_sink_client, DisplayPrivateAssociatedRequest display_private) = 0;

  virtual void CreateCompositorFrameSink(const viz::FrameSinkId& frame_sink_id, ::cc::mojom::CompositorFrameSinkRequest compositor_frame_sink, ::cc::mojom::CompositorFrameSinkPrivateRequest compositor_frame_sink_private, ::cc::mojom::CompositorFrameSinkClientPtr compositor_frame_sink_client) = 0;

  virtual void RegisterFrameSinkHierarchy(const viz::FrameSinkId& parent_frame_sink_id, const viz::FrameSinkId& child_frame_sink_id) = 0;

  virtual void UnregisterFrameSinkHierarchy(const viz::FrameSinkId& parent_frame_sink_id, const viz::FrameSinkId& child_frame_sink_id) = 0;

  virtual void DropTemporaryReference(const viz::SurfaceId& surface_id) = 0;
};

class FrameSinkManagerClientProxy;

template <typename ImplRefTraits>
class FrameSinkManagerClientStub;

class FrameSinkManagerClientRequestValidator;

class  FrameSinkManagerClient
    : public FrameSinkManagerClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = FrameSinkManagerClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = FrameSinkManagerClientStub<ImplRefTraits>;

  using RequestValidator_ = FrameSinkManagerClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnSurfaceCreatedMinVersion = 0,
    kOnClientConnectionClosedMinVersion = 0,
  };
  virtual ~FrameSinkManagerClient() {}

  virtual void OnSurfaceCreated(const viz::SurfaceInfo& surface_info) = 0;

  virtual void OnClientConnectionClosed(const viz::FrameSinkId& frame_sink_id) = 0;
};

class  DisplayPrivateProxy
    : public DisplayPrivate {
 public:
  explicit DisplayPrivateProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetDisplayVisible(bool visible) override;
  void ResizeDisplay(const gfx::Size& size_in_pixel) override;
  void SetDisplayColorSpace(const gfx::ColorSpace& color_space) override;
  void SetOutputIsSecure(bool secure) override;
  void SetLocalSurfaceId(const viz::LocalSurfaceId& local_surface_id, float scale_factor) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  FrameSinkManagerProxy
    : public FrameSinkManager {
 public:
  explicit FrameSinkManagerProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateRootCompositorFrameSink(const viz::FrameSinkId& frame_sink_id, ::gpu::SurfaceHandle widget, ::cc::mojom::CompositorFrameSinkAssociatedRequest compositor_frame_sink, ::cc::mojom::CompositorFrameSinkPrivateRequest compositor_frame_sink_private, ::cc::mojom::CompositorFrameSinkClientPtr compositor_frame_sink_client, DisplayPrivateAssociatedRequest display_private) override;
  void CreateCompositorFrameSink(const viz::FrameSinkId& frame_sink_id, ::cc::mojom::CompositorFrameSinkRequest compositor_frame_sink, ::cc::mojom::CompositorFrameSinkPrivateRequest compositor_frame_sink_private, ::cc::mojom::CompositorFrameSinkClientPtr compositor_frame_sink_client) override;
  void RegisterFrameSinkHierarchy(const viz::FrameSinkId& parent_frame_sink_id, const viz::FrameSinkId& child_frame_sink_id) override;
  void UnregisterFrameSinkHierarchy(const viz::FrameSinkId& parent_frame_sink_id, const viz::FrameSinkId& child_frame_sink_id) override;
  void DropTemporaryReference(const viz::SurfaceId& surface_id) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  FrameSinkManagerClientProxy
    : public FrameSinkManagerClient {
 public:
  explicit FrameSinkManagerClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnSurfaceCreated(const viz::SurfaceInfo& surface_info) override;
  void OnClientConnectionClosed(const viz::FrameSinkId& frame_sink_id) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  DisplayPrivateStubDispatch {
 public:
  static bool Accept(DisplayPrivate* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      DisplayPrivate* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<DisplayPrivate>>
class DisplayPrivateStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  DisplayPrivateStub() {}
  ~DisplayPrivateStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DisplayPrivateStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DisplayPrivateStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  FrameSinkManagerStubDispatch {
 public:
  static bool Accept(FrameSinkManager* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      FrameSinkManager* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<FrameSinkManager>>
class FrameSinkManagerStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FrameSinkManagerStub() {}
  ~FrameSinkManagerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameSinkManagerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameSinkManagerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  FrameSinkManagerClientStubDispatch {
 public:
  static bool Accept(FrameSinkManagerClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      FrameSinkManagerClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<FrameSinkManagerClient>>
class FrameSinkManagerClientStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FrameSinkManagerClientStub() {}
  ~FrameSinkManagerClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameSinkManagerClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameSinkManagerClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  DisplayPrivateRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  FrameSinkManagerRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  FrameSinkManagerClientRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace cc

namespace mojo {

}  // namespace mojo

#endif  // CC_IPC_FRAME_SINK_MANAGER_MOJOM_H_