// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_LOCAL_SURFACE_ID_MOJOM_H_
#define CC_IPC_LOCAL_SURFACE_ID_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/local_surface_id.mojom-shared.h"
#include "mojo/common/unguessable_token.mojom.h"
#include <string>
#include <vector>
#include "components/viz/common/surfaces/local_surface_id.h"


namespace cc {
namespace mojom {
class LocalSurfaceId;
using LocalSurfaceIdPtr = mojo::StructPtr<LocalSurfaceId>;






class  LocalSurfaceId {
 public:
  using DataView = LocalSurfaceIdDataView;
  using Data_ = internal::LocalSurfaceId_Data;

  template <typename... Args>
  static LocalSurfaceIdPtr New(Args&&... args) {
    return LocalSurfaceIdPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static LocalSurfaceIdPtr From(const U& u) {
    return mojo::TypeConverter<LocalSurfaceIdPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, LocalSurfaceId>::Convert(*this);
  }


  LocalSurfaceId();

  LocalSurfaceId(
      uint32_t local_id,
      const base::UnguessableToken& nonce);

  ~LocalSurfaceId();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = LocalSurfaceIdPtr>
  LocalSurfaceIdPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, LocalSurfaceId>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        LocalSurfaceId::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        LocalSurfaceId::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint32_t local_id;
  base::UnguessableToken nonce;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
LocalSurfaceIdPtr LocalSurfaceId::Clone() const {
  return New(
      mojo::Clone(local_id),
      mojo::Clone(nonce)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, LocalSurfaceId>::value>::type*>
bool LocalSurfaceId::Equals(const T& other) const {
  if (!mojo::Equals(this->local_id, other.local_id))
    return false;
  if (!mojo::Equals(this->nonce, other.nonce))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::LocalSurfaceId::DataView,
                                         ::cc::mojom::LocalSurfaceIdPtr> {
  static bool IsNull(const ::cc::mojom::LocalSurfaceIdPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::LocalSurfaceIdPtr* output) { output->reset(); }

  static decltype(::cc::mojom::LocalSurfaceId::local_id) local_id(
      const ::cc::mojom::LocalSurfaceIdPtr& input) {
    return input->local_id;
  }

  static const decltype(::cc::mojom::LocalSurfaceId::nonce)& nonce(
      const ::cc::mojom::LocalSurfaceIdPtr& input) {
    return input->nonce;
  }

  static bool Read(::cc::mojom::LocalSurfaceId::DataView input, ::cc::mojom::LocalSurfaceIdPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_LOCAL_SURFACE_ID_MOJOM_H_