// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_SELECTION_MOJOM_BLINK_H_
#define CC_IPC_SELECTION_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/selection.mojom-shared.h"
#include "ui/gfx/mojo/selection_bound.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


namespace cc {
namespace mojom {
namespace blink {
class Selection;
using SelectionPtr = mojo::StructPtr<Selection>;






class  Selection {
 public:
  using DataView = SelectionDataView;
  using Data_ = internal::Selection_Data;

  template <typename... Args>
  static SelectionPtr New(Args&&... args) {
    return SelectionPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static SelectionPtr From(const U& u) {
    return mojo::TypeConverter<SelectionPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, Selection>::Convert(*this);
  }


  Selection();

  Selection(
      ::gfx::mojom::blink::SelectionBoundPtr start,
      ::gfx::mojom::blink::SelectionBoundPtr end);

  ~Selection();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SelectionPtr>
  SelectionPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, Selection>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        Selection::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        Selection::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  ::gfx::mojom::blink::SelectionBoundPtr start;
  ::gfx::mojom::blink::SelectionBoundPtr end;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(Selection);
};


template <typename StructPtrType>
SelectionPtr Selection::Clone() const {
  return New(
      mojo::Clone(start),
      mojo::Clone(end)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, Selection>::value>::type*>
bool Selection::Equals(const T& other) const {
  if (!mojo::Equals(this->start, other.start))
    return false;
  if (!mojo::Equals(this->end, other.end))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::blink::Selection::DataView,
                                         ::cc::mojom::blink::SelectionPtr> {
  static bool IsNull(const ::cc::mojom::blink::SelectionPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::blink::SelectionPtr* output) { output->reset(); }

  static const decltype(::cc::mojom::blink::Selection::start)& start(
      const ::cc::mojom::blink::SelectionPtr& input) {
    return input->start;
  }

  static const decltype(::cc::mojom::blink::Selection::end)& end(
      const ::cc::mojom::blink::SelectionPtr& input) {
    return input->end;
  }

  static bool Read(::cc::mojom::blink::Selection::DataView input, ::cc::mojom::blink::SelectionPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_SELECTION_MOJOM_BLINK_H_