// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_TEXTURE_MAILBOX_MOJOM_SHARED_H_
#define CC_IPC_TEXTURE_MAILBOX_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "cc/ipc/texture_mailbox.mojom-shared-internal.h"
#include "gpu/ipc/common/mailbox_holder.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"
#include "ui/gfx/mojo/color_space.mojom-shared.h"


namespace cc {
namespace mojom {
class TextureMailboxDataView;



}  // namespace mojom
}  // namespace cc

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::cc::mojom::TextureMailboxDataView> {
  using Data = ::cc::mojom::internal::TextureMailbox_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace cc {
namespace mojom {
class TextureMailboxDataView {
 public:
  TextureMailboxDataView() {}

  TextureMailboxDataView(
      internal::TextureMailbox_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMailboxHolderDataView(
      ::gpu::mojom::MailboxHolderDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMailboxHolder(UserType* output) {
    auto* pointer = data_->mailbox_holder.Get();
    return mojo::internal::Deserialize<::gpu::mojom::MailboxHolderDataView>(
        pointer, output, context_);
  }
  inline void GetSizeInPixelsDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSizeInPixels(UserType* output) {
    auto* pointer = data_->size_in_pixels.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  bool is_overlay_candidate() const {
    return data_->is_overlay_candidate;
  }
  bool is_backed_by_surface_texture() const {
    return data_->is_backed_by_surface_texture;
  }
  bool wants_promotion_hint() const {
    return data_->wants_promotion_hint;
  }
  bool secure_output_only() const {
    return data_->secure_output_only;
  }
  bool nearest_neighbor() const {
    return data_->nearest_neighbor;
  }
  inline void GetColorSpaceDataView(
      ::gfx::mojom::ColorSpaceDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadColorSpace(UserType* output) {
    auto* pointer = data_->color_space.Get();
    return mojo::internal::Deserialize<::gfx::mojom::ColorSpaceDataView>(
        pointer, output, context_);
  }
 private:
  internal::TextureMailbox_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace cc

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::cc::mojom::TextureMailboxDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::cc::mojom::TextureMailboxDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::cc::mojom::internal::TextureMailbox_Data);
    decltype(CallWithContext(Traits::mailbox_holder, input, custom_context)) in_mailbox_holder = CallWithContext(Traits::mailbox_holder, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gpu::mojom::MailboxHolderDataView>(
        in_mailbox_holder, context);
    decltype(CallWithContext(Traits::size_in_pixels, input, custom_context)) in_size_in_pixels = CallWithContext(Traits::size_in_pixels, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::SizeDataView>(
        in_size_in_pixels, context);
    decltype(CallWithContext(Traits::color_space, input, custom_context)) in_color_space = CallWithContext(Traits::color_space, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::ColorSpaceDataView>(
        in_color_space, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::cc::mojom::internal::TextureMailbox_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::cc::mojom::internal::TextureMailbox_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::mailbox_holder, input, custom_context)) in_mailbox_holder = CallWithContext(Traits::mailbox_holder, input, custom_context);
    typename decltype(result->mailbox_holder)::BaseType* mailbox_holder_ptr;
    mojo::internal::Serialize<::gpu::mojom::MailboxHolderDataView>(
        in_mailbox_holder, buffer, &mailbox_holder_ptr, context);
    result->mailbox_holder.Set(mailbox_holder_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->mailbox_holder.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null mailbox_holder in TextureMailbox struct");
    decltype(CallWithContext(Traits::size_in_pixels, input, custom_context)) in_size_in_pixels = CallWithContext(Traits::size_in_pixels, input, custom_context);
    typename decltype(result->size_in_pixels)::BaseType* size_in_pixels_ptr;
    mojo::internal::Serialize<::gfx::mojom::SizeDataView>(
        in_size_in_pixels, buffer, &size_in_pixels_ptr, context);
    result->size_in_pixels.Set(size_in_pixels_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->size_in_pixels.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null size_in_pixels in TextureMailbox struct");
    result->is_overlay_candidate = CallWithContext(Traits::is_overlay_candidate, input, custom_context);
    result->is_backed_by_surface_texture = CallWithContext(Traits::is_backed_by_surface_texture, input, custom_context);
    result->wants_promotion_hint = CallWithContext(Traits::wants_promotion_hint, input, custom_context);
    result->secure_output_only = CallWithContext(Traits::secure_output_only, input, custom_context);
    result->nearest_neighbor = CallWithContext(Traits::nearest_neighbor, input, custom_context);
    decltype(CallWithContext(Traits::color_space, input, custom_context)) in_color_space = CallWithContext(Traits::color_space, input, custom_context);
    typename decltype(result->color_space)::BaseType* color_space_ptr;
    mojo::internal::Serialize<::gfx::mojom::ColorSpaceDataView>(
        in_color_space, buffer, &color_space_ptr, context);
    result->color_space.Set(color_space_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->color_space.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null color_space in TextureMailbox struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::cc::mojom::internal::TextureMailbox_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::cc::mojom::TextureMailboxDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace cc {
namespace mojom {

inline void TextureMailboxDataView::GetMailboxHolderDataView(
    ::gpu::mojom::MailboxHolderDataView* output) {
  auto pointer = data_->mailbox_holder.Get();
  *output = ::gpu::mojom::MailboxHolderDataView(pointer, context_);
}
inline void TextureMailboxDataView::GetSizeInPixelsDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->size_in_pixels.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}
inline void TextureMailboxDataView::GetColorSpaceDataView(
    ::gfx::mojom::ColorSpaceDataView* output) {
  auto pointer = data_->color_space.Get();
  *output = ::gfx::mojom::ColorSpaceDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace cc

#endif  // CC_IPC_TEXTURE_MAILBOX_MOJOM_SHARED_H_
