// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_TEXTURE_MAILBOX_MOJOM_H_
#define CC_IPC_TEXTURE_MAILBOX_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/texture_mailbox.mojom-shared.h"
#include "gpu/ipc/common/mailbox_holder.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/color_space.mojom.h"
#include <string>
#include <vector>
#include "components/viz/common/quads/texture_mailbox.h"


namespace cc {
namespace mojom {
class TextureMailbox;
using TextureMailboxPtr = mojo::StructPtr<TextureMailbox>;






class  TextureMailbox {
 public:
  using DataView = TextureMailboxDataView;
  using Data_ = internal::TextureMailbox_Data;

  template <typename... Args>
  static TextureMailboxPtr New(Args&&... args) {
    return TextureMailboxPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static TextureMailboxPtr From(const U& u) {
    return mojo::TypeConverter<TextureMailboxPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, TextureMailbox>::Convert(*this);
  }


  TextureMailbox();

  TextureMailbox(
      const ::gpu::MailboxHolder& mailbox_holder,
      const gfx::Size& size_in_pixels,
      bool is_overlay_candidate,
      bool is_backed_by_surface_texture,
      bool wants_promotion_hint,
      bool secure_output_only,
      bool nearest_neighbor,
      const gfx::ColorSpace& color_space);

  ~TextureMailbox();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = TextureMailboxPtr>
  TextureMailboxPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, TextureMailbox>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        TextureMailbox::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        TextureMailbox::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  ::gpu::MailboxHolder mailbox_holder;
  gfx::Size size_in_pixels;
  bool is_overlay_candidate;
  bool is_backed_by_surface_texture;
  bool wants_promotion_hint;
  bool secure_output_only;
  bool nearest_neighbor;
  gfx::ColorSpace color_space;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
TextureMailboxPtr TextureMailbox::Clone() const {
  return New(
      mojo::Clone(mailbox_holder),
      mojo::Clone(size_in_pixels),
      mojo::Clone(is_overlay_candidate),
      mojo::Clone(is_backed_by_surface_texture),
      mojo::Clone(wants_promotion_hint),
      mojo::Clone(secure_output_only),
      mojo::Clone(nearest_neighbor),
      mojo::Clone(color_space)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, TextureMailbox>::value>::type*>
bool TextureMailbox::Equals(const T& other) const {
  if (!mojo::Equals(this->mailbox_holder, other.mailbox_holder))
    return false;
  if (!mojo::Equals(this->size_in_pixels, other.size_in_pixels))
    return false;
  if (!mojo::Equals(this->is_overlay_candidate, other.is_overlay_candidate))
    return false;
  if (!mojo::Equals(this->is_backed_by_surface_texture, other.is_backed_by_surface_texture))
    return false;
  if (!mojo::Equals(this->wants_promotion_hint, other.wants_promotion_hint))
    return false;
  if (!mojo::Equals(this->secure_output_only, other.secure_output_only))
    return false;
  if (!mojo::Equals(this->nearest_neighbor, other.nearest_neighbor))
    return false;
  if (!mojo::Equals(this->color_space, other.color_space))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::TextureMailbox::DataView,
                                         ::cc::mojom::TextureMailboxPtr> {
  static bool IsNull(const ::cc::mojom::TextureMailboxPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::TextureMailboxPtr* output) { output->reset(); }

  static const decltype(::cc::mojom::TextureMailbox::mailbox_holder)& mailbox_holder(
      const ::cc::mojom::TextureMailboxPtr& input) {
    return input->mailbox_holder;
  }

  static const decltype(::cc::mojom::TextureMailbox::size_in_pixels)& size_in_pixels(
      const ::cc::mojom::TextureMailboxPtr& input) {
    return input->size_in_pixels;
  }

  static decltype(::cc::mojom::TextureMailbox::is_overlay_candidate) is_overlay_candidate(
      const ::cc::mojom::TextureMailboxPtr& input) {
    return input->is_overlay_candidate;
  }

  static decltype(::cc::mojom::TextureMailbox::is_backed_by_surface_texture) is_backed_by_surface_texture(
      const ::cc::mojom::TextureMailboxPtr& input) {
    return input->is_backed_by_surface_texture;
  }

  static decltype(::cc::mojom::TextureMailbox::wants_promotion_hint) wants_promotion_hint(
      const ::cc::mojom::TextureMailboxPtr& input) {
    return input->wants_promotion_hint;
  }

  static decltype(::cc::mojom::TextureMailbox::secure_output_only) secure_output_only(
      const ::cc::mojom::TextureMailboxPtr& input) {
    return input->secure_output_only;
  }

  static decltype(::cc::mojom::TextureMailbox::nearest_neighbor) nearest_neighbor(
      const ::cc::mojom::TextureMailboxPtr& input) {
    return input->nearest_neighbor;
  }

  static const decltype(::cc::mojom::TextureMailbox::color_space)& color_space(
      const ::cc::mojom::TextureMailboxPtr& input) {
    return input->color_space;
  }

  static bool Read(::cc::mojom::TextureMailbox::DataView input, ::cc::mojom::TextureMailboxPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_TEXTURE_MAILBOX_MOJOM_H_