// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_INPUT_INPUT_HANDLER_MOJOM_SHARED_INTERNAL_H_
#define CONTENT_COMMON_INPUT_INPUT_HANDLER_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "content/common/native_types.mojom-shared-internal.h"
#include "mojo/common/string16.mojom-shared-internal.h"
#include "services/ui/public/interfaces/ime/ime.mojom-shared-internal.h"
#include "ui/events/mojo/event.mojom-shared-internal.h"
#include "ui/events/mojo/event_constants.mojom-shared-internal.h"
#include "ui/latency/mojo/latency_info.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared-internal.h"
#include "ui/gfx/range/mojo/range.mojom-shared-internal.h"
#include "content/common/mojo_bindings_shared_export.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace content {
namespace mojom {
namespace internal {
class KeyData_Data;
class PointerData_Data;
class WheelData_Data;
class MouseData_Data;
class ScrollUpdate_Data;
class ScrollData_Data;
class PinchData_Data;
class FlingData_Data;
class TapData_Data;
class GestureData_Data;
class TouchPoint_Data;
class TouchData_Data;
class Event_Data;

#pragma pack(push, 1)
class MOJOM_SHARED_CONTENT_EXPORT KeyData_Data {
 public:
  static KeyData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t dom_key;
  int32_t dom_code;
  int32_t windows_key_code;
  int32_t native_key_code;
  uint8_t is_system_key : 1;
  uint8_t is_browser_shortcut : 1;
  uint8_t pad5_[7];
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> text;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> unmodified_text;

 private:
  KeyData_Data();
  ~KeyData_Data() = delete;
};
static_assert(sizeof(KeyData_Data) == 48,
              "Bad sizeof(KeyData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT PointerData_Data {
 public:
  static PointerData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t pointer_id;
  float force;
  int32_t tilt_x;
  int32_t tilt_y;
  float tangential_pressure;
  int32_t twist;
  int32_t button;
  int32_t pointer_type;
  int32_t movement_x;
  int32_t movement_y;
  mojo::internal::Pointer<::gfx::mojom::internal::PointF_Data> widget_position;
  mojo::internal::Pointer<::gfx::mojom::internal::PointF_Data> screen_position;
  mojo::internal::Pointer<internal::MouseData_Data> mouse_data;

 private:
  PointerData_Data();
  ~PointerData_Data() = delete;
};
static_assert(sizeof(PointerData_Data) == 72,
              "Bad sizeof(PointerData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT WheelData_Data {
 public:
  static WheelData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  float delta_x;
  float delta_y;
  float wheel_ticks_x;
  float wheel_ticks_y;
  float acceleration_ratio_x;
  float acceleration_ratio_y;
  int32_t resending_plugin_id;
  uint8_t phase;
  uint8_t momentum_phase;
  uint8_t scroll_by_page : 1;
  uint8_t has_precise_scrolling_deltas : 1;
  uint8_t pad10_[1];
  int32_t cancelable;
  uint8_t padfinal_[4];

 private:
  WheelData_Data();
  ~WheelData_Data() = delete;
};
static_assert(sizeof(WheelData_Data) == 48,
              "Bad sizeof(WheelData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT MouseData_Data {
 public:
  static MouseData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t click_count;
  uint8_t pad0_[4];
  mojo::internal::Pointer<internal::WheelData_Data> wheel_data;

 private:
  MouseData_Data();
  ~MouseData_Data() = delete;
};
static_assert(sizeof(MouseData_Data) == 24,
              "Bad sizeof(MouseData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT ScrollUpdate_Data {
 public:
  static ScrollUpdate_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  float velocity_x;
  float velocity_y;
  uint8_t previous_update_in_sequence_prevented : 1;
  uint8_t prevent_propagation : 1;
  uint8_t padfinal_[7];

 private:
  ScrollUpdate_Data();
  ~ScrollUpdate_Data() = delete;
};
static_assert(sizeof(ScrollUpdate_Data) == 24,
              "Bad sizeof(ScrollUpdate_Data)");
class MOJOM_SHARED_CONTENT_EXPORT ScrollData_Data {
 public:
  static ScrollData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  float delta_x;
  float delta_y;
  int32_t delta_units;
  uint8_t target_viewport : 1;
  uint8_t synthetic : 1;
  uint8_t pad4_[3];
  int32_t inertial_phase;
  int32_t pointer_count;
  mojo::internal::Pointer<internal::ScrollUpdate_Data> update_details;

 private:
  ScrollData_Data();
  ~ScrollData_Data() = delete;
};
static_assert(sizeof(ScrollData_Data) == 40,
              "Bad sizeof(ScrollData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT PinchData_Data {
 public:
  static PinchData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t zoom_disabled : 1;
  uint8_t pad0_[3];
  float scale;

 private:
  PinchData_Data();
  ~PinchData_Data() = delete;
};
static_assert(sizeof(PinchData_Data) == 16,
              "Bad sizeof(PinchData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT FlingData_Data {
 public:
  static FlingData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  float velocity_x;
  float velocity_y;
  uint8_t target_viewport : 1;
  uint8_t prevent_boosting : 1;
  uint8_t padfinal_[7];

 private:
  FlingData_Data();
  ~FlingData_Data() = delete;
};
static_assert(sizeof(FlingData_Data) == 24,
              "Bad sizeof(FlingData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT TapData_Data {
 public:
  static TapData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t tap_count;
  uint8_t padfinal_[4];

 private:
  TapData_Data();
  ~TapData_Data() = delete;
};
static_assert(sizeof(TapData_Data) == 16,
              "Bad sizeof(TapData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT GestureData_Data {
 public:
  static GestureData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::gfx::mojom::internal::PointF_Data> screen_position;
  mojo::internal::Pointer<::gfx::mojom::internal::PointF_Data> widget_position;
  int32_t source_device;
  int32_t unique_touch_event_id;
  int32_t resending_plugin_id;
  uint8_t pad4_[4];
  mojo::internal::Pointer<::gfx::mojom::internal::Size_Data> contact_size;
  mojo::internal::Pointer<internal::ScrollData_Data> scroll_data;
  mojo::internal::Pointer<internal::PinchData_Data> pinch_data;
  mojo::internal::Pointer<internal::TapData_Data> tap_data;
  mojo::internal::Pointer<internal::FlingData_Data> fling_data;

 private:
  GestureData_Data();
  ~GestureData_Data() = delete;
};
static_assert(sizeof(GestureData_Data) == 80,
              "Bad sizeof(GestureData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT TouchPoint_Data {
 public:
  static TouchPoint_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t state;
  float radius_x;
  float radius_y;
  float rotation_angle;
  mojo::internal::Pointer<internal::PointerData_Data> pointer_data;

 private:
  TouchPoint_Data();
  ~TouchPoint_Data() = delete;
};
static_assert(sizeof(TouchPoint_Data) == 32,
              "Bad sizeof(TouchPoint_Data)");
class MOJOM_SHARED_CONTENT_EXPORT TouchData_Data {
 public:
  static TouchData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t cancelable;
  uint8_t moved_beyond_slop_region : 1;
  uint8_t touch_start_or_first_move : 1;
  uint8_t pad2_[3];
  uint32_t unique_touch_event_id;
  uint8_t pad3_[4];
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::TouchPoint_Data>>> touches;

 private:
  TouchData_Data();
  ~TouchData_Data() = delete;
};
static_assert(sizeof(TouchData_Data) == 32,
              "Bad sizeof(TouchData_Data)");
class MOJOM_SHARED_CONTENT_EXPORT Event_Data {
 public:
  static Event_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t type;
  int32_t modifiers;
  double timestamp_seconds;
  mojo::internal::Pointer<::ui::mojom::internal::LatencyInfo_Data> latency;
  mojo::internal::Pointer<internal::KeyData_Data> key_data;
  mojo::internal::Pointer<internal::PointerData_Data> pointer_data;
  mojo::internal::Pointer<internal::GestureData_Data> gesture_data;
  mojo::internal::Pointer<internal::TouchData_Data> touch_data;

 private:
  Event_Data();
  ~Event_Data() = delete;
};
static_assert(sizeof(Event_Data) == 64,
              "Bad sizeof(Event_Data)");
constexpr uint32_t kWidgetInputHandler_SetFocus_Name = 0;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_SetFocus_Params_Data {
 public:
  static WidgetInputHandler_SetFocus_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t focused : 1;
  uint8_t padfinal_[7];

 private:
  WidgetInputHandler_SetFocus_Params_Data();
  ~WidgetInputHandler_SetFocus_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_SetFocus_Params_Data) == 16,
              "Bad sizeof(WidgetInputHandler_SetFocus_Params_Data)");
constexpr uint32_t kWidgetInputHandler_MouseCaptureLost_Name = 1;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_MouseCaptureLost_Params_Data {
 public:
  static WidgetInputHandler_MouseCaptureLost_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  WidgetInputHandler_MouseCaptureLost_Params_Data();
  ~WidgetInputHandler_MouseCaptureLost_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_MouseCaptureLost_Params_Data) == 8,
              "Bad sizeof(WidgetInputHandler_MouseCaptureLost_Params_Data)");
constexpr uint32_t kWidgetInputHandler_SetEditCommandsForNextKeyEvent_Name = 2;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data {
 public:
  static WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::content::mojom::internal::EditCommand_Data>>> commands;

 private:
  WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data();
  ~WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data) == 16,
              "Bad sizeof(WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data)");
constexpr uint32_t kWidgetInputHandler_CursorVisibilityChanged_Name = 3;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_CursorVisibilityChanged_Params_Data {
 public:
  static WidgetInputHandler_CursorVisibilityChanged_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t visible : 1;
  uint8_t padfinal_[7];

 private:
  WidgetInputHandler_CursorVisibilityChanged_Params_Data();
  ~WidgetInputHandler_CursorVisibilityChanged_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_CursorVisibilityChanged_Params_Data) == 16,
              "Bad sizeof(WidgetInputHandler_CursorVisibilityChanged_Params_Data)");
constexpr uint32_t kWidgetInputHandler_ImeSetComposition_Name = 4;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_ImeSetComposition_Params_Data {
 public:
  static WidgetInputHandler_ImeSetComposition_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> text;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::ui::mojom::internal::CompositionUnderline_Data>>> underlines;
  mojo::internal::Pointer<::gfx::mojom::internal::Range_Data> range;
  int32_t start;
  int32_t end;

 private:
  WidgetInputHandler_ImeSetComposition_Params_Data();
  ~WidgetInputHandler_ImeSetComposition_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_ImeSetComposition_Params_Data) == 40,
              "Bad sizeof(WidgetInputHandler_ImeSetComposition_Params_Data)");
constexpr uint32_t kWidgetInputHandler_ImeCommitText_Name = 5;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_ImeCommitText_Params_Data {
 public:
  static WidgetInputHandler_ImeCommitText_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> text;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::ui::mojom::internal::CompositionUnderline_Data>>> underlines;
  mojo::internal::Pointer<::gfx::mojom::internal::Range_Data> range;
  int32_t relative_cursor_position;
  uint8_t padfinal_[4];

 private:
  WidgetInputHandler_ImeCommitText_Params_Data();
  ~WidgetInputHandler_ImeCommitText_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_ImeCommitText_Params_Data) == 40,
              "Bad sizeof(WidgetInputHandler_ImeCommitText_Params_Data)");
constexpr uint32_t kWidgetInputHandler_ImeFinishComposingText_Name = 6;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_ImeFinishComposingText_Params_Data {
 public:
  static WidgetInputHandler_ImeFinishComposingText_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t keep_selection : 1;
  uint8_t padfinal_[7];

 private:
  WidgetInputHandler_ImeFinishComposingText_Params_Data();
  ~WidgetInputHandler_ImeFinishComposingText_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_ImeFinishComposingText_Params_Data) == 16,
              "Bad sizeof(WidgetInputHandler_ImeFinishComposingText_Params_Data)");
constexpr uint32_t kWidgetInputHandler_RequestTextInputStateUpdate_Name = 7;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_RequestTextInputStateUpdate_Params_Data {
 public:
  static WidgetInputHandler_RequestTextInputStateUpdate_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  WidgetInputHandler_RequestTextInputStateUpdate_Params_Data();
  ~WidgetInputHandler_RequestTextInputStateUpdate_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_RequestTextInputStateUpdate_Params_Data) == 8,
              "Bad sizeof(WidgetInputHandler_RequestTextInputStateUpdate_Params_Data)");
constexpr uint32_t kWidgetInputHandler_RequestCompositionUpdates_Name = 8;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_RequestCompositionUpdates_Params_Data {
 public:
  static WidgetInputHandler_RequestCompositionUpdates_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t immediate_request : 1;
  uint8_t monitor_request : 1;
  uint8_t padfinal_[7];

 private:
  WidgetInputHandler_RequestCompositionUpdates_Params_Data();
  ~WidgetInputHandler_RequestCompositionUpdates_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_RequestCompositionUpdates_Params_Data) == 16,
              "Bad sizeof(WidgetInputHandler_RequestCompositionUpdates_Params_Data)");
constexpr uint32_t kWidgetInputHandler_DispatchEvent_Name = 9;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_DispatchEvent_Params_Data {
 public:
  static WidgetInputHandler_DispatchEvent_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::Event_Data> event;

 private:
  WidgetInputHandler_DispatchEvent_Params_Data();
  ~WidgetInputHandler_DispatchEvent_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_DispatchEvent_Params_Data) == 16,
              "Bad sizeof(WidgetInputHandler_DispatchEvent_Params_Data)");
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_DispatchEvent_ResponseParams_Data {
 public:
  static WidgetInputHandler_DispatchEvent_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t source;
  int32_t state;
  mojo::internal::Pointer<::ui::mojom::internal::LatencyInfo_Data> updated_latency;
  mojo::internal::Pointer<::content::mojom::internal::DidOverscrollParams_Data> overscroll;

 private:
  WidgetInputHandler_DispatchEvent_ResponseParams_Data();
  ~WidgetInputHandler_DispatchEvent_ResponseParams_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_DispatchEvent_ResponseParams_Data) == 32,
              "Bad sizeof(WidgetInputHandler_DispatchEvent_ResponseParams_Data)");
constexpr uint32_t kWidgetInputHandler_DispatchNonBlockingEvent_Name = 10;
class MOJOM_SHARED_CONTENT_EXPORT WidgetInputHandler_DispatchNonBlockingEvent_Params_Data {
 public:
  static WidgetInputHandler_DispatchNonBlockingEvent_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::Event_Data> event;

 private:
  WidgetInputHandler_DispatchNonBlockingEvent_Params_Data();
  ~WidgetInputHandler_DispatchNonBlockingEvent_Params_Data() = delete;
};
static_assert(sizeof(WidgetInputHandler_DispatchNonBlockingEvent_Params_Data) == 16,
              "Bad sizeof(WidgetInputHandler_DispatchNonBlockingEvent_Params_Data)");
constexpr uint32_t kFrameInputHandler_SetCompositionFromExistingText_Name = 0;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_SetCompositionFromExistingText_Params_Data {
 public:
  static FrameInputHandler_SetCompositionFromExistingText_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t start;
  int32_t end;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::ui::mojom::internal::CompositionUnderline_Data>>> underlines;

 private:
  FrameInputHandler_SetCompositionFromExistingText_Params_Data();
  ~FrameInputHandler_SetCompositionFromExistingText_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_SetCompositionFromExistingText_Params_Data) == 24,
              "Bad sizeof(FrameInputHandler_SetCompositionFromExistingText_Params_Data)");
constexpr uint32_t kFrameInputHandler_ExtendSelectionAndDelete_Name = 1;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_ExtendSelectionAndDelete_Params_Data {
 public:
  static FrameInputHandler_ExtendSelectionAndDelete_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t before;
  int32_t after;

 private:
  FrameInputHandler_ExtendSelectionAndDelete_Params_Data();
  ~FrameInputHandler_ExtendSelectionAndDelete_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_ExtendSelectionAndDelete_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_ExtendSelectionAndDelete_Params_Data)");
constexpr uint32_t kFrameInputHandler_DeleteSurroundingText_Name = 2;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_DeleteSurroundingText_Params_Data {
 public:
  static FrameInputHandler_DeleteSurroundingText_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t before;
  int32_t after;

 private:
  FrameInputHandler_DeleteSurroundingText_Params_Data();
  ~FrameInputHandler_DeleteSurroundingText_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_DeleteSurroundingText_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_DeleteSurroundingText_Params_Data)");
constexpr uint32_t kFrameInputHandler_DeleteSurroundingTextInCodePoints_Name = 3;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data {
 public:
  static FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t before;
  int32_t after;

 private:
  FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data();
  ~FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data)");
constexpr uint32_t kFrameInputHandler_SetEditableSelectionOffsets_Name = 4;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_SetEditableSelectionOffsets_Params_Data {
 public:
  static FrameInputHandler_SetEditableSelectionOffsets_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t start;
  int32_t end;

 private:
  FrameInputHandler_SetEditableSelectionOffsets_Params_Data();
  ~FrameInputHandler_SetEditableSelectionOffsets_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_SetEditableSelectionOffsets_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_SetEditableSelectionOffsets_Params_Data)");
constexpr uint32_t kFrameInputHandler_ExecuteEditCommand_Name = 5;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_ExecuteEditCommand_Params_Data {
 public:
  static FrameInputHandler_ExecuteEditCommand_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> command;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> value;

 private:
  FrameInputHandler_ExecuteEditCommand_Params_Data();
  ~FrameInputHandler_ExecuteEditCommand_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_ExecuteEditCommand_Params_Data) == 24,
              "Bad sizeof(FrameInputHandler_ExecuteEditCommand_Params_Data)");
constexpr uint32_t kFrameInputHandler_Undo_Name = 6;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_Undo_Params_Data {
 public:
  static FrameInputHandler_Undo_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_Undo_Params_Data();
  ~FrameInputHandler_Undo_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_Undo_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_Undo_Params_Data)");
constexpr uint32_t kFrameInputHandler_Redo_Name = 7;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_Redo_Params_Data {
 public:
  static FrameInputHandler_Redo_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_Redo_Params_Data();
  ~FrameInputHandler_Redo_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_Redo_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_Redo_Params_Data)");
constexpr uint32_t kFrameInputHandler_Cut_Name = 8;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_Cut_Params_Data {
 public:
  static FrameInputHandler_Cut_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_Cut_Params_Data();
  ~FrameInputHandler_Cut_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_Cut_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_Cut_Params_Data)");
constexpr uint32_t kFrameInputHandler_Copy_Name = 9;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_Copy_Params_Data {
 public:
  static FrameInputHandler_Copy_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_Copy_Params_Data();
  ~FrameInputHandler_Copy_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_Copy_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_Copy_Params_Data)");
constexpr uint32_t kFrameInputHandler_CopyToFindPboard_Name = 10;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_CopyToFindPboard_Params_Data {
 public:
  static FrameInputHandler_CopyToFindPboard_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_CopyToFindPboard_Params_Data();
  ~FrameInputHandler_CopyToFindPboard_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_CopyToFindPboard_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_CopyToFindPboard_Params_Data)");
constexpr uint32_t kFrameInputHandler_Paste_Name = 11;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_Paste_Params_Data {
 public:
  static FrameInputHandler_Paste_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_Paste_Params_Data();
  ~FrameInputHandler_Paste_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_Paste_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_Paste_Params_Data)");
constexpr uint32_t kFrameInputHandler_PasteAndMatchStyle_Name = 12;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_PasteAndMatchStyle_Params_Data {
 public:
  static FrameInputHandler_PasteAndMatchStyle_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_PasteAndMatchStyle_Params_Data();
  ~FrameInputHandler_PasteAndMatchStyle_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_PasteAndMatchStyle_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_PasteAndMatchStyle_Params_Data)");
constexpr uint32_t kFrameInputHandler_Delete_Name = 13;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_Delete_Params_Data {
 public:
  static FrameInputHandler_Delete_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_Delete_Params_Data();
  ~FrameInputHandler_Delete_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_Delete_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_Delete_Params_Data)");
constexpr uint32_t kFrameInputHandler_SelectAll_Name = 14;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_SelectAll_Params_Data {
 public:
  static FrameInputHandler_SelectAll_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_SelectAll_Params_Data();
  ~FrameInputHandler_SelectAll_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_SelectAll_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_SelectAll_Params_Data)");
constexpr uint32_t kFrameInputHandler_CollapseSelection_Name = 15;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_CollapseSelection_Params_Data {
 public:
  static FrameInputHandler_CollapseSelection_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FrameInputHandler_CollapseSelection_Params_Data();
  ~FrameInputHandler_CollapseSelection_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_CollapseSelection_Params_Data) == 8,
              "Bad sizeof(FrameInputHandler_CollapseSelection_Params_Data)");
constexpr uint32_t kFrameInputHandler_Replace_Name = 16;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_Replace_Params_Data {
 public:
  static FrameInputHandler_Replace_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> word;

 private:
  FrameInputHandler_Replace_Params_Data();
  ~FrameInputHandler_Replace_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_Replace_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_Replace_Params_Data)");
constexpr uint32_t kFrameInputHandler_ReplaceMisspelling_Name = 17;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_ReplaceMisspelling_Params_Data {
 public:
  static FrameInputHandler_ReplaceMisspelling_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> word;

 private:
  FrameInputHandler_ReplaceMisspelling_Params_Data();
  ~FrameInputHandler_ReplaceMisspelling_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_ReplaceMisspelling_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_ReplaceMisspelling_Params_Data)");
constexpr uint32_t kFrameInputHandler_SelectRange_Name = 18;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_SelectRange_Params_Data {
 public:
  static FrameInputHandler_SelectRange_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::gfx::mojom::internal::Point_Data> base;
  mojo::internal::Pointer<::gfx::mojom::internal::Point_Data> extent;

 private:
  FrameInputHandler_SelectRange_Params_Data();
  ~FrameInputHandler_SelectRange_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_SelectRange_Params_Data) == 24,
              "Bad sizeof(FrameInputHandler_SelectRange_Params_Data)");
constexpr uint32_t kFrameInputHandler_AdjustSelectionByCharacterOffset_Name = 19;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data {
 public:
  static FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t start;
  int32_t end;

 private:
  FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data();
  ~FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data)");
constexpr uint32_t kFrameInputHandler_MoveRangeSelectionExtent_Name = 20;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_MoveRangeSelectionExtent_Params_Data {
 public:
  static FrameInputHandler_MoveRangeSelectionExtent_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::gfx::mojom::internal::Point_Data> extent;

 private:
  FrameInputHandler_MoveRangeSelectionExtent_Params_Data();
  ~FrameInputHandler_MoveRangeSelectionExtent_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_MoveRangeSelectionExtent_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_MoveRangeSelectionExtent_Params_Data)");
constexpr uint32_t kFrameInputHandler_ScrollFocusedEditableNodeIntoRect_Name = 21;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data {
 public:
  static FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::gfx::mojom::internal::Rect_Data> rect;

 private:
  FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data();
  ~FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data)");
constexpr uint32_t kFrameInputHandler_MoveCaret_Name = 22;
class MOJOM_SHARED_CONTENT_EXPORT FrameInputHandler_MoveCaret_Params_Data {
 public:
  static FrameInputHandler_MoveCaret_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::gfx::mojom::internal::Point_Data> point;

 private:
  FrameInputHandler_MoveCaret_Params_Data();
  ~FrameInputHandler_MoveCaret_Params_Data() = delete;
};
static_assert(sizeof(FrameInputHandler_MoveCaret_Params_Data) == 16,
              "Bad sizeof(FrameInputHandler_MoveCaret_Params_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace content

#endif  // CONTENT_COMMON_INPUT_INPUT_HANDLER_MOJOM_SHARED_INTERNAL_H_