// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "content/common/input/input_handler.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace content {
namespace mojom {

namespace internal {


// static
KeyData_Data* KeyData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(KeyData_Data))) KeyData_Data();
}

// static
bool KeyData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const KeyData_Data* object = static_cast<const KeyData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 48 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->text, "null text field in KeyData",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->text, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->unmodified_text, "null unmodified_text field in KeyData",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->unmodified_text, validation_context))
    return false;

  return true;
}

KeyData_Data::KeyData_Data()
    : header_({sizeof(*this), 0}) {}


// static
PointerData_Data* PointerData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PointerData_Data))) PointerData_Data();
}

// static
bool PointerData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PointerData_Data* object = static_cast<const PointerData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 72 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::content::mojom::internal::Button_Data
        ::Validate(object->button, validation_context))
    return false;


  if (!::content::mojom::internal::PointerType_Data
        ::Validate(object->pointer_type, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->widget_position, "null widget_position field in PointerData",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->widget_position, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->screen_position, "null screen_position field in PointerData",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->screen_position, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->mouse_data, validation_context))
    return false;

  return true;
}

PointerData_Data::PointerData_Data()
    : header_({sizeof(*this), 0}) {}


// static
WheelData_Data* WheelData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WheelData_Data))) WheelData_Data();
}

// static
bool WheelData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WheelData_Data* object = static_cast<const WheelData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 48 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::content::mojom::internal::Cancelability_Data
        ::Validate(object->cancelable, validation_context))
    return false;

  return true;
}

WheelData_Data::WheelData_Data()
    : header_({sizeof(*this), 0}) {}


// static
MouseData_Data* MouseData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(MouseData_Data))) MouseData_Data();
}

// static
bool MouseData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const MouseData_Data* object = static_cast<const MouseData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->wheel_data, validation_context))
    return false;

  return true;
}

MouseData_Data::MouseData_Data()
    : header_({sizeof(*this), 0}) {}


// static
ScrollUpdate_Data* ScrollUpdate_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ScrollUpdate_Data))) ScrollUpdate_Data();
}

// static
bool ScrollUpdate_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ScrollUpdate_Data* object = static_cast<const ScrollUpdate_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

ScrollUpdate_Data::ScrollUpdate_Data()
    : header_({sizeof(*this), 0}) {}


// static
ScrollData_Data* ScrollData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ScrollData_Data))) ScrollData_Data();
}

// static
bool ScrollData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ScrollData_Data* object = static_cast<const ScrollData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::content::mojom::internal::ScrollUnits_Data
        ::Validate(object->delta_units, validation_context))
    return false;


  if (!::content::mojom::internal::InertialPhaseState_Data
        ::Validate(object->inertial_phase, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->update_details, validation_context))
    return false;

  return true;
}

ScrollData_Data::ScrollData_Data()
    : header_({sizeof(*this), 0}) {}


// static
PinchData_Data* PinchData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PinchData_Data))) PinchData_Data();
}

// static
bool PinchData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PinchData_Data* object = static_cast<const PinchData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

PinchData_Data::PinchData_Data()
    : header_({sizeof(*this), 0}) {}


// static
FlingData_Data* FlingData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FlingData_Data))) FlingData_Data();
}

// static
bool FlingData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FlingData_Data* object = static_cast<const FlingData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FlingData_Data::FlingData_Data()
    : header_({sizeof(*this), 0}) {}


// static
TapData_Data* TapData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(TapData_Data))) TapData_Data();
}

// static
bool TapData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const TapData_Data* object = static_cast<const TapData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

TapData_Data::TapData_Data()
    : header_({sizeof(*this), 0}) {}


// static
GestureData_Data* GestureData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(GestureData_Data))) GestureData_Data();
}

// static
bool GestureData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const GestureData_Data* object = static_cast<const GestureData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 80 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->screen_position, "null screen_position field in GestureData",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->screen_position, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->widget_position, "null widget_position field in GestureData",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->widget_position, validation_context))
    return false;


  if (!::content::mojom::internal::GestureDevice_Data
        ::Validate(object->source_device, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->contact_size, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->scroll_data, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->pinch_data, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->tap_data, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->fling_data, validation_context))
    return false;

  return true;
}

GestureData_Data::GestureData_Data()
    : header_({sizeof(*this), 0}) {}


// static
TouchPoint_Data* TouchPoint_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(TouchPoint_Data))) TouchPoint_Data();
}

// static
bool TouchPoint_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const TouchPoint_Data* object = static_cast<const TouchPoint_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::content::mojom::internal::TouchState_Data
        ::Validate(object->state, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->pointer_data, "null pointer_data field in TouchPoint",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->pointer_data, validation_context))
    return false;

  return true;
}

TouchPoint_Data::TouchPoint_Data()
    : header_({sizeof(*this), 0}) {}


// static
TouchData_Data* TouchData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(TouchData_Data))) TouchData_Data();
}

// static
bool TouchData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const TouchData_Data* object = static_cast<const TouchData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::content::mojom::internal::Cancelability_Data
        ::Validate(object->cancelable, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->touches, "null touches field in TouchData",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams touches_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->touches, validation_context,
                                         &touches_validate_params)) {
    return false;
  }

  return true;
}

TouchData_Data::TouchData_Data()
    : header_({sizeof(*this), 0}) {}


// static
Event_Data* Event_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Event_Data))) Event_Data();
}

// static
bool Event_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Event_Data* object = static_cast<const Event_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 64 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::content::mojom::internal::EventType_Data
        ::Validate(object->type, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->latency, "null latency field in Event",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->latency, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->key_data, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->pointer_data, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->gesture_data, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->touch_data, validation_context))
    return false;

  return true;
}

Event_Data::Event_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_SetFocus_Params_Data* WidgetInputHandler_SetFocus_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_SetFocus_Params_Data))) WidgetInputHandler_SetFocus_Params_Data();
}

// static
bool WidgetInputHandler_SetFocus_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_SetFocus_Params_Data* object = static_cast<const WidgetInputHandler_SetFocus_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WidgetInputHandler_SetFocus_Params_Data::WidgetInputHandler_SetFocus_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_MouseCaptureLost_Params_Data* WidgetInputHandler_MouseCaptureLost_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_MouseCaptureLost_Params_Data))) WidgetInputHandler_MouseCaptureLost_Params_Data();
}

// static
bool WidgetInputHandler_MouseCaptureLost_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_MouseCaptureLost_Params_Data* object = static_cast<const WidgetInputHandler_MouseCaptureLost_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WidgetInputHandler_MouseCaptureLost_Params_Data::WidgetInputHandler_MouseCaptureLost_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data* WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data))) WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data();
}

// static
bool WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data* object = static_cast<const WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->commands, "null commands field in WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams commands_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->commands, validation_context,
                                         &commands_validate_params)) {
    return false;
  }

  return true;
}

WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data::WidgetInputHandler_SetEditCommandsForNextKeyEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_CursorVisibilityChanged_Params_Data* WidgetInputHandler_CursorVisibilityChanged_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_CursorVisibilityChanged_Params_Data))) WidgetInputHandler_CursorVisibilityChanged_Params_Data();
}

// static
bool WidgetInputHandler_CursorVisibilityChanged_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_CursorVisibilityChanged_Params_Data* object = static_cast<const WidgetInputHandler_CursorVisibilityChanged_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WidgetInputHandler_CursorVisibilityChanged_Params_Data::WidgetInputHandler_CursorVisibilityChanged_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_ImeSetComposition_Params_Data* WidgetInputHandler_ImeSetComposition_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_ImeSetComposition_Params_Data))) WidgetInputHandler_ImeSetComposition_Params_Data();
}

// static
bool WidgetInputHandler_ImeSetComposition_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_ImeSetComposition_Params_Data* object = static_cast<const WidgetInputHandler_ImeSetComposition_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->text, "null text field in WidgetInputHandler_ImeSetComposition_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->text, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->underlines, "null underlines field in WidgetInputHandler_ImeSetComposition_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams underlines_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->underlines, validation_context,
                                         &underlines_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->range, "null range field in WidgetInputHandler_ImeSetComposition_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->range, validation_context))
    return false;

  return true;
}

WidgetInputHandler_ImeSetComposition_Params_Data::WidgetInputHandler_ImeSetComposition_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_ImeCommitText_Params_Data* WidgetInputHandler_ImeCommitText_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_ImeCommitText_Params_Data))) WidgetInputHandler_ImeCommitText_Params_Data();
}

// static
bool WidgetInputHandler_ImeCommitText_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_ImeCommitText_Params_Data* object = static_cast<const WidgetInputHandler_ImeCommitText_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->text, "null text field in WidgetInputHandler_ImeCommitText_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->text, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->underlines, "null underlines field in WidgetInputHandler_ImeCommitText_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams underlines_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->underlines, validation_context,
                                         &underlines_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->range, "null range field in WidgetInputHandler_ImeCommitText_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->range, validation_context))
    return false;

  return true;
}

WidgetInputHandler_ImeCommitText_Params_Data::WidgetInputHandler_ImeCommitText_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_ImeFinishComposingText_Params_Data* WidgetInputHandler_ImeFinishComposingText_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_ImeFinishComposingText_Params_Data))) WidgetInputHandler_ImeFinishComposingText_Params_Data();
}

// static
bool WidgetInputHandler_ImeFinishComposingText_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_ImeFinishComposingText_Params_Data* object = static_cast<const WidgetInputHandler_ImeFinishComposingText_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WidgetInputHandler_ImeFinishComposingText_Params_Data::WidgetInputHandler_ImeFinishComposingText_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_RequestTextInputStateUpdate_Params_Data* WidgetInputHandler_RequestTextInputStateUpdate_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_RequestTextInputStateUpdate_Params_Data))) WidgetInputHandler_RequestTextInputStateUpdate_Params_Data();
}

// static
bool WidgetInputHandler_RequestTextInputStateUpdate_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_RequestTextInputStateUpdate_Params_Data* object = static_cast<const WidgetInputHandler_RequestTextInputStateUpdate_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WidgetInputHandler_RequestTextInputStateUpdate_Params_Data::WidgetInputHandler_RequestTextInputStateUpdate_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_RequestCompositionUpdates_Params_Data* WidgetInputHandler_RequestCompositionUpdates_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_RequestCompositionUpdates_Params_Data))) WidgetInputHandler_RequestCompositionUpdates_Params_Data();
}

// static
bool WidgetInputHandler_RequestCompositionUpdates_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_RequestCompositionUpdates_Params_Data* object = static_cast<const WidgetInputHandler_RequestCompositionUpdates_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WidgetInputHandler_RequestCompositionUpdates_Params_Data::WidgetInputHandler_RequestCompositionUpdates_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_DispatchEvent_Params_Data* WidgetInputHandler_DispatchEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_DispatchEvent_Params_Data))) WidgetInputHandler_DispatchEvent_Params_Data();
}

// static
bool WidgetInputHandler_DispatchEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_DispatchEvent_Params_Data* object = static_cast<const WidgetInputHandler_DispatchEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->event, "null event field in WidgetInputHandler_DispatchEvent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->event, validation_context))
    return false;

  return true;
}

WidgetInputHandler_DispatchEvent_Params_Data::WidgetInputHandler_DispatchEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_DispatchEvent_ResponseParams_Data* WidgetInputHandler_DispatchEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_DispatchEvent_ResponseParams_Data))) WidgetInputHandler_DispatchEvent_ResponseParams_Data();
}

// static
bool WidgetInputHandler_DispatchEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_DispatchEvent_ResponseParams_Data* object = static_cast<const WidgetInputHandler_DispatchEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::content::mojom::internal::InputEventAckSource_Data
        ::Validate(object->source, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->updated_latency, "null updated_latency field in WidgetInputHandler_DispatchEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->updated_latency, validation_context))
    return false;


  if (!::content::mojom::internal::InputEventAckState_Data
        ::Validate(object->state, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->overscroll, validation_context))
    return false;

  return true;
}

WidgetInputHandler_DispatchEvent_ResponseParams_Data::WidgetInputHandler_DispatchEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
WidgetInputHandler_DispatchNonBlockingEvent_Params_Data* WidgetInputHandler_DispatchNonBlockingEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WidgetInputHandler_DispatchNonBlockingEvent_Params_Data))) WidgetInputHandler_DispatchNonBlockingEvent_Params_Data();
}

// static
bool WidgetInputHandler_DispatchNonBlockingEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WidgetInputHandler_DispatchNonBlockingEvent_Params_Data* object = static_cast<const WidgetInputHandler_DispatchNonBlockingEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->event, "null event field in WidgetInputHandler_DispatchNonBlockingEvent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->event, validation_context))
    return false;

  return true;
}

WidgetInputHandler_DispatchNonBlockingEvent_Params_Data::WidgetInputHandler_DispatchNonBlockingEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_SetCompositionFromExistingText_Params_Data* FrameInputHandler_SetCompositionFromExistingText_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_SetCompositionFromExistingText_Params_Data))) FrameInputHandler_SetCompositionFromExistingText_Params_Data();
}

// static
bool FrameInputHandler_SetCompositionFromExistingText_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_SetCompositionFromExistingText_Params_Data* object = static_cast<const FrameInputHandler_SetCompositionFromExistingText_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->underlines, "null underlines field in FrameInputHandler_SetCompositionFromExistingText_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams underlines_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->underlines, validation_context,
                                         &underlines_validate_params)) {
    return false;
  }

  return true;
}

FrameInputHandler_SetCompositionFromExistingText_Params_Data::FrameInputHandler_SetCompositionFromExistingText_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_ExtendSelectionAndDelete_Params_Data* FrameInputHandler_ExtendSelectionAndDelete_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_ExtendSelectionAndDelete_Params_Data))) FrameInputHandler_ExtendSelectionAndDelete_Params_Data();
}

// static
bool FrameInputHandler_ExtendSelectionAndDelete_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_ExtendSelectionAndDelete_Params_Data* object = static_cast<const FrameInputHandler_ExtendSelectionAndDelete_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_ExtendSelectionAndDelete_Params_Data::FrameInputHandler_ExtendSelectionAndDelete_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_DeleteSurroundingText_Params_Data* FrameInputHandler_DeleteSurroundingText_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_DeleteSurroundingText_Params_Data))) FrameInputHandler_DeleteSurroundingText_Params_Data();
}

// static
bool FrameInputHandler_DeleteSurroundingText_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_DeleteSurroundingText_Params_Data* object = static_cast<const FrameInputHandler_DeleteSurroundingText_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_DeleteSurroundingText_Params_Data::FrameInputHandler_DeleteSurroundingText_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data* FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data))) FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data();
}

// static
bool FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data* object = static_cast<const FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data::FrameInputHandler_DeleteSurroundingTextInCodePoints_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_SetEditableSelectionOffsets_Params_Data* FrameInputHandler_SetEditableSelectionOffsets_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_SetEditableSelectionOffsets_Params_Data))) FrameInputHandler_SetEditableSelectionOffsets_Params_Data();
}

// static
bool FrameInputHandler_SetEditableSelectionOffsets_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_SetEditableSelectionOffsets_Params_Data* object = static_cast<const FrameInputHandler_SetEditableSelectionOffsets_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_SetEditableSelectionOffsets_Params_Data::FrameInputHandler_SetEditableSelectionOffsets_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_ExecuteEditCommand_Params_Data* FrameInputHandler_ExecuteEditCommand_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_ExecuteEditCommand_Params_Data))) FrameInputHandler_ExecuteEditCommand_Params_Data();
}

// static
bool FrameInputHandler_ExecuteEditCommand_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_ExecuteEditCommand_Params_Data* object = static_cast<const FrameInputHandler_ExecuteEditCommand_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->command, "null command field in FrameInputHandler_ExecuteEditCommand_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams command_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->command, validation_context,
                                         &command_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->value, validation_context))
    return false;

  return true;
}

FrameInputHandler_ExecuteEditCommand_Params_Data::FrameInputHandler_ExecuteEditCommand_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_Undo_Params_Data* FrameInputHandler_Undo_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_Undo_Params_Data))) FrameInputHandler_Undo_Params_Data();
}

// static
bool FrameInputHandler_Undo_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_Undo_Params_Data* object = static_cast<const FrameInputHandler_Undo_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_Undo_Params_Data::FrameInputHandler_Undo_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_Redo_Params_Data* FrameInputHandler_Redo_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_Redo_Params_Data))) FrameInputHandler_Redo_Params_Data();
}

// static
bool FrameInputHandler_Redo_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_Redo_Params_Data* object = static_cast<const FrameInputHandler_Redo_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_Redo_Params_Data::FrameInputHandler_Redo_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_Cut_Params_Data* FrameInputHandler_Cut_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_Cut_Params_Data))) FrameInputHandler_Cut_Params_Data();
}

// static
bool FrameInputHandler_Cut_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_Cut_Params_Data* object = static_cast<const FrameInputHandler_Cut_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_Cut_Params_Data::FrameInputHandler_Cut_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_Copy_Params_Data* FrameInputHandler_Copy_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_Copy_Params_Data))) FrameInputHandler_Copy_Params_Data();
}

// static
bool FrameInputHandler_Copy_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_Copy_Params_Data* object = static_cast<const FrameInputHandler_Copy_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_Copy_Params_Data::FrameInputHandler_Copy_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_CopyToFindPboard_Params_Data* FrameInputHandler_CopyToFindPboard_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_CopyToFindPboard_Params_Data))) FrameInputHandler_CopyToFindPboard_Params_Data();
}

// static
bool FrameInputHandler_CopyToFindPboard_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_CopyToFindPboard_Params_Data* object = static_cast<const FrameInputHandler_CopyToFindPboard_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_CopyToFindPboard_Params_Data::FrameInputHandler_CopyToFindPboard_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_Paste_Params_Data* FrameInputHandler_Paste_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_Paste_Params_Data))) FrameInputHandler_Paste_Params_Data();
}

// static
bool FrameInputHandler_Paste_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_Paste_Params_Data* object = static_cast<const FrameInputHandler_Paste_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_Paste_Params_Data::FrameInputHandler_Paste_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_PasteAndMatchStyle_Params_Data* FrameInputHandler_PasteAndMatchStyle_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_PasteAndMatchStyle_Params_Data))) FrameInputHandler_PasteAndMatchStyle_Params_Data();
}

// static
bool FrameInputHandler_PasteAndMatchStyle_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_PasteAndMatchStyle_Params_Data* object = static_cast<const FrameInputHandler_PasteAndMatchStyle_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_PasteAndMatchStyle_Params_Data::FrameInputHandler_PasteAndMatchStyle_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_Delete_Params_Data* FrameInputHandler_Delete_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_Delete_Params_Data))) FrameInputHandler_Delete_Params_Data();
}

// static
bool FrameInputHandler_Delete_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_Delete_Params_Data* object = static_cast<const FrameInputHandler_Delete_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_Delete_Params_Data::FrameInputHandler_Delete_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_SelectAll_Params_Data* FrameInputHandler_SelectAll_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_SelectAll_Params_Data))) FrameInputHandler_SelectAll_Params_Data();
}

// static
bool FrameInputHandler_SelectAll_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_SelectAll_Params_Data* object = static_cast<const FrameInputHandler_SelectAll_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_SelectAll_Params_Data::FrameInputHandler_SelectAll_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_CollapseSelection_Params_Data* FrameInputHandler_CollapseSelection_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_CollapseSelection_Params_Data))) FrameInputHandler_CollapseSelection_Params_Data();
}

// static
bool FrameInputHandler_CollapseSelection_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_CollapseSelection_Params_Data* object = static_cast<const FrameInputHandler_CollapseSelection_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_CollapseSelection_Params_Data::FrameInputHandler_CollapseSelection_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_Replace_Params_Data* FrameInputHandler_Replace_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_Replace_Params_Data))) FrameInputHandler_Replace_Params_Data();
}

// static
bool FrameInputHandler_Replace_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_Replace_Params_Data* object = static_cast<const FrameInputHandler_Replace_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->word, "null word field in FrameInputHandler_Replace_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->word, validation_context))
    return false;

  return true;
}

FrameInputHandler_Replace_Params_Data::FrameInputHandler_Replace_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_ReplaceMisspelling_Params_Data* FrameInputHandler_ReplaceMisspelling_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_ReplaceMisspelling_Params_Data))) FrameInputHandler_ReplaceMisspelling_Params_Data();
}

// static
bool FrameInputHandler_ReplaceMisspelling_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_ReplaceMisspelling_Params_Data* object = static_cast<const FrameInputHandler_ReplaceMisspelling_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->word, "null word field in FrameInputHandler_ReplaceMisspelling_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->word, validation_context))
    return false;

  return true;
}

FrameInputHandler_ReplaceMisspelling_Params_Data::FrameInputHandler_ReplaceMisspelling_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_SelectRange_Params_Data* FrameInputHandler_SelectRange_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_SelectRange_Params_Data))) FrameInputHandler_SelectRange_Params_Data();
}

// static
bool FrameInputHandler_SelectRange_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_SelectRange_Params_Data* object = static_cast<const FrameInputHandler_SelectRange_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->base, "null base field in FrameInputHandler_SelectRange_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->base, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->extent, "null extent field in FrameInputHandler_SelectRange_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->extent, validation_context))
    return false;

  return true;
}

FrameInputHandler_SelectRange_Params_Data::FrameInputHandler_SelectRange_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data* FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data))) FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data();
}

// static
bool FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data* object = static_cast<const FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data::FrameInputHandler_AdjustSelectionByCharacterOffset_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_MoveRangeSelectionExtent_Params_Data* FrameInputHandler_MoveRangeSelectionExtent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_MoveRangeSelectionExtent_Params_Data))) FrameInputHandler_MoveRangeSelectionExtent_Params_Data();
}

// static
bool FrameInputHandler_MoveRangeSelectionExtent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_MoveRangeSelectionExtent_Params_Data* object = static_cast<const FrameInputHandler_MoveRangeSelectionExtent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->extent, "null extent field in FrameInputHandler_MoveRangeSelectionExtent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->extent, validation_context))
    return false;

  return true;
}

FrameInputHandler_MoveRangeSelectionExtent_Params_Data::FrameInputHandler_MoveRangeSelectionExtent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data* FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data))) FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data();
}

// static
bool FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data* object = static_cast<const FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->rect, "null rect field in FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->rect, validation_context))
    return false;

  return true;
}

FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data::FrameInputHandler_ScrollFocusedEditableNodeIntoRect_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
FrameInputHandler_MoveCaret_Params_Data* FrameInputHandler_MoveCaret_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FrameInputHandler_MoveCaret_Params_Data))) FrameInputHandler_MoveCaret_Params_Data();
}

// static
bool FrameInputHandler_MoveCaret_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FrameInputHandler_MoveCaret_Params_Data* object = static_cast<const FrameInputHandler_MoveCaret_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->point, "null point field in FrameInputHandler_MoveCaret_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->point, validation_context))
    return false;

  return true;
}

FrameInputHandler_MoveCaret_Params_Data::FrameInputHandler_MoveCaret_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace content

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
