// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "content/common/media/media_devices.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "content/common/media/media_devices_param_traits.h"
#include "media/base/ipc/media_param_traits.h"
#include "media/base/ipc/media_param_traits_macros.h"
#include "media/capture/ipc/capture_param_traits.h"
#include "media/capture/ipc/capture_param_traits_macros.h"
#include "media/capture/mojo/video_capture_types_typemap_traits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace mojom {
VideoInputDeviceCapabilities::VideoInputDeviceCapabilities()
    : device_id(),
      formats(),
      facing_mode() {}

VideoInputDeviceCapabilities::VideoInputDeviceCapabilities(
    const std::string& device_id_in,
    const std::vector<media::VideoCaptureFormat>& formats_in,
    FacingMode facing_mode_in)
    : device_id(std::move(device_id_in)),
      formats(std::move(formats_in)),
      facing_mode(std::move(facing_mode_in)) {}

VideoInputDeviceCapabilities::~VideoInputDeviceCapabilities() = default;

bool VideoInputDeviceCapabilities::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
AudioInputDeviceCapabilities::AudioInputDeviceCapabilities()
    : device_id(),
      parameters() {}

AudioInputDeviceCapabilities::AudioInputDeviceCapabilities(
    const std::string& device_id_in,
    const media::AudioParameters& parameters_in)
    : device_id(std::move(device_id_in)),
      parameters(std::move(parameters_in)) {}

AudioInputDeviceCapabilities::~AudioInputDeviceCapabilities() = default;

bool AudioInputDeviceCapabilities::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char MediaDevicesDispatcherHost::Name_[] = "mojom::MediaDevicesDispatcherHost";

class MediaDevicesDispatcherHost_EnumerateDevices_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  MediaDevicesDispatcherHost_EnumerateDevices_ForwardToCallback(
      MediaDevicesDispatcherHost::EnumerateDevicesCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  MediaDevicesDispatcherHost::EnumerateDevicesCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(MediaDevicesDispatcherHost_EnumerateDevices_ForwardToCallback);
};

class MediaDevicesDispatcherHost_GetVideoInputCapabilities_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  MediaDevicesDispatcherHost_GetVideoInputCapabilities_ForwardToCallback(
      MediaDevicesDispatcherHost::GetVideoInputCapabilitiesCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  MediaDevicesDispatcherHost::GetVideoInputCapabilitiesCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(MediaDevicesDispatcherHost_GetVideoInputCapabilities_ForwardToCallback);
};

class MediaDevicesDispatcherHost_GetAudioInputCapabilities_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  MediaDevicesDispatcherHost_GetAudioInputCapabilities_ForwardToCallback(
      MediaDevicesDispatcherHost::GetAudioInputCapabilitiesCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  MediaDevicesDispatcherHost::GetAudioInputCapabilitiesCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(MediaDevicesDispatcherHost_GetAudioInputCapabilities_ForwardToCallback);
};

MediaDevicesDispatcherHostProxy::MediaDevicesDispatcherHostProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void MediaDevicesDispatcherHostProxy::EnumerateDevices(
    bool in_request_audio_input, bool in_request_video_input, bool in_request_audio_output, EnumerateDevicesCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::mojom::internal::MediaDevicesDispatcherHost_EnumerateDevices_Params_Data);
  serialization_context.PrepareMessage(
      internal::kMediaDevicesDispatcherHost_EnumerateDevices_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::mojom::internal::MediaDevicesDispatcherHost_EnumerateDevices_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->request_audio_input = in_request_audio_input;
  params->request_video_input = in_request_video_input;
  params->request_audio_output = in_request_audio_output;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new MediaDevicesDispatcherHost_EnumerateDevices_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void MediaDevicesDispatcherHostProxy::GetVideoInputCapabilities(
    GetVideoInputCapabilitiesCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::mojom::internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data);
  serialization_context.PrepareMessage(
      internal::kMediaDevicesDispatcherHost_GetVideoInputCapabilities_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::mojom::internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new MediaDevicesDispatcherHost_GetVideoInputCapabilities_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void MediaDevicesDispatcherHostProxy::GetAudioInputCapabilities(
    GetAudioInputCapabilitiesCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::mojom::internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data);
  serialization_context.PrepareMessage(
      internal::kMediaDevicesDispatcherHost_GetAudioInputCapabilities_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::mojom::internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new MediaDevicesDispatcherHost_GetAudioInputCapabilities_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void MediaDevicesDispatcherHostProxy::SubscribeDeviceChangeNotifications(
    ::content::MediaDeviceType in_type, uint32_t in_subscription_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::mojom::internal::MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data);
  serialization_context.PrepareMessage(
      internal::kMediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::mojom::internal::MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::mojom::MediaDeviceType>(
      in_type, &params->type);
  params->subscription_id = in_subscription_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void MediaDevicesDispatcherHostProxy::UnsubscribeDeviceChangeNotifications(
    ::content::MediaDeviceType in_type, uint32_t in_subscription_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::mojom::internal::MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data);
  serialization_context.PrepareMessage(
      internal::kMediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::mojom::internal::MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::mojom::MediaDeviceType>(
      in_type, &params->type);
  params->subscription_id = in_subscription_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class MediaDevicesDispatcherHost_EnumerateDevices_ProxyToResponder {
 public:
  static MediaDevicesDispatcherHost::EnumerateDevicesCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<MediaDevicesDispatcherHost_EnumerateDevices_ProxyToResponder> proxy(
        new MediaDevicesDispatcherHost_EnumerateDevices_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&MediaDevicesDispatcherHost_EnumerateDevices_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~MediaDevicesDispatcherHost_EnumerateDevices_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "MediaDevicesDispatcherHost::EnumerateDevices() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  MediaDevicesDispatcherHost_EnumerateDevices_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      const std::vector<std::vector<::content::MediaDeviceInfo>>& in_enumeration);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(MediaDevicesDispatcherHost_EnumerateDevices_ProxyToResponder);
};

bool MediaDevicesDispatcherHost_EnumerateDevices_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data* params =
      reinterpret_cast<
          internal::MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<std::vector<::content::MediaDeviceInfo>> p_enumeration{};
  MediaDevicesDispatcherHost_EnumerateDevices_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadEnumeration(&p_enumeration))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "MediaDevicesDispatcherHost::EnumerateDevices response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_enumeration));
  return true;
}

void MediaDevicesDispatcherHost_EnumerateDevices_ProxyToResponder::Run(
    const std::vector<std::vector<::content::MediaDeviceInfo>>& in_enumeration) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::mojom::internal::MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<mojo::ArrayDataView<::mojom::MediaDeviceInfoDataView>>>(
      in_enumeration, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kMediaDevicesDispatcherHost_EnumerateDevices_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::mojom::internal::MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->enumeration)::BaseType* enumeration_ptr;
  const mojo::internal::ContainerValidateParams enumeration_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::ArrayDataView<::mojom::MediaDeviceInfoDataView>>>(
      in_enumeration, buffer, &enumeration_ptr, &enumeration_validate_params,
      &serialization_context);
  params->enumeration.Set(enumeration_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->enumeration.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null enumeration in ");

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class MediaDevicesDispatcherHost_GetVideoInputCapabilities_ProxyToResponder {
 public:
  static MediaDevicesDispatcherHost::GetVideoInputCapabilitiesCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<MediaDevicesDispatcherHost_GetVideoInputCapabilities_ProxyToResponder> proxy(
        new MediaDevicesDispatcherHost_GetVideoInputCapabilities_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&MediaDevicesDispatcherHost_GetVideoInputCapabilities_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~MediaDevicesDispatcherHost_GetVideoInputCapabilities_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "MediaDevicesDispatcherHost::GetVideoInputCapabilities() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  MediaDevicesDispatcherHost_GetVideoInputCapabilities_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      std::vector<VideoInputDeviceCapabilitiesPtr> in_video_input_device_capabilities);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(MediaDevicesDispatcherHost_GetVideoInputCapabilities_ProxyToResponder);
};

bool MediaDevicesDispatcherHost_GetVideoInputCapabilities_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data* params =
      reinterpret_cast<
          internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<VideoInputDeviceCapabilitiesPtr> p_video_input_device_capabilities{};
  MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadVideoInputDeviceCapabilities(&p_video_input_device_capabilities))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "MediaDevicesDispatcherHost::GetVideoInputCapabilities response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_video_input_device_capabilities));
  return true;
}

void MediaDevicesDispatcherHost_GetVideoInputCapabilities_ProxyToResponder::Run(
    std::vector<VideoInputDeviceCapabilitiesPtr> in_video_input_device_capabilities) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::mojom::internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::mojom::VideoInputDeviceCapabilitiesDataView>>(
      in_video_input_device_capabilities, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kMediaDevicesDispatcherHost_GetVideoInputCapabilities_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::mojom::internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->video_input_device_capabilities)::BaseType* video_input_device_capabilities_ptr;
  const mojo::internal::ContainerValidateParams video_input_device_capabilities_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::mojom::VideoInputDeviceCapabilitiesDataView>>(
      in_video_input_device_capabilities, buffer, &video_input_device_capabilities_ptr, &video_input_device_capabilities_validate_params,
      &serialization_context);
  params->video_input_device_capabilities.Set(video_input_device_capabilities_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->video_input_device_capabilities.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null video_input_device_capabilities in ");

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class MediaDevicesDispatcherHost_GetAudioInputCapabilities_ProxyToResponder {
 public:
  static MediaDevicesDispatcherHost::GetAudioInputCapabilitiesCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<MediaDevicesDispatcherHost_GetAudioInputCapabilities_ProxyToResponder> proxy(
        new MediaDevicesDispatcherHost_GetAudioInputCapabilities_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&MediaDevicesDispatcherHost_GetAudioInputCapabilities_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~MediaDevicesDispatcherHost_GetAudioInputCapabilities_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "MediaDevicesDispatcherHost::GetAudioInputCapabilities() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  MediaDevicesDispatcherHost_GetAudioInputCapabilities_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      std::vector<AudioInputDeviceCapabilitiesPtr> in_audio_input_device_capabilities);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(MediaDevicesDispatcherHost_GetAudioInputCapabilities_ProxyToResponder);
};

bool MediaDevicesDispatcherHost_GetAudioInputCapabilities_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data* params =
      reinterpret_cast<
          internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<AudioInputDeviceCapabilitiesPtr> p_audio_input_device_capabilities{};
  MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadAudioInputDeviceCapabilities(&p_audio_input_device_capabilities))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "MediaDevicesDispatcherHost::GetAudioInputCapabilities response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_audio_input_device_capabilities));
  return true;
}

void MediaDevicesDispatcherHost_GetAudioInputCapabilities_ProxyToResponder::Run(
    std::vector<AudioInputDeviceCapabilitiesPtr> in_audio_input_device_capabilities) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::mojom::internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::mojom::AudioInputDeviceCapabilitiesDataView>>(
      in_audio_input_device_capabilities, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kMediaDevicesDispatcherHost_GetAudioInputCapabilities_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::mojom::internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->audio_input_device_capabilities)::BaseType* audio_input_device_capabilities_ptr;
  const mojo::internal::ContainerValidateParams audio_input_device_capabilities_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::mojom::AudioInputDeviceCapabilitiesDataView>>(
      in_audio_input_device_capabilities, buffer, &audio_input_device_capabilities_ptr, &audio_input_device_capabilities_validate_params,
      &serialization_context);
  params->audio_input_device_capabilities.Set(audio_input_device_capabilities_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->audio_input_device_capabilities.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null audio_input_device_capabilities in ");

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool MediaDevicesDispatcherHostStubDispatch::Accept(
    MediaDevicesDispatcherHost* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kMediaDevicesDispatcherHost_EnumerateDevices_Name: {
      break;
    }
    case internal::kMediaDevicesDispatcherHost_GetVideoInputCapabilities_Name: {
      break;
    }
    case internal::kMediaDevicesDispatcherHost_GetAudioInputCapabilities_Name: {
      break;
    }
    case internal::kMediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data* params =
          reinterpret_cast<internal::MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::content::MediaDeviceType p_type{};
      uint32_t p_subscription_id{};
      MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadType(&p_type))
        success = false;
      p_subscription_id = input_data_view.subscription_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "MediaDevicesDispatcherHost::SubscribeDeviceChangeNotifications deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SubscribeDeviceChangeNotifications(
std::move(p_type), 
std::move(p_subscription_id));
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data* params =
          reinterpret_cast<internal::MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::content::MediaDeviceType p_type{};
      uint32_t p_subscription_id{};
      MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadType(&p_type))
        success = false;
      p_subscription_id = input_data_view.subscription_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "MediaDevicesDispatcherHost::UnsubscribeDeviceChangeNotifications deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->UnsubscribeDeviceChangeNotifications(
std::move(p_type), 
std::move(p_subscription_id));
      return true;
    }
  }
  return false;
}

// static
bool MediaDevicesDispatcherHostStubDispatch::AcceptWithResponder(
    MediaDevicesDispatcherHost* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kMediaDevicesDispatcherHost_EnumerateDevices_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::MediaDevicesDispatcherHost_EnumerateDevices_Params_Data* params =
          reinterpret_cast<
              internal::MediaDevicesDispatcherHost_EnumerateDevices_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_request_audio_input{};
      bool p_request_video_input{};
      bool p_request_audio_output{};
      MediaDevicesDispatcherHost_EnumerateDevices_ParamsDataView input_data_view(params, &serialization_context);
      
      p_request_audio_input = input_data_view.request_audio_input();
      p_request_video_input = input_data_view.request_video_input();
      p_request_audio_output = input_data_view.request_audio_output();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "MediaDevicesDispatcherHost::EnumerateDevices deserializer");
        return false;
      }
      MediaDevicesDispatcherHost::EnumerateDevicesCallback callback =
          MediaDevicesDispatcherHost_EnumerateDevices_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->EnumerateDevices(
std::move(p_request_audio_input), 
std::move(p_request_video_input), 
std::move(p_request_audio_output), std::move(callback));
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_GetVideoInputCapabilities_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data* params =
          reinterpret_cast<
              internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      MediaDevicesDispatcherHost_GetVideoInputCapabilities_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "MediaDevicesDispatcherHost::GetVideoInputCapabilities deserializer");
        return false;
      }
      MediaDevicesDispatcherHost::GetVideoInputCapabilitiesCallback callback =
          MediaDevicesDispatcherHost_GetVideoInputCapabilities_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetVideoInputCapabilities(std::move(callback));
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_GetAudioInputCapabilities_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data* params =
          reinterpret_cast<
              internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      MediaDevicesDispatcherHost_GetAudioInputCapabilities_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "MediaDevicesDispatcherHost::GetAudioInputCapabilities deserializer");
        return false;
      }
      MediaDevicesDispatcherHost::GetAudioInputCapabilitiesCallback callback =
          MediaDevicesDispatcherHost_GetAudioInputCapabilities_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetAudioInputCapabilities(std::move(callback));
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Name: {
      break;
    }
    case internal::kMediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Name: {
      break;
    }
  }
  return false;
}

bool MediaDevicesDispatcherHostRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "MediaDevicesDispatcherHost RequestValidator");

  switch (message->header()->name) {
    case internal::kMediaDevicesDispatcherHost_EnumerateDevices_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::MediaDevicesDispatcherHost_EnumerateDevices_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_GetVideoInputCapabilities_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_GetAudioInputCapabilities_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool MediaDevicesDispatcherHostResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "MediaDevicesDispatcherHost ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kMediaDevicesDispatcherHost_EnumerateDevices_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_GetVideoInputCapabilities_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kMediaDevicesDispatcherHost_GetAudioInputCapabilities_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char MediaDevicesListener::Name_[] = "mojom::MediaDevicesListener";

MediaDevicesListenerProxy::MediaDevicesListenerProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void MediaDevicesListenerProxy::OnDevicesChanged(
    ::content::MediaDeviceType in_type, uint32_t in_subscription_id, const std::vector<::content::MediaDeviceInfo>& in_device_infos) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::mojom::internal::MediaDevicesListener_OnDevicesChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::mojom::MediaDeviceInfoDataView>>(
      in_device_infos, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kMediaDevicesListener_OnDevicesChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::mojom::internal::MediaDevicesListener_OnDevicesChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::mojom::MediaDeviceType>(
      in_type, &params->type);
  params->subscription_id = in_subscription_id;
  typename decltype(params->device_infos)::BaseType* device_infos_ptr;
  const mojo::internal::ContainerValidateParams device_infos_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::mojom::MediaDeviceInfoDataView>>(
      in_device_infos, buffer, &device_infos_ptr, &device_infos_validate_params,
      &serialization_context);
  params->device_infos.Set(device_infos_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->device_infos.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null device_infos in MediaDevicesListener.OnDevicesChanged request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool MediaDevicesListenerStubDispatch::Accept(
    MediaDevicesListener* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kMediaDevicesListener_OnDevicesChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::MediaDevicesListener_OnDevicesChanged_Params_Data* params =
          reinterpret_cast<internal::MediaDevicesListener_OnDevicesChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::content::MediaDeviceType p_type{};
      uint32_t p_subscription_id{};
      std::vector<::content::MediaDeviceInfo> p_device_infos{};
      MediaDevicesListener_OnDevicesChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadType(&p_type))
        success = false;
      p_subscription_id = input_data_view.subscription_id();
      if (!input_data_view.ReadDeviceInfos(&p_device_infos))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "MediaDevicesListener::OnDevicesChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDevicesChanged(
std::move(p_type), 
std::move(p_subscription_id), 
std::move(p_device_infos));
      return true;
    }
  }
  return false;
}

// static
bool MediaDevicesListenerStubDispatch::AcceptWithResponder(
    MediaDevicesListener* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kMediaDevicesListener_OnDevicesChanged_Name: {
      break;
    }
  }
  return false;
}

bool MediaDevicesListenerRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "MediaDevicesListener RequestValidator");

  switch (message->header()->name) {
    case internal::kMediaDevicesListener_OnDevicesChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::MediaDevicesListener_OnDevicesChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom

namespace mojo {


// static
bool StructTraits<::mojom::VideoInputDeviceCapabilities::DataView, ::mojom::VideoInputDeviceCapabilitiesPtr>::Read(
    ::mojom::VideoInputDeviceCapabilities::DataView input,
    ::mojom::VideoInputDeviceCapabilitiesPtr* output) {
  bool success = true;
  ::mojom::VideoInputDeviceCapabilitiesPtr result(::mojom::VideoInputDeviceCapabilities::New());
  
      if (!input.ReadDeviceId(&result->device_id))
        success = false;
      if (!input.ReadFormats(&result->formats))
        success = false;
      if (!input.ReadFacingMode(&result->facing_mode))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::mojom::AudioInputDeviceCapabilities::DataView, ::mojom::AudioInputDeviceCapabilitiesPtr>::Read(
    ::mojom::AudioInputDeviceCapabilities::DataView input,
    ::mojom::AudioInputDeviceCapabilitiesPtr* output) {
  bool success = true;
  ::mojom::AudioInputDeviceCapabilitiesPtr result(::mojom::AudioInputDeviceCapabilities::New());
  
      if (!input.ReadDeviceId(&result->device_id))
        success = false;
      if (!input.ReadParameters(&result->parameters))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif