// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "content/common/renderer.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "content/common/frame_messages.h"
#include "content/common/input/input_event_struct_traits.h"
#include "content/common/input_messages.h"
#include "content/common/view_messages.h"
#include "content/public/common/common_param_traits.h"
#include "content/public/common/common_param_traits_macros.h"
#include "ipc/ipc_message_utils.h"
#include "mojo/common/common_custom_types_struct_traits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
#include "ui/gfx/ipc/color/gfx_param_traits.h"
namespace content {
namespace mojom {
CreateViewParams::CreateViewParams()
    : renderer_preferences(),
      web_preferences(),
      view_id(::IPC::mojom::kRoutingIdNone),
      main_frame_routing_id(::IPC::mojom::kRoutingIdNone),
      main_frame_widget_routing_id(::IPC::mojom::kRoutingIdNone),
      session_storage_namespace_id(),
      opener_frame_route_id(::IPC::mojom::kRoutingIdNone),
      swapped_out(),
      replicated_frame_state(),
      proxy_routing_id(::IPC::mojom::kRoutingIdNone),
      hidden(),
      never_visible(),
      window_was_created_with_opener(),
      initial_size(),
      enable_auto_resize(),
      min_size(),
      max_size(),
      page_zoom_level(),
      image_decode_color_space() {}

CreateViewParams::CreateViewParams(
    const content::RendererPreferences& renderer_preferences_in,
    const content::WebPreferences& web_preferences_in,
    int32_t view_id_in,
    int32_t main_frame_routing_id_in,
    int32_t main_frame_widget_routing_id_in,
    int64_t session_storage_namespace_id_in,
    int32_t opener_frame_route_id_in,
    bool swapped_out_in,
    const content::FrameReplicationState& replicated_frame_state_in,
    int32_t proxy_routing_id_in,
    bool hidden_in,
    bool never_visible_in,
    bool window_was_created_with_opener_in,
    const content::ResizeParams& initial_size_in,
    bool enable_auto_resize_in,
    const gfx::Size& min_size_in,
    const gfx::Size& max_size_in,
    double page_zoom_level_in,
    const gfx::ICCProfile& image_decode_color_space_in)
    : renderer_preferences(std::move(renderer_preferences_in)),
      web_preferences(std::move(web_preferences_in)),
      view_id(std::move(view_id_in)),
      main_frame_routing_id(std::move(main_frame_routing_id_in)),
      main_frame_widget_routing_id(std::move(main_frame_widget_routing_id_in)),
      session_storage_namespace_id(std::move(session_storage_namespace_id_in)),
      opener_frame_route_id(std::move(opener_frame_route_id_in)),
      swapped_out(std::move(swapped_out_in)),
      replicated_frame_state(std::move(replicated_frame_state_in)),
      proxy_routing_id(std::move(proxy_routing_id_in)),
      hidden(std::move(hidden_in)),
      never_visible(std::move(never_visible_in)),
      window_was_created_with_opener(std::move(window_was_created_with_opener_in)),
      initial_size(std::move(initial_size_in)),
      enable_auto_resize(std::move(enable_auto_resize_in)),
      min_size(std::move(min_size_in)),
      max_size(std::move(max_size_in)),
      page_zoom_level(std::move(page_zoom_level_in)),
      image_decode_color_space(std::move(image_decode_color_space_in)) {}

CreateViewParams::~CreateViewParams() = default;

bool CreateViewParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
CreateFrameWidgetParams::CreateFrameWidgetParams()
    : routing_id(),
      hidden() {}

CreateFrameWidgetParams::CreateFrameWidgetParams(
    int32_t routing_id_in,
    bool hidden_in)
    : routing_id(std::move(routing_id_in)),
      hidden(std::move(hidden_in)) {}

CreateFrameWidgetParams::~CreateFrameWidgetParams() = default;
size_t CreateFrameWidgetParams::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->routing_id);
  seed = mojo::internal::Hash(seed, this->hidden);
  return seed;
}

bool CreateFrameWidgetParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
CreateFrameParams::CreateFrameParams()
    : routing_id(),
      proxy_routing_id(),
      opener_routing_id(),
      parent_routing_id(),
      previous_sibling_routing_id(),
      replication_state(),
      frame_owner_properties(),
      widget_params() {}

CreateFrameParams::CreateFrameParams(
    int32_t routing_id_in,
    int32_t proxy_routing_id_in,
    int32_t opener_routing_id_in,
    int32_t parent_routing_id_in,
    int32_t previous_sibling_routing_id_in,
    const content::FrameReplicationState& replication_state_in,
    const content::FrameOwnerProperties& frame_owner_properties_in,
    CreateFrameWidgetParamsPtr widget_params_in)
    : routing_id(std::move(routing_id_in)),
      proxy_routing_id(std::move(proxy_routing_id_in)),
      opener_routing_id(std::move(opener_routing_id_in)),
      parent_routing_id(std::move(parent_routing_id_in)),
      previous_sibling_routing_id(std::move(previous_sibling_routing_id_in)),
      replication_state(std::move(replication_state_in)),
      frame_owner_properties(std::move(frame_owner_properties_in)),
      widget_params(std::move(widget_params_in)) {}

CreateFrameParams::~CreateFrameParams() = default;

bool CreateFrameParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
UpdateScrollbarThemeParams::UpdateScrollbarThemeParams()
    : initial_button_delay(),
      autoscroll_button_delay(),
      jump_on_track_click(),
      preferred_scroller_style(),
      redraw(),
      button_placement() {}

UpdateScrollbarThemeParams::UpdateScrollbarThemeParams(
    float initial_button_delay_in,
    float autoscroll_button_delay_in,
    bool jump_on_track_click_in,
    ::content::mojom::ScrollerStyle preferred_scroller_style_in,
    bool redraw_in,
    ::content::mojom::ScrollbarButtonsPlacement button_placement_in)
    : initial_button_delay(std::move(initial_button_delay_in)),
      autoscroll_button_delay(std::move(autoscroll_button_delay_in)),
      jump_on_track_click(std::move(jump_on_track_click_in)),
      preferred_scroller_style(std::move(preferred_scroller_style_in)),
      redraw(std::move(redraw_in)),
      button_placement(std::move(button_placement_in)) {}

UpdateScrollbarThemeParams::~UpdateScrollbarThemeParams() = default;
size_t UpdateScrollbarThemeParams::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->initial_button_delay);
  seed = mojo::internal::Hash(seed, this->autoscroll_button_delay);
  seed = mojo::internal::Hash(seed, this->jump_on_track_click);
  seed = mojo::internal::Hash(seed, this->preferred_scroller_style);
  seed = mojo::internal::Hash(seed, this->redraw);
  seed = mojo::internal::Hash(seed, this->button_placement);
  return seed;
}

bool UpdateScrollbarThemeParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char Renderer::Name_[] = "content::mojom::Renderer";

RendererProxy::RendererProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void RendererProxy::CreateView(
    CreateViewParamsPtr in_params) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::Renderer_CreateView_Params_Data);
  size += mojo::internal::PrepareToSerialize<::content::mojom::CreateViewParamsDataView>(
      in_params, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kRenderer_CreateView_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::Renderer_CreateView_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->params)::BaseType* params_ptr;
  mojo::internal::Serialize<::content::mojom::CreateViewParamsDataView>(
      in_params, buffer, &params_ptr, &serialization_context);
  params->params.Set(params_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null params in Renderer.CreateView request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererProxy::CreateFrame(
    CreateFrameParamsPtr in_params) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::Renderer_CreateFrame_Params_Data);
  size += mojo::internal::PrepareToSerialize<::content::mojom::CreateFrameParamsDataView>(
      in_params, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kRenderer_CreateFrame_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::Renderer_CreateFrame_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->params)::BaseType* params_ptr;
  mojo::internal::Serialize<::content::mojom::CreateFrameParamsDataView>(
      in_params, buffer, &params_ptr, &serialization_context);
  params->params.Set(params_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null params in Renderer.CreateFrame request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererProxy::CreateFrameProxy(
    int32_t in_routing_id, int32_t in_render_view_routing_id, int32_t in_opener_routing_id, int32_t in_parent_routing_id, const content::FrameReplicationState& in_replication_state) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::Renderer_CreateFrameProxy_Params_Data);
  size += mojo::internal::PrepareToSerialize<::content::mojom::FrameReplicationStateDataView>(
      in_replication_state, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kRenderer_CreateFrameProxy_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::Renderer_CreateFrameProxy_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->routing_id = in_routing_id;
  params->render_view_routing_id = in_render_view_routing_id;
  params->opener_routing_id = in_opener_routing_id;
  params->parent_routing_id = in_parent_routing_id;
  typename decltype(params->replication_state)::BaseType* replication_state_ptr;
  mojo::internal::Serialize<::content::mojom::FrameReplicationStateDataView>(
      in_replication_state, buffer, &replication_state_ptr, &serialization_context);
  params->replication_state.Set(replication_state_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->replication_state.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null replication_state in Renderer.CreateFrameProxy request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererProxy::OnNetworkConnectionChanged(
    net::NetworkChangeNotifier::ConnectionType in_connection_type, double in_max_bandwidth_mbps) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::Renderer_OnNetworkConnectionChanged_Params_Data);
  serialization_context.PrepareMessage(
      internal::kRenderer_OnNetworkConnectionChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::Renderer_OnNetworkConnectionChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::content::mojom::NetworkConnectionType>(
      in_connection_type, &params->connection_type);
  params->max_bandwidth_mbps = in_max_bandwidth_mbps;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererProxy::OnNetworkQualityChanged(
    net::EffectiveConnectionType in_effective_connection_type, base::TimeDelta in_http_rtt, base::TimeDelta in_transport_rtt, double in_bandwidth_kbps) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::Renderer_OnNetworkQualityChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::TimeDeltaDataView>(
      in_http_rtt, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::TimeDeltaDataView>(
      in_transport_rtt, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kRenderer_OnNetworkQualityChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::Renderer_OnNetworkQualityChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::content::mojom::EffectiveConnectionType>(
      in_effective_connection_type, &params->effective_connection_type);
  typename decltype(params->http_rtt)::BaseType* http_rtt_ptr;
  mojo::internal::Serialize<::mojo::common::mojom::TimeDeltaDataView>(
      in_http_rtt, buffer, &http_rtt_ptr, &serialization_context);
  params->http_rtt.Set(http_rtt_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->http_rtt.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null http_rtt in Renderer.OnNetworkQualityChanged request");
  typename decltype(params->transport_rtt)::BaseType* transport_rtt_ptr;
  mojo::internal::Serialize<::mojo::common::mojom::TimeDeltaDataView>(
      in_transport_rtt, buffer, &transport_rtt_ptr, &serialization_context);
  params->transport_rtt.Set(transport_rtt_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->transport_rtt.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null transport_rtt in Renderer.OnNetworkQualityChanged request");
  params->bandwidth_kbps = in_bandwidth_kbps;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererProxy::SetWebKitSharedTimersSuspended(
    bool in_suspend) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::Renderer_SetWebKitSharedTimersSuspended_Params_Data);
  serialization_context.PrepareMessage(
      internal::kRenderer_SetWebKitSharedTimersSuspended_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::Renderer_SetWebKitSharedTimersSuspended_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->suspend = in_suspend;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererProxy::UpdateScrollbarTheme(
    UpdateScrollbarThemeParamsPtr in_params) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::Renderer_UpdateScrollbarTheme_Params_Data);
  size += mojo::internal::PrepareToSerialize<::content::mojom::UpdateScrollbarThemeParamsDataView>(
      in_params, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kRenderer_UpdateScrollbarTheme_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::Renderer_UpdateScrollbarTheme_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->params)::BaseType* params_ptr;
  mojo::internal::Serialize<::content::mojom::UpdateScrollbarThemeParamsDataView>(
      in_params, buffer, &params_ptr, &serialization_context);
  params->params.Set(params_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null params in Renderer.UpdateScrollbarTheme request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererProxy::OnSystemColorsChanged(
    int32_t in_aqua_color_variant, const std::string& in_highlight_text_color, const std::string& in_highlight_color) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::Renderer_OnSystemColorsChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_highlight_text_color, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_highlight_color, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kRenderer_OnSystemColorsChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::Renderer_OnSystemColorsChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->aqua_color_variant = in_aqua_color_variant;
  typename decltype(params->highlight_text_color)::BaseType* highlight_text_color_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_highlight_text_color, buffer, &highlight_text_color_ptr, &serialization_context);
  params->highlight_text_color.Set(highlight_text_color_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->highlight_text_color.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null highlight_text_color in Renderer.OnSystemColorsChanged request");
  typename decltype(params->highlight_color)::BaseType* highlight_color_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_highlight_color, buffer, &highlight_color_ptr, &serialization_context);
  params->highlight_color.Set(highlight_color_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->highlight_color.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null highlight_color in Renderer.OnSystemColorsChanged request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererProxy::PurgePluginListCache(
    bool in_reload_pages) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::Renderer_PurgePluginListCache_Params_Data);
  serialization_context.PrepareMessage(
      internal::kRenderer_PurgePluginListCache_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::Renderer_PurgePluginListCache_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->reload_pages = in_reload_pages;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool RendererStubDispatch::Accept(
    Renderer* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kRenderer_CreateView_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Renderer_CreateView_Params_Data* params =
          reinterpret_cast<internal::Renderer_CreateView_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      CreateViewParamsPtr p_params{};
      Renderer_CreateView_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadParams(&p_params))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Renderer::CreateView deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateView(
std::move(p_params));
      return true;
    }
    case internal::kRenderer_CreateFrame_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Renderer_CreateFrame_Params_Data* params =
          reinterpret_cast<internal::Renderer_CreateFrame_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      CreateFrameParamsPtr p_params{};
      Renderer_CreateFrame_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadParams(&p_params))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Renderer::CreateFrame deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateFrame(
std::move(p_params));
      return true;
    }
    case internal::kRenderer_CreateFrameProxy_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Renderer_CreateFrameProxy_Params_Data* params =
          reinterpret_cast<internal::Renderer_CreateFrameProxy_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_routing_id{};
      int32_t p_render_view_routing_id{};
      int32_t p_opener_routing_id{};
      int32_t p_parent_routing_id{};
      content::FrameReplicationState p_replication_state{};
      Renderer_CreateFrameProxy_ParamsDataView input_data_view(params, &serialization_context);
      
      p_routing_id = input_data_view.routing_id();
      p_render_view_routing_id = input_data_view.render_view_routing_id();
      p_opener_routing_id = input_data_view.opener_routing_id();
      p_parent_routing_id = input_data_view.parent_routing_id();
      if (!input_data_view.ReadReplicationState(&p_replication_state))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Renderer::CreateFrameProxy deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateFrameProxy(
std::move(p_routing_id), 
std::move(p_render_view_routing_id), 
std::move(p_opener_routing_id), 
std::move(p_parent_routing_id), 
std::move(p_replication_state));
      return true;
    }
    case internal::kRenderer_OnNetworkConnectionChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Renderer_OnNetworkConnectionChanged_Params_Data* params =
          reinterpret_cast<internal::Renderer_OnNetworkConnectionChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      net::NetworkChangeNotifier::ConnectionType p_connection_type{};
      double p_max_bandwidth_mbps{};
      Renderer_OnNetworkConnectionChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadConnectionType(&p_connection_type))
        success = false;
      p_max_bandwidth_mbps = input_data_view.max_bandwidth_mbps();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Renderer::OnNetworkConnectionChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnNetworkConnectionChanged(
std::move(p_connection_type), 
std::move(p_max_bandwidth_mbps));
      return true;
    }
    case internal::kRenderer_OnNetworkQualityChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Renderer_OnNetworkQualityChanged_Params_Data* params =
          reinterpret_cast<internal::Renderer_OnNetworkQualityChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      net::EffectiveConnectionType p_effective_connection_type{};
      base::TimeDelta p_http_rtt{};
      base::TimeDelta p_transport_rtt{};
      double p_bandwidth_kbps{};
      Renderer_OnNetworkQualityChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadEffectiveConnectionType(&p_effective_connection_type))
        success = false;
      if (!input_data_view.ReadHttpRtt(&p_http_rtt))
        success = false;
      if (!input_data_view.ReadTransportRtt(&p_transport_rtt))
        success = false;
      p_bandwidth_kbps = input_data_view.bandwidth_kbps();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Renderer::OnNetworkQualityChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnNetworkQualityChanged(
std::move(p_effective_connection_type), 
std::move(p_http_rtt), 
std::move(p_transport_rtt), 
std::move(p_bandwidth_kbps));
      return true;
    }
    case internal::kRenderer_SetWebKitSharedTimersSuspended_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Renderer_SetWebKitSharedTimersSuspended_Params_Data* params =
          reinterpret_cast<internal::Renderer_SetWebKitSharedTimersSuspended_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_suspend{};
      Renderer_SetWebKitSharedTimersSuspended_ParamsDataView input_data_view(params, &serialization_context);
      
      p_suspend = input_data_view.suspend();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Renderer::SetWebKitSharedTimersSuspended deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetWebKitSharedTimersSuspended(
std::move(p_suspend));
      return true;
    }
    case internal::kRenderer_UpdateScrollbarTheme_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Renderer_UpdateScrollbarTheme_Params_Data* params =
          reinterpret_cast<internal::Renderer_UpdateScrollbarTheme_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      UpdateScrollbarThemeParamsPtr p_params{};
      Renderer_UpdateScrollbarTheme_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadParams(&p_params))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Renderer::UpdateScrollbarTheme deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->UpdateScrollbarTheme(
std::move(p_params));
      return true;
    }
    case internal::kRenderer_OnSystemColorsChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Renderer_OnSystemColorsChanged_Params_Data* params =
          reinterpret_cast<internal::Renderer_OnSystemColorsChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_aqua_color_variant{};
      std::string p_highlight_text_color{};
      std::string p_highlight_color{};
      Renderer_OnSystemColorsChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_aqua_color_variant = input_data_view.aqua_color_variant();
      if (!input_data_view.ReadHighlightTextColor(&p_highlight_text_color))
        success = false;
      if (!input_data_view.ReadHighlightColor(&p_highlight_color))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Renderer::OnSystemColorsChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnSystemColorsChanged(
std::move(p_aqua_color_variant), 
std::move(p_highlight_text_color), 
std::move(p_highlight_color));
      return true;
    }
    case internal::kRenderer_PurgePluginListCache_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Renderer_PurgePluginListCache_Params_Data* params =
          reinterpret_cast<internal::Renderer_PurgePluginListCache_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_reload_pages{};
      Renderer_PurgePluginListCache_ParamsDataView input_data_view(params, &serialization_context);
      
      p_reload_pages = input_data_view.reload_pages();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Renderer::PurgePluginListCache deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->PurgePluginListCache(
std::move(p_reload_pages));
      return true;
    }
  }
  return false;
}

// static
bool RendererStubDispatch::AcceptWithResponder(
    Renderer* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kRenderer_CreateView_Name: {
      break;
    }
    case internal::kRenderer_CreateFrame_Name: {
      break;
    }
    case internal::kRenderer_CreateFrameProxy_Name: {
      break;
    }
    case internal::kRenderer_OnNetworkConnectionChanged_Name: {
      break;
    }
    case internal::kRenderer_OnNetworkQualityChanged_Name: {
      break;
    }
    case internal::kRenderer_SetWebKitSharedTimersSuspended_Name: {
      break;
    }
    case internal::kRenderer_UpdateScrollbarTheme_Name: {
      break;
    }
    case internal::kRenderer_OnSystemColorsChanged_Name: {
      break;
    }
    case internal::kRenderer_PurgePluginListCache_Name: {
      break;
    }
  }
  return false;
}

bool RendererRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "Renderer RequestValidator");

  switch (message->header()->name) {
    case internal::kRenderer_CreateView_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Renderer_CreateView_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRenderer_CreateFrame_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Renderer_CreateFrame_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRenderer_CreateFrameProxy_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Renderer_CreateFrameProxy_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRenderer_OnNetworkConnectionChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Renderer_OnNetworkConnectionChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRenderer_OnNetworkQualityChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Renderer_OnNetworkQualityChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRenderer_SetWebKitSharedTimersSuspended_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Renderer_SetWebKitSharedTimersSuspended_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRenderer_UpdateScrollbarTheme_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Renderer_UpdateScrollbarTheme_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRenderer_OnSystemColorsChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Renderer_OnSystemColorsChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRenderer_PurgePluginListCache_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Renderer_PurgePluginListCache_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace content

namespace mojo {


// static
bool StructTraits<::content::mojom::CreateViewParams::DataView, ::content::mojom::CreateViewParamsPtr>::Read(
    ::content::mojom::CreateViewParams::DataView input,
    ::content::mojom::CreateViewParamsPtr* output) {
  bool success = true;
  ::content::mojom::CreateViewParamsPtr result(::content::mojom::CreateViewParams::New());
  
      if (!input.ReadRendererPreferences(&result->renderer_preferences))
        success = false;
      if (!input.ReadWebPreferences(&result->web_preferences))
        success = false;
      result->view_id = input.view_id();
      result->main_frame_routing_id = input.main_frame_routing_id();
      result->main_frame_widget_routing_id = input.main_frame_widget_routing_id();
      result->session_storage_namespace_id = input.session_storage_namespace_id();
      result->opener_frame_route_id = input.opener_frame_route_id();
      result->swapped_out = input.swapped_out();
      if (!input.ReadReplicatedFrameState(&result->replicated_frame_state))
        success = false;
      result->proxy_routing_id = input.proxy_routing_id();
      result->hidden = input.hidden();
      result->never_visible = input.never_visible();
      result->window_was_created_with_opener = input.window_was_created_with_opener();
      if (!input.ReadInitialSize(&result->initial_size))
        success = false;
      result->enable_auto_resize = input.enable_auto_resize();
      if (!input.ReadMinSize(&result->min_size))
        success = false;
      if (!input.ReadMaxSize(&result->max_size))
        success = false;
      result->page_zoom_level = input.page_zoom_level();
      if (!input.ReadImageDecodeColorSpace(&result->image_decode_color_space))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::content::mojom::CreateFrameWidgetParams::DataView, ::content::mojom::CreateFrameWidgetParamsPtr>::Read(
    ::content::mojom::CreateFrameWidgetParams::DataView input,
    ::content::mojom::CreateFrameWidgetParamsPtr* output) {
  bool success = true;
  ::content::mojom::CreateFrameWidgetParamsPtr result(::content::mojom::CreateFrameWidgetParams::New());
  
      result->routing_id = input.routing_id();
      result->hidden = input.hidden();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::content::mojom::CreateFrameParams::DataView, ::content::mojom::CreateFrameParamsPtr>::Read(
    ::content::mojom::CreateFrameParams::DataView input,
    ::content::mojom::CreateFrameParamsPtr* output) {
  bool success = true;
  ::content::mojom::CreateFrameParamsPtr result(::content::mojom::CreateFrameParams::New());
  
      result->routing_id = input.routing_id();
      result->proxy_routing_id = input.proxy_routing_id();
      result->opener_routing_id = input.opener_routing_id();
      result->parent_routing_id = input.parent_routing_id();
      result->previous_sibling_routing_id = input.previous_sibling_routing_id();
      if (!input.ReadReplicationState(&result->replication_state))
        success = false;
      if (!input.ReadFrameOwnerProperties(&result->frame_owner_properties))
        success = false;
      if (!input.ReadWidgetParams(&result->widget_params))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::content::mojom::UpdateScrollbarThemeParams::DataView, ::content::mojom::UpdateScrollbarThemeParamsPtr>::Read(
    ::content::mojom::UpdateScrollbarThemeParams::DataView input,
    ::content::mojom::UpdateScrollbarThemeParamsPtr* output) {
  bool success = true;
  ::content::mojom::UpdateScrollbarThemeParamsPtr result(::content::mojom::UpdateScrollbarThemeParams::New());
  
      result->initial_button_delay = input.initial_button_delay();
      result->autoscroll_button_delay = input.autoscroll_button_delay();
      result->jump_on_track_click = input.jump_on_track_click();
      if (!input.ReadPreferredScrollerStyle(&result->preferred_scroller_style))
        success = false;
      result->redraw = input.redraw();
      if (!input.ReadButtonPlacement(&result->button_placement))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif