// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_RENDERER_HOST_MOJOM_H_
#define CONTENT_COMMON_RENDERER_HOST_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/renderer_host.mojom-shared.h"
#include "content/public/common/url_loader_factory.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class RendererHost;
using RendererHostPtr = mojo::InterfacePtr<RendererHost>;
using RendererHostPtrInfo = mojo::InterfacePtrInfo<RendererHost>;
using ThreadSafeRendererHostPtr =
    mojo::ThreadSafeInterfacePtr<RendererHost>;
using RendererHostRequest = mojo::InterfaceRequest<RendererHost>;
using RendererHostAssociatedPtr =
    mojo::AssociatedInterfacePtr<RendererHost>;
using ThreadSafeRendererHostAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<RendererHost>;
using RendererHostAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<RendererHost>;
using RendererHostAssociatedRequest =
    mojo::AssociatedInterfaceRequest<RendererHost>;


class RendererHostProxy;

template <typename ImplRefTraits>
class RendererHostStub;

class RendererHostRequestValidator;

class CONTENT_EXPORT RendererHost
    : public RendererHostInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = RendererHostProxy;

  template <typename ImplRefTraits>
  using Stub_ = RendererHostStub<ImplRefTraits>;

  using RequestValidator_ = RendererHostRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kGetBlobURLLoaderFactoryMinVersion = 0,
  };
  virtual ~RendererHost() {}

  virtual void GetBlobURLLoaderFactory(::content::mojom::URLLoaderFactoryRequest loader) = 0;
};

class CONTENT_EXPORT RendererHostProxy
    : public RendererHost {
 public:
  explicit RendererHostProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetBlobURLLoaderFactory(::content::mojom::URLLoaderFactoryRequest loader) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT RendererHostStubDispatch {
 public:
  static bool Accept(RendererHost* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      RendererHost* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<RendererHost>>
class RendererHostStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RendererHostStub() {}
  ~RendererHostStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RendererHostStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RendererHostStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT RendererHostRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace content

namespace mojo {

}  // namespace mojo

#endif  // CONTENT_COMMON_RENDERER_HOST_MOJOM_H_