// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "content/common/service_worker/service_worker_event_dispatcher.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace content {
namespace mojom {

namespace internal {


// static
BackgroundFetchSettledFetch_Data* BackgroundFetchSettledFetch_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(BackgroundFetchSettledFetch_Data))) BackgroundFetchSettledFetch_Data();
}

// static
bool BackgroundFetchSettledFetch_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const BackgroundFetchSettledFetch_Data* object = static_cast<const BackgroundFetchSettledFetch_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->request, "null request field in BackgroundFetchSettledFetch",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->request, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->response, "null response field in BackgroundFetchSettledFetch",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->response, validation_context))
    return false;

  return true;
}

BackgroundFetchSettledFetch_Data::BackgroundFetchSettledFetch_Data()
    : header_({sizeof(*this), 0}) {}


// static
FetchEventPreloadHandle_Data* FetchEventPreloadHandle_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(FetchEventPreloadHandle_Data))) FetchEventPreloadHandle_Data();
}

// static
bool FetchEventPreloadHandle_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const FetchEventPreloadHandle_Data* object = static_cast<const FetchEventPreloadHandle_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->url_loader,
          "invalid url_loader field in FetchEventPreloadHandle", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->url_loader,
                                                 validation_context)) {
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->url_loader_client_request,
          "invalid url_loader_client_request field in FetchEventPreloadHandle", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->url_loader_client_request,
                                                 validation_context)) {
    return false;
  }

  return true;
}

FetchEventPreloadHandle_Data::FetchEventPreloadHandle_Data()
    : header_({sizeof(*this), 0}) {}


// static
ExtendableMessageEvent_Data* ExtendableMessageEvent_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ExtendableMessageEvent_Data))) ExtendableMessageEvent_Data();
}

// static
bool ExtendableMessageEvent_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ExtendableMessageEvent_Data* object = static_cast<const ExtendableMessageEvent_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->message, "null message field in ExtendableMessageEvent",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->message, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->source_origin, "null source_origin field in ExtendableMessageEvent",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->source_origin, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->message_ports, "null message_ports field in ExtendableMessageEvent",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams message_ports_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->message_ports, validation_context,
                                         &message_ports_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->source, "null source field in ExtendableMessageEvent",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->source, validation_context))
    return false;

  return true;
}

ExtendableMessageEvent_Data::ExtendableMessageEvent_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerFetchResponseCallback_OnResponse_Params_Data* ServiceWorkerFetchResponseCallback_OnResponse_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerFetchResponseCallback_OnResponse_Params_Data))) ServiceWorkerFetchResponseCallback_OnResponse_Params_Data();
}

// static
bool ServiceWorkerFetchResponseCallback_OnResponse_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerFetchResponseCallback_OnResponse_Params_Data* object = static_cast<const ServiceWorkerFetchResponseCallback_OnResponse_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->response, "null response field in ServiceWorkerFetchResponseCallback_OnResponse_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->response, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerFetchResponseCallback_OnResponse_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerFetchResponseCallback_OnResponse_Params_Data::ServiceWorkerFetchResponseCallback_OnResponse_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerFetchResponseCallback_OnResponseStream_Params_Data* ServiceWorkerFetchResponseCallback_OnResponseStream_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerFetchResponseCallback_OnResponseStream_Params_Data))) ServiceWorkerFetchResponseCallback_OnResponseStream_Params_Data();
}

// static
bool ServiceWorkerFetchResponseCallback_OnResponseStream_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerFetchResponseCallback_OnResponseStream_Params_Data* object = static_cast<const ServiceWorkerFetchResponseCallback_OnResponseStream_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->response, "null response field in ServiceWorkerFetchResponseCallback_OnResponseStream_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->response, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->body_as_stream, "null body_as_stream field in ServiceWorkerFetchResponseCallback_OnResponseStream_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->body_as_stream, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerFetchResponseCallback_OnResponseStream_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerFetchResponseCallback_OnResponseStream_Params_Data::ServiceWorkerFetchResponseCallback_OnResponseStream_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerFetchResponseCallback_OnFallback_Params_Data* ServiceWorkerFetchResponseCallback_OnFallback_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerFetchResponseCallback_OnFallback_Params_Data))) ServiceWorkerFetchResponseCallback_OnFallback_Params_Data();
}

// static
bool ServiceWorkerFetchResponseCallback_OnFallback_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerFetchResponseCallback_OnFallback_Params_Data* object = static_cast<const ServiceWorkerFetchResponseCallback_OnFallback_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerFetchResponseCallback_OnFallback_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerFetchResponseCallback_OnFallback_Params_Data::ServiceWorkerFetchResponseCallback_OnFallback_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchInstallEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchInstallEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchInstallEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchInstallEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchInstallEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchInstallEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchInstallEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->method,
          "invalid method field in ServiceWorkerEventDispatcher_DispatchInstallEvent_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->method,
                                                 validation_context)) {
    return false;
  }

  return true;
}

ServiceWorkerEventDispatcher_DispatchInstallEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchInstallEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchInstallEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchActivateEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchActivateEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchActivateEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchActivateEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchActivateEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchActivateEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchActivateEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

ServiceWorkerEventDispatcher_DispatchActivateEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchActivateEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchActivateEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->tag, "null tag field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams tag_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->tag, validation_context,
                                         &tag_validate_params)) {
    return false;
  }

  return true;
}

ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchBackgroundFetchAbortEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->tag, "null tag field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams tag_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->tag, validation_context,
                                         &tag_validate_params)) {
    return false;
  }


  if (!::content::mojom::internal::BackgroundFetchState_Data
        ::Validate(object->state, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchBackgroundFetchClickEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->tag, "null tag field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams tag_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->tag, validation_context,
                                         &tag_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->fetches, "null fetches field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams fetches_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->fetches, validation_context,
                                         &fetches_validate_params)) {
    return false;
  }

  return true;
}

ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchBackgroundFetchFailEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->tag, "null tag field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams tag_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->tag, validation_context,
                                         &tag_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->fetches, "null fetches field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams fetches_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->fetches, validation_context,
                                         &fetches_validate_params)) {
    return false;
  }

  return true;
}

ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchBackgroundFetchedEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchFetchEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchFetchEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchFetchEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchFetchEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchFetchEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchFetchEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchFetchEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->request, "null request field in ServiceWorkerEventDispatcher_DispatchFetchEvent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->request, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->preload_handle, validation_context))
    return false;

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->response_callback,
          "invalid response_callback field in ServiceWorkerEventDispatcher_DispatchFetchEvent_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->response_callback,
                                                 validation_context)) {
    return false;
  }

  return true;
}

ServiceWorkerEventDispatcher_DispatchFetchEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchFetchEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchFetchEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->notification_id, "null notification_id field in ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams notification_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->notification_id, validation_context,
                                         &notification_id_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->notification_data, "null notification_data field in ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->notification_data, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->reply, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchNotificationClickEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->notification_id, "null notification_id field in ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams notification_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->notification_id, validation_context,
                                         &notification_id_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->notification_data, "null notification_data field in ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->notification_data, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchNotificationCloseEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchPushEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchPushEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchPushEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchPushEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchPushEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchPushEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchPushEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->payload, "null payload field in ServiceWorkerEventDispatcher_DispatchPushEvent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->payload, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchPushEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchPushEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchPushEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchSyncEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchSyncEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchSyncEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchSyncEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchSyncEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchSyncEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchSyncEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->tag, "null tag field in ServiceWorkerEventDispatcher_DispatchSyncEvent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams tag_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->tag, validation_context,
                                         &tag_validate_params)) {
    return false;
  }


  if (!::blink::mojom::internal::BackgroundSyncEventLastChance_Data
        ::Validate(object->last_chance, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchSyncEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchSyncEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchSyncEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->request_data, "null request_data field in ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->request_data, validation_context))
    return false;

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->response_callback,
          "invalid response_callback field in ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->response_callback,
                                                 validation_context)) {
    return false;
  }

  return true;
}

ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchPaymentRequestEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params_Data* ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params_Data))) ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->event, "null event field in ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->event, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params_Data::ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams_Data* ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams_Data))) ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ServiceWorkerEventStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams_Data::ServiceWorkerEventDispatcher_DispatchExtendableMessageEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_Ping_Params_Data* ServiceWorkerEventDispatcher_Ping_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_Ping_Params_Data))) ServiceWorkerEventDispatcher_Ping_Params_Data();
}

// static
bool ServiceWorkerEventDispatcher_Ping_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_Ping_Params_Data* object = static_cast<const ServiceWorkerEventDispatcher_Ping_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

ServiceWorkerEventDispatcher_Ping_Params_Data::ServiceWorkerEventDispatcher_Ping_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerEventDispatcher_Ping_ResponseParams_Data* ServiceWorkerEventDispatcher_Ping_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerEventDispatcher_Ping_ResponseParams_Data))) ServiceWorkerEventDispatcher_Ping_ResponseParams_Data();
}

// static
bool ServiceWorkerEventDispatcher_Ping_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerEventDispatcher_Ping_ResponseParams_Data* object = static_cast<const ServiceWorkerEventDispatcher_Ping_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

ServiceWorkerEventDispatcher_Ping_ResponseParams_Data::ServiceWorkerEventDispatcher_Ping_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params_Data* ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params_Data))) ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params_Data();
}

// static
bool ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params_Data* object = static_cast<const ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->sub_scopes, "null sub_scopes field in ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams sub_scopes_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->sub_scopes, validation_context,
                                         &sub_scopes_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->origins, "null origins field in ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams origins_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->origins, validation_context,
                                         &origins_validate_params)) {
    return false;
  }

  return true;
}

ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params_Data::ServiceWorkerInstallEventMethods_RegisterForeignFetchScopes_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace content

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
