// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "content/common/service_worker/service_worker_installed_scripts_manager.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace content {
namespace mojom {

namespace internal {


// static
ServiceWorkerInstalledScriptsInfo_Data* ServiceWorkerInstalledScriptsInfo_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerInstalledScriptsInfo_Data))) ServiceWorkerInstalledScriptsInfo_Data();
}

// static
bool ServiceWorkerInstalledScriptsInfo_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerInstalledScriptsInfo_Data* object = static_cast<const ServiceWorkerInstalledScriptsInfo_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->manager_request,
          "invalid manager_request field in ServiceWorkerInstalledScriptsInfo", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->manager_request,
                                                 validation_context)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->installed_urls, "null installed_urls field in ServiceWorkerInstalledScriptsInfo",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams installed_urls_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->installed_urls, validation_context,
                                         &installed_urls_validate_params)) {
    return false;
  }

  return true;
}

ServiceWorkerInstalledScriptsInfo_Data::ServiceWorkerInstalledScriptsInfo_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerScriptInfo_Data* ServiceWorkerScriptInfo_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerScriptInfo_Data))) ServiceWorkerScriptInfo_Data();
}

// static
bool ServiceWorkerScriptInfo_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerScriptInfo_Data* object = static_cast<const ServiceWorkerScriptInfo_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->script_url, "null script_url field in ServiceWorkerScriptInfo",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->script_url, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->encoding, "null encoding field in ServiceWorkerScriptInfo",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams encoding_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->encoding, validation_context,
                                         &encoding_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->headers, "null headers field in ServiceWorkerScriptInfo",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams headers_validate_params(
      new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
  if (!mojo::internal::ValidateContainer(object->headers, validation_context,
                                         &headers_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->body,
          "invalid body field in ServiceWorkerScriptInfo", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->body,
                                                 validation_context)) {
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->meta_data,
          "invalid meta_data field in ServiceWorkerScriptInfo", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->meta_data,
                                                 validation_context)) {
    return false;
  }

  return true;
}

ServiceWorkerScriptInfo_Data::ServiceWorkerScriptInfo_Data()
    : header_({sizeof(*this), 0}) {}


// static
ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data* ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data))) ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data();
}

// static
bool ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data* object = static_cast<const ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->script_info, "null script_info field in ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->script_info, validation_context))
    return false;

  return true;
}

ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data::ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace content

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
