// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_INSTALLED_SCRIPTS_MANAGER_MOJOM_H_
#define CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_INSTALLED_SCRIPTS_MANAGER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/service_worker/service_worker_installed_scripts_manager.mojom-shared.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class ServiceWorkerInstalledScriptsManager;
using ServiceWorkerInstalledScriptsManagerPtr = mojo::InterfacePtr<ServiceWorkerInstalledScriptsManager>;
using ServiceWorkerInstalledScriptsManagerPtrInfo = mojo::InterfacePtrInfo<ServiceWorkerInstalledScriptsManager>;
using ThreadSafeServiceWorkerInstalledScriptsManagerPtr =
    mojo::ThreadSafeInterfacePtr<ServiceWorkerInstalledScriptsManager>;
using ServiceWorkerInstalledScriptsManagerRequest = mojo::InterfaceRequest<ServiceWorkerInstalledScriptsManager>;
using ServiceWorkerInstalledScriptsManagerAssociatedPtr =
    mojo::AssociatedInterfacePtr<ServiceWorkerInstalledScriptsManager>;
using ThreadSafeServiceWorkerInstalledScriptsManagerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ServiceWorkerInstalledScriptsManager>;
using ServiceWorkerInstalledScriptsManagerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ServiceWorkerInstalledScriptsManager>;
using ServiceWorkerInstalledScriptsManagerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ServiceWorkerInstalledScriptsManager>;

class ServiceWorkerInstalledScriptsInfo;
using ServiceWorkerInstalledScriptsInfoPtr = mojo::StructPtr<ServiceWorkerInstalledScriptsInfo>;

class ServiceWorkerScriptInfo;
using ServiceWorkerScriptInfoPtr = mojo::StructPtr<ServiceWorkerScriptInfo>;


class ServiceWorkerInstalledScriptsManagerProxy;

template <typename ImplRefTraits>
class ServiceWorkerInstalledScriptsManagerStub;

class ServiceWorkerInstalledScriptsManagerRequestValidator;

class CONTENT_EXPORT ServiceWorkerInstalledScriptsManager
    : public ServiceWorkerInstalledScriptsManagerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ServiceWorkerInstalledScriptsManagerProxy;

  template <typename ImplRefTraits>
  using Stub_ = ServiceWorkerInstalledScriptsManagerStub<ImplRefTraits>;

  using RequestValidator_ = ServiceWorkerInstalledScriptsManagerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kTransferInstalledScriptMinVersion = 0,
  };
  virtual ~ServiceWorkerInstalledScriptsManager() {}

  virtual void TransferInstalledScript(ServiceWorkerScriptInfoPtr script_info) = 0;
};

class CONTENT_EXPORT ServiceWorkerInstalledScriptsManagerProxy
    : public ServiceWorkerInstalledScriptsManager {
 public:
  explicit ServiceWorkerInstalledScriptsManagerProxy(mojo::MessageReceiverWithResponder* receiver);
  void TransferInstalledScript(ServiceWorkerScriptInfoPtr script_info) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT ServiceWorkerInstalledScriptsManagerStubDispatch {
 public:
  static bool Accept(ServiceWorkerInstalledScriptsManager* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ServiceWorkerInstalledScriptsManager* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ServiceWorkerInstalledScriptsManager>>
class ServiceWorkerInstalledScriptsManagerStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ServiceWorkerInstalledScriptsManagerStub() {}
  ~ServiceWorkerInstalledScriptsManagerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceWorkerInstalledScriptsManagerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceWorkerInstalledScriptsManagerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT ServiceWorkerInstalledScriptsManagerRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};






class CONTENT_EXPORT ServiceWorkerInstalledScriptsInfo {
 public:
  using DataView = ServiceWorkerInstalledScriptsInfoDataView;
  using Data_ = internal::ServiceWorkerInstalledScriptsInfo_Data;

  template <typename... Args>
  static ServiceWorkerInstalledScriptsInfoPtr New(Args&&... args) {
    return ServiceWorkerInstalledScriptsInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ServiceWorkerInstalledScriptsInfoPtr From(const U& u) {
    return mojo::TypeConverter<ServiceWorkerInstalledScriptsInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ServiceWorkerInstalledScriptsInfo>::Convert(*this);
  }


  ServiceWorkerInstalledScriptsInfo();

  ServiceWorkerInstalledScriptsInfo(
      ServiceWorkerInstalledScriptsManagerRequest manager_request,
      const std::vector<GURL>& installed_urls);

  ~ServiceWorkerInstalledScriptsInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ServiceWorkerInstalledScriptsInfoPtr>
  ServiceWorkerInstalledScriptsInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ServiceWorkerInstalledScriptsInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ServiceWorkerInstalledScriptsInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ServiceWorkerInstalledScriptsInfo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  ServiceWorkerInstalledScriptsManagerRequest manager_request;
  std::vector<GURL> installed_urls;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(ServiceWorkerInstalledScriptsInfo);
};



class CONTENT_EXPORT ServiceWorkerScriptInfo {
 public:
  using DataView = ServiceWorkerScriptInfoDataView;
  using Data_ = internal::ServiceWorkerScriptInfo_Data;

  template <typename... Args>
  static ServiceWorkerScriptInfoPtr New(Args&&... args) {
    return ServiceWorkerScriptInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ServiceWorkerScriptInfoPtr From(const U& u) {
    return mojo::TypeConverter<ServiceWorkerScriptInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ServiceWorkerScriptInfo>::Convert(*this);
  }


  ServiceWorkerScriptInfo();

  ServiceWorkerScriptInfo(
      const GURL& script_url,
      const std::string& encoding,
      const std::unordered_map<std::string, std::string>& headers,
      mojo::ScopedDataPipeConsumerHandle body,
      mojo::ScopedDataPipeConsumerHandle meta_data);

  ~ServiceWorkerScriptInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ServiceWorkerScriptInfoPtr>
  ServiceWorkerScriptInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ServiceWorkerScriptInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ServiceWorkerScriptInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ServiceWorkerScriptInfo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  GURL script_url;
  std::string encoding;
  std::unordered_map<std::string, std::string> headers;
  mojo::ScopedDataPipeConsumerHandle body;
  mojo::ScopedDataPipeConsumerHandle meta_data;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(ServiceWorkerScriptInfo);
};


template <typename StructPtrType>
ServiceWorkerInstalledScriptsInfoPtr ServiceWorkerInstalledScriptsInfo::Clone() const {
  return New(
      mojo::Clone(manager_request),
      mojo::Clone(installed_urls)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ServiceWorkerInstalledScriptsInfo>::value>::type*>
bool ServiceWorkerInstalledScriptsInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->manager_request, other.manager_request))
    return false;
  if (!mojo::Equals(this->installed_urls, other.installed_urls))
    return false;
  return true;
}
template <typename StructPtrType>
ServiceWorkerScriptInfoPtr ServiceWorkerScriptInfo::Clone() const {
  return New(
      mojo::Clone(script_url),
      mojo::Clone(encoding),
      mojo::Clone(headers),
      mojo::Clone(body),
      mojo::Clone(meta_data)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ServiceWorkerScriptInfo>::value>::type*>
bool ServiceWorkerScriptInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->script_url, other.script_url))
    return false;
  if (!mojo::Equals(this->encoding, other.encoding))
    return false;
  if (!mojo::Equals(this->headers, other.headers))
    return false;
  if (!mojo::Equals(this->body, other.body))
    return false;
  if (!mojo::Equals(this->meta_data, other.meta_data))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace content

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::ServiceWorkerInstalledScriptsInfo::DataView,
                                         ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr> {
  static bool IsNull(const ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr& input) { return !input; }
  static void SetToNull(::content::mojom::ServiceWorkerInstalledScriptsInfoPtr* output) { output->reset(); }

  static  decltype(::content::mojom::ServiceWorkerInstalledScriptsInfo::manager_request)& manager_request(
       ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr& input) {
    return input->manager_request;
  }

  static const decltype(::content::mojom::ServiceWorkerInstalledScriptsInfo::installed_urls)& installed_urls(
      const ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr& input) {
    return input->installed_urls;
  }

  static bool Read(::content::mojom::ServiceWorkerInstalledScriptsInfo::DataView input, ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::ServiceWorkerScriptInfo::DataView,
                                         ::content::mojom::ServiceWorkerScriptInfoPtr> {
  static bool IsNull(const ::content::mojom::ServiceWorkerScriptInfoPtr& input) { return !input; }
  static void SetToNull(::content::mojom::ServiceWorkerScriptInfoPtr* output) { output->reset(); }

  static const decltype(::content::mojom::ServiceWorkerScriptInfo::script_url)& script_url(
      const ::content::mojom::ServiceWorkerScriptInfoPtr& input) {
    return input->script_url;
  }

  static const decltype(::content::mojom::ServiceWorkerScriptInfo::encoding)& encoding(
      const ::content::mojom::ServiceWorkerScriptInfoPtr& input) {
    return input->encoding;
  }

  static const decltype(::content::mojom::ServiceWorkerScriptInfo::headers)& headers(
      const ::content::mojom::ServiceWorkerScriptInfoPtr& input) {
    return input->headers;
  }

  static  decltype(::content::mojom::ServiceWorkerScriptInfo::body)& body(
       ::content::mojom::ServiceWorkerScriptInfoPtr& input) {
    return input->body;
  }

  static  decltype(::content::mojom::ServiceWorkerScriptInfo::meta_data)& meta_data(
       ::content::mojom::ServiceWorkerScriptInfoPtr& input) {
    return input->meta_data;
  }

  static bool Read(::content::mojom::ServiceWorkerScriptInfo::DataView input, ::content::mojom::ServiceWorkerScriptInfoPtr* output);
};

}  // namespace mojo

#endif  // CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_INSTALLED_SCRIPTS_MANAGER_MOJOM_H_