// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_PROVIDER_MOJOM_SHARED_H_
#define CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_PROVIDER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "content/common/service_worker/service_worker_provider.mojom-shared-internal.h"
#include "content/common/service_worker/service_worker_provider_interfaces.mojom-shared.h"
#include "content/common/service_worker/service_worker_types.mojom-shared.h"
#include "content/common/mojo_bindings_shared_export.h"


namespace content {
namespace mojom {
class ServiceWorkerProviderHostInfoDataView;



}  // namespace mojom
}  // namespace content

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::content::mojom::ServiceWorkerProviderHostInfoDataView> {
  using Data = ::content::mojom::internal::ServiceWorkerProviderHostInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace content {
namespace mojom {
class ServiceWorkerProviderHostInfoDataView {
 public:
  ServiceWorkerProviderHostInfoDataView() {}

  ServiceWorkerProviderHostInfoDataView(
      internal::ServiceWorkerProviderHostInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t provider_id() const {
    return data_->provider_id;
  }
  int32_t route_id() const {
    return data_->route_id;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::content::mojom::ServiceWorkerProviderType>(
        data_value, output);
  }

  ::content::mojom::ServiceWorkerProviderType type() const {
    return static_cast<::content::mojom::ServiceWorkerProviderType>(data_->type);
  }
  bool is_parent_frame_secure() const {
    return data_->is_parent_frame_secure;
  }
  template <typename UserType>
  UserType TakeHostRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::content::mojom::ServiceWorkerProviderHostAssociatedRequestDataView>(
            &data_->host_request, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeClientPtrInfo() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::content::mojom::ServiceWorkerProviderAssociatedPtrInfoDataView>(
            &data_->client_ptr_info, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::ServiceWorkerProviderHostInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace content

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::content::mojom::ServiceWorkerProviderHostInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::content::mojom::ServiceWorkerProviderHostInfoDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::content::mojom::internal::ServiceWorkerProviderHostInfo_Data);
    decltype(CallWithContext(Traits::host_request, input, custom_context)) in_host_request = CallWithContext(Traits::host_request, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::content::mojom::ServiceWorkerProviderHostAssociatedRequestDataView>(
        in_host_request, context);
    decltype(CallWithContext(Traits::client_ptr_info, input, custom_context)) in_client_ptr_info = CallWithContext(Traits::client_ptr_info, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::content::mojom::ServiceWorkerProviderAssociatedPtrInfoDataView>(
        in_client_ptr_info, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::content::mojom::internal::ServiceWorkerProviderHostInfo_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::content::mojom::internal::ServiceWorkerProviderHostInfo_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->provider_id = CallWithContext(Traits::provider_id, input, custom_context);
    result->route_id = CallWithContext(Traits::route_id, input, custom_context);
    mojo::internal::Serialize<::content::mojom::ServiceWorkerProviderType>(
        CallWithContext(Traits::type, input, custom_context), &result->type);
    result->is_parent_frame_secure = CallWithContext(Traits::is_parent_frame_secure, input, custom_context);
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::host_request, input, custom_context))>::type
        in_host_request;
    mojo::internal::Serialize<::content::mojom::ServiceWorkerProviderHostAssociatedRequestDataView>(
        in_host_request, &result->host_request, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->host_request),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_INTERFACE_ID,
        "invalid host_request in ServiceWorkerProviderHostInfo struct");
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::client_ptr_info, input, custom_context))>::type
        in_client_ptr_info;
    mojo::internal::Serialize<::content::mojom::ServiceWorkerProviderAssociatedPtrInfoDataView>(
        in_client_ptr_info, &result->client_ptr_info, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->client_ptr_info),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_INTERFACE_ID,
        "invalid client_ptr_info in ServiceWorkerProviderHostInfo struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::content::mojom::internal::ServiceWorkerProviderHostInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::content::mojom::ServiceWorkerProviderHostInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace content {
namespace mojom {




}  // namespace mojom
}  // namespace content

#endif  // CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_PROVIDER_MOJOM_SHARED_H_
