// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_PROVIDER_INTERFACES_MOJOM_H_
#define CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_PROVIDER_INTERFACES_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/service_worker/service_worker_provider_interfaces.mojom-shared.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class ServiceWorkerProviderHost;
using ServiceWorkerProviderHostPtr = mojo::InterfacePtr<ServiceWorkerProviderHost>;
using ServiceWorkerProviderHostPtrInfo = mojo::InterfacePtrInfo<ServiceWorkerProviderHost>;
using ThreadSafeServiceWorkerProviderHostPtr =
    mojo::ThreadSafeInterfacePtr<ServiceWorkerProviderHost>;
using ServiceWorkerProviderHostRequest = mojo::InterfaceRequest<ServiceWorkerProviderHost>;
using ServiceWorkerProviderHostAssociatedPtr =
    mojo::AssociatedInterfacePtr<ServiceWorkerProviderHost>;
using ThreadSafeServiceWorkerProviderHostAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ServiceWorkerProviderHost>;
using ServiceWorkerProviderHostAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ServiceWorkerProviderHost>;
using ServiceWorkerProviderHostAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ServiceWorkerProviderHost>;

class ServiceWorkerProvider;
using ServiceWorkerProviderPtr = mojo::InterfacePtr<ServiceWorkerProvider>;
using ServiceWorkerProviderPtrInfo = mojo::InterfacePtrInfo<ServiceWorkerProvider>;
using ThreadSafeServiceWorkerProviderPtr =
    mojo::ThreadSafeInterfacePtr<ServiceWorkerProvider>;
using ServiceWorkerProviderRequest = mojo::InterfaceRequest<ServiceWorkerProvider>;
using ServiceWorkerProviderAssociatedPtr =
    mojo::AssociatedInterfacePtr<ServiceWorkerProvider>;
using ThreadSafeServiceWorkerProviderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ServiceWorkerProvider>;
using ServiceWorkerProviderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ServiceWorkerProvider>;
using ServiceWorkerProviderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ServiceWorkerProvider>;


class ServiceWorkerProviderHostProxy;

template <typename ImplRefTraits>
class ServiceWorkerProviderHostStub;

class ServiceWorkerProviderHostRequestValidator;

class CONTENT_EXPORT ServiceWorkerProviderHost
    : public ServiceWorkerProviderHostInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ServiceWorkerProviderHostProxy;

  template <typename ImplRefTraits>
  using Stub_ = ServiceWorkerProviderHostStub<ImplRefTraits>;

  using RequestValidator_ = ServiceWorkerProviderHostRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
  };
  virtual ~ServiceWorkerProviderHost() {}
};

class ServiceWorkerProviderProxy;

template <typename ImplRefTraits>
class ServiceWorkerProviderStub;

class ServiceWorkerProviderRequestValidator;

class CONTENT_EXPORT ServiceWorkerProvider
    : public ServiceWorkerProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ServiceWorkerProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = ServiceWorkerProviderStub<ImplRefTraits>;

  using RequestValidator_ = ServiceWorkerProviderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
  };
  virtual ~ServiceWorkerProvider() {}
};

class CONTENT_EXPORT ServiceWorkerProviderHostProxy
    : public ServiceWorkerProviderHost {
 public:
  explicit ServiceWorkerProviderHostProxy(mojo::MessageReceiverWithResponder* receiver);

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT ServiceWorkerProviderProxy
    : public ServiceWorkerProvider {
 public:
  explicit ServiceWorkerProviderProxy(mojo::MessageReceiverWithResponder* receiver);

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT ServiceWorkerProviderHostStubDispatch {
 public:
  static bool Accept(ServiceWorkerProviderHost* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ServiceWorkerProviderHost* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ServiceWorkerProviderHost>>
class ServiceWorkerProviderHostStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ServiceWorkerProviderHostStub() {}
  ~ServiceWorkerProviderHostStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceWorkerProviderHostStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceWorkerProviderHostStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT ServiceWorkerProviderStubDispatch {
 public:
  static bool Accept(ServiceWorkerProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ServiceWorkerProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ServiceWorkerProvider>>
class ServiceWorkerProviderStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ServiceWorkerProviderStub() {}
  ~ServiceWorkerProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceWorkerProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceWorkerProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT ServiceWorkerProviderHostRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT ServiceWorkerProviderRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace content

namespace mojo {

}  // namespace mojo

#endif  // CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_PROVIDER_INTERFACES_MOJOM_H_