// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_WORKER_URL_LOADER_FACTORY_PROVIDER_MOJOM_H_
#define CONTENT_COMMON_WORKER_URL_LOADER_FACTORY_PROVIDER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/worker_url_loader_factory_provider.mojom-shared.h"
#include "content/public/common/url_loader_factory.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class ServiceWorkerWorkerClient;
using ServiceWorkerWorkerClientPtr = mojo::InterfacePtr<ServiceWorkerWorkerClient>;
using ServiceWorkerWorkerClientPtrInfo = mojo::InterfacePtrInfo<ServiceWorkerWorkerClient>;
using ThreadSafeServiceWorkerWorkerClientPtr =
    mojo::ThreadSafeInterfacePtr<ServiceWorkerWorkerClient>;
using ServiceWorkerWorkerClientRequest = mojo::InterfaceRequest<ServiceWorkerWorkerClient>;
using ServiceWorkerWorkerClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<ServiceWorkerWorkerClient>;
using ThreadSafeServiceWorkerWorkerClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ServiceWorkerWorkerClient>;
using ServiceWorkerWorkerClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ServiceWorkerWorkerClient>;
using ServiceWorkerWorkerClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ServiceWorkerWorkerClient>;

class WorkerURLLoaderFactoryProvider;
using WorkerURLLoaderFactoryProviderPtr = mojo::InterfacePtr<WorkerURLLoaderFactoryProvider>;
using WorkerURLLoaderFactoryProviderPtrInfo = mojo::InterfacePtrInfo<WorkerURLLoaderFactoryProvider>;
using ThreadSafeWorkerURLLoaderFactoryProviderPtr =
    mojo::ThreadSafeInterfacePtr<WorkerURLLoaderFactoryProvider>;
using WorkerURLLoaderFactoryProviderRequest = mojo::InterfaceRequest<WorkerURLLoaderFactoryProvider>;
using WorkerURLLoaderFactoryProviderAssociatedPtr =
    mojo::AssociatedInterfacePtr<WorkerURLLoaderFactoryProvider>;
using ThreadSafeWorkerURLLoaderFactoryProviderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<WorkerURLLoaderFactoryProvider>;
using WorkerURLLoaderFactoryProviderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<WorkerURLLoaderFactoryProvider>;
using WorkerURLLoaderFactoryProviderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<WorkerURLLoaderFactoryProvider>;


class ServiceWorkerWorkerClientProxy;

template <typename ImplRefTraits>
class ServiceWorkerWorkerClientStub;

class ServiceWorkerWorkerClientRequestValidator;

class CONTENT_EXPORT ServiceWorkerWorkerClient
    : public ServiceWorkerWorkerClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ServiceWorkerWorkerClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = ServiceWorkerWorkerClientStub<ImplRefTraits>;

  using RequestValidator_ = ServiceWorkerWorkerClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetControllerServiceWorkerMinVersion = 0,
  };
  virtual ~ServiceWorkerWorkerClient() {}

  virtual void SetControllerServiceWorker(int64_t controller_version_id) = 0;
};

class WorkerURLLoaderFactoryProviderProxy;

template <typename ImplRefTraits>
class WorkerURLLoaderFactoryProviderStub;

class WorkerURLLoaderFactoryProviderRequestValidator;

class CONTENT_EXPORT WorkerURLLoaderFactoryProvider
    : public WorkerURLLoaderFactoryProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = true;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = WorkerURLLoaderFactoryProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = WorkerURLLoaderFactoryProviderStub<ImplRefTraits>;

  using RequestValidator_ = WorkerURLLoaderFactoryProviderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kGetURLLoaderFactoryAndRegisterClientMinVersion = 0,
    kGetURLLoaderFactoryMinVersion = 0,
  };
  virtual ~WorkerURLLoaderFactoryProvider() {}

  virtual void GetURLLoaderFactoryAndRegisterClient(::content::mojom::URLLoaderFactoryAssociatedRequest loader, ServiceWorkerWorkerClientAssociatedPtrInfo client, int32_t service_worker_provider_id) = 0;

  virtual void GetURLLoaderFactory(::content::mojom::URLLoaderFactoryAssociatedRequest loader) = 0;
};

class CONTENT_EXPORT ServiceWorkerWorkerClientProxy
    : public ServiceWorkerWorkerClient {
 public:
  explicit ServiceWorkerWorkerClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetControllerServiceWorker(int64_t controller_version_id) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT WorkerURLLoaderFactoryProviderProxy
    : public WorkerURLLoaderFactoryProvider {
 public:
  explicit WorkerURLLoaderFactoryProviderProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetURLLoaderFactoryAndRegisterClient(::content::mojom::URLLoaderFactoryAssociatedRequest loader, ServiceWorkerWorkerClientAssociatedPtrInfo client, int32_t service_worker_provider_id) override;
  void GetURLLoaderFactory(::content::mojom::URLLoaderFactoryAssociatedRequest loader) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT ServiceWorkerWorkerClientStubDispatch {
 public:
  static bool Accept(ServiceWorkerWorkerClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ServiceWorkerWorkerClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ServiceWorkerWorkerClient>>
class ServiceWorkerWorkerClientStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ServiceWorkerWorkerClientStub() {}
  ~ServiceWorkerWorkerClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceWorkerWorkerClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceWorkerWorkerClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT WorkerURLLoaderFactoryProviderStubDispatch {
 public:
  static bool Accept(WorkerURLLoaderFactoryProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      WorkerURLLoaderFactoryProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<WorkerURLLoaderFactoryProvider>>
class WorkerURLLoaderFactoryProviderStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  WorkerURLLoaderFactoryProviderStub() {}
  ~WorkerURLLoaderFactoryProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WorkerURLLoaderFactoryProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WorkerURLLoaderFactoryProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT ServiceWorkerWorkerClientRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT WorkerURLLoaderFactoryProviderRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace content

namespace mojo {

}  // namespace mojo

#endif  // CONTENT_COMMON_WORKER_URL_LOADER_FACTORY_PROVIDER_MOJOM_H_