// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_PUBLIC_COMMON_NETWORK_SERVICE_TEST_MOJOM_H_
#define CONTENT_PUBLIC_COMMON_NETWORK_SERVICE_TEST_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/public/common/network_service_test.mojom-shared.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class NetworkServiceTest;
using NetworkServiceTestPtr = mojo::InterfacePtr<NetworkServiceTest>;
using NetworkServiceTestPtrInfo = mojo::InterfacePtrInfo<NetworkServiceTest>;
using ThreadSafeNetworkServiceTestPtr =
    mojo::ThreadSafeInterfacePtr<NetworkServiceTest>;
using NetworkServiceTestRequest = mojo::InterfaceRequest<NetworkServiceTest>;
using NetworkServiceTestAssociatedPtr =
    mojo::AssociatedInterfacePtr<NetworkServiceTest>;
using ThreadSafeNetworkServiceTestAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NetworkServiceTest>;
using NetworkServiceTestAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NetworkServiceTest>;
using NetworkServiceTestAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NetworkServiceTest>;

class Rule;
using RulePtr = mojo::InlinedStructPtr<Rule>;


class NetworkServiceTestProxy;

template <typename ImplRefTraits>
class NetworkServiceTestStub;

class NetworkServiceTestRequestValidator;
class NetworkServiceTestResponseValidator;

class CONTENT_EXPORT NetworkServiceTest
    : public NetworkServiceTestInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = NetworkServiceTestProxy;

  template <typename ImplRefTraits>
  using Stub_ = NetworkServiceTestStub<ImplRefTraits>;

  using RequestValidator_ = NetworkServiceTestRequestValidator;
  using ResponseValidator_ = NetworkServiceTestResponseValidator;
  enum MethodMinVersions : uint32_t {
    kAddRulesMinVersion = 0,
  };
  virtual ~NetworkServiceTest() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool AddRules(std::vector<RulePtr> rules);

  using AddRulesCallback = base::OnceCallback<void()>;
  virtual void AddRules(std::vector<RulePtr> rules, AddRulesCallback callback) = 0;
};

class CONTENT_EXPORT NetworkServiceTestProxy
    : public NetworkServiceTest {
 public:
  explicit NetworkServiceTestProxy(mojo::MessageReceiverWithResponder* receiver);
  bool AddRules(std::vector<RulePtr> rules) override;
  void AddRules(std::vector<RulePtr> rules, AddRulesCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT NetworkServiceTestStubDispatch {
 public:
  static bool Accept(NetworkServiceTest* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NetworkServiceTest* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NetworkServiceTest>>
class NetworkServiceTestStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NetworkServiceTestStub() {}
  ~NetworkServiceTestStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkServiceTestStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkServiceTestStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT NetworkServiceTestRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT NetworkServiceTestResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};


class CONTENT_EXPORT Rule {
 public:
  using DataView = RuleDataView;
  using Data_ = internal::Rule_Data;

  template <typename... Args>
  static RulePtr New(Args&&... args) {
    return RulePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static RulePtr From(const U& u) {
    return mojo::TypeConverter<RulePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, Rule>::Convert(*this);
  }


  Rule();

  Rule(
      const std::string& host_pattern,
      const std::string& replacement);

  ~Rule();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = RulePtr>
  RulePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, Rule>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        Rule::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        Rule::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::string host_pattern;
  std::string replacement;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





template <typename StructPtrType>
RulePtr Rule::Clone() const {
  return New(
      mojo::Clone(host_pattern),
      mojo::Clone(replacement)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, Rule>::value>::type*>
bool Rule::Equals(const T& other) const {
  if (!mojo::Equals(this->host_pattern, other.host_pattern))
    return false;
  if (!mojo::Equals(this->replacement, other.replacement))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace content

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::Rule::DataView,
                                         ::content::mojom::RulePtr> {
  static bool IsNull(const ::content::mojom::RulePtr& input) { return !input; }
  static void SetToNull(::content::mojom::RulePtr* output) { output->reset(); }

  static const decltype(::content::mojom::Rule::host_pattern)& host_pattern(
      const ::content::mojom::RulePtr& input) {
    return input->host_pattern;
  }

  static const decltype(::content::mojom::Rule::replacement)& replacement(
      const ::content::mojom::RulePtr& input) {
    return input->replacement;
  }

  static bool Read(::content::mojom::Rule::DataView input, ::content::mojom::RulePtr* output);
};

}  // namespace mojo

#endif  // CONTENT_PUBLIC_COMMON_NETWORK_SERVICE_TEST_MOJOM_H_