// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef DEVICE_SERIAL_SERIAL_MOJOM_SHARED_H_
#define DEVICE_SERIAL_SERIAL_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "device/serial/serial.mojom-shared-internal.h"


namespace device {
namespace serial {
class DeviceInfoDataView;

class ConnectionOptionsDataView;

class ConnectionInfoDataView;

class HostControlSignalsDataView;

class DeviceControlSignalsDataView;



}  // namespace serial
}  // namespace device

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::device::serial::DeviceInfoDataView> {
  using Data = ::device::serial::internal::DeviceInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::device::serial::ConnectionOptionsDataView> {
  using Data = ::device::serial::internal::ConnectionOptions_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::device::serial::ConnectionInfoDataView> {
  using Data = ::device::serial::internal::ConnectionInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::device::serial::HostControlSignalsDataView> {
  using Data = ::device::serial::internal::HostControlSignals_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::device::serial::DeviceControlSignalsDataView> {
  using Data = ::device::serial::internal::DeviceControlSignals_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace device {
namespace serial {

enum class SendError : int32_t {
  NONE,
  DISCONNECTED,
  PENDING,
  TIMEOUT,
  SYSTEM_ERROR,
};

inline std::ostream& operator<<(std::ostream& os, SendError value) {
  switch(value) {
    case SendError::NONE:
      return os << "SendError::NONE";
    case SendError::DISCONNECTED:
      return os << "SendError::DISCONNECTED";
    case SendError::PENDING:
      return os << "SendError::PENDING";
    case SendError::TIMEOUT:
      return os << "SendError::TIMEOUT";
    case SendError::SYSTEM_ERROR:
      return os << "SendError::SYSTEM_ERROR";
    default:
      return os << "Unknown SendError value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(SendError value) {
  return internal::SendError_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class ReceiveError : int32_t {
  NONE,
  DISCONNECTED,
  TIMEOUT,
  DEVICE_LOST,
  BREAK,
  FRAME_ERROR,
  OVERRUN,
  BUFFER_OVERFLOW,
  PARITY_ERROR,
  SYSTEM_ERROR,
};

inline std::ostream& operator<<(std::ostream& os, ReceiveError value) {
  switch(value) {
    case ReceiveError::NONE:
      return os << "ReceiveError::NONE";
    case ReceiveError::DISCONNECTED:
      return os << "ReceiveError::DISCONNECTED";
    case ReceiveError::TIMEOUT:
      return os << "ReceiveError::TIMEOUT";
    case ReceiveError::DEVICE_LOST:
      return os << "ReceiveError::DEVICE_LOST";
    case ReceiveError::BREAK:
      return os << "ReceiveError::BREAK";
    case ReceiveError::FRAME_ERROR:
      return os << "ReceiveError::FRAME_ERROR";
    case ReceiveError::OVERRUN:
      return os << "ReceiveError::OVERRUN";
    case ReceiveError::BUFFER_OVERFLOW:
      return os << "ReceiveError::BUFFER_OVERFLOW";
    case ReceiveError::PARITY_ERROR:
      return os << "ReceiveError::PARITY_ERROR";
    case ReceiveError::SYSTEM_ERROR:
      return os << "ReceiveError::SYSTEM_ERROR";
    default:
      return os << "Unknown ReceiveError value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(ReceiveError value) {
  return internal::ReceiveError_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class DataBits : int32_t {
  NONE,
  SEVEN,
  EIGHT,
};

inline std::ostream& operator<<(std::ostream& os, DataBits value) {
  switch(value) {
    case DataBits::NONE:
      return os << "DataBits::NONE";
    case DataBits::SEVEN:
      return os << "DataBits::SEVEN";
    case DataBits::EIGHT:
      return os << "DataBits::EIGHT";
    default:
      return os << "Unknown DataBits value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(DataBits value) {
  return internal::DataBits_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class ParityBit : int32_t {
  NONE,
  NO_PARITY,
  ODD,
  EVEN,
};

inline std::ostream& operator<<(std::ostream& os, ParityBit value) {
  switch(value) {
    case ParityBit::NONE:
      return os << "ParityBit::NONE";
    case ParityBit::NO_PARITY:
      return os << "ParityBit::NO_PARITY";
    case ParityBit::ODD:
      return os << "ParityBit::ODD";
    case ParityBit::EVEN:
      return os << "ParityBit::EVEN";
    default:
      return os << "Unknown ParityBit value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(ParityBit value) {
  return internal::ParityBit_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class StopBits : int32_t {
  NONE,
  ONE,
  TWO,
};

inline std::ostream& operator<<(std::ostream& os, StopBits value) {
  switch(value) {
    case StopBits::NONE:
      return os << "StopBits::NONE";
    case StopBits::ONE:
      return os << "StopBits::ONE";
    case StopBits::TWO:
      return os << "StopBits::TWO";
    default:
      return os << "Unknown StopBits value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(StopBits value) {
  return internal::StopBits_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class DeviceInfoDataView {
 public:
  DeviceInfoDataView() {}

  DeviceInfoDataView(
      internal::DeviceInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPathDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPath(UserType* output) {
    auto* pointer = data_->path.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  uint16_t vendor_id() const {
    return data_->vendor_id;
  }
  bool has_vendor_id() const {
    return data_->has_vendor_id;
  }
  uint16_t product_id() const {
    return data_->product_id;
  }
  bool has_product_id() const {
    return data_->has_product_id;
  }
  inline void GetDisplayNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDisplayName(UserType* output) {
    auto* pointer = data_->display_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::DeviceInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ConnectionOptionsDataView {
 public:
  ConnectionOptionsDataView() {}

  ConnectionOptionsDataView(
      internal::ConnectionOptions_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint32_t bitrate() const {
    return data_->bitrate;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDataBits(UserType* output) const {
    auto data_value = data_->data_bits;
    return mojo::internal::Deserialize<::device::serial::DataBits>(
        data_value, output);
  }

  DataBits data_bits() const {
    return static_cast<DataBits>(data_->data_bits);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadParityBit(UserType* output) const {
    auto data_value = data_->parity_bit;
    return mojo::internal::Deserialize<::device::serial::ParityBit>(
        data_value, output);
  }

  ParityBit parity_bit() const {
    return static_cast<ParityBit>(data_->parity_bit);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStopBits(UserType* output) const {
    auto data_value = data_->stop_bits;
    return mojo::internal::Deserialize<::device::serial::StopBits>(
        data_value, output);
  }

  StopBits stop_bits() const {
    return static_cast<StopBits>(data_->stop_bits);
  }
  bool cts_flow_control() const {
    return data_->cts_flow_control;
  }
  bool has_cts_flow_control() const {
    return data_->has_cts_flow_control;
  }
 private:
  internal::ConnectionOptions_Data* data_ = nullptr;
};

class ConnectionInfoDataView {
 public:
  ConnectionInfoDataView() {}

  ConnectionInfoDataView(
      internal::ConnectionInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint32_t bitrate() const {
    return data_->bitrate;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDataBits(UserType* output) const {
    auto data_value = data_->data_bits;
    return mojo::internal::Deserialize<::device::serial::DataBits>(
        data_value, output);
  }

  DataBits data_bits() const {
    return static_cast<DataBits>(data_->data_bits);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadParityBit(UserType* output) const {
    auto data_value = data_->parity_bit;
    return mojo::internal::Deserialize<::device::serial::ParityBit>(
        data_value, output);
  }

  ParityBit parity_bit() const {
    return static_cast<ParityBit>(data_->parity_bit);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStopBits(UserType* output) const {
    auto data_value = data_->stop_bits;
    return mojo::internal::Deserialize<::device::serial::StopBits>(
        data_value, output);
  }

  StopBits stop_bits() const {
    return static_cast<StopBits>(data_->stop_bits);
  }
  bool cts_flow_control() const {
    return data_->cts_flow_control;
  }
 private:
  internal::ConnectionInfo_Data* data_ = nullptr;
};

class HostControlSignalsDataView {
 public:
  HostControlSignalsDataView() {}

  HostControlSignalsDataView(
      internal::HostControlSignals_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool dtr() const {
    return data_->dtr;
  }
  bool has_dtr() const {
    return data_->has_dtr;
  }
  bool rts() const {
    return data_->rts;
  }
  bool has_rts() const {
    return data_->has_rts;
  }
 private:
  internal::HostControlSignals_Data* data_ = nullptr;
};

class DeviceControlSignalsDataView {
 public:
  DeviceControlSignalsDataView() {}

  DeviceControlSignalsDataView(
      internal::DeviceControlSignals_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool dcd() const {
    return data_->dcd;
  }
  bool cts() const {
    return data_->cts;
  }
  bool ri() const {
    return data_->ri;
  }
  bool dsr() const {
    return data_->dsr;
  }
 private:
  internal::DeviceControlSignals_Data* data_ = nullptr;
};



}  // namespace serial
}  // namespace device

namespace std {

template <>
struct hash<::device::serial::SendError>
    : public mojo::internal::EnumHashImpl<::device::serial::SendError> {};

template <>
struct hash<::device::serial::ReceiveError>
    : public mojo::internal::EnumHashImpl<::device::serial::ReceiveError> {};

template <>
struct hash<::device::serial::DataBits>
    : public mojo::internal::EnumHashImpl<::device::serial::DataBits> {};

template <>
struct hash<::device::serial::ParityBit>
    : public mojo::internal::EnumHashImpl<::device::serial::ParityBit> {};

template <>
struct hash<::device::serial::StopBits>
    : public mojo::internal::EnumHashImpl<::device::serial::StopBits> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::device::serial::SendError, ::device::serial::SendError> {
  static ::device::serial::SendError ToMojom(::device::serial::SendError input) { return input; }
  static bool FromMojom(::device::serial::SendError input, ::device::serial::SendError* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::SendError, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::device::serial::SendError, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::device::serial::SendError>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::device::serial::ReceiveError, ::device::serial::ReceiveError> {
  static ::device::serial::ReceiveError ToMojom(::device::serial::ReceiveError input) { return input; }
  static bool FromMojom(::device::serial::ReceiveError input, ::device::serial::ReceiveError* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::ReceiveError, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::device::serial::ReceiveError, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::device::serial::ReceiveError>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::device::serial::DataBits, ::device::serial::DataBits> {
  static ::device::serial::DataBits ToMojom(::device::serial::DataBits input) { return input; }
  static bool FromMojom(::device::serial::DataBits input, ::device::serial::DataBits* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::DataBits, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::device::serial::DataBits, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::device::serial::DataBits>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::device::serial::ParityBit, ::device::serial::ParityBit> {
  static ::device::serial::ParityBit ToMojom(::device::serial::ParityBit input) { return input; }
  static bool FromMojom(::device::serial::ParityBit input, ::device::serial::ParityBit* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::ParityBit, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::device::serial::ParityBit, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::device::serial::ParityBit>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::device::serial::StopBits, ::device::serial::StopBits> {
  static ::device::serial::StopBits ToMojom(::device::serial::StopBits input) { return input; }
  static bool FromMojom(::device::serial::StopBits input, ::device::serial::StopBits* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::StopBits, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::device::serial::StopBits, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::device::serial::StopBits>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::DeviceInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::device::serial::DeviceInfoDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::device::serial::internal::DeviceInfo_Data);
    decltype(CallWithContext(Traits::path, input, custom_context)) in_path = CallWithContext(Traits::path, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_path, context);
    decltype(CallWithContext(Traits::display_name, input, custom_context)) in_display_name = CallWithContext(Traits::display_name, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_display_name, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::device::serial::internal::DeviceInfo_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::device::serial::internal::DeviceInfo_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::path, input, custom_context)) in_path = CallWithContext(Traits::path, input, custom_context);
    typename decltype(result->path)::BaseType* path_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_path, buffer, &path_ptr, context);
    result->path.Set(path_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->path.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null path in DeviceInfo struct");
    result->vendor_id = CallWithContext(Traits::vendor_id, input, custom_context);
    result->has_vendor_id = CallWithContext(Traits::has_vendor_id, input, custom_context);
    result->product_id = CallWithContext(Traits::product_id, input, custom_context);
    result->has_product_id = CallWithContext(Traits::has_product_id, input, custom_context);
    decltype(CallWithContext(Traits::display_name, input, custom_context)) in_display_name = CallWithContext(Traits::display_name, input, custom_context);
    typename decltype(result->display_name)::BaseType* display_name_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_display_name, buffer, &display_name_ptr, context);
    result->display_name.Set(display_name_ptr);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::device::serial::internal::DeviceInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::device::serial::DeviceInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::ConnectionOptionsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::device::serial::ConnectionOptionsDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::device::serial::internal::ConnectionOptions_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::device::serial::internal::ConnectionOptions_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::device::serial::internal::ConnectionOptions_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->bitrate = CallWithContext(Traits::bitrate, input, custom_context);
    mojo::internal::Serialize<::device::serial::DataBits>(
        CallWithContext(Traits::data_bits, input, custom_context), &result->data_bits);
    mojo::internal::Serialize<::device::serial::ParityBit>(
        CallWithContext(Traits::parity_bit, input, custom_context), &result->parity_bit);
    mojo::internal::Serialize<::device::serial::StopBits>(
        CallWithContext(Traits::stop_bits, input, custom_context), &result->stop_bits);
    result->cts_flow_control = CallWithContext(Traits::cts_flow_control, input, custom_context);
    result->has_cts_flow_control = CallWithContext(Traits::has_cts_flow_control, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::device::serial::internal::ConnectionOptions_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::device::serial::ConnectionOptionsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::ConnectionInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::device::serial::ConnectionInfoDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::device::serial::internal::ConnectionInfo_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::device::serial::internal::ConnectionInfo_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::device::serial::internal::ConnectionInfo_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->bitrate = CallWithContext(Traits::bitrate, input, custom_context);
    mojo::internal::Serialize<::device::serial::DataBits>(
        CallWithContext(Traits::data_bits, input, custom_context), &result->data_bits);
    mojo::internal::Serialize<::device::serial::ParityBit>(
        CallWithContext(Traits::parity_bit, input, custom_context), &result->parity_bit);
    mojo::internal::Serialize<::device::serial::StopBits>(
        CallWithContext(Traits::stop_bits, input, custom_context), &result->stop_bits);
    result->cts_flow_control = CallWithContext(Traits::cts_flow_control, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::device::serial::internal::ConnectionInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::device::serial::ConnectionInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::HostControlSignalsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::device::serial::HostControlSignalsDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::device::serial::internal::HostControlSignals_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::device::serial::internal::HostControlSignals_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::device::serial::internal::HostControlSignals_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->dtr = CallWithContext(Traits::dtr, input, custom_context);
    result->has_dtr = CallWithContext(Traits::has_dtr, input, custom_context);
    result->rts = CallWithContext(Traits::rts, input, custom_context);
    result->has_rts = CallWithContext(Traits::has_rts, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::device::serial::internal::HostControlSignals_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::device::serial::HostControlSignalsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::device::serial::DeviceControlSignalsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::device::serial::DeviceControlSignalsDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::device::serial::internal::DeviceControlSignals_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::device::serial::internal::DeviceControlSignals_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::device::serial::internal::DeviceControlSignals_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->dcd = CallWithContext(Traits::dcd, input, custom_context);
    result->cts = CallWithContext(Traits::cts, input, custom_context);
    result->ri = CallWithContext(Traits::ri, input, custom_context);
    result->dsr = CallWithContext(Traits::dsr, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::device::serial::internal::DeviceControlSignals_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::device::serial::DeviceControlSignalsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace device {
namespace serial {

inline void DeviceInfoDataView::GetPathDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->path.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void DeviceInfoDataView::GetDisplayNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->display_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}











}  // namespace serial
}  // namespace device

#endif  // DEVICE_SERIAL_SERIAL_MOJOM_SHARED_H_
