// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "device/vr/vr_service.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "gpu/ipc/common/mailbox_holder_struct_traits.h"
#include "gpu/ipc/common/mailbox_struct_traits.h"
#include "gpu/ipc/common/sync_token_struct_traits.h"
#include "ipc/ipc_message_utils.h"
#include "mojo/common/common_custom_types_struct_traits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace device {
namespace mojom {
VRFieldOfView::VRFieldOfView()
    : upDegrees(),
      downDegrees(),
      leftDegrees(),
      rightDegrees() {}

VRFieldOfView::VRFieldOfView(
    float upDegrees_in,
    float downDegrees_in,
    float leftDegrees_in,
    float rightDegrees_in)
    : upDegrees(std::move(upDegrees_in)),
      downDegrees(std::move(downDegrees_in)),
      leftDegrees(std::move(leftDegrees_in)),
      rightDegrees(std::move(rightDegrees_in)) {}

VRFieldOfView::~VRFieldOfView() = default;
size_t VRFieldOfView::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->upDegrees);
  seed = mojo::internal::Hash(seed, this->downDegrees);
  seed = mojo::internal::Hash(seed, this->leftDegrees);
  seed = mojo::internal::Hash(seed, this->rightDegrees);
  return seed;
}

bool VRFieldOfView::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
VRPose::VRPose()
    : orientation(),
      position(),
      angularVelocity(),
      linearVelocity(),
      angularAcceleration(),
      linearAcceleration() {}

VRPose::VRPose(
    const base::Optional<std::vector<float>>& orientation_in,
    const base::Optional<std::vector<float>>& position_in,
    const base::Optional<std::vector<float>>& angularVelocity_in,
    const base::Optional<std::vector<float>>& linearVelocity_in,
    const base::Optional<std::vector<float>>& angularAcceleration_in,
    const base::Optional<std::vector<float>>& linearAcceleration_in)
    : orientation(std::move(orientation_in)),
      position(std::move(position_in)),
      angularVelocity(std::move(angularVelocity_in)),
      linearVelocity(std::move(linearVelocity_in)),
      angularAcceleration(std::move(angularAcceleration_in)),
      linearAcceleration(std::move(linearAcceleration_in)) {}

VRPose::~VRPose() = default;

bool VRPose::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
VRDisplayCapabilities::VRDisplayCapabilities()
    : hasPosition(),
      hasExternalDisplay(),
      canPresent() {}

VRDisplayCapabilities::VRDisplayCapabilities(
    bool hasPosition_in,
    bool hasExternalDisplay_in,
    bool canPresent_in)
    : hasPosition(std::move(hasPosition_in)),
      hasExternalDisplay(std::move(hasExternalDisplay_in)),
      canPresent(std::move(canPresent_in)) {}

VRDisplayCapabilities::~VRDisplayCapabilities() = default;
size_t VRDisplayCapabilities::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->hasPosition);
  seed = mojo::internal::Hash(seed, this->hasExternalDisplay);
  seed = mojo::internal::Hash(seed, this->canPresent);
  return seed;
}

bool VRDisplayCapabilities::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
VREyeParameters::VREyeParameters()
    : fieldOfView(),
      offset(),
      renderWidth(),
      renderHeight() {}

VREyeParameters::VREyeParameters(
    VRFieldOfViewPtr fieldOfView_in,
    const std::vector<float>& offset_in,
    uint32_t renderWidth_in,
    uint32_t renderHeight_in)
    : fieldOfView(std::move(fieldOfView_in)),
      offset(std::move(offset_in)),
      renderWidth(std::move(renderWidth_in)),
      renderHeight(std::move(renderHeight_in)) {}

VREyeParameters::~VREyeParameters() = default;

bool VREyeParameters::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
VRStageParameters::VRStageParameters()
    : standingTransform(),
      sizeX(),
      sizeZ() {}

VRStageParameters::VRStageParameters(
    const std::vector<float>& standingTransform_in,
    float sizeX_in,
    float sizeZ_in)
    : standingTransform(std::move(standingTransform_in)),
      sizeX(std::move(sizeX_in)),
      sizeZ(std::move(sizeZ_in)) {}

VRStageParameters::~VRStageParameters() = default;

bool VRStageParameters::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
VRDisplayInfo::VRDisplayInfo()
    : index(),
      displayName(),
      capabilities(),
      stageParameters(),
      leftEye(),
      rightEye() {}

VRDisplayInfo::VRDisplayInfo(
    uint32_t index_in,
    const std::string& displayName_in,
    VRDisplayCapabilitiesPtr capabilities_in,
    VRStageParametersPtr stageParameters_in,
    VREyeParametersPtr leftEye_in,
    VREyeParametersPtr rightEye_in)
    : index(std::move(index_in)),
      displayName(std::move(displayName_in)),
      capabilities(std::move(capabilities_in)),
      stageParameters(std::move(stageParameters_in)),
      leftEye(std::move(leftEye_in)),
      rightEye(std::move(rightEye_in)) {}

VRDisplayInfo::~VRDisplayInfo() = default;

bool VRDisplayInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char VRService::Name_[] = "device::mojom::VRService";

class VRService_SetClient_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  VRService_SetClient_ForwardToCallback(
      VRService::SetClientCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  VRService::SetClientCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(VRService_SetClient_ForwardToCallback);
};

VRServiceProxy::VRServiceProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void VRServiceProxy::SetClient(
    VRServiceClientPtr in_client, SetClientCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRService_SetClient_Params_Data);
  size += mojo::internal::PrepareToSerialize<::device::mojom::VRServiceClientPtrDataView>(
      in_client, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kVRService_SetClient_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRService_SetClient_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::device::mojom::VRServiceClientPtrDataView>(
      in_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in VRService.SetClient request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new VRService_SetClient_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void VRServiceProxy::SetListeningForActivate(
    bool in_listening) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRService_SetListeningForActivate_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRService_SetListeningForActivate_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRService_SetListeningForActivate_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->listening = in_listening;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class VRService_SetClient_ProxyToResponder {
 public:
  static VRService::SetClientCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<VRService_SetClient_ProxyToResponder> proxy(
        new VRService_SetClient_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&VRService_SetClient_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~VRService_SetClient_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "VRService::SetClient() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  VRService_SetClient_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      uint32_t in_number_of_connected_devices);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(VRService_SetClient_ProxyToResponder);
};

bool VRService_SetClient_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::VRService_SetClient_ResponseParams_Data* params =
      reinterpret_cast<
          internal::VRService_SetClient_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  uint32_t p_number_of_connected_devices{};
  VRService_SetClient_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_number_of_connected_devices = input_data_view.number_of_connected_devices();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "VRService::SetClient response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_number_of_connected_devices));
  return true;
}

void VRService_SetClient_ProxyToResponder::Run(
    uint32_t in_number_of_connected_devices) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRService_SetClient_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kVRService_SetClient_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRService_SetClient_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->number_of_connected_devices = in_number_of_connected_devices;

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool VRServiceStubDispatch::Accept(
    VRService* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kVRService_SetClient_Name: {
      break;
    }
    case internal::kVRService_SetListeningForActivate_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRService_SetListeningForActivate_Params_Data* params =
          reinterpret_cast<internal::VRService_SetListeningForActivate_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_listening{};
      VRService_SetListeningForActivate_ParamsDataView input_data_view(params, &serialization_context);
      
      p_listening = input_data_view.listening();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRService::SetListeningForActivate deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetListeningForActivate(
std::move(p_listening));
      return true;
    }
  }
  return false;
}

// static
bool VRServiceStubDispatch::AcceptWithResponder(
    VRService* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kVRService_SetClient_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::VRService_SetClient_Params_Data* params =
          reinterpret_cast<
              internal::VRService_SetClient_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRServiceClientPtr p_client{};
      VRService_SetClient_ParamsDataView input_data_view(params, &serialization_context);
      
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRService::SetClient deserializer");
        return false;
      }
      VRService::SetClientCallback callback =
          VRService_SetClient_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetClient(
std::move(p_client), std::move(callback));
      return true;
    }
    case internal::kVRService_SetListeningForActivate_Name: {
      break;
    }
  }
  return false;
}

bool VRServiceRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRService RequestValidator");

  switch (message->header()->name) {
    case internal::kVRService_SetClient_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRService_SetClient_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRService_SetListeningForActivate_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRService_SetListeningForActivate_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool VRServiceResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRService ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kVRService_SetClient_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRService_SetClient_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char VRServiceClient::Name_[] = "device::mojom::VRServiceClient";

VRServiceClientProxy::VRServiceClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void VRServiceClientProxy::OnDisplayConnected(
    VRDisplayPtr in_display, VRDisplayClientRequest in_request, VRDisplayInfoPtr in_display_info) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRServiceClient_OnDisplayConnected_Params_Data);
  size += mojo::internal::PrepareToSerialize<::device::mojom::VRDisplayPtrDataView>(
      in_display, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::device::mojom::VRDisplayClientRequestDataView>(
      in_request, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::device::mojom::VRDisplayInfoDataView>(
      in_display_info, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kVRServiceClient_OnDisplayConnected_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRServiceClient_OnDisplayConnected_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::device::mojom::VRDisplayPtrDataView>(
      in_display, &params->display, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->display),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid display in VRServiceClient.OnDisplayConnected request");
  mojo::internal::Serialize<::device::mojom::VRDisplayClientRequestDataView>(
      in_request, &params->request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in VRServiceClient.OnDisplayConnected request");
  typename decltype(params->display_info)::BaseType* display_info_ptr;
  mojo::internal::Serialize<::device::mojom::VRDisplayInfoDataView>(
      in_display_info, buffer, &display_info_ptr, &serialization_context);
  params->display_info.Set(display_info_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->display_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null display_info in VRServiceClient.OnDisplayConnected request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool VRServiceClientStubDispatch::Accept(
    VRServiceClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kVRServiceClient_OnDisplayConnected_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRServiceClient_OnDisplayConnected_Params_Data* params =
          reinterpret_cast<internal::VRServiceClient_OnDisplayConnected_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRDisplayPtr p_display{};
      VRDisplayClientRequest p_request{};
      VRDisplayInfoPtr p_display_info{};
      VRServiceClient_OnDisplayConnected_ParamsDataView input_data_view(params, &serialization_context);
      
      p_display =
          input_data_view.TakeDisplay<decltype(p_display)>();
      p_request =
          input_data_view.TakeRequest<decltype(p_request)>();
      if (!input_data_view.ReadDisplayInfo(&p_display_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRServiceClient::OnDisplayConnected deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDisplayConnected(
std::move(p_display), 
std::move(p_request), 
std::move(p_display_info));
      return true;
    }
  }
  return false;
}

// static
bool VRServiceClientStubDispatch::AcceptWithResponder(
    VRServiceClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kVRServiceClient_OnDisplayConnected_Name: {
      break;
    }
  }
  return false;
}

bool VRServiceClientRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRServiceClient RequestValidator");

  switch (message->header()->name) {
    case internal::kVRServiceClient_OnDisplayConnected_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRServiceClient_OnDisplayConnected_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

const char VRSubmitFrameClient::Name_[] = "device::mojom::VRSubmitFrameClient";

VRSubmitFrameClientProxy::VRSubmitFrameClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void VRSubmitFrameClientProxy::OnSubmitFrameTransferred(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRSubmitFrameClient_OnSubmitFrameTransferred_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void VRSubmitFrameClientProxy::OnSubmitFrameRendered(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRSubmitFrameClient_OnSubmitFrameRendered_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool VRSubmitFrameClientStubDispatch::Accept(
    VRSubmitFrameClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kVRSubmitFrameClient_OnSubmitFrameTransferred_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data* params =
          reinterpret_cast<internal::VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRSubmitFrameClient_OnSubmitFrameTransferred_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRSubmitFrameClient::OnSubmitFrameTransferred deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnSubmitFrameTransferred();
      return true;
    }
    case internal::kVRSubmitFrameClient_OnSubmitFrameRendered_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data* params =
          reinterpret_cast<internal::VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRSubmitFrameClient_OnSubmitFrameRendered_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRSubmitFrameClient::OnSubmitFrameRendered deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnSubmitFrameRendered();
      return true;
    }
  }
  return false;
}

// static
bool VRSubmitFrameClientStubDispatch::AcceptWithResponder(
    VRSubmitFrameClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kVRSubmitFrameClient_OnSubmitFrameTransferred_Name: {
      break;
    }
    case internal::kVRSubmitFrameClient_OnSubmitFrameRendered_Name: {
      break;
    }
  }
  return false;
}

bool VRSubmitFrameClientRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRSubmitFrameClient RequestValidator");

  switch (message->header()->name) {
    case internal::kVRSubmitFrameClient_OnSubmitFrameTransferred_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRSubmitFrameClient_OnSubmitFrameRendered_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

const char VRDisplay::Name_[] = "device::mojom::VRDisplay";

class VRDisplay_RequestPresent_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  VRDisplay_RequestPresent_ForwardToCallback(
      VRDisplay::RequestPresentCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  VRDisplay::RequestPresentCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(VRDisplay_RequestPresent_ForwardToCallback);
};

class VRDisplay_GetNextMagicWindowPose_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  VRDisplay_GetNextMagicWindowPose_ForwardToCallback(
      VRDisplay::GetNextMagicWindowPoseCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  VRDisplay::GetNextMagicWindowPoseCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(VRDisplay_GetNextMagicWindowPose_ForwardToCallback);
};

VRDisplayProxy::VRDisplayProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void VRDisplayProxy::RequestPresent(
    bool in_secure_origin, VRSubmitFrameClientPtr in_client, VRPresentationProviderRequest in_request, RequestPresentCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplay_RequestPresent_Params_Data);
  size += mojo::internal::PrepareToSerialize<::device::mojom::VRSubmitFrameClientPtrDataView>(
      in_client, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::device::mojom::VRPresentationProviderRequestDataView>(
      in_request, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kVRDisplay_RequestPresent_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplay_RequestPresent_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->secure_origin = in_secure_origin;
  mojo::internal::Serialize<::device::mojom::VRSubmitFrameClientPtrDataView>(
      in_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in VRDisplay.RequestPresent request");
  mojo::internal::Serialize<::device::mojom::VRPresentationProviderRequestDataView>(
      in_request, &params->request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in VRDisplay.RequestPresent request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new VRDisplay_RequestPresent_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void VRDisplayProxy::ExitPresent(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplay_ExitPresent_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRDisplay_ExitPresent_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplay_ExitPresent_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void VRDisplayProxy::GetNextMagicWindowPose(
    GetNextMagicWindowPoseCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplay_GetNextMagicWindowPose_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRDisplay_GetNextMagicWindowPose_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplay_GetNextMagicWindowPose_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new VRDisplay_GetNextMagicWindowPose_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class VRDisplay_RequestPresent_ProxyToResponder {
 public:
  static VRDisplay::RequestPresentCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<VRDisplay_RequestPresent_ProxyToResponder> proxy(
        new VRDisplay_RequestPresent_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&VRDisplay_RequestPresent_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~VRDisplay_RequestPresent_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "VRDisplay::RequestPresent() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  VRDisplay_RequestPresent_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      bool in_success);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(VRDisplay_RequestPresent_ProxyToResponder);
};

bool VRDisplay_RequestPresent_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::VRDisplay_RequestPresent_ResponseParams_Data* params =
      reinterpret_cast<
          internal::VRDisplay_RequestPresent_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_success{};
  VRDisplay_RequestPresent_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_success = input_data_view.success();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "VRDisplay::RequestPresent response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_success));
  return true;
}

void VRDisplay_RequestPresent_ProxyToResponder::Run(
    bool in_success) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplay_RequestPresent_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kVRDisplay_RequestPresent_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplay_RequestPresent_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->success = in_success;

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class VRDisplay_GetNextMagicWindowPose_ProxyToResponder {
 public:
  static VRDisplay::GetNextMagicWindowPoseCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<VRDisplay_GetNextMagicWindowPose_ProxyToResponder> proxy(
        new VRDisplay_GetNextMagicWindowPose_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&VRDisplay_GetNextMagicWindowPose_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~VRDisplay_GetNextMagicWindowPose_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "VRDisplay::GetNextMagicWindowPose() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  VRDisplay_GetNextMagicWindowPose_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      VRPosePtr in_pose);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(VRDisplay_GetNextMagicWindowPose_ProxyToResponder);
};

bool VRDisplay_GetNextMagicWindowPose_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::VRDisplay_GetNextMagicWindowPose_ResponseParams_Data* params =
      reinterpret_cast<
          internal::VRDisplay_GetNextMagicWindowPose_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  VRPosePtr p_pose{};
  VRDisplay_GetNextMagicWindowPose_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadPose(&p_pose))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "VRDisplay::GetNextMagicWindowPose response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_pose));
  return true;
}

void VRDisplay_GetNextMagicWindowPose_ProxyToResponder::Run(
    VRPosePtr in_pose) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplay_GetNextMagicWindowPose_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::device::mojom::VRPoseDataView>(
      in_pose, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kVRDisplay_GetNextMagicWindowPose_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplay_GetNextMagicWindowPose_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->pose)::BaseType* pose_ptr;
  mojo::internal::Serialize<::device::mojom::VRPoseDataView>(
      in_pose, buffer, &pose_ptr, &serialization_context);
  params->pose.Set(pose_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool VRDisplayStubDispatch::Accept(
    VRDisplay* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kVRDisplay_RequestPresent_Name: {
      break;
    }
    case internal::kVRDisplay_ExitPresent_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRDisplay_ExitPresent_Params_Data* params =
          reinterpret_cast<internal::VRDisplay_ExitPresent_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRDisplay_ExitPresent_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRDisplay::ExitPresent deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ExitPresent();
      return true;
    }
    case internal::kVRDisplay_GetNextMagicWindowPose_Name: {
      break;
    }
  }
  return false;
}

// static
bool VRDisplayStubDispatch::AcceptWithResponder(
    VRDisplay* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kVRDisplay_RequestPresent_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::VRDisplay_RequestPresent_Params_Data* params =
          reinterpret_cast<
              internal::VRDisplay_RequestPresent_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_secure_origin{};
      VRSubmitFrameClientPtr p_client{};
      VRPresentationProviderRequest p_request{};
      VRDisplay_RequestPresent_ParamsDataView input_data_view(params, &serialization_context);
      
      p_secure_origin = input_data_view.secure_origin();
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      p_request =
          input_data_view.TakeRequest<decltype(p_request)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRDisplay::RequestPresent deserializer");
        return false;
      }
      VRDisplay::RequestPresentCallback callback =
          VRDisplay_RequestPresent_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RequestPresent(
std::move(p_secure_origin), 
std::move(p_client), 
std::move(p_request), std::move(callback));
      return true;
    }
    case internal::kVRDisplay_ExitPresent_Name: {
      break;
    }
    case internal::kVRDisplay_GetNextMagicWindowPose_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::VRDisplay_GetNextMagicWindowPose_Params_Data* params =
          reinterpret_cast<
              internal::VRDisplay_GetNextMagicWindowPose_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRDisplay_GetNextMagicWindowPose_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRDisplay::GetNextMagicWindowPose deserializer");
        return false;
      }
      VRDisplay::GetNextMagicWindowPoseCallback callback =
          VRDisplay_GetNextMagicWindowPose_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetNextMagicWindowPose(std::move(callback));
      return true;
    }
  }
  return false;
}

bool VRDisplayRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRDisplay RequestValidator");

  switch (message->header()->name) {
    case internal::kVRDisplay_RequestPresent_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplay_RequestPresent_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRDisplay_ExitPresent_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplay_ExitPresent_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRDisplay_GetNextMagicWindowPose_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplay_GetNextMagicWindowPose_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool VRDisplayResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRDisplay ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kVRDisplay_RequestPresent_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplay_RequestPresent_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRDisplay_GetNextMagicWindowPose_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplay_GetNextMagicWindowPose_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char VRPresentationProvider::Name_[] = "device::mojom::VRPresentationProvider";

class VRPresentationProvider_GetVSync_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  VRPresentationProvider_GetVSync_ForwardToCallback(
      VRPresentationProvider::GetVSyncCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  VRPresentationProvider::GetVSyncCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(VRPresentationProvider_GetVSync_ForwardToCallback);
};

VRPresentationProviderProxy::VRPresentationProviderProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void VRPresentationProviderProxy::GetVSync(
    GetVSyncCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRPresentationProvider_GetVSync_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRPresentationProvider_GetVSync_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRPresentationProvider_GetVSync_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new VRPresentationProvider_GetVSync_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void VRPresentationProviderProxy::SubmitFrame(
    int16_t in_frame_id, const ::gpu::MailboxHolder& in_mailbox_holder) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRPresentationProvider_SubmitFrame_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gpu::mojom::MailboxHolderDataView>(
      in_mailbox_holder, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kVRPresentationProvider_SubmitFrame_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRPresentationProvider_SubmitFrame_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->frame_id = in_frame_id;
  typename decltype(params->mailbox_holder)::BaseType* mailbox_holder_ptr;
  mojo::internal::Serialize<::gpu::mojom::MailboxHolderDataView>(
      in_mailbox_holder, buffer, &mailbox_holder_ptr, &serialization_context);
  params->mailbox_holder.Set(mailbox_holder_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->mailbox_holder.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null mailbox_holder in VRPresentationProvider.SubmitFrame request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void VRPresentationProviderProxy::UpdateLayerBounds(
    int16_t in_frame_id, const gfx::RectF& in_left_bounds, const gfx::RectF& in_right_bounds, const gfx::Size& in_source_size) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRPresentationProvider_UpdateLayerBounds_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectFDataView>(
      in_left_bounds, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectFDataView>(
      in_right_bounds, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::SizeDataView>(
      in_source_size, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kVRPresentationProvider_UpdateLayerBounds_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRPresentationProvider_UpdateLayerBounds_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->frame_id = in_frame_id;
  typename decltype(params->left_bounds)::BaseType* left_bounds_ptr;
  mojo::internal::Serialize<::gfx::mojom::RectFDataView>(
      in_left_bounds, buffer, &left_bounds_ptr, &serialization_context);
  params->left_bounds.Set(left_bounds_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->left_bounds.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null left_bounds in VRPresentationProvider.UpdateLayerBounds request");
  typename decltype(params->right_bounds)::BaseType* right_bounds_ptr;
  mojo::internal::Serialize<::gfx::mojom::RectFDataView>(
      in_right_bounds, buffer, &right_bounds_ptr, &serialization_context);
  params->right_bounds.Set(right_bounds_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->right_bounds.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null right_bounds in VRPresentationProvider.UpdateLayerBounds request");
  typename decltype(params->source_size)::BaseType* source_size_ptr;
  mojo::internal::Serialize<::gfx::mojom::SizeDataView>(
      in_source_size, buffer, &source_size_ptr, &serialization_context);
  params->source_size.Set(source_size_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->source_size.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null source_size in VRPresentationProvider.UpdateLayerBounds request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class VRPresentationProvider_GetVSync_ProxyToResponder {
 public:
  static VRPresentationProvider::GetVSyncCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<VRPresentationProvider_GetVSync_ProxyToResponder> proxy(
        new VRPresentationProvider_GetVSync_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&VRPresentationProvider_GetVSync_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~VRPresentationProvider_GetVSync_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "VRPresentationProvider::GetVSync() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  VRPresentationProvider_GetVSync_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      VRPosePtr in_pose, base::TimeDelta in_time, int16_t in_frame_id, VRPresentationProvider::VSyncStatus in_status);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(VRPresentationProvider_GetVSync_ProxyToResponder);
};

bool VRPresentationProvider_GetVSync_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::VRPresentationProvider_GetVSync_ResponseParams_Data* params =
      reinterpret_cast<
          internal::VRPresentationProvider_GetVSync_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  VRPosePtr p_pose{};
  base::TimeDelta p_time{};
  int16_t p_frame_id{};
  VRPresentationProvider::VSyncStatus p_status{};
  VRPresentationProvider_GetVSync_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadPose(&p_pose))
    success = false;
  if (!input_data_view.ReadTime(&p_time))
    success = false;
  p_frame_id = input_data_view.frame_id();
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "VRPresentationProvider::GetVSync response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_pose), 
std::move(p_time), 
std::move(p_frame_id), 
std::move(p_status));
  return true;
}

void VRPresentationProvider_GetVSync_ProxyToResponder::Run(
    VRPosePtr in_pose, base::TimeDelta in_time, int16_t in_frame_id, VRPresentationProvider::VSyncStatus in_status) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRPresentationProvider_GetVSync_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::device::mojom::VRPoseDataView>(
      in_pose, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::TimeDeltaDataView>(
      in_time, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kVRPresentationProvider_GetVSync_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRPresentationProvider_GetVSync_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->pose)::BaseType* pose_ptr;
  mojo::internal::Serialize<::device::mojom::VRPoseDataView>(
      in_pose, buffer, &pose_ptr, &serialization_context);
  params->pose.Set(pose_ptr);
  typename decltype(params->time)::BaseType* time_ptr;
  mojo::internal::Serialize<::mojo::common::mojom::TimeDeltaDataView>(
      in_time, buffer, &time_ptr, &serialization_context);
  params->time.Set(time_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null time in ");
  params->frame_id = in_frame_id;
  mojo::internal::Serialize<::device::mojom::VRPresentationProvider_VSyncStatus>(
      in_status, &params->status);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool VRPresentationProviderStubDispatch::Accept(
    VRPresentationProvider* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kVRPresentationProvider_GetVSync_Name: {
      break;
    }
    case internal::kVRPresentationProvider_SubmitFrame_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRPresentationProvider_SubmitFrame_Params_Data* params =
          reinterpret_cast<internal::VRPresentationProvider_SubmitFrame_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int16_t p_frame_id{};
      ::gpu::MailboxHolder p_mailbox_holder{};
      VRPresentationProvider_SubmitFrame_ParamsDataView input_data_view(params, &serialization_context);
      
      p_frame_id = input_data_view.frame_id();
      if (!input_data_view.ReadMailboxHolder(&p_mailbox_holder))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRPresentationProvider::SubmitFrame deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SubmitFrame(
std::move(p_frame_id), 
std::move(p_mailbox_holder));
      return true;
    }
    case internal::kVRPresentationProvider_UpdateLayerBounds_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRPresentationProvider_UpdateLayerBounds_Params_Data* params =
          reinterpret_cast<internal::VRPresentationProvider_UpdateLayerBounds_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int16_t p_frame_id{};
      gfx::RectF p_left_bounds{};
      gfx::RectF p_right_bounds{};
      gfx::Size p_source_size{};
      VRPresentationProvider_UpdateLayerBounds_ParamsDataView input_data_view(params, &serialization_context);
      
      p_frame_id = input_data_view.frame_id();
      if (!input_data_view.ReadLeftBounds(&p_left_bounds))
        success = false;
      if (!input_data_view.ReadRightBounds(&p_right_bounds))
        success = false;
      if (!input_data_view.ReadSourceSize(&p_source_size))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRPresentationProvider::UpdateLayerBounds deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->UpdateLayerBounds(
std::move(p_frame_id), 
std::move(p_left_bounds), 
std::move(p_right_bounds), 
std::move(p_source_size));
      return true;
    }
  }
  return false;
}

// static
bool VRPresentationProviderStubDispatch::AcceptWithResponder(
    VRPresentationProvider* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kVRPresentationProvider_GetVSync_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::VRPresentationProvider_GetVSync_Params_Data* params =
          reinterpret_cast<
              internal::VRPresentationProvider_GetVSync_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRPresentationProvider_GetVSync_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRPresentationProvider::GetVSync deserializer");
        return false;
      }
      VRPresentationProvider::GetVSyncCallback callback =
          VRPresentationProvider_GetVSync_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetVSync(std::move(callback));
      return true;
    }
    case internal::kVRPresentationProvider_SubmitFrame_Name: {
      break;
    }
    case internal::kVRPresentationProvider_UpdateLayerBounds_Name: {
      break;
    }
  }
  return false;
}

bool VRPresentationProviderRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRPresentationProvider RequestValidator");

  switch (message->header()->name) {
    case internal::kVRPresentationProvider_GetVSync_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRPresentationProvider_GetVSync_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRPresentationProvider_SubmitFrame_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRPresentationProvider_SubmitFrame_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRPresentationProvider_UpdateLayerBounds_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRPresentationProvider_UpdateLayerBounds_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool VRPresentationProviderResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRPresentationProvider ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kVRPresentationProvider_GetVSync_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRPresentationProvider_GetVSync_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char VRDisplayClient::Name_[] = "device::mojom::VRDisplayClient";

class VRDisplayClient_OnActivate_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  VRDisplayClient_OnActivate_ForwardToCallback(
      VRDisplayClient::OnActivateCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  VRDisplayClient::OnActivateCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(VRDisplayClient_OnActivate_ForwardToCallback);
};

VRDisplayClientProxy::VRDisplayClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void VRDisplayClientProxy::OnChanged(
    VRDisplayInfoPtr in_display) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplayClient_OnChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<::device::mojom::VRDisplayInfoDataView>(
      in_display, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kVRDisplayClient_OnChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplayClient_OnChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->display)::BaseType* display_ptr;
  mojo::internal::Serialize<::device::mojom::VRDisplayInfoDataView>(
      in_display, buffer, &display_ptr, &serialization_context);
  params->display.Set(display_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->display.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null display in VRDisplayClient.OnChanged request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void VRDisplayClientProxy::OnExitPresent(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplayClient_OnExitPresent_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRDisplayClient_OnExitPresent_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplayClient_OnExitPresent_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void VRDisplayClientProxy::OnBlur(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplayClient_OnBlur_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRDisplayClient_OnBlur_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplayClient_OnBlur_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void VRDisplayClientProxy::OnFocus(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplayClient_OnFocus_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRDisplayClient_OnFocus_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplayClient_OnFocus_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void VRDisplayClientProxy::OnActivate(
    VRDisplayEventReason in_reason, OnActivateCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplayClient_OnActivate_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRDisplayClient_OnActivate_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplayClient_OnActivate_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::device::mojom::VRDisplayEventReason>(
      in_reason, &params->reason);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new VRDisplayClient_OnActivate_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void VRDisplayClientProxy::OnDeactivate(
    VRDisplayEventReason in_reason) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplayClient_OnDeactivate_Params_Data);
  serialization_context.PrepareMessage(
      internal::kVRDisplayClient_OnDeactivate_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplayClient_OnDeactivate_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::device::mojom::VRDisplayEventReason>(
      in_reason, &params->reason);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class VRDisplayClient_OnActivate_ProxyToResponder {
 public:
  static VRDisplayClient::OnActivateCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<VRDisplayClient_OnActivate_ProxyToResponder> proxy(
        new VRDisplayClient_OnActivate_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&VRDisplayClient_OnActivate_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~VRDisplayClient_OnActivate_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "VRDisplayClient::OnActivate() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  VRDisplayClient_OnActivate_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      bool in_will_not_present);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(VRDisplayClient_OnActivate_ProxyToResponder);
};

bool VRDisplayClient_OnActivate_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::VRDisplayClient_OnActivate_ResponseParams_Data* params =
      reinterpret_cast<
          internal::VRDisplayClient_OnActivate_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_will_not_present{};
  VRDisplayClient_OnActivate_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_will_not_present = input_data_view.will_not_present();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "VRDisplayClient::OnActivate response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_will_not_present));
  return true;
}

void VRDisplayClient_OnActivate_ProxyToResponder::Run(
    bool in_will_not_present) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::VRDisplayClient_OnActivate_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kVRDisplayClient_OnActivate_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::VRDisplayClient_OnActivate_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->will_not_present = in_will_not_present;

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool VRDisplayClientStubDispatch::Accept(
    VRDisplayClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kVRDisplayClient_OnChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRDisplayClient_OnChanged_Params_Data* params =
          reinterpret_cast<internal::VRDisplayClient_OnChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRDisplayInfoPtr p_display{};
      VRDisplayClient_OnChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadDisplay(&p_display))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRDisplayClient::OnChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnChanged(
std::move(p_display));
      return true;
    }
    case internal::kVRDisplayClient_OnExitPresent_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRDisplayClient_OnExitPresent_Params_Data* params =
          reinterpret_cast<internal::VRDisplayClient_OnExitPresent_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRDisplayClient_OnExitPresent_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRDisplayClient::OnExitPresent deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnExitPresent();
      return true;
    }
    case internal::kVRDisplayClient_OnBlur_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRDisplayClient_OnBlur_Params_Data* params =
          reinterpret_cast<internal::VRDisplayClient_OnBlur_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRDisplayClient_OnBlur_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRDisplayClient::OnBlur deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnBlur();
      return true;
    }
    case internal::kVRDisplayClient_OnFocus_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRDisplayClient_OnFocus_Params_Data* params =
          reinterpret_cast<internal::VRDisplayClient_OnFocus_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRDisplayClient_OnFocus_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRDisplayClient::OnFocus deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnFocus();
      return true;
    }
    case internal::kVRDisplayClient_OnActivate_Name: {
      break;
    }
    case internal::kVRDisplayClient_OnDeactivate_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VRDisplayClient_OnDeactivate_Params_Data* params =
          reinterpret_cast<internal::VRDisplayClient_OnDeactivate_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRDisplayEventReason p_reason{};
      VRDisplayClient_OnDeactivate_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadReason(&p_reason))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRDisplayClient::OnDeactivate deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDeactivate(
std::move(p_reason));
      return true;
    }
  }
  return false;
}

// static
bool VRDisplayClientStubDispatch::AcceptWithResponder(
    VRDisplayClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kVRDisplayClient_OnChanged_Name: {
      break;
    }
    case internal::kVRDisplayClient_OnExitPresent_Name: {
      break;
    }
    case internal::kVRDisplayClient_OnBlur_Name: {
      break;
    }
    case internal::kVRDisplayClient_OnFocus_Name: {
      break;
    }
    case internal::kVRDisplayClient_OnActivate_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::VRDisplayClient_OnActivate_Params_Data* params =
          reinterpret_cast<
              internal::VRDisplayClient_OnActivate_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      VRDisplayEventReason p_reason{};
      VRDisplayClient_OnActivate_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadReason(&p_reason))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VRDisplayClient::OnActivate deserializer");
        return false;
      }
      VRDisplayClient::OnActivateCallback callback =
          VRDisplayClient_OnActivate_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnActivate(
std::move(p_reason), std::move(callback));
      return true;
    }
    case internal::kVRDisplayClient_OnDeactivate_Name: {
      break;
    }
  }
  return false;
}

bool VRDisplayClientRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRDisplayClient RequestValidator");

  switch (message->header()->name) {
    case internal::kVRDisplayClient_OnChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplayClient_OnChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRDisplayClient_OnExitPresent_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplayClient_OnExitPresent_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRDisplayClient_OnBlur_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplayClient_OnBlur_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRDisplayClient_OnFocus_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplayClient_OnFocus_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRDisplayClient_OnActivate_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplayClient_OnActivate_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVRDisplayClient_OnDeactivate_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplayClient_OnDeactivate_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool VRDisplayClientResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VRDisplayClient ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kVRDisplayClient_OnActivate_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::VRDisplayClient_OnActivate_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
}  // namespace mojom
}  // namespace device

namespace mojo {


// static
bool StructTraits<::device::mojom::VRFieldOfView::DataView, ::device::mojom::VRFieldOfViewPtr>::Read(
    ::device::mojom::VRFieldOfView::DataView input,
    ::device::mojom::VRFieldOfViewPtr* output) {
  bool success = true;
  ::device::mojom::VRFieldOfViewPtr result(::device::mojom::VRFieldOfView::New());
  
      result->upDegrees = input.upDegrees();
      result->downDegrees = input.downDegrees();
      result->leftDegrees = input.leftDegrees();
      result->rightDegrees = input.rightDegrees();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::device::mojom::VRPose::DataView, ::device::mojom::VRPosePtr>::Read(
    ::device::mojom::VRPose::DataView input,
    ::device::mojom::VRPosePtr* output) {
  bool success = true;
  ::device::mojom::VRPosePtr result(::device::mojom::VRPose::New());
  
      if (!input.ReadOrientation(&result->orientation))
        success = false;
      if (!input.ReadPosition(&result->position))
        success = false;
      if (!input.ReadAngularVelocity(&result->angularVelocity))
        success = false;
      if (!input.ReadLinearVelocity(&result->linearVelocity))
        success = false;
      if (!input.ReadAngularAcceleration(&result->angularAcceleration))
        success = false;
      if (!input.ReadLinearAcceleration(&result->linearAcceleration))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::device::mojom::VRDisplayCapabilities::DataView, ::device::mojom::VRDisplayCapabilitiesPtr>::Read(
    ::device::mojom::VRDisplayCapabilities::DataView input,
    ::device::mojom::VRDisplayCapabilitiesPtr* output) {
  bool success = true;
  ::device::mojom::VRDisplayCapabilitiesPtr result(::device::mojom::VRDisplayCapabilities::New());
  
      result->hasPosition = input.hasPosition();
      result->hasExternalDisplay = input.hasExternalDisplay();
      result->canPresent = input.canPresent();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::device::mojom::VREyeParameters::DataView, ::device::mojom::VREyeParametersPtr>::Read(
    ::device::mojom::VREyeParameters::DataView input,
    ::device::mojom::VREyeParametersPtr* output) {
  bool success = true;
  ::device::mojom::VREyeParametersPtr result(::device::mojom::VREyeParameters::New());
  
      if (!input.ReadFieldOfView(&result->fieldOfView))
        success = false;
      if (!input.ReadOffset(&result->offset))
        success = false;
      result->renderWidth = input.renderWidth();
      result->renderHeight = input.renderHeight();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::device::mojom::VRStageParameters::DataView, ::device::mojom::VRStageParametersPtr>::Read(
    ::device::mojom::VRStageParameters::DataView input,
    ::device::mojom::VRStageParametersPtr* output) {
  bool success = true;
  ::device::mojom::VRStageParametersPtr result(::device::mojom::VRStageParameters::New());
  
      if (!input.ReadStandingTransform(&result->standingTransform))
        success = false;
      result->sizeX = input.sizeX();
      result->sizeZ = input.sizeZ();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::device::mojom::VRDisplayInfo::DataView, ::device::mojom::VRDisplayInfoPtr>::Read(
    ::device::mojom::VRDisplayInfo::DataView input,
    ::device::mojom::VRDisplayInfoPtr* output) {
  bool success = true;
  ::device::mojom::VRDisplayInfoPtr result(::device::mojom::VRDisplayInfo::New());
  
      result->index = input.index();
      if (!input.ReadDisplayName(&result->displayName))
        success = false;
      if (!input.ReadCapabilities(&result->capabilities))
        success = false;
      if (!input.ReadStageParameters(&result->stageParameters))
        success = false;
      if (!input.ReadLeftEye(&result->leftEye))
        success = false;
      if (!input.ReadRightEye(&result->rightEye))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif