// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef GPU_IPC_COMMON_GPU_PREFERENCES_MOJOM_BLINK_H_
#define GPU_IPC_COMMON_GPU_PREFERENCES_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "gpu/ipc/common/gpu_preferences.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


namespace WTF {
struct gpu_mojom_internal_VpxDecodeVendors_DataHashFn {
  static unsigned GetHash(const ::gpu::mojom::VpxDecodeVendors& value) {
    using utype = std::underlying_type<::gpu::mojom::VpxDecodeVendors>::type;
    return DefaultHash<utype>::Hash().GetHash(static_cast<utype>(value));
  }
  static bool Equal(const ::gpu::mojom::VpxDecodeVendors& left, const ::gpu::mojom::VpxDecodeVendors& right) {
    return left == right;
  }
  static const bool safe_to_compare_to_empty_or_deleted = true;
};

template <>
struct DefaultHash<::gpu::mojom::VpxDecodeVendors> {
  using Hash = gpu_mojom_internal_VpxDecodeVendors_DataHashFn;
};

template <>
struct HashTraits<::gpu::mojom::VpxDecodeVendors>
    : public GenericHashTraits<::gpu::mojom::VpxDecodeVendors> {
  static_assert(true,
                "-1000000 is a reserved enum value");
  static_assert(true,
                "-1000001 is a reserved enum value");
  static const bool hasIsEmptyValueFunction = true;
  static bool IsEmptyValue(const ::gpu::mojom::VpxDecodeVendors& value) {
    return value == static_cast<::gpu::mojom::VpxDecodeVendors>(-1000000);
  }
  static void ConstructDeletedValue(::gpu::mojom::VpxDecodeVendors& slot, bool) {
    slot = static_cast<::gpu::mojom::VpxDecodeVendors>(-1000001);
  }
  static bool IsDeletedValue(const ::gpu::mojom::VpxDecodeVendors& value) {
    return value == static_cast<::gpu::mojom::VpxDecodeVendors>(-1000001);
  }
};
}  // namespace WTF


namespace gpu {
namespace mojom {
namespace blink {
using VpxDecodeVendors = VpxDecodeVendors;  // Alias for definition in the parent namespace.
class GpuPreferences;
using GpuPreferencesPtr = mojo::StructPtr<GpuPreferences>;






class  GpuPreferences {
 public:
  using DataView = GpuPreferencesDataView;
  using Data_ = internal::GpuPreferences_Data;

  template <typename... Args>
  static GpuPreferencesPtr New(Args&&... args) {
    return GpuPreferencesPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static GpuPreferencesPtr From(const U& u) {
    return mojo::TypeConverter<GpuPreferencesPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, GpuPreferences>::Convert(*this);
  }


  GpuPreferences();

  GpuPreferences(
      bool single_process,
      bool in_process_gpu,
      bool ui_prioritize_in_gpu_process,
      bool enable_gpu_scheduler,
      bool disable_accelerated_video_decode,
      bool disable_vaapi_accelerated_video_encode,
      bool disable_web_rtc_hw_encoding,
      VpxDecodeVendors enable_accelerated_vpx_decode,
      bool enable_low_latency_dxva,
      bool enable_zero_copy_dxgi_video,
      bool enable_nv12_dxgi_video,
      bool compile_shader_always_succeeds,
      bool disable_gl_error_limit,
      bool disable_glsl_translator,
      bool disable_gpu_driver_bug_workarounds,
      bool disable_shader_name_hashing,
      bool enable_gpu_command_logging,
      bool enable_gpu_debugging,
      bool enable_gpu_service_logging_gpu,
      bool enable_gpu_driver_debug_logging,
      bool disable_gpu_program_cache,
      bool enforce_gl_minimums,
      uint32_t force_gpu_mem_available,
      uint32_t gpu_program_cache_size,
      bool disable_gpu_shader_disk_cache,
      bool enable_threaded_texture_mailboxes,
      bool gl_shader_interm_output,
      bool emulate_shader_precision,
      bool enable_gpu_service_logging,
      bool enable_gpu_service_tracing,
      bool enable_es3_apis,
      bool use_passthrough_cmd_decoder);

  ~GpuPreferences();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = GpuPreferencesPtr>
  GpuPreferencesPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, GpuPreferences>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        GpuPreferences::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        GpuPreferences::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  bool single_process;
  bool in_process_gpu;
  bool ui_prioritize_in_gpu_process;
  bool enable_gpu_scheduler;
  bool disable_accelerated_video_decode;
  bool disable_vaapi_accelerated_video_encode;
  bool disable_web_rtc_hw_encoding;
  VpxDecodeVendors enable_accelerated_vpx_decode;
  bool enable_low_latency_dxva;
  bool enable_zero_copy_dxgi_video;
  bool enable_nv12_dxgi_video;
  bool compile_shader_always_succeeds;
  bool disable_gl_error_limit;
  bool disable_glsl_translator;
  bool disable_gpu_driver_bug_workarounds;
  bool disable_shader_name_hashing;
  bool enable_gpu_command_logging;
  bool enable_gpu_debugging;
  bool enable_gpu_service_logging_gpu;
  bool enable_gpu_driver_debug_logging;
  bool disable_gpu_program_cache;
  bool enforce_gl_minimums;
  uint32_t force_gpu_mem_available;
  uint32_t gpu_program_cache_size;
  bool disable_gpu_shader_disk_cache;
  bool enable_threaded_texture_mailboxes;
  bool gl_shader_interm_output;
  bool emulate_shader_precision;
  bool enable_gpu_service_logging;
  bool enable_gpu_service_tracing;
  bool enable_es3_apis;
  bool use_passthrough_cmd_decoder;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
GpuPreferencesPtr GpuPreferences::Clone() const {
  return New(
      mojo::Clone(single_process),
      mojo::Clone(in_process_gpu),
      mojo::Clone(ui_prioritize_in_gpu_process),
      mojo::Clone(enable_gpu_scheduler),
      mojo::Clone(disable_accelerated_video_decode),
      mojo::Clone(disable_vaapi_accelerated_video_encode),
      mojo::Clone(disable_web_rtc_hw_encoding),
      mojo::Clone(enable_accelerated_vpx_decode),
      mojo::Clone(enable_low_latency_dxva),
      mojo::Clone(enable_zero_copy_dxgi_video),
      mojo::Clone(enable_nv12_dxgi_video),
      mojo::Clone(compile_shader_always_succeeds),
      mojo::Clone(disable_gl_error_limit),
      mojo::Clone(disable_glsl_translator),
      mojo::Clone(disable_gpu_driver_bug_workarounds),
      mojo::Clone(disable_shader_name_hashing),
      mojo::Clone(enable_gpu_command_logging),
      mojo::Clone(enable_gpu_debugging),
      mojo::Clone(enable_gpu_service_logging_gpu),
      mojo::Clone(enable_gpu_driver_debug_logging),
      mojo::Clone(disable_gpu_program_cache),
      mojo::Clone(enforce_gl_minimums),
      mojo::Clone(force_gpu_mem_available),
      mojo::Clone(gpu_program_cache_size),
      mojo::Clone(disable_gpu_shader_disk_cache),
      mojo::Clone(enable_threaded_texture_mailboxes),
      mojo::Clone(gl_shader_interm_output),
      mojo::Clone(emulate_shader_precision),
      mojo::Clone(enable_gpu_service_logging),
      mojo::Clone(enable_gpu_service_tracing),
      mojo::Clone(enable_es3_apis),
      mojo::Clone(use_passthrough_cmd_decoder)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, GpuPreferences>::value>::type*>
bool GpuPreferences::Equals(const T& other) const {
  if (!mojo::Equals(this->single_process, other.single_process))
    return false;
  if (!mojo::Equals(this->in_process_gpu, other.in_process_gpu))
    return false;
  if (!mojo::Equals(this->ui_prioritize_in_gpu_process, other.ui_prioritize_in_gpu_process))
    return false;
  if (!mojo::Equals(this->enable_gpu_scheduler, other.enable_gpu_scheduler))
    return false;
  if (!mojo::Equals(this->disable_accelerated_video_decode, other.disable_accelerated_video_decode))
    return false;
  if (!mojo::Equals(this->disable_vaapi_accelerated_video_encode, other.disable_vaapi_accelerated_video_encode))
    return false;
  if (!mojo::Equals(this->disable_web_rtc_hw_encoding, other.disable_web_rtc_hw_encoding))
    return false;
  if (!mojo::Equals(this->enable_accelerated_vpx_decode, other.enable_accelerated_vpx_decode))
    return false;
  if (!mojo::Equals(this->enable_low_latency_dxva, other.enable_low_latency_dxva))
    return false;
  if (!mojo::Equals(this->enable_zero_copy_dxgi_video, other.enable_zero_copy_dxgi_video))
    return false;
  if (!mojo::Equals(this->enable_nv12_dxgi_video, other.enable_nv12_dxgi_video))
    return false;
  if (!mojo::Equals(this->compile_shader_always_succeeds, other.compile_shader_always_succeeds))
    return false;
  if (!mojo::Equals(this->disable_gl_error_limit, other.disable_gl_error_limit))
    return false;
  if (!mojo::Equals(this->disable_glsl_translator, other.disable_glsl_translator))
    return false;
  if (!mojo::Equals(this->disable_gpu_driver_bug_workarounds, other.disable_gpu_driver_bug_workarounds))
    return false;
  if (!mojo::Equals(this->disable_shader_name_hashing, other.disable_shader_name_hashing))
    return false;
  if (!mojo::Equals(this->enable_gpu_command_logging, other.enable_gpu_command_logging))
    return false;
  if (!mojo::Equals(this->enable_gpu_debugging, other.enable_gpu_debugging))
    return false;
  if (!mojo::Equals(this->enable_gpu_service_logging_gpu, other.enable_gpu_service_logging_gpu))
    return false;
  if (!mojo::Equals(this->enable_gpu_driver_debug_logging, other.enable_gpu_driver_debug_logging))
    return false;
  if (!mojo::Equals(this->disable_gpu_program_cache, other.disable_gpu_program_cache))
    return false;
  if (!mojo::Equals(this->enforce_gl_minimums, other.enforce_gl_minimums))
    return false;
  if (!mojo::Equals(this->force_gpu_mem_available, other.force_gpu_mem_available))
    return false;
  if (!mojo::Equals(this->gpu_program_cache_size, other.gpu_program_cache_size))
    return false;
  if (!mojo::Equals(this->disable_gpu_shader_disk_cache, other.disable_gpu_shader_disk_cache))
    return false;
  if (!mojo::Equals(this->enable_threaded_texture_mailboxes, other.enable_threaded_texture_mailboxes))
    return false;
  if (!mojo::Equals(this->gl_shader_interm_output, other.gl_shader_interm_output))
    return false;
  if (!mojo::Equals(this->emulate_shader_precision, other.emulate_shader_precision))
    return false;
  if (!mojo::Equals(this->enable_gpu_service_logging, other.enable_gpu_service_logging))
    return false;
  if (!mojo::Equals(this->enable_gpu_service_tracing, other.enable_gpu_service_tracing))
    return false;
  if (!mojo::Equals(this->enable_es3_apis, other.enable_es3_apis))
    return false;
  if (!mojo::Equals(this->use_passthrough_cmd_decoder, other.use_passthrough_cmd_decoder))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace gpu

namespace mojo {


template <>
struct  StructTraits<::gpu::mojom::blink::GpuPreferences::DataView,
                                         ::gpu::mojom::blink::GpuPreferencesPtr> {
  static bool IsNull(const ::gpu::mojom::blink::GpuPreferencesPtr& input) { return !input; }
  static void SetToNull(::gpu::mojom::blink::GpuPreferencesPtr* output) { output->reset(); }

  static decltype(::gpu::mojom::blink::GpuPreferences::single_process) single_process(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->single_process;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::in_process_gpu) in_process_gpu(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->in_process_gpu;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::ui_prioritize_in_gpu_process) ui_prioritize_in_gpu_process(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->ui_prioritize_in_gpu_process;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_gpu_scheduler) enable_gpu_scheduler(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_gpu_scheduler;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::disable_accelerated_video_decode) disable_accelerated_video_decode(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->disable_accelerated_video_decode;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::disable_vaapi_accelerated_video_encode) disable_vaapi_accelerated_video_encode(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->disable_vaapi_accelerated_video_encode;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::disable_web_rtc_hw_encoding) disable_web_rtc_hw_encoding(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->disable_web_rtc_hw_encoding;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_accelerated_vpx_decode) enable_accelerated_vpx_decode(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_accelerated_vpx_decode;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_low_latency_dxva) enable_low_latency_dxva(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_low_latency_dxva;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_zero_copy_dxgi_video) enable_zero_copy_dxgi_video(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_zero_copy_dxgi_video;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_nv12_dxgi_video) enable_nv12_dxgi_video(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_nv12_dxgi_video;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::compile_shader_always_succeeds) compile_shader_always_succeeds(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->compile_shader_always_succeeds;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::disable_gl_error_limit) disable_gl_error_limit(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->disable_gl_error_limit;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::disable_glsl_translator) disable_glsl_translator(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->disable_glsl_translator;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::disable_gpu_driver_bug_workarounds) disable_gpu_driver_bug_workarounds(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->disable_gpu_driver_bug_workarounds;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::disable_shader_name_hashing) disable_shader_name_hashing(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->disable_shader_name_hashing;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_gpu_command_logging) enable_gpu_command_logging(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_gpu_command_logging;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_gpu_debugging) enable_gpu_debugging(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_gpu_debugging;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_gpu_service_logging_gpu) enable_gpu_service_logging_gpu(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_gpu_service_logging_gpu;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_gpu_driver_debug_logging) enable_gpu_driver_debug_logging(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_gpu_driver_debug_logging;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::disable_gpu_program_cache) disable_gpu_program_cache(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->disable_gpu_program_cache;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enforce_gl_minimums) enforce_gl_minimums(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enforce_gl_minimums;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::force_gpu_mem_available) force_gpu_mem_available(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->force_gpu_mem_available;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::gpu_program_cache_size) gpu_program_cache_size(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->gpu_program_cache_size;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::disable_gpu_shader_disk_cache) disable_gpu_shader_disk_cache(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->disable_gpu_shader_disk_cache;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_threaded_texture_mailboxes) enable_threaded_texture_mailboxes(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_threaded_texture_mailboxes;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::gl_shader_interm_output) gl_shader_interm_output(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->gl_shader_interm_output;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::emulate_shader_precision) emulate_shader_precision(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->emulate_shader_precision;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_gpu_service_logging) enable_gpu_service_logging(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_gpu_service_logging;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_gpu_service_tracing) enable_gpu_service_tracing(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_gpu_service_tracing;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::enable_es3_apis) enable_es3_apis(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->enable_es3_apis;
  }

  static decltype(::gpu::mojom::blink::GpuPreferences::use_passthrough_cmd_decoder) use_passthrough_cmd_decoder(
      const ::gpu::mojom::blink::GpuPreferencesPtr& input) {
    return input->use_passthrough_cmd_decoder;
  }

  static bool Read(::gpu::mojom::blink::GpuPreferences::DataView input, ::gpu::mojom::blink::GpuPreferencesPtr* output);
};

}  // namespace mojo

#endif  // GPU_IPC_COMMON_GPU_PREFERENCES_MOJOM_BLINK_H_