// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_CAPTURE_MOJO_IMAGE_CAPTURE_MOJOM_SHARED_INTERNAL_H_
#define MEDIA_CAPTURE_MOJO_IMAGE_CAPTURE_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace media {
namespace mojom {
namespace internal {
class Range_Data;
class PhotoState_Data;
class Point2D_Data;
class PhotoSettings_Data;
class Blob_Data;

struct MeteringMode_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct RedEyeReduction_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct FillLightMode_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class  Range_Data {
 public:
  static Range_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  double max;
  double min;
  double current;
  double step;

 private:
  Range_Data();
  ~Range_Data() = delete;
};
static_assert(sizeof(Range_Data) == 40,
              "Bad sizeof(Range_Data)");
class  PhotoState_Data {
 public:
  static PhotoState_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<int32_t>> supported_white_balance_modes;
  int32_t current_white_balance_mode;
  int32_t current_exposure_mode;
  mojo::internal::Pointer<mojo::internal::Array_Data<int32_t>> supported_exposure_modes;
  mojo::internal::Pointer<mojo::internal::Array_Data<int32_t>> supported_focus_modes;
  int32_t current_focus_mode;
  uint8_t supports_torch : 1;
  uint8_t torch : 1;
  uint8_t pad7_[3];
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::Point2D_Data>>> points_of_interest;
  mojo::internal::Pointer<internal::Range_Data> exposure_compensation;
  mojo::internal::Pointer<internal::Range_Data> color_temperature;
  mojo::internal::Pointer<internal::Range_Data> iso;
  mojo::internal::Pointer<internal::Range_Data> brightness;
  mojo::internal::Pointer<internal::Range_Data> contrast;
  mojo::internal::Pointer<internal::Range_Data> saturation;
  mojo::internal::Pointer<internal::Range_Data> sharpness;
  mojo::internal::Pointer<internal::Range_Data> zoom;
  int32_t red_eye_reduction;
  uint8_t pad17_[4];
  mojo::internal::Pointer<internal::Range_Data> height;
  mojo::internal::Pointer<internal::Range_Data> width;
  mojo::internal::Pointer<mojo::internal::Array_Data<int32_t>> fill_light_mode;

 private:
  PhotoState_Data();
  ~PhotoState_Data() = delete;
};
static_assert(sizeof(PhotoState_Data) == 152,
              "Bad sizeof(PhotoState_Data)");
class  Point2D_Data {
 public:
  static Point2D_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  float x;
  float y;

 private:
  Point2D_Data();
  ~Point2D_Data() = delete;
};
static_assert(sizeof(Point2D_Data) == 16,
              "Bad sizeof(Point2D_Data)");
class  PhotoSettings_Data {
 public:
  static PhotoSettings_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t has_white_balance_mode : 1;
  uint8_t has_exposure_mode : 1;
  uint8_t has_focus_mode : 1;
  uint8_t has_exposure_compensation : 1;
  uint8_t has_color_temperature : 1;
  uint8_t has_iso : 1;
  uint8_t has_brightness : 1;
  uint8_t has_contrast : 1;
  uint8_t has_saturation : 1;
  uint8_t has_sharpness : 1;
  uint8_t has_zoom : 1;
  uint8_t has_torch : 1;
  uint8_t torch : 1;
  uint8_t has_fill_light_mode : 1;
  uint8_t has_width : 1;
  uint8_t has_height : 1;
  uint8_t has_red_eye_reduction : 1;
  uint8_t red_eye_reduction : 1;
  uint8_t pad17_[1];
  int32_t white_balance_mode;
  int32_t exposure_mode;
  int32_t focus_mode;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::Point2D_Data>>> points_of_interest;
  double exposure_compensation;
  double color_temperature;
  double iso;
  double brightness;
  double contrast;
  double saturation;
  double sharpness;
  double zoom;
  int32_t fill_light_mode;
  uint8_t pad30_[4];
  double width;
  double height;

 private:
  PhotoSettings_Data();
  ~PhotoSettings_Data() = delete;
};
static_assert(sizeof(PhotoSettings_Data) == 120,
              "Bad sizeof(PhotoSettings_Data)");
class  Blob_Data {
 public:
  static Blob_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> mime_type;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> data;

 private:
  Blob_Data();
  ~Blob_Data() = delete;
};
static_assert(sizeof(Blob_Data) == 24,
              "Bad sizeof(Blob_Data)");
constexpr uint32_t kImageCapture_GetPhotoState_Name = 0;
class  ImageCapture_GetPhotoState_Params_Data {
 public:
  static ImageCapture_GetPhotoState_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> source_id;

 private:
  ImageCapture_GetPhotoState_Params_Data();
  ~ImageCapture_GetPhotoState_Params_Data() = delete;
};
static_assert(sizeof(ImageCapture_GetPhotoState_Params_Data) == 16,
              "Bad sizeof(ImageCapture_GetPhotoState_Params_Data)");
class  ImageCapture_GetPhotoState_ResponseParams_Data {
 public:
  static ImageCapture_GetPhotoState_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::PhotoState_Data> state;

 private:
  ImageCapture_GetPhotoState_ResponseParams_Data();
  ~ImageCapture_GetPhotoState_ResponseParams_Data() = delete;
};
static_assert(sizeof(ImageCapture_GetPhotoState_ResponseParams_Data) == 16,
              "Bad sizeof(ImageCapture_GetPhotoState_ResponseParams_Data)");
constexpr uint32_t kImageCapture_SetOptions_Name = 1;
class  ImageCapture_SetOptions_Params_Data {
 public:
  static ImageCapture_SetOptions_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> source_id;
  mojo::internal::Pointer<internal::PhotoSettings_Data> settings;

 private:
  ImageCapture_SetOptions_Params_Data();
  ~ImageCapture_SetOptions_Params_Data() = delete;
};
static_assert(sizeof(ImageCapture_SetOptions_Params_Data) == 24,
              "Bad sizeof(ImageCapture_SetOptions_Params_Data)");
class  ImageCapture_SetOptions_ResponseParams_Data {
 public:
  static ImageCapture_SetOptions_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t success : 1;
  uint8_t padfinal_[7];

 private:
  ImageCapture_SetOptions_ResponseParams_Data();
  ~ImageCapture_SetOptions_ResponseParams_Data() = delete;
};
static_assert(sizeof(ImageCapture_SetOptions_ResponseParams_Data) == 16,
              "Bad sizeof(ImageCapture_SetOptions_ResponseParams_Data)");
constexpr uint32_t kImageCapture_TakePhoto_Name = 2;
class  ImageCapture_TakePhoto_Params_Data {
 public:
  static ImageCapture_TakePhoto_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> source_id;

 private:
  ImageCapture_TakePhoto_Params_Data();
  ~ImageCapture_TakePhoto_Params_Data() = delete;
};
static_assert(sizeof(ImageCapture_TakePhoto_Params_Data) == 16,
              "Bad sizeof(ImageCapture_TakePhoto_Params_Data)");
class  ImageCapture_TakePhoto_ResponseParams_Data {
 public:
  static ImageCapture_TakePhoto_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::Blob_Data> blob;

 private:
  ImageCapture_TakePhoto_ResponseParams_Data();
  ~ImageCapture_TakePhoto_ResponseParams_Data() = delete;
};
static_assert(sizeof(ImageCapture_TakePhoto_ResponseParams_Data) == 16,
              "Bad sizeof(ImageCapture_TakePhoto_ResponseParams_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace media

#endif  // MEDIA_CAPTURE_MOJO_IMAGE_CAPTURE_MOJOM_SHARED_INTERNAL_H_