// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "media/capture/mojo/image_capture.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace media {
namespace mojom {

namespace internal {


// static
Range_Data* Range_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Range_Data))) Range_Data();
}

// static
bool Range_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Range_Data* object = static_cast<const Range_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Range_Data::Range_Data()
    : header_({sizeof(*this), 0}) {}


// static
PhotoState_Data* PhotoState_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PhotoState_Data))) PhotoState_Data();
}

// static
bool PhotoState_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PhotoState_Data* object = static_cast<const PhotoState_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 152 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->supported_white_balance_modes, "null supported_white_balance_modes field in PhotoState",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams supported_white_balance_modes_validate_params(
      0, ::media::mojom::internal::MeteringMode_Data::Validate);
  if (!mojo::internal::ValidateContainer(object->supported_white_balance_modes, validation_context,
                                         &supported_white_balance_modes_validate_params)) {
    return false;
  }


  if (!::media::mojom::internal::MeteringMode_Data
        ::Validate(object->current_white_balance_mode, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->supported_exposure_modes, "null supported_exposure_modes field in PhotoState",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams supported_exposure_modes_validate_params(
      0, ::media::mojom::internal::MeteringMode_Data::Validate);
  if (!mojo::internal::ValidateContainer(object->supported_exposure_modes, validation_context,
                                         &supported_exposure_modes_validate_params)) {
    return false;
  }


  if (!::media::mojom::internal::MeteringMode_Data
        ::Validate(object->current_exposure_mode, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->supported_focus_modes, "null supported_focus_modes field in PhotoState",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams supported_focus_modes_validate_params(
      0, ::media::mojom::internal::MeteringMode_Data::Validate);
  if (!mojo::internal::ValidateContainer(object->supported_focus_modes, validation_context,
                                         &supported_focus_modes_validate_params)) {
    return false;
  }


  if (!::media::mojom::internal::MeteringMode_Data
        ::Validate(object->current_focus_mode, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->points_of_interest, "null points_of_interest field in PhotoState",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams points_of_interest_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->points_of_interest, validation_context,
                                         &points_of_interest_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->exposure_compensation, "null exposure_compensation field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->exposure_compensation, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->color_temperature, "null color_temperature field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->color_temperature, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->iso, "null iso field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->iso, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->brightness, "null brightness field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->brightness, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->contrast, "null contrast field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->contrast, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->saturation, "null saturation field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->saturation, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->sharpness, "null sharpness field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->sharpness, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->zoom, "null zoom field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->zoom, validation_context))
    return false;


  if (!::media::mojom::internal::RedEyeReduction_Data
        ::Validate(object->red_eye_reduction, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->height, "null height field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->height, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->width, "null width field in PhotoState",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->width, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->fill_light_mode, "null fill_light_mode field in PhotoState",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams fill_light_mode_validate_params(
      0, ::media::mojom::internal::FillLightMode_Data::Validate);
  if (!mojo::internal::ValidateContainer(object->fill_light_mode, validation_context,
                                         &fill_light_mode_validate_params)) {
    return false;
  }

  return true;
}

PhotoState_Data::PhotoState_Data()
    : header_({sizeof(*this), 0}) {}


// static
Point2D_Data* Point2D_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Point2D_Data))) Point2D_Data();
}

// static
bool Point2D_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Point2D_Data* object = static_cast<const Point2D_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Point2D_Data::Point2D_Data()
    : header_({sizeof(*this), 0}) {}


// static
PhotoSettings_Data* PhotoSettings_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PhotoSettings_Data))) PhotoSettings_Data();
}

// static
bool PhotoSettings_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PhotoSettings_Data* object = static_cast<const PhotoSettings_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 120 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::media::mojom::internal::MeteringMode_Data
        ::Validate(object->white_balance_mode, validation_context))
    return false;


  if (!::media::mojom::internal::MeteringMode_Data
        ::Validate(object->exposure_mode, validation_context))
    return false;


  if (!::media::mojom::internal::MeteringMode_Data
        ::Validate(object->focus_mode, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->points_of_interest, "null points_of_interest field in PhotoSettings",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams points_of_interest_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->points_of_interest, validation_context,
                                         &points_of_interest_validate_params)) {
    return false;
  }


  if (!::media::mojom::internal::FillLightMode_Data
        ::Validate(object->fill_light_mode, validation_context))
    return false;

  return true;
}

PhotoSettings_Data::PhotoSettings_Data()
    : header_({sizeof(*this), 0}) {}


// static
Blob_Data* Blob_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Blob_Data))) Blob_Data();
}

// static
bool Blob_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Blob_Data* object = static_cast<const Blob_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->mime_type, "null mime_type field in Blob",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams mime_type_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->mime_type, validation_context,
                                         &mime_type_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->data, "null data field in Blob",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams data_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->data, validation_context,
                                         &data_validate_params)) {
    return false;
  }

  return true;
}

Blob_Data::Blob_Data()
    : header_({sizeof(*this), 0}) {}


// static
ImageCapture_GetPhotoState_Params_Data* ImageCapture_GetPhotoState_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ImageCapture_GetPhotoState_Params_Data))) ImageCapture_GetPhotoState_Params_Data();
}

// static
bool ImageCapture_GetPhotoState_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ImageCapture_GetPhotoState_Params_Data* object = static_cast<const ImageCapture_GetPhotoState_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->source_id, "null source_id field in ImageCapture_GetPhotoState_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams source_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->source_id, validation_context,
                                         &source_id_validate_params)) {
    return false;
  }

  return true;
}

ImageCapture_GetPhotoState_Params_Data::ImageCapture_GetPhotoState_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ImageCapture_GetPhotoState_ResponseParams_Data* ImageCapture_GetPhotoState_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ImageCapture_GetPhotoState_ResponseParams_Data))) ImageCapture_GetPhotoState_ResponseParams_Data();
}

// static
bool ImageCapture_GetPhotoState_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ImageCapture_GetPhotoState_ResponseParams_Data* object = static_cast<const ImageCapture_GetPhotoState_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->state, "null state field in ImageCapture_GetPhotoState_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->state, validation_context))
    return false;

  return true;
}

ImageCapture_GetPhotoState_ResponseParams_Data::ImageCapture_GetPhotoState_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ImageCapture_SetOptions_Params_Data* ImageCapture_SetOptions_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ImageCapture_SetOptions_Params_Data))) ImageCapture_SetOptions_Params_Data();
}

// static
bool ImageCapture_SetOptions_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ImageCapture_SetOptions_Params_Data* object = static_cast<const ImageCapture_SetOptions_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->source_id, "null source_id field in ImageCapture_SetOptions_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams source_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->source_id, validation_context,
                                         &source_id_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->settings, "null settings field in ImageCapture_SetOptions_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->settings, validation_context))
    return false;

  return true;
}

ImageCapture_SetOptions_Params_Data::ImageCapture_SetOptions_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ImageCapture_SetOptions_ResponseParams_Data* ImageCapture_SetOptions_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ImageCapture_SetOptions_ResponseParams_Data))) ImageCapture_SetOptions_ResponseParams_Data();
}

// static
bool ImageCapture_SetOptions_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ImageCapture_SetOptions_ResponseParams_Data* object = static_cast<const ImageCapture_SetOptions_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

ImageCapture_SetOptions_ResponseParams_Data::ImageCapture_SetOptions_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ImageCapture_TakePhoto_Params_Data* ImageCapture_TakePhoto_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ImageCapture_TakePhoto_Params_Data))) ImageCapture_TakePhoto_Params_Data();
}

// static
bool ImageCapture_TakePhoto_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ImageCapture_TakePhoto_Params_Data* object = static_cast<const ImageCapture_TakePhoto_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->source_id, "null source_id field in ImageCapture_TakePhoto_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams source_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->source_id, validation_context,
                                         &source_id_validate_params)) {
    return false;
  }

  return true;
}

ImageCapture_TakePhoto_Params_Data::ImageCapture_TakePhoto_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ImageCapture_TakePhoto_ResponseParams_Data* ImageCapture_TakePhoto_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ImageCapture_TakePhoto_ResponseParams_Data))) ImageCapture_TakePhoto_ResponseParams_Data();
}

// static
bool ImageCapture_TakePhoto_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ImageCapture_TakePhoto_ResponseParams_Data* object = static_cast<const ImageCapture_TakePhoto_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->blob, "null blob field in ImageCapture_TakePhoto_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->blob, validation_context))
    return false;

  return true;
}

ImageCapture_TakePhoto_ResponseParams_Data::ImageCapture_TakePhoto_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace media

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
