// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_CAPTURE_MOJO_VIDEO_CAPTURE_TYPES_MOJOM_H_
#define MEDIA_CAPTURE_MOJO_VIDEO_CAPTURE_TYPES_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "media/capture/mojo/video_capture_types.mojom-shared.h"
#include "media/mojo/interfaces/media_types.mojom.h"
#include "mojo/common/time.mojom.h"
#include "mojo/common/values.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>
#include "media/capture/video/video_capture_device_descriptor.h"
#include "media/capture/video/video_capture_device_info.h"
#include "media/capture/video_capture_types.h"


namespace media {
namespace mojom {
class VideoCaptureFormat;
using VideoCaptureFormatPtr = mojo::StructPtr<VideoCaptureFormat>;

class VideoCaptureParams;
using VideoCaptureParamsPtr = mojo::StructPtr<VideoCaptureParams>;

class VideoFrameInfo;
using VideoFrameInfoPtr = mojo::StructPtr<VideoFrameInfo>;

class VideoCaptureDeviceDescriptor;
using VideoCaptureDeviceDescriptorPtr = mojo::StructPtr<VideoCaptureDeviceDescriptor>;

class VideoCaptureDeviceInfo;
using VideoCaptureDeviceInfoPtr = mojo::StructPtr<VideoCaptureDeviceInfo>;










class  VideoCaptureFormat {
 public:
  using DataView = VideoCaptureFormatDataView;
  using Data_ = internal::VideoCaptureFormat_Data;

  template <typename... Args>
  static VideoCaptureFormatPtr New(Args&&... args) {
    return VideoCaptureFormatPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static VideoCaptureFormatPtr From(const U& u) {
    return mojo::TypeConverter<VideoCaptureFormatPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, VideoCaptureFormat>::Convert(*this);
  }


  VideoCaptureFormat();

  VideoCaptureFormat(
      const gfx::Size& frame_size,
      float frame_rate,
      media::VideoPixelFormat pixel_format,
      media::VideoPixelStorage pixel_storage);

  ~VideoCaptureFormat();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = VideoCaptureFormatPtr>
  VideoCaptureFormatPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, VideoCaptureFormat>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        VideoCaptureFormat::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        VideoCaptureFormat::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  gfx::Size frame_size;
  float frame_rate;
  media::VideoPixelFormat pixel_format;
  media::VideoPixelStorage pixel_storage;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class  VideoCaptureParams {
 public:
  using DataView = VideoCaptureParamsDataView;
  using Data_ = internal::VideoCaptureParams_Data;

  template <typename... Args>
  static VideoCaptureParamsPtr New(Args&&... args) {
    return VideoCaptureParamsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static VideoCaptureParamsPtr From(const U& u) {
    return mojo::TypeConverter<VideoCaptureParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, VideoCaptureParams>::Convert(*this);
  }


  VideoCaptureParams();

  VideoCaptureParams(
      const media::VideoCaptureFormat& requested_format,
      media::ResolutionChangePolicy resolution_change_policy,
      media::PowerLineFrequency power_line_frequency);

  ~VideoCaptureParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = VideoCaptureParamsPtr>
  VideoCaptureParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, VideoCaptureParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        VideoCaptureParams::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        VideoCaptureParams::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  media::VideoCaptureFormat requested_format;
  media::ResolutionChangePolicy resolution_change_policy;
  media::PowerLineFrequency power_line_frequency;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class  VideoFrameInfo {
 public:
  using DataView = VideoFrameInfoDataView;
  using Data_ = internal::VideoFrameInfo_Data;

  template <typename... Args>
  static VideoFrameInfoPtr New(Args&&... args) {
    return VideoFrameInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static VideoFrameInfoPtr From(const U& u) {
    return mojo::TypeConverter<VideoFrameInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, VideoFrameInfo>::Convert(*this);
  }


  VideoFrameInfo();

  VideoFrameInfo(
      base::TimeDelta timestamp,
      std::unique_ptr<base::DictionaryValue> metadata,
      media::VideoPixelFormat pixel_format,
      media::VideoPixelStorage storage_type,
      const gfx::Size& coded_size,
      const gfx::Rect& visible_rect);

  ~VideoFrameInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = VideoFrameInfoPtr>
  VideoFrameInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, VideoFrameInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        VideoFrameInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        VideoFrameInfo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::TimeDelta timestamp;
  std::unique_ptr<base::DictionaryValue> metadata;
  media::VideoPixelFormat pixel_format;
  media::VideoPixelStorage storage_type;
  gfx::Size coded_size;
  gfx::Rect visible_rect;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(VideoFrameInfo);
};



class  VideoCaptureDeviceDescriptor {
 public:
  using DataView = VideoCaptureDeviceDescriptorDataView;
  using Data_ = internal::VideoCaptureDeviceDescriptor_Data;

  template <typename... Args>
  static VideoCaptureDeviceDescriptorPtr New(Args&&... args) {
    return VideoCaptureDeviceDescriptorPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static VideoCaptureDeviceDescriptorPtr From(const U& u) {
    return mojo::TypeConverter<VideoCaptureDeviceDescriptorPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, VideoCaptureDeviceDescriptor>::Convert(*this);
  }


  VideoCaptureDeviceDescriptor();

  VideoCaptureDeviceDescriptor(
      const std::string& display_name,
      const std::string& device_id,
      const std::string& model_id,
      VideoCaptureApi capture_api,
      VideoCaptureTransportType transport_type);

  ~VideoCaptureDeviceDescriptor();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = VideoCaptureDeviceDescriptorPtr>
  VideoCaptureDeviceDescriptorPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, VideoCaptureDeviceDescriptor>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        VideoCaptureDeviceDescriptor::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        VideoCaptureDeviceDescriptor::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::string display_name;
  std::string device_id;
  std::string model_id;
  VideoCaptureApi capture_api;
  VideoCaptureTransportType transport_type;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class  VideoCaptureDeviceInfo {
 public:
  using DataView = VideoCaptureDeviceInfoDataView;
  using Data_ = internal::VideoCaptureDeviceInfo_Data;

  template <typename... Args>
  static VideoCaptureDeviceInfoPtr New(Args&&... args) {
    return VideoCaptureDeviceInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static VideoCaptureDeviceInfoPtr From(const U& u) {
    return mojo::TypeConverter<VideoCaptureDeviceInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, VideoCaptureDeviceInfo>::Convert(*this);
  }


  VideoCaptureDeviceInfo();

  VideoCaptureDeviceInfo(
      const media::VideoCaptureDeviceDescriptor& descriptor,
      const std::vector<media::VideoCaptureFormat>& supported_formats);

  ~VideoCaptureDeviceInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = VideoCaptureDeviceInfoPtr>
  VideoCaptureDeviceInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, VideoCaptureDeviceInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        VideoCaptureDeviceInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        VideoCaptureDeviceInfo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  media::VideoCaptureDeviceDescriptor descriptor;
  std::vector<media::VideoCaptureFormat> supported_formats;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
VideoCaptureFormatPtr VideoCaptureFormat::Clone() const {
  return New(
      mojo::Clone(frame_size),
      mojo::Clone(frame_rate),
      mojo::Clone(pixel_format),
      mojo::Clone(pixel_storage)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, VideoCaptureFormat>::value>::type*>
bool VideoCaptureFormat::Equals(const T& other) const {
  if (!mojo::Equals(this->frame_size, other.frame_size))
    return false;
  if (!mojo::Equals(this->frame_rate, other.frame_rate))
    return false;
  if (!mojo::Equals(this->pixel_format, other.pixel_format))
    return false;
  if (!mojo::Equals(this->pixel_storage, other.pixel_storage))
    return false;
  return true;
}
template <typename StructPtrType>
VideoCaptureParamsPtr VideoCaptureParams::Clone() const {
  return New(
      mojo::Clone(requested_format),
      mojo::Clone(resolution_change_policy),
      mojo::Clone(power_line_frequency)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, VideoCaptureParams>::value>::type*>
bool VideoCaptureParams::Equals(const T& other) const {
  if (!mojo::Equals(this->requested_format, other.requested_format))
    return false;
  if (!mojo::Equals(this->resolution_change_policy, other.resolution_change_policy))
    return false;
  if (!mojo::Equals(this->power_line_frequency, other.power_line_frequency))
    return false;
  return true;
}
template <typename StructPtrType>
VideoFrameInfoPtr VideoFrameInfo::Clone() const {
  return New(
      mojo::Clone(timestamp),
      mojo::Clone(metadata),
      mojo::Clone(pixel_format),
      mojo::Clone(storage_type),
      mojo::Clone(coded_size),
      mojo::Clone(visible_rect)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, VideoFrameInfo>::value>::type*>
bool VideoFrameInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->timestamp, other.timestamp))
    return false;
  if (!mojo::Equals(this->metadata, other.metadata))
    return false;
  if (!mojo::Equals(this->pixel_format, other.pixel_format))
    return false;
  if (!mojo::Equals(this->storage_type, other.storage_type))
    return false;
  if (!mojo::Equals(this->coded_size, other.coded_size))
    return false;
  if (!mojo::Equals(this->visible_rect, other.visible_rect))
    return false;
  return true;
}
template <typename StructPtrType>
VideoCaptureDeviceDescriptorPtr VideoCaptureDeviceDescriptor::Clone() const {
  return New(
      mojo::Clone(display_name),
      mojo::Clone(device_id),
      mojo::Clone(model_id),
      mojo::Clone(capture_api),
      mojo::Clone(transport_type)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, VideoCaptureDeviceDescriptor>::value>::type*>
bool VideoCaptureDeviceDescriptor::Equals(const T& other) const {
  if (!mojo::Equals(this->display_name, other.display_name))
    return false;
  if (!mojo::Equals(this->device_id, other.device_id))
    return false;
  if (!mojo::Equals(this->model_id, other.model_id))
    return false;
  if (!mojo::Equals(this->capture_api, other.capture_api))
    return false;
  if (!mojo::Equals(this->transport_type, other.transport_type))
    return false;
  return true;
}
template <typename StructPtrType>
VideoCaptureDeviceInfoPtr VideoCaptureDeviceInfo::Clone() const {
  return New(
      mojo::Clone(descriptor),
      mojo::Clone(supported_formats)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, VideoCaptureDeviceInfo>::value>::type*>
bool VideoCaptureDeviceInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->descriptor, other.descriptor))
    return false;
  if (!mojo::Equals(this->supported_formats, other.supported_formats))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace media

namespace mojo {


template <>
struct  StructTraits<::media::mojom::VideoCaptureFormat::DataView,
                                         ::media::mojom::VideoCaptureFormatPtr> {
  static bool IsNull(const ::media::mojom::VideoCaptureFormatPtr& input) { return !input; }
  static void SetToNull(::media::mojom::VideoCaptureFormatPtr* output) { output->reset(); }

  static const decltype(::media::mojom::VideoCaptureFormat::frame_size)& frame_size(
      const ::media::mojom::VideoCaptureFormatPtr& input) {
    return input->frame_size;
  }

  static decltype(::media::mojom::VideoCaptureFormat::frame_rate) frame_rate(
      const ::media::mojom::VideoCaptureFormatPtr& input) {
    return input->frame_rate;
  }

  static decltype(::media::mojom::VideoCaptureFormat::pixel_format) pixel_format(
      const ::media::mojom::VideoCaptureFormatPtr& input) {
    return input->pixel_format;
  }

  static decltype(::media::mojom::VideoCaptureFormat::pixel_storage) pixel_storage(
      const ::media::mojom::VideoCaptureFormatPtr& input) {
    return input->pixel_storage;
  }

  static bool Read(::media::mojom::VideoCaptureFormat::DataView input, ::media::mojom::VideoCaptureFormatPtr* output);
};


template <>
struct  StructTraits<::media::mojom::VideoCaptureParams::DataView,
                                         ::media::mojom::VideoCaptureParamsPtr> {
  static bool IsNull(const ::media::mojom::VideoCaptureParamsPtr& input) { return !input; }
  static void SetToNull(::media::mojom::VideoCaptureParamsPtr* output) { output->reset(); }

  static const decltype(::media::mojom::VideoCaptureParams::requested_format)& requested_format(
      const ::media::mojom::VideoCaptureParamsPtr& input) {
    return input->requested_format;
  }

  static decltype(::media::mojom::VideoCaptureParams::resolution_change_policy) resolution_change_policy(
      const ::media::mojom::VideoCaptureParamsPtr& input) {
    return input->resolution_change_policy;
  }

  static decltype(::media::mojom::VideoCaptureParams::power_line_frequency) power_line_frequency(
      const ::media::mojom::VideoCaptureParamsPtr& input) {
    return input->power_line_frequency;
  }

  static bool Read(::media::mojom::VideoCaptureParams::DataView input, ::media::mojom::VideoCaptureParamsPtr* output);
};


template <>
struct  StructTraits<::media::mojom::VideoFrameInfo::DataView,
                                         ::media::mojom::VideoFrameInfoPtr> {
  static bool IsNull(const ::media::mojom::VideoFrameInfoPtr& input) { return !input; }
  static void SetToNull(::media::mojom::VideoFrameInfoPtr* output) { output->reset(); }

  static const decltype(::media::mojom::VideoFrameInfo::timestamp)& timestamp(
      const ::media::mojom::VideoFrameInfoPtr& input) {
    return input->timestamp;
  }

  static const decltype(::media::mojom::VideoFrameInfo::metadata)& metadata(
      const ::media::mojom::VideoFrameInfoPtr& input) {
    return input->metadata;
  }

  static decltype(::media::mojom::VideoFrameInfo::pixel_format) pixel_format(
      const ::media::mojom::VideoFrameInfoPtr& input) {
    return input->pixel_format;
  }

  static decltype(::media::mojom::VideoFrameInfo::storage_type) storage_type(
      const ::media::mojom::VideoFrameInfoPtr& input) {
    return input->storage_type;
  }

  static const decltype(::media::mojom::VideoFrameInfo::coded_size)& coded_size(
      const ::media::mojom::VideoFrameInfoPtr& input) {
    return input->coded_size;
  }

  static const decltype(::media::mojom::VideoFrameInfo::visible_rect)& visible_rect(
      const ::media::mojom::VideoFrameInfoPtr& input) {
    return input->visible_rect;
  }

  static bool Read(::media::mojom::VideoFrameInfo::DataView input, ::media::mojom::VideoFrameInfoPtr* output);
};


template <>
struct  StructTraits<::media::mojom::VideoCaptureDeviceDescriptor::DataView,
                                         ::media::mojom::VideoCaptureDeviceDescriptorPtr> {
  static bool IsNull(const ::media::mojom::VideoCaptureDeviceDescriptorPtr& input) { return !input; }
  static void SetToNull(::media::mojom::VideoCaptureDeviceDescriptorPtr* output) { output->reset(); }

  static const decltype(::media::mojom::VideoCaptureDeviceDescriptor::display_name)& display_name(
      const ::media::mojom::VideoCaptureDeviceDescriptorPtr& input) {
    return input->display_name;
  }

  static const decltype(::media::mojom::VideoCaptureDeviceDescriptor::device_id)& device_id(
      const ::media::mojom::VideoCaptureDeviceDescriptorPtr& input) {
    return input->device_id;
  }

  static const decltype(::media::mojom::VideoCaptureDeviceDescriptor::model_id)& model_id(
      const ::media::mojom::VideoCaptureDeviceDescriptorPtr& input) {
    return input->model_id;
  }

  static decltype(::media::mojom::VideoCaptureDeviceDescriptor::capture_api) capture_api(
      const ::media::mojom::VideoCaptureDeviceDescriptorPtr& input) {
    return input->capture_api;
  }

  static decltype(::media::mojom::VideoCaptureDeviceDescriptor::transport_type) transport_type(
      const ::media::mojom::VideoCaptureDeviceDescriptorPtr& input) {
    return input->transport_type;
  }

  static bool Read(::media::mojom::VideoCaptureDeviceDescriptor::DataView input, ::media::mojom::VideoCaptureDeviceDescriptorPtr* output);
};


template <>
struct  StructTraits<::media::mojom::VideoCaptureDeviceInfo::DataView,
                                         ::media::mojom::VideoCaptureDeviceInfoPtr> {
  static bool IsNull(const ::media::mojom::VideoCaptureDeviceInfoPtr& input) { return !input; }
  static void SetToNull(::media::mojom::VideoCaptureDeviceInfoPtr* output) { output->reset(); }

  static const decltype(::media::mojom::VideoCaptureDeviceInfo::descriptor)& descriptor(
      const ::media::mojom::VideoCaptureDeviceInfoPtr& input) {
    return input->descriptor;
  }

  static const decltype(::media::mojom::VideoCaptureDeviceInfo::supported_formats)& supported_formats(
      const ::media::mojom::VideoCaptureDeviceInfoPtr& input) {
    return input->supported_formats;
  }

  static bool Read(::media::mojom::VideoCaptureDeviceInfo::DataView input, ::media::mojom::VideoCaptureDeviceInfoPtr* output);
};

}  // namespace mojo

#endif  // MEDIA_CAPTURE_MOJO_VIDEO_CAPTURE_TYPES_MOJOM_H_