// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_GPU_MOJO_JPEG_DECODER_MOJOM_SHARED_H_
#define MEDIA_GPU_MOJO_JPEG_DECODER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "media/gpu/mojo/jpeg_decoder.mojom-shared-internal.h"
#include "media/mojo/interfaces/media_types.mojom-shared.h"
#include "mojo/common/time.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"


namespace media {
namespace mojom {
class BitstreamBufferDataView;



}  // namespace mojom
}  // namespace media

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::media::mojom::BitstreamBufferDataView> {
  using Data = ::media::mojom::internal::BitstreamBuffer_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace media {
namespace mojom {

enum class DecodeError : int32_t {
  NO_ERRORS,
  INVALID_ARGUMENT,
  UNREADABLE_INPUT,
  PARSE_JPEG_FAILED,
  UNSUPPORTED_JPEG,
  PLATFORM_FAILURE,
};

inline std::ostream& operator<<(std::ostream& os, DecodeError value) {
  switch(value) {
    case DecodeError::NO_ERRORS:
      return os << "DecodeError::NO_ERRORS";
    case DecodeError::INVALID_ARGUMENT:
      return os << "DecodeError::INVALID_ARGUMENT";
    case DecodeError::UNREADABLE_INPUT:
      return os << "DecodeError::UNREADABLE_INPUT";
    case DecodeError::PARSE_JPEG_FAILED:
      return os << "DecodeError::PARSE_JPEG_FAILED";
    case DecodeError::UNSUPPORTED_JPEG:
      return os << "DecodeError::UNSUPPORTED_JPEG";
    case DecodeError::PLATFORM_FAILURE:
      return os << "DecodeError::PLATFORM_FAILURE";
    default:
      return os << "Unknown DecodeError value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(DecodeError value) {
  return internal::DecodeError_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class GpuJpegDecodeAcceleratorInterfaceBase {};

using GpuJpegDecodeAcceleratorPtrDataView =
    mojo::InterfacePtrDataView<GpuJpegDecodeAcceleratorInterfaceBase>;
using GpuJpegDecodeAcceleratorRequestDataView =
    mojo::InterfaceRequestDataView<GpuJpegDecodeAcceleratorInterfaceBase>;
using GpuJpegDecodeAcceleratorAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<GpuJpegDecodeAcceleratorInterfaceBase>;
using GpuJpegDecodeAcceleratorAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<GpuJpegDecodeAcceleratorInterfaceBase>;
class BitstreamBufferDataView {
 public:
  BitstreamBufferDataView() {}

  BitstreamBufferDataView(
      internal::BitstreamBuffer_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t id() const {
    return data_->id;
  }
  mojo::ScopedSharedBufferHandle TakeMemoryHandle() {
    mojo::ScopedSharedBufferHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedSharedBufferHandle>(
            &data_->memory_handle, &result, context_);
    DCHECK(ret);
    return result;
  }
  uint32_t size() const {
    return data_->size;
  }
  int64_t offset() const {
    return data_->offset;
  }
  inline void GetTimestampDataView(
      ::mojo::common::mojom::TimeDeltaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTimestamp(UserType* output) {
    auto* pointer = data_->timestamp.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::TimeDeltaDataView>(
        pointer, output, context_);
  }
  inline void GetKeyIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadKeyId(UserType* output) {
    auto* pointer = data_->key_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetIvDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIv(UserType* output) {
    auto* pointer = data_->iv.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetSubsamplesDataView(
      mojo::ArrayDataView<::media::mojom::SubsampleEntryDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSubsamples(UserType* output) {
    auto* pointer = data_->subsamples.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::media::mojom::SubsampleEntryDataView>>(
        pointer, output, context_);
  }
 private:
  internal::BitstreamBuffer_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class GpuJpegDecodeAccelerator_Initialize_ParamsDataView {
 public:
  GpuJpegDecodeAccelerator_Initialize_ParamsDataView() {}

  GpuJpegDecodeAccelerator_Initialize_ParamsDataView(
      internal::GpuJpegDecodeAccelerator_Initialize_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::GpuJpegDecodeAccelerator_Initialize_Params_Data* data_ = nullptr;
};

class GpuJpegDecodeAccelerator_Initialize_ResponseParamsDataView {
 public:
  GpuJpegDecodeAccelerator_Initialize_ResponseParamsDataView() {}

  GpuJpegDecodeAccelerator_Initialize_ResponseParamsDataView(
      internal::GpuJpegDecodeAccelerator_Initialize_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
 private:
  internal::GpuJpegDecodeAccelerator_Initialize_ResponseParams_Data* data_ = nullptr;
};

class GpuJpegDecodeAccelerator_Decode_ParamsDataView {
 public:
  GpuJpegDecodeAccelerator_Decode_ParamsDataView() {}

  GpuJpegDecodeAccelerator_Decode_ParamsDataView(
      internal::GpuJpegDecodeAccelerator_Decode_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInputBufferDataView(
      BitstreamBufferDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInputBuffer(UserType* output) {
    auto* pointer = data_->input_buffer.Get();
    return mojo::internal::Deserialize<::media::mojom::BitstreamBufferDataView>(
        pointer, output, context_);
  }
  inline void GetCodedSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCodedSize(UserType* output) {
    auto* pointer = data_->coded_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  mojo::ScopedSharedBufferHandle TakeOutputHandle() {
    mojo::ScopedSharedBufferHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedSharedBufferHandle>(
            &data_->output_handle, &result, context_);
    DCHECK(ret);
    return result;
  }
  uint32_t output_buffer_size() const {
    return data_->output_buffer_size;
  }
 private:
  internal::GpuJpegDecodeAccelerator_Decode_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class GpuJpegDecodeAccelerator_Decode_ResponseParamsDataView {
 public:
  GpuJpegDecodeAccelerator_Decode_ResponseParamsDataView() {}

  GpuJpegDecodeAccelerator_Decode_ResponseParamsDataView(
      internal::GpuJpegDecodeAccelerator_Decode_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t bitstream_buffer_id() const {
    return data_->bitstream_buffer_id;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadError(UserType* output) const {
    auto data_value = data_->error;
    return mojo::internal::Deserialize<::media::mojom::DecodeError>(
        data_value, output);
  }

  DecodeError error() const {
    return static_cast<DecodeError>(data_->error);
  }
 private:
  internal::GpuJpegDecodeAccelerator_Decode_ResponseParams_Data* data_ = nullptr;
};

class GpuJpegDecodeAccelerator_Uninitialize_ParamsDataView {
 public:
  GpuJpegDecodeAccelerator_Uninitialize_ParamsDataView() {}

  GpuJpegDecodeAccelerator_Uninitialize_ParamsDataView(
      internal::GpuJpegDecodeAccelerator_Uninitialize_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::GpuJpegDecodeAccelerator_Uninitialize_Params_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace media

namespace std {

template <>
struct hash<::media::mojom::DecodeError>
    : public mojo::internal::EnumHashImpl<::media::mojom::DecodeError> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::media::mojom::DecodeError, ::media::mojom::DecodeError> {
  static ::media::mojom::DecodeError ToMojom(::media::mojom::DecodeError input) { return input; }
  static bool FromMojom(::media::mojom::DecodeError input, ::media::mojom::DecodeError* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media::mojom::DecodeError, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media::mojom::DecodeError, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media::mojom::DecodeError>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media::mojom::BitstreamBufferDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::media::mojom::BitstreamBufferDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::media::mojom::internal::BitstreamBuffer_Data);
    decltype(CallWithContext(Traits::memory_handle, input, custom_context)) in_memory_handle = CallWithContext(Traits::memory_handle, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ScopedSharedBufferHandle>(
        in_memory_handle, context);
    decltype(CallWithContext(Traits::timestamp, input, custom_context)) in_timestamp = CallWithContext(Traits::timestamp, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::TimeDeltaDataView>(
        in_timestamp, context);
    decltype(CallWithContext(Traits::key_id, input, custom_context)) in_key_id = CallWithContext(Traits::key_id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_key_id, context);
    decltype(CallWithContext(Traits::iv, input, custom_context)) in_iv = CallWithContext(Traits::iv, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_iv, context);
    decltype(CallWithContext(Traits::subsamples, input, custom_context)) in_subsamples = CallWithContext(Traits::subsamples, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::media::mojom::SubsampleEntryDataView>>(
        in_subsamples, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::media::mojom::internal::BitstreamBuffer_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::media::mojom::internal::BitstreamBuffer_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->id = CallWithContext(Traits::id, input, custom_context);
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::memory_handle, input, custom_context))>::type
        in_memory_handle;
    mojo::internal::Serialize<mojo::ScopedSharedBufferHandle>(
        in_memory_handle, &result->memory_handle, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->memory_handle),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid memory_handle in BitstreamBuffer struct");
    result->size = CallWithContext(Traits::size, input, custom_context);
    result->offset = CallWithContext(Traits::offset, input, custom_context);
    decltype(CallWithContext(Traits::timestamp, input, custom_context)) in_timestamp = CallWithContext(Traits::timestamp, input, custom_context);
    typename decltype(result->timestamp)::BaseType* timestamp_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::TimeDeltaDataView>(
        in_timestamp, buffer, &timestamp_ptr, context);
    result->timestamp.Set(timestamp_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->timestamp.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null timestamp in BitstreamBuffer struct");
    decltype(CallWithContext(Traits::key_id, input, custom_context)) in_key_id = CallWithContext(Traits::key_id, input, custom_context);
    typename decltype(result->key_id)::BaseType* key_id_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_key_id, buffer, &key_id_ptr, context);
    result->key_id.Set(key_id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->key_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null key_id in BitstreamBuffer struct");
    decltype(CallWithContext(Traits::iv, input, custom_context)) in_iv = CallWithContext(Traits::iv, input, custom_context);
    typename decltype(result->iv)::BaseType* iv_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_iv, buffer, &iv_ptr, context);
    result->iv.Set(iv_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->iv.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null iv in BitstreamBuffer struct");
    decltype(CallWithContext(Traits::subsamples, input, custom_context)) in_subsamples = CallWithContext(Traits::subsamples, input, custom_context);
    typename decltype(result->subsamples)::BaseType* subsamples_ptr;
    const mojo::internal::ContainerValidateParams subsamples_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::media::mojom::SubsampleEntryDataView>>(
        in_subsamples, buffer, &subsamples_ptr, &subsamples_validate_params,
        context);
    result->subsamples.Set(subsamples_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->subsamples.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null subsamples in BitstreamBuffer struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::media::mojom::internal::BitstreamBuffer_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::media::mojom::BitstreamBufferDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace media {
namespace mojom {

inline void BitstreamBufferDataView::GetTimestampDataView(
    ::mojo::common::mojom::TimeDeltaDataView* output) {
  auto pointer = data_->timestamp.Get();
  *output = ::mojo::common::mojom::TimeDeltaDataView(pointer, context_);
}
inline void BitstreamBufferDataView::GetKeyIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->key_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void BitstreamBufferDataView::GetIvDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->iv.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void BitstreamBufferDataView::GetSubsamplesDataView(
    mojo::ArrayDataView<::media::mojom::SubsampleEntryDataView>* output) {
  auto pointer = data_->subsamples.Get();
  *output = mojo::ArrayDataView<::media::mojom::SubsampleEntryDataView>(pointer, context_);
}






inline void GpuJpegDecodeAccelerator_Decode_ParamsDataView::GetInputBufferDataView(
    BitstreamBufferDataView* output) {
  auto pointer = data_->input_buffer.Get();
  *output = BitstreamBufferDataView(pointer, context_);
}
inline void GpuJpegDecodeAccelerator_Decode_ParamsDataView::GetCodedSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->coded_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}







}  // namespace mojom
}  // namespace media

#endif  // MEDIA_GPU_MOJO_JPEG_DECODER_MOJOM_SHARED_H_
