// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_MOJO_INTERFACES_REMOTING_COMMON_MOJOM_SHARED_H_
#define MEDIA_MOJO_INTERFACES_REMOTING_COMMON_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "media/mojo/interfaces/remoting_common.mojom-shared-internal.h"


namespace media {
namespace mojom {
class SinkCapabilitiesDataView;



}  // namespace mojom
}  // namespace media

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::media::mojom::SinkCapabilitiesDataView> {
  using Data = ::media::mojom::internal::SinkCapabilities_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace media {
namespace mojom {

enum class RemotingStopReason : int32_t {
  ROUTE_TERMINATED,
  LOCAL_PLAYBACK,
  SOURCE_GONE,
  MESSAGE_SEND_FAILED,
  DATA_SEND_FAILED,
  UNEXPECTED_FAILURE,
  SERVICE_GONE,
};

inline std::ostream& operator<<(std::ostream& os, RemotingStopReason value) {
  switch(value) {
    case RemotingStopReason::ROUTE_TERMINATED:
      return os << "RemotingStopReason::ROUTE_TERMINATED";
    case RemotingStopReason::LOCAL_PLAYBACK:
      return os << "RemotingStopReason::LOCAL_PLAYBACK";
    case RemotingStopReason::SOURCE_GONE:
      return os << "RemotingStopReason::SOURCE_GONE";
    case RemotingStopReason::MESSAGE_SEND_FAILED:
      return os << "RemotingStopReason::MESSAGE_SEND_FAILED";
    case RemotingStopReason::DATA_SEND_FAILED:
      return os << "RemotingStopReason::DATA_SEND_FAILED";
    case RemotingStopReason::UNEXPECTED_FAILURE:
      return os << "RemotingStopReason::UNEXPECTED_FAILURE";
    case RemotingStopReason::SERVICE_GONE:
      return os << "RemotingStopReason::SERVICE_GONE";
    default:
      return os << "Unknown RemotingStopReason value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(RemotingStopReason value) {
  return internal::RemotingStopReason_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class RemotingStartFailReason : int32_t {
  CANNOT_START_MULTIPLE,
  ROUTE_TERMINATED,
  SERVICE_NOT_CONNECTED,
};

inline std::ostream& operator<<(std::ostream& os, RemotingStartFailReason value) {
  switch(value) {
    case RemotingStartFailReason::CANNOT_START_MULTIPLE:
      return os << "RemotingStartFailReason::CANNOT_START_MULTIPLE";
    case RemotingStartFailReason::ROUTE_TERMINATED:
      return os << "RemotingStartFailReason::ROUTE_TERMINATED";
    case RemotingStartFailReason::SERVICE_NOT_CONNECTED:
      return os << "RemotingStartFailReason::SERVICE_NOT_CONNECTED";
    default:
      return os << "Unknown RemotingStartFailReason value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(RemotingStartFailReason value) {
  return internal::RemotingStartFailReason_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class RemotingSinkFeatures : int32_t {
  RENDERING,
  CONTENT_DECRYPTION,
};

inline std::ostream& operator<<(std::ostream& os, RemotingSinkFeatures value) {
  switch(value) {
    case RemotingSinkFeatures::RENDERING:
      return os << "RemotingSinkFeatures::RENDERING";
    case RemotingSinkFeatures::CONTENT_DECRYPTION:
      return os << "RemotingSinkFeatures::CONTENT_DECRYPTION";
    default:
      return os << "Unknown RemotingSinkFeatures value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(RemotingSinkFeatures value) {
  return internal::RemotingSinkFeatures_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class RemotingSinkAudioCapabilities : int32_t {
  CODEC_BASELINE_SET,
  CODEC_AAC,
  CODEC_OPUS,
};

inline std::ostream& operator<<(std::ostream& os, RemotingSinkAudioCapabilities value) {
  switch(value) {
    case RemotingSinkAudioCapabilities::CODEC_BASELINE_SET:
      return os << "RemotingSinkAudioCapabilities::CODEC_BASELINE_SET";
    case RemotingSinkAudioCapabilities::CODEC_AAC:
      return os << "RemotingSinkAudioCapabilities::CODEC_AAC";
    case RemotingSinkAudioCapabilities::CODEC_OPUS:
      return os << "RemotingSinkAudioCapabilities::CODEC_OPUS";
    default:
      return os << "Unknown RemotingSinkAudioCapabilities value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(RemotingSinkAudioCapabilities value) {
  return internal::RemotingSinkAudioCapabilities_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class RemotingSinkVideoCapabilities : int32_t {
  SUPPORT_4K,
  CODEC_BASELINE_SET,
  CODEC_H264,
  CODEC_VP8,
  CODEC_VP9,
  CODEC_HEVC,
};

inline std::ostream& operator<<(std::ostream& os, RemotingSinkVideoCapabilities value) {
  switch(value) {
    case RemotingSinkVideoCapabilities::SUPPORT_4K:
      return os << "RemotingSinkVideoCapabilities::SUPPORT_4K";
    case RemotingSinkVideoCapabilities::CODEC_BASELINE_SET:
      return os << "RemotingSinkVideoCapabilities::CODEC_BASELINE_SET";
    case RemotingSinkVideoCapabilities::CODEC_H264:
      return os << "RemotingSinkVideoCapabilities::CODEC_H264";
    case RemotingSinkVideoCapabilities::CODEC_VP8:
      return os << "RemotingSinkVideoCapabilities::CODEC_VP8";
    case RemotingSinkVideoCapabilities::CODEC_VP9:
      return os << "RemotingSinkVideoCapabilities::CODEC_VP9";
    case RemotingSinkVideoCapabilities::CODEC_HEVC:
      return os << "RemotingSinkVideoCapabilities::CODEC_HEVC";
    default:
      return os << "Unknown RemotingSinkVideoCapabilities value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(RemotingSinkVideoCapabilities value) {
  return internal::RemotingSinkVideoCapabilities_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class SinkCapabilitiesDataView {
 public:
  SinkCapabilitiesDataView() {}

  SinkCapabilitiesDataView(
      internal::SinkCapabilities_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFeaturesDataView(
      mojo::ArrayDataView<RemotingSinkFeatures>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFeatures(UserType* output) {
    auto* pointer = data_->features.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::media::mojom::RemotingSinkFeatures>>(
        pointer, output, context_);
  }
  inline void GetAudioDataView(
      mojo::ArrayDataView<RemotingSinkAudioCapabilities>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAudio(UserType* output) {
    auto* pointer = data_->audio.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::media::mojom::RemotingSinkAudioCapabilities>>(
        pointer, output, context_);
  }
  inline void GetVideoDataView(
      mojo::ArrayDataView<RemotingSinkVideoCapabilities>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadVideo(UserType* output) {
    auto* pointer = data_->video.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::media::mojom::RemotingSinkVideoCapabilities>>(
        pointer, output, context_);
  }
 private:
  internal::SinkCapabilities_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace media

namespace std {

template <>
struct hash<::media::mojom::RemotingStopReason>
    : public mojo::internal::EnumHashImpl<::media::mojom::RemotingStopReason> {};

template <>
struct hash<::media::mojom::RemotingStartFailReason>
    : public mojo::internal::EnumHashImpl<::media::mojom::RemotingStartFailReason> {};

template <>
struct hash<::media::mojom::RemotingSinkFeatures>
    : public mojo::internal::EnumHashImpl<::media::mojom::RemotingSinkFeatures> {};

template <>
struct hash<::media::mojom::RemotingSinkAudioCapabilities>
    : public mojo::internal::EnumHashImpl<::media::mojom::RemotingSinkAudioCapabilities> {};

template <>
struct hash<::media::mojom::RemotingSinkVideoCapabilities>
    : public mojo::internal::EnumHashImpl<::media::mojom::RemotingSinkVideoCapabilities> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::media::mojom::RemotingStopReason, ::media::mojom::RemotingStopReason> {
  static ::media::mojom::RemotingStopReason ToMojom(::media::mojom::RemotingStopReason input) { return input; }
  static bool FromMojom(::media::mojom::RemotingStopReason input, ::media::mojom::RemotingStopReason* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media::mojom::RemotingStopReason, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media::mojom::RemotingStopReason, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media::mojom::RemotingStopReason>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::media::mojom::RemotingStartFailReason, ::media::mojom::RemotingStartFailReason> {
  static ::media::mojom::RemotingStartFailReason ToMojom(::media::mojom::RemotingStartFailReason input) { return input; }
  static bool FromMojom(::media::mojom::RemotingStartFailReason input, ::media::mojom::RemotingStartFailReason* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media::mojom::RemotingStartFailReason, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media::mojom::RemotingStartFailReason, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media::mojom::RemotingStartFailReason>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::media::mojom::RemotingSinkFeatures, ::media::mojom::RemotingSinkFeatures> {
  static ::media::mojom::RemotingSinkFeatures ToMojom(::media::mojom::RemotingSinkFeatures input) { return input; }
  static bool FromMojom(::media::mojom::RemotingSinkFeatures input, ::media::mojom::RemotingSinkFeatures* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media::mojom::RemotingSinkFeatures, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media::mojom::RemotingSinkFeatures, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media::mojom::RemotingSinkFeatures>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::media::mojom::RemotingSinkAudioCapabilities, ::media::mojom::RemotingSinkAudioCapabilities> {
  static ::media::mojom::RemotingSinkAudioCapabilities ToMojom(::media::mojom::RemotingSinkAudioCapabilities input) { return input; }
  static bool FromMojom(::media::mojom::RemotingSinkAudioCapabilities input, ::media::mojom::RemotingSinkAudioCapabilities* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media::mojom::RemotingSinkAudioCapabilities, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media::mojom::RemotingSinkAudioCapabilities, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media::mojom::RemotingSinkAudioCapabilities>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::media::mojom::RemotingSinkVideoCapabilities, ::media::mojom::RemotingSinkVideoCapabilities> {
  static ::media::mojom::RemotingSinkVideoCapabilities ToMojom(::media::mojom::RemotingSinkVideoCapabilities input) { return input; }
  static bool FromMojom(::media::mojom::RemotingSinkVideoCapabilities input, ::media::mojom::RemotingSinkVideoCapabilities* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media::mojom::RemotingSinkVideoCapabilities, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media::mojom::RemotingSinkVideoCapabilities, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media::mojom::RemotingSinkVideoCapabilities>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media::mojom::SinkCapabilitiesDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::media::mojom::SinkCapabilitiesDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::media::mojom::internal::SinkCapabilities_Data);
    decltype(CallWithContext(Traits::features, input, custom_context)) in_features = CallWithContext(Traits::features, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::media::mojom::RemotingSinkFeatures>>(
        in_features, context);
    decltype(CallWithContext(Traits::audio, input, custom_context)) in_audio = CallWithContext(Traits::audio, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::media::mojom::RemotingSinkAudioCapabilities>>(
        in_audio, context);
    decltype(CallWithContext(Traits::video, input, custom_context)) in_video = CallWithContext(Traits::video, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::media::mojom::RemotingSinkVideoCapabilities>>(
        in_video, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::media::mojom::internal::SinkCapabilities_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::media::mojom::internal::SinkCapabilities_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::features, input, custom_context)) in_features = CallWithContext(Traits::features, input, custom_context);
    typename decltype(result->features)::BaseType* features_ptr;
    const mojo::internal::ContainerValidateParams features_validate_params(
        0, ::media::mojom::internal::RemotingSinkFeatures_Data::Validate);
    mojo::internal::Serialize<mojo::ArrayDataView<::media::mojom::RemotingSinkFeatures>>(
        in_features, buffer, &features_ptr, &features_validate_params,
        context);
    result->features.Set(features_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->features.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null features in SinkCapabilities struct");
    decltype(CallWithContext(Traits::audio, input, custom_context)) in_audio = CallWithContext(Traits::audio, input, custom_context);
    typename decltype(result->audio)::BaseType* audio_ptr;
    const mojo::internal::ContainerValidateParams audio_validate_params(
        0, ::media::mojom::internal::RemotingSinkAudioCapabilities_Data::Validate);
    mojo::internal::Serialize<mojo::ArrayDataView<::media::mojom::RemotingSinkAudioCapabilities>>(
        in_audio, buffer, &audio_ptr, &audio_validate_params,
        context);
    result->audio.Set(audio_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->audio.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null audio in SinkCapabilities struct");
    decltype(CallWithContext(Traits::video, input, custom_context)) in_video = CallWithContext(Traits::video, input, custom_context);
    typename decltype(result->video)::BaseType* video_ptr;
    const mojo::internal::ContainerValidateParams video_validate_params(
        0, ::media::mojom::internal::RemotingSinkVideoCapabilities_Data::Validate);
    mojo::internal::Serialize<mojo::ArrayDataView<::media::mojom::RemotingSinkVideoCapabilities>>(
        in_video, buffer, &video_ptr, &video_validate_params,
        context);
    result->video.Set(video_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->video.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null video in SinkCapabilities struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::media::mojom::internal::SinkCapabilities_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::media::mojom::SinkCapabilitiesDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace media {
namespace mojom {

inline void SinkCapabilitiesDataView::GetFeaturesDataView(
    mojo::ArrayDataView<RemotingSinkFeatures>* output) {
  auto pointer = data_->features.Get();
  *output = mojo::ArrayDataView<RemotingSinkFeatures>(pointer, context_);
}
inline void SinkCapabilitiesDataView::GetAudioDataView(
    mojo::ArrayDataView<RemotingSinkAudioCapabilities>* output) {
  auto pointer = data_->audio.Get();
  *output = mojo::ArrayDataView<RemotingSinkAudioCapabilities>(pointer, context_);
}
inline void SinkCapabilitiesDataView::GetVideoDataView(
    mojo::ArrayDataView<RemotingSinkVideoCapabilities>* output) {
  auto pointer = data_->video.Get();
  *output = mojo::ArrayDataView<RemotingSinkVideoCapabilities>(pointer, context_);
}



}  // namespace mojom
}  // namespace media

#endif  // MEDIA_MOJO_INTERFACES_REMOTING_COMMON_MOJOM_SHARED_H_
