// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EVENTS_MOJOM_BLINK_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EVENTS_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/interfaces/events.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"
#include "services/resource_coordinator/public/cpp/resource_coordinator_export.h"


namespace WTF {
struct resource_coordinator_mojom_internal_EventType_DataHashFn {
  static unsigned GetHash(const ::resource_coordinator::mojom::EventType& value) {
    using utype = std::underlying_type<::resource_coordinator::mojom::EventType>::type;
    return DefaultHash<utype>::Hash().GetHash(static_cast<utype>(value));
  }
  static bool Equal(const ::resource_coordinator::mojom::EventType& left, const ::resource_coordinator::mojom::EventType& right) {
    return left == right;
  }
  static const bool safe_to_compare_to_empty_or_deleted = true;
};

template <>
struct DefaultHash<::resource_coordinator::mojom::EventType> {
  using Hash = resource_coordinator_mojom_internal_EventType_DataHashFn;
};

template <>
struct HashTraits<::resource_coordinator::mojom::EventType>
    : public GenericHashTraits<::resource_coordinator::mojom::EventType> {
  static_assert(true,
                "-1000000 is a reserved enum value");
  static_assert(true,
                "-1000001 is a reserved enum value");
  static const bool hasIsEmptyValueFunction = true;
  static bool IsEmptyValue(const ::resource_coordinator::mojom::EventType& value) {
    return value == static_cast<::resource_coordinator::mojom::EventType>(-1000000);
  }
  static void ConstructDeletedValue(::resource_coordinator::mojom::EventType& slot, bool) {
    slot = static_cast<::resource_coordinator::mojom::EventType>(-1000001);
  }
  static bool IsDeletedValue(const ::resource_coordinator::mojom::EventType& value) {
    return value == static_cast<::resource_coordinator::mojom::EventType>(-1000001);
  }
};
}  // namespace WTF


namespace resource_coordinator {
namespace mojom {
namespace blink {
using EventType = EventType;  // Alias for definition in the parent namespace.
class Event;
using EventPtr = mojo::InlinedStructPtr<Event>;



class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT Event {
 public:
  using DataView = EventDataView;
  using Data_ = internal::Event_Data;

  template <typename... Args>
  static EventPtr New(Args&&... args) {
    return EventPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static EventPtr From(const U& u) {
    return mojo::TypeConverter<EventPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, Event>::Convert(*this);
  }


  Event();

  explicit Event(
      EventType type);

  ~Event();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = EventPtr>
  EventPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, Event>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        Event::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        Event::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  EventType type;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





template <typename StructPtrType>
EventPtr Event::Clone() const {
  return New(
      mojo::Clone(type)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, Event>::value>::type*>
bool Event::Equals(const T& other) const {
  if (!mojo::Equals(this->type, other.type))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::resource_coordinator::mojom::blink::Event::DataView,
                                         ::resource_coordinator::mojom::blink::EventPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::blink::EventPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::blink::EventPtr* output) { output->reset(); }

  static decltype(::resource_coordinator::mojom::blink::Event::type) type(
      const ::resource_coordinator::mojom::blink::EventPtr& input) {
    return input->type;
  }

  static bool Read(::resource_coordinator::mojom::blink::Event::DataView input, ::resource_coordinator::mojom::blink::EventPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EVENTS_MOJOM_BLINK_H_