// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_MEMORY_INSTRUMENTATION_MEMORY_INSTRUMENTATION_MOJOM_SHARED_INTERNAL_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_MEMORY_INSTRUMENTATION_MEMORY_INSTRUMENTATION_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "mojo/common/process_id.mojom-shared-internal.h"
#include "services/resource_coordinator/public/interfaces/interfaces_internal_shared_export.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace memory_instrumentation {
namespace mojom {
namespace internal {
class RequestArgs_Data;
class ChromeMemDump_Data;
class PlatformPrivateFootprint_Data;
class RawOSMemDump_Data;
class RawProcessMemoryDump_Data;
class OSMemDump_Data;
class ProcessMemoryDump_Data;
class GlobalMemoryDump_Data;

struct DumpType_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct LevelOfDetail_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct ProcessType_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT RequestArgs_Data {
 public:
  static RequestArgs_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint64_t dump_guid;
  int32_t dump_type;
  int32_t level_of_detail;

 private:
  RequestArgs_Data();
  ~RequestArgs_Data() = delete;
};
static_assert(sizeof(RequestArgs_Data) == 24,
              "Bad sizeof(RequestArgs_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ChromeMemDump_Data {
 public:
  static ChromeMemDump_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint32_t malloc_total_kb;
  uint32_t command_buffer_total_kb;
  uint32_t partition_alloc_total_kb;
  uint32_t blink_gc_total_kb;
  uint32_t v8_total_kb;
  uint8_t padfinal_[4];

 private:
  ChromeMemDump_Data();
  ~ChromeMemDump_Data() = delete;
};
static_assert(sizeof(ChromeMemDump_Data) == 32,
              "Bad sizeof(ChromeMemDump_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT PlatformPrivateFootprint_Data {
 public:
  static PlatformPrivateFootprint_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint64_t phys_footprint_bytes;
  uint64_t internal_bytes;
  uint64_t compressed_bytes;
  uint64_t rss_anon_bytes;
  uint64_t vm_swap_bytes;
  uint64_t private_bytes;

 private:
  PlatformPrivateFootprint_Data();
  ~PlatformPrivateFootprint_Data() = delete;
};
static_assert(sizeof(PlatformPrivateFootprint_Data) == 56,
              "Bad sizeof(PlatformPrivateFootprint_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT RawOSMemDump_Data {
 public:
  static RawOSMemDump_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint32_t resident_set_kb;
  uint8_t pad0_[4];
  mojo::internal::Pointer<internal::PlatformPrivateFootprint_Data> platform_private_footprint;

 private:
  RawOSMemDump_Data();
  ~RawOSMemDump_Data() = delete;
};
static_assert(sizeof(RawOSMemDump_Data) == 24,
              "Bad sizeof(RawOSMemDump_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT RawProcessMemoryDump_Data {
 public:
  static RawProcessMemoryDump_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::RawOSMemDump_Data> os_dump;
  mojo::internal::Pointer<internal::ChromeMemDump_Data> chrome_dump;
  mojo::internal::Pointer<mojo::internal::Map_Data<mojo::internal::Pointer<::mojo::common::mojom::internal::ProcessId_Data>, mojo::internal::Pointer<internal::RawOSMemDump_Data>>> extra_processes_dumps;

 private:
  RawProcessMemoryDump_Data();
  ~RawProcessMemoryDump_Data() = delete;
};
static_assert(sizeof(RawProcessMemoryDump_Data) == 32,
              "Bad sizeof(RawProcessMemoryDump_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT OSMemDump_Data {
 public:
  static OSMemDump_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint32_t resident_set_kb;
  uint32_t private_footprint_kb;

 private:
  OSMemDump_Data();
  ~OSMemDump_Data() = delete;
};
static_assert(sizeof(OSMemDump_Data) == 16,
              "Bad sizeof(OSMemDump_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ProcessMemoryDump_Data {
 public:
  static ProcessMemoryDump_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t process_type;
  uint8_t pad0_[4];
  mojo::internal::Pointer<internal::OSMemDump_Data> os_dump;
  mojo::internal::Pointer<internal::ChromeMemDump_Data> chrome_dump;

 private:
  ProcessMemoryDump_Data();
  ~ProcessMemoryDump_Data() = delete;
};
static_assert(sizeof(ProcessMemoryDump_Data) == 32,
              "Bad sizeof(ProcessMemoryDump_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT GlobalMemoryDump_Data {
 public:
  static GlobalMemoryDump_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::ProcessMemoryDump_Data>>> process_dumps;

 private:
  GlobalMemoryDump_Data();
  ~GlobalMemoryDump_Data() = delete;
};
static_assert(sizeof(GlobalMemoryDump_Data) == 16,
              "Bad sizeof(GlobalMemoryDump_Data)");
constexpr uint32_t kClientProcess_RequestProcessMemoryDump_Name = 0;
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ClientProcess_RequestProcessMemoryDump_Params_Data {
 public:
  static ClientProcess_RequestProcessMemoryDump_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::RequestArgs_Data> args;

 private:
  ClientProcess_RequestProcessMemoryDump_Params_Data();
  ~ClientProcess_RequestProcessMemoryDump_Params_Data() = delete;
};
static_assert(sizeof(ClientProcess_RequestProcessMemoryDump_Params_Data) == 16,
              "Bad sizeof(ClientProcess_RequestProcessMemoryDump_Params_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ClientProcess_RequestProcessMemoryDump_ResponseParams_Data {
 public:
  static ClientProcess_RequestProcessMemoryDump_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t success : 1;
  uint8_t pad0_[7];
  uint64_t dump_id;
  mojo::internal::Pointer<internal::RawProcessMemoryDump_Data> process_memory_dump;

 private:
  ClientProcess_RequestProcessMemoryDump_ResponseParams_Data();
  ~ClientProcess_RequestProcessMemoryDump_ResponseParams_Data() = delete;
};
static_assert(sizeof(ClientProcess_RequestProcessMemoryDump_ResponseParams_Data) == 32,
              "Bad sizeof(ClientProcess_RequestProcessMemoryDump_ResponseParams_Data)");
constexpr uint32_t kClientProcess_RequestOSMemoryDump_Name = 1;
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ClientProcess_RequestOSMemoryDump_Params_Data {
 public:
  static ClientProcess_RequestOSMemoryDump_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::mojo::common::mojom::internal::ProcessId_Data>>> pids;

 private:
  ClientProcess_RequestOSMemoryDump_Params_Data();
  ~ClientProcess_RequestOSMemoryDump_Params_Data() = delete;
};
static_assert(sizeof(ClientProcess_RequestOSMemoryDump_Params_Data) == 16,
              "Bad sizeof(ClientProcess_RequestOSMemoryDump_Params_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ClientProcess_RequestOSMemoryDump_ResponseParams_Data {
 public:
  static ClientProcess_RequestOSMemoryDump_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t success : 1;
  uint8_t pad0_[7];
  mojo::internal::Pointer<mojo::internal::Map_Data<mojo::internal::Pointer<::mojo::common::mojom::internal::ProcessId_Data>, mojo::internal::Pointer<internal::RawOSMemDump_Data>>> dumps;

 private:
  ClientProcess_RequestOSMemoryDump_ResponseParams_Data();
  ~ClientProcess_RequestOSMemoryDump_ResponseParams_Data() = delete;
};
static_assert(sizeof(ClientProcess_RequestOSMemoryDump_ResponseParams_Data) == 24,
              "Bad sizeof(ClientProcess_RequestOSMemoryDump_ResponseParams_Data)");
constexpr uint32_t kCoordinator_RegisterClientProcess_Name = 0;
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT Coordinator_RegisterClientProcess_Params_Data {
 public:
  static Coordinator_RegisterClientProcess_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Interface_Data client_process;
  int32_t process_type;
  uint8_t padfinal_[4];

 private:
  Coordinator_RegisterClientProcess_Params_Data();
  ~Coordinator_RegisterClientProcess_Params_Data() = delete;
};
static_assert(sizeof(Coordinator_RegisterClientProcess_Params_Data) == 24,
              "Bad sizeof(Coordinator_RegisterClientProcess_Params_Data)");
constexpr uint32_t kCoordinator_RequestGlobalMemoryDump_Name = 1;
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT Coordinator_RequestGlobalMemoryDump_Params_Data {
 public:
  static Coordinator_RequestGlobalMemoryDump_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::RequestArgs_Data> args;

 private:
  Coordinator_RequestGlobalMemoryDump_Params_Data();
  ~Coordinator_RequestGlobalMemoryDump_Params_Data() = delete;
};
static_assert(sizeof(Coordinator_RequestGlobalMemoryDump_Params_Data) == 16,
              "Bad sizeof(Coordinator_RequestGlobalMemoryDump_Params_Data)");
class MOJOM_SHARED_SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT Coordinator_RequestGlobalMemoryDump_ResponseParams_Data {
 public:
  static Coordinator_RequestGlobalMemoryDump_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t success : 1;
  uint8_t pad0_[7];
  uint64_t dump_id;
  mojo::internal::Pointer<internal::GlobalMemoryDump_Data> global_memory_dump;

 private:
  Coordinator_RequestGlobalMemoryDump_ResponseParams_Data();
  ~Coordinator_RequestGlobalMemoryDump_ResponseParams_Data() = delete;
};
static_assert(sizeof(Coordinator_RequestGlobalMemoryDump_ResponseParams_Data) == 32,
              "Bad sizeof(Coordinator_RequestGlobalMemoryDump_ResponseParams_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace memory_instrumentation

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_MEMORY_INSTRUMENTATION_MEMORY_INSTRUMENTATION_MOJOM_SHARED_INTERNAL_H_