// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_MEMORY_INSTRUMENTATION_MEMORY_INSTRUMENTATION_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_MEMORY_INSTRUMENTATION_MEMORY_INSTRUMENTATION_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/interfaces/memory_instrumentation/memory_instrumentation.mojom-shared.h"
#include "mojo/common/process_id.mojom.h"
#include <string>
#include <vector>
#include "base/trace_event/memory_dump_request_args.h"
#include "base/trace_event/process_memory_totals.h"
#include "services/resource_coordinator/public/cpp/resource_coordinator_export.h"


namespace memory_instrumentation {
namespace mojom {
class ClientProcess;
using ClientProcessPtr = mojo::InterfacePtr<ClientProcess>;
using ClientProcessPtrInfo = mojo::InterfacePtrInfo<ClientProcess>;
using ThreadSafeClientProcessPtr =
    mojo::ThreadSafeInterfacePtr<ClientProcess>;
using ClientProcessRequest = mojo::InterfaceRequest<ClientProcess>;
using ClientProcessAssociatedPtr =
    mojo::AssociatedInterfacePtr<ClientProcess>;
using ThreadSafeClientProcessAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ClientProcess>;
using ClientProcessAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ClientProcess>;
using ClientProcessAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ClientProcess>;

class Coordinator;
using CoordinatorPtr = mojo::InterfacePtr<Coordinator>;
using CoordinatorPtrInfo = mojo::InterfacePtrInfo<Coordinator>;
using ThreadSafeCoordinatorPtr =
    mojo::ThreadSafeInterfacePtr<Coordinator>;
using CoordinatorRequest = mojo::InterfaceRequest<Coordinator>;
using CoordinatorAssociatedPtr =
    mojo::AssociatedInterfacePtr<Coordinator>;
using ThreadSafeCoordinatorAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Coordinator>;
using CoordinatorAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Coordinator>;
using CoordinatorAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Coordinator>;

class RequestArgs;
using RequestArgsPtr = mojo::InlinedStructPtr<RequestArgs>;

class ChromeMemDump;
using ChromeMemDumpPtr = mojo::StructPtr<ChromeMemDump>;

class PlatformPrivateFootprint;
using PlatformPrivateFootprintPtr = mojo::StructPtr<PlatformPrivateFootprint>;

class RawOSMemDump;
using RawOSMemDumpPtr = mojo::StructPtr<RawOSMemDump>;

class RawProcessMemoryDump;
using RawProcessMemoryDumpPtr = mojo::StructPtr<RawProcessMemoryDump>;

class OSMemDump;
using OSMemDumpPtr = mojo::InlinedStructPtr<OSMemDump>;

class ProcessMemoryDump;
using ProcessMemoryDumpPtr = mojo::StructPtr<ProcessMemoryDump>;

class GlobalMemoryDump;
using GlobalMemoryDumpPtr = mojo::StructPtr<GlobalMemoryDump>;


class ClientProcessProxy;

template <typename ImplRefTraits>
class ClientProcessStub;

class ClientProcessRequestValidator;
class ClientProcessResponseValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ClientProcess
    : public ClientProcessInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ClientProcessProxy;

  template <typename ImplRefTraits>
  using Stub_ = ClientProcessStub<ImplRefTraits>;

  using RequestValidator_ = ClientProcessRequestValidator;
  using ResponseValidator_ = ClientProcessResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRequestProcessMemoryDumpMinVersion = 0,
    kRequestOSMemoryDumpMinVersion = 0,
  };
  virtual ~ClientProcess() {}


  using RequestProcessMemoryDumpCallback = base::Callback<void(bool, uint64_t, RawProcessMemoryDumpPtr)>;
  virtual void RequestProcessMemoryDump(const base::trace_event::MemoryDumpRequestArgs& args, const RequestProcessMemoryDumpCallback& callback) = 0;


  using RequestOSMemoryDumpCallback = base::Callback<void(bool, const std::unordered_map<base::ProcessId, base::trace_event::MemoryDumpCallbackResult::OSMemDump>&)>;
  virtual void RequestOSMemoryDump(const std::vector<base::ProcessId>& pids, const RequestOSMemoryDumpCallback& callback) = 0;
};

class CoordinatorProxy;

template <typename ImplRefTraits>
class CoordinatorStub;

class CoordinatorRequestValidator;
class CoordinatorResponseValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT Coordinator
    : public CoordinatorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CoordinatorProxy;

  template <typename ImplRefTraits>
  using Stub_ = CoordinatorStub<ImplRefTraits>;

  using RequestValidator_ = CoordinatorRequestValidator;
  using ResponseValidator_ = CoordinatorResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRegisterClientProcessMinVersion = 0,
    kRequestGlobalMemoryDumpMinVersion = 0,
  };
  virtual ~Coordinator() {}

  virtual void RegisterClientProcess(ClientProcessPtr client_process, ProcessType process_type) = 0;


  using RequestGlobalMemoryDumpCallback = base::Callback<void(bool, uint64_t, GlobalMemoryDumpPtr)>;
  virtual void RequestGlobalMemoryDump(const base::trace_event::MemoryDumpRequestArgs& args, const RequestGlobalMemoryDumpCallback& callback) = 0;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ClientProcessProxy
    : public ClientProcess {
 public:
  explicit ClientProcessProxy(mojo::MessageReceiverWithResponder* receiver);
  void RequestProcessMemoryDump(const base::trace_event::MemoryDumpRequestArgs& args, const RequestProcessMemoryDumpCallback& callback) override;
  void RequestOSMemoryDump(const std::vector<base::ProcessId>& pids, const RequestOSMemoryDumpCallback& callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinatorProxy
    : public Coordinator {
 public:
  explicit CoordinatorProxy(mojo::MessageReceiverWithResponder* receiver);
  void RegisterClientProcess(ClientProcessPtr client_process, ProcessType process_type) override;
  void RequestGlobalMemoryDump(const base::trace_event::MemoryDumpRequestArgs& args, const RequestGlobalMemoryDumpCallback& callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ClientProcessStubDispatch {
 public:
  static bool Accept(ClientProcess* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ClientProcess* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ClientProcess>>
class ClientProcessStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ClientProcessStub() {}
  ~ClientProcessStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ClientProcessStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ClientProcessStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinatorStubDispatch {
 public:
  static bool Accept(Coordinator* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Coordinator* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Coordinator>>
class CoordinatorStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CoordinatorStub() {}
  ~CoordinatorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinatorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinatorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ClientProcessRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinatorRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ClientProcessResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinatorResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};


class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT RequestArgs {
 public:
  using DataView = RequestArgsDataView;
  using Data_ = internal::RequestArgs_Data;

  template <typename... Args>
  static RequestArgsPtr New(Args&&... args) {
    return RequestArgsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static RequestArgsPtr From(const U& u) {
    return mojo::TypeConverter<RequestArgsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, RequestArgs>::Convert(*this);
  }


  RequestArgs();

  RequestArgs(
      uint64_t dump_guid,
      base::trace_event::MemoryDumpType dump_type,
      base::trace_event::MemoryDumpLevelOfDetail level_of_detail);

  ~RequestArgs();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = RequestArgsPtr>
  RequestArgsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, RequestArgs>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        RequestArgs::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        RequestArgs::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint64_t dump_guid;
  base::trace_event::MemoryDumpType dump_type;
  base::trace_event::MemoryDumpLevelOfDetail level_of_detail;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};







class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT OSMemDump {
 public:
  using DataView = OSMemDumpDataView;
  using Data_ = internal::OSMemDump_Data;

  template <typename... Args>
  static OSMemDumpPtr New(Args&&... args) {
    return OSMemDumpPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static OSMemDumpPtr From(const U& u) {
    return mojo::TypeConverter<OSMemDumpPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, OSMemDump>::Convert(*this);
  }


  OSMemDump();

  OSMemDump(
      uint32_t resident_set_kb,
      uint32_t private_footprint_kb);

  ~OSMemDump();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = OSMemDumpPtr>
  OSMemDumpPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, OSMemDump>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        OSMemDump::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        OSMemDump::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint32_t resident_set_kb;
  uint32_t private_footprint_kb;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};








class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ChromeMemDump {
 public:
  using DataView = ChromeMemDumpDataView;
  using Data_ = internal::ChromeMemDump_Data;

  template <typename... Args>
  static ChromeMemDumpPtr New(Args&&... args) {
    return ChromeMemDumpPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ChromeMemDumpPtr From(const U& u) {
    return mojo::TypeConverter<ChromeMemDumpPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ChromeMemDump>::Convert(*this);
  }


  ChromeMemDump();

  ChromeMemDump(
      uint32_t malloc_total_kb,
      uint32_t command_buffer_total_kb,
      uint32_t partition_alloc_total_kb,
      uint32_t blink_gc_total_kb,
      uint32_t v8_total_kb);

  ~ChromeMemDump();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ChromeMemDumpPtr>
  ChromeMemDumpPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ChromeMemDump>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ChromeMemDump::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ChromeMemDump::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint32_t malloc_total_kb;
  uint32_t command_buffer_total_kb;
  uint32_t partition_alloc_total_kb;
  uint32_t blink_gc_total_kb;
  uint32_t v8_total_kb;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT PlatformPrivateFootprint {
 public:
  using DataView = PlatformPrivateFootprintDataView;
  using Data_ = internal::PlatformPrivateFootprint_Data;

  template <typename... Args>
  static PlatformPrivateFootprintPtr New(Args&&... args) {
    return PlatformPrivateFootprintPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PlatformPrivateFootprintPtr From(const U& u) {
    return mojo::TypeConverter<PlatformPrivateFootprintPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PlatformPrivateFootprint>::Convert(*this);
  }


  PlatformPrivateFootprint();

  PlatformPrivateFootprint(
      uint64_t phys_footprint_bytes,
      uint64_t internal_bytes,
      uint64_t compressed_bytes,
      uint64_t rss_anon_bytes,
      uint64_t vm_swap_bytes,
      uint64_t private_bytes);

  ~PlatformPrivateFootprint();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PlatformPrivateFootprintPtr>
  PlatformPrivateFootprintPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PlatformPrivateFootprint>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PlatformPrivateFootprint::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PlatformPrivateFootprint::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint64_t phys_footprint_bytes;
  uint64_t internal_bytes;
  uint64_t compressed_bytes;
  uint64_t rss_anon_bytes;
  uint64_t vm_swap_bytes;
  uint64_t private_bytes;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT RawOSMemDump {
 public:
  using DataView = RawOSMemDumpDataView;
  using Data_ = internal::RawOSMemDump_Data;

  template <typename... Args>
  static RawOSMemDumpPtr New(Args&&... args) {
    return RawOSMemDumpPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static RawOSMemDumpPtr From(const U& u) {
    return mojo::TypeConverter<RawOSMemDumpPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, RawOSMemDump>::Convert(*this);
  }


  RawOSMemDump();

  RawOSMemDump(
      uint32_t resident_set_kb,
      const base::trace_event::ProcessMemoryTotals::PlatformPrivateFootprint& platform_private_footprint);

  ~RawOSMemDump();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = RawOSMemDumpPtr>
  RawOSMemDumpPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, RawOSMemDump>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        RawOSMemDump::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        RawOSMemDump::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint32_t resident_set_kb;
  base::trace_event::ProcessMemoryTotals::PlatformPrivateFootprint platform_private_footprint;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT RawProcessMemoryDump {
 public:
  using DataView = RawProcessMemoryDumpDataView;
  using Data_ = internal::RawProcessMemoryDump_Data;

  template <typename... Args>
  static RawProcessMemoryDumpPtr New(Args&&... args) {
    return RawProcessMemoryDumpPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static RawProcessMemoryDumpPtr From(const U& u) {
    return mojo::TypeConverter<RawProcessMemoryDumpPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, RawProcessMemoryDump>::Convert(*this);
  }


  RawProcessMemoryDump();

  RawProcessMemoryDump(
      const base::trace_event::MemoryDumpCallbackResult::OSMemDump& os_dump,
      const base::trace_event::MemoryDumpCallbackResult::ChromeMemDump& chrome_dump,
      const std::unordered_map<base::ProcessId, base::trace_event::MemoryDumpCallbackResult::OSMemDump>& extra_processes_dumps);

  ~RawProcessMemoryDump();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = RawProcessMemoryDumpPtr>
  RawProcessMemoryDumpPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, RawProcessMemoryDump>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        RawProcessMemoryDump::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        RawProcessMemoryDump::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::trace_event::MemoryDumpCallbackResult::OSMemDump os_dump;
  base::trace_event::MemoryDumpCallbackResult::ChromeMemDump chrome_dump;
  std::unordered_map<base::ProcessId, base::trace_event::MemoryDumpCallbackResult::OSMemDump> extra_processes_dumps;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ProcessMemoryDump {
 public:
  using DataView = ProcessMemoryDumpDataView;
  using Data_ = internal::ProcessMemoryDump_Data;

  template <typename... Args>
  static ProcessMemoryDumpPtr New(Args&&... args) {
    return ProcessMemoryDumpPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ProcessMemoryDumpPtr From(const U& u) {
    return mojo::TypeConverter<ProcessMemoryDumpPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ProcessMemoryDump>::Convert(*this);
  }


  ProcessMemoryDump();

  ProcessMemoryDump(
      ProcessType process_type,
      OSMemDumpPtr os_dump,
      const base::trace_event::MemoryDumpCallbackResult::ChromeMemDump& chrome_dump);

  ~ProcessMemoryDump();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ProcessMemoryDumpPtr>
  ProcessMemoryDumpPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ProcessMemoryDump>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ProcessMemoryDump::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ProcessMemoryDump::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  ProcessType process_type;
  OSMemDumpPtr os_dump;
  base::trace_event::MemoryDumpCallbackResult::ChromeMemDump chrome_dump;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(ProcessMemoryDump);
};



class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT GlobalMemoryDump {
 public:
  using DataView = GlobalMemoryDumpDataView;
  using Data_ = internal::GlobalMemoryDump_Data;

  template <typename... Args>
  static GlobalMemoryDumpPtr New(Args&&... args) {
    return GlobalMemoryDumpPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static GlobalMemoryDumpPtr From(const U& u) {
    return mojo::TypeConverter<GlobalMemoryDumpPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, GlobalMemoryDump>::Convert(*this);
  }


  GlobalMemoryDump();

  explicit GlobalMemoryDump(
      std::vector<ProcessMemoryDumpPtr> process_dumps);

  ~GlobalMemoryDump();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = GlobalMemoryDumpPtr>
  GlobalMemoryDumpPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, GlobalMemoryDump>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        GlobalMemoryDump::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        GlobalMemoryDump::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::vector<ProcessMemoryDumpPtr> process_dumps;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(GlobalMemoryDump);
};


template <typename StructPtrType>
RequestArgsPtr RequestArgs::Clone() const {
  return New(
      mojo::Clone(dump_guid),
      mojo::Clone(dump_type),
      mojo::Clone(level_of_detail)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, RequestArgs>::value>::type*>
bool RequestArgs::Equals(const T& other) const {
  if (!mojo::Equals(this->dump_guid, other.dump_guid))
    return false;
  if (!mojo::Equals(this->dump_type, other.dump_type))
    return false;
  if (!mojo::Equals(this->level_of_detail, other.level_of_detail))
    return false;
  return true;
}
template <typename StructPtrType>
ChromeMemDumpPtr ChromeMemDump::Clone() const {
  return New(
      mojo::Clone(malloc_total_kb),
      mojo::Clone(command_buffer_total_kb),
      mojo::Clone(partition_alloc_total_kb),
      mojo::Clone(blink_gc_total_kb),
      mojo::Clone(v8_total_kb)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ChromeMemDump>::value>::type*>
bool ChromeMemDump::Equals(const T& other) const {
  if (!mojo::Equals(this->malloc_total_kb, other.malloc_total_kb))
    return false;
  if (!mojo::Equals(this->command_buffer_total_kb, other.command_buffer_total_kb))
    return false;
  if (!mojo::Equals(this->partition_alloc_total_kb, other.partition_alloc_total_kb))
    return false;
  if (!mojo::Equals(this->blink_gc_total_kb, other.blink_gc_total_kb))
    return false;
  if (!mojo::Equals(this->v8_total_kb, other.v8_total_kb))
    return false;
  return true;
}
template <typename StructPtrType>
PlatformPrivateFootprintPtr PlatformPrivateFootprint::Clone() const {
  return New(
      mojo::Clone(phys_footprint_bytes),
      mojo::Clone(internal_bytes),
      mojo::Clone(compressed_bytes),
      mojo::Clone(rss_anon_bytes),
      mojo::Clone(vm_swap_bytes),
      mojo::Clone(private_bytes)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PlatformPrivateFootprint>::value>::type*>
bool PlatformPrivateFootprint::Equals(const T& other) const {
  if (!mojo::Equals(this->phys_footprint_bytes, other.phys_footprint_bytes))
    return false;
  if (!mojo::Equals(this->internal_bytes, other.internal_bytes))
    return false;
  if (!mojo::Equals(this->compressed_bytes, other.compressed_bytes))
    return false;
  if (!mojo::Equals(this->rss_anon_bytes, other.rss_anon_bytes))
    return false;
  if (!mojo::Equals(this->vm_swap_bytes, other.vm_swap_bytes))
    return false;
  if (!mojo::Equals(this->private_bytes, other.private_bytes))
    return false;
  return true;
}
template <typename StructPtrType>
RawOSMemDumpPtr RawOSMemDump::Clone() const {
  return New(
      mojo::Clone(resident_set_kb),
      mojo::Clone(platform_private_footprint)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, RawOSMemDump>::value>::type*>
bool RawOSMemDump::Equals(const T& other) const {
  if (!mojo::Equals(this->resident_set_kb, other.resident_set_kb))
    return false;
  if (!mojo::Equals(this->platform_private_footprint, other.platform_private_footprint))
    return false;
  return true;
}
template <typename StructPtrType>
RawProcessMemoryDumpPtr RawProcessMemoryDump::Clone() const {
  return New(
      mojo::Clone(os_dump),
      mojo::Clone(chrome_dump),
      mojo::Clone(extra_processes_dumps)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, RawProcessMemoryDump>::value>::type*>
bool RawProcessMemoryDump::Equals(const T& other) const {
  if (!mojo::Equals(this->os_dump, other.os_dump))
    return false;
  if (!mojo::Equals(this->chrome_dump, other.chrome_dump))
    return false;
  if (!mojo::Equals(this->extra_processes_dumps, other.extra_processes_dumps))
    return false;
  return true;
}
template <typename StructPtrType>
OSMemDumpPtr OSMemDump::Clone() const {
  return New(
      mojo::Clone(resident_set_kb),
      mojo::Clone(private_footprint_kb)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, OSMemDump>::value>::type*>
bool OSMemDump::Equals(const T& other) const {
  if (!mojo::Equals(this->resident_set_kb, other.resident_set_kb))
    return false;
  if (!mojo::Equals(this->private_footprint_kb, other.private_footprint_kb))
    return false;
  return true;
}
template <typename StructPtrType>
ProcessMemoryDumpPtr ProcessMemoryDump::Clone() const {
  return New(
      mojo::Clone(process_type),
      mojo::Clone(os_dump),
      mojo::Clone(chrome_dump)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ProcessMemoryDump>::value>::type*>
bool ProcessMemoryDump::Equals(const T& other) const {
  if (!mojo::Equals(this->process_type, other.process_type))
    return false;
  if (!mojo::Equals(this->os_dump, other.os_dump))
    return false;
  if (!mojo::Equals(this->chrome_dump, other.chrome_dump))
    return false;
  return true;
}
template <typename StructPtrType>
GlobalMemoryDumpPtr GlobalMemoryDump::Clone() const {
  return New(
      mojo::Clone(process_dumps)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, GlobalMemoryDump>::value>::type*>
bool GlobalMemoryDump::Equals(const T& other) const {
  if (!mojo::Equals(this->process_dumps, other.process_dumps))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace memory_instrumentation

namespace mojo {


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::memory_instrumentation::mojom::RequestArgs::DataView,
                                         ::memory_instrumentation::mojom::RequestArgsPtr> {
  static bool IsNull(const ::memory_instrumentation::mojom::RequestArgsPtr& input) { return !input; }
  static void SetToNull(::memory_instrumentation::mojom::RequestArgsPtr* output) { output->reset(); }

  static decltype(::memory_instrumentation::mojom::RequestArgs::dump_guid) dump_guid(
      const ::memory_instrumentation::mojom::RequestArgsPtr& input) {
    return input->dump_guid;
  }

  static decltype(::memory_instrumentation::mojom::RequestArgs::dump_type) dump_type(
      const ::memory_instrumentation::mojom::RequestArgsPtr& input) {
    return input->dump_type;
  }

  static decltype(::memory_instrumentation::mojom::RequestArgs::level_of_detail) level_of_detail(
      const ::memory_instrumentation::mojom::RequestArgsPtr& input) {
    return input->level_of_detail;
  }

  static bool Read(::memory_instrumentation::mojom::RequestArgs::DataView input, ::memory_instrumentation::mojom::RequestArgsPtr* output);
};


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::memory_instrumentation::mojom::ChromeMemDump::DataView,
                                         ::memory_instrumentation::mojom::ChromeMemDumpPtr> {
  static bool IsNull(const ::memory_instrumentation::mojom::ChromeMemDumpPtr& input) { return !input; }
  static void SetToNull(::memory_instrumentation::mojom::ChromeMemDumpPtr* output) { output->reset(); }

  static decltype(::memory_instrumentation::mojom::ChromeMemDump::malloc_total_kb) malloc_total_kb(
      const ::memory_instrumentation::mojom::ChromeMemDumpPtr& input) {
    return input->malloc_total_kb;
  }

  static decltype(::memory_instrumentation::mojom::ChromeMemDump::command_buffer_total_kb) command_buffer_total_kb(
      const ::memory_instrumentation::mojom::ChromeMemDumpPtr& input) {
    return input->command_buffer_total_kb;
  }

  static decltype(::memory_instrumentation::mojom::ChromeMemDump::partition_alloc_total_kb) partition_alloc_total_kb(
      const ::memory_instrumentation::mojom::ChromeMemDumpPtr& input) {
    return input->partition_alloc_total_kb;
  }

  static decltype(::memory_instrumentation::mojom::ChromeMemDump::blink_gc_total_kb) blink_gc_total_kb(
      const ::memory_instrumentation::mojom::ChromeMemDumpPtr& input) {
    return input->blink_gc_total_kb;
  }

  static decltype(::memory_instrumentation::mojom::ChromeMemDump::v8_total_kb) v8_total_kb(
      const ::memory_instrumentation::mojom::ChromeMemDumpPtr& input) {
    return input->v8_total_kb;
  }

  static bool Read(::memory_instrumentation::mojom::ChromeMemDump::DataView input, ::memory_instrumentation::mojom::ChromeMemDumpPtr* output);
};


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::memory_instrumentation::mojom::PlatformPrivateFootprint::DataView,
                                         ::memory_instrumentation::mojom::PlatformPrivateFootprintPtr> {
  static bool IsNull(const ::memory_instrumentation::mojom::PlatformPrivateFootprintPtr& input) { return !input; }
  static void SetToNull(::memory_instrumentation::mojom::PlatformPrivateFootprintPtr* output) { output->reset(); }

  static decltype(::memory_instrumentation::mojom::PlatformPrivateFootprint::phys_footprint_bytes) phys_footprint_bytes(
      const ::memory_instrumentation::mojom::PlatformPrivateFootprintPtr& input) {
    return input->phys_footprint_bytes;
  }

  static decltype(::memory_instrumentation::mojom::PlatformPrivateFootprint::internal_bytes) internal_bytes(
      const ::memory_instrumentation::mojom::PlatformPrivateFootprintPtr& input) {
    return input->internal_bytes;
  }

  static decltype(::memory_instrumentation::mojom::PlatformPrivateFootprint::compressed_bytes) compressed_bytes(
      const ::memory_instrumentation::mojom::PlatformPrivateFootprintPtr& input) {
    return input->compressed_bytes;
  }

  static decltype(::memory_instrumentation::mojom::PlatformPrivateFootprint::rss_anon_bytes) rss_anon_bytes(
      const ::memory_instrumentation::mojom::PlatformPrivateFootprintPtr& input) {
    return input->rss_anon_bytes;
  }

  static decltype(::memory_instrumentation::mojom::PlatformPrivateFootprint::vm_swap_bytes) vm_swap_bytes(
      const ::memory_instrumentation::mojom::PlatformPrivateFootprintPtr& input) {
    return input->vm_swap_bytes;
  }

  static decltype(::memory_instrumentation::mojom::PlatformPrivateFootprint::private_bytes) private_bytes(
      const ::memory_instrumentation::mojom::PlatformPrivateFootprintPtr& input) {
    return input->private_bytes;
  }

  static bool Read(::memory_instrumentation::mojom::PlatformPrivateFootprint::DataView input, ::memory_instrumentation::mojom::PlatformPrivateFootprintPtr* output);
};


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::memory_instrumentation::mojom::RawOSMemDump::DataView,
                                         ::memory_instrumentation::mojom::RawOSMemDumpPtr> {
  static bool IsNull(const ::memory_instrumentation::mojom::RawOSMemDumpPtr& input) { return !input; }
  static void SetToNull(::memory_instrumentation::mojom::RawOSMemDumpPtr* output) { output->reset(); }

  static decltype(::memory_instrumentation::mojom::RawOSMemDump::resident_set_kb) resident_set_kb(
      const ::memory_instrumentation::mojom::RawOSMemDumpPtr& input) {
    return input->resident_set_kb;
  }

  static const decltype(::memory_instrumentation::mojom::RawOSMemDump::platform_private_footprint)& platform_private_footprint(
      const ::memory_instrumentation::mojom::RawOSMemDumpPtr& input) {
    return input->platform_private_footprint;
  }

  static bool Read(::memory_instrumentation::mojom::RawOSMemDump::DataView input, ::memory_instrumentation::mojom::RawOSMemDumpPtr* output);
};


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::memory_instrumentation::mojom::RawProcessMemoryDump::DataView,
                                         ::memory_instrumentation::mojom::RawProcessMemoryDumpPtr> {
  static bool IsNull(const ::memory_instrumentation::mojom::RawProcessMemoryDumpPtr& input) { return !input; }
  static void SetToNull(::memory_instrumentation::mojom::RawProcessMemoryDumpPtr* output) { output->reset(); }

  static const decltype(::memory_instrumentation::mojom::RawProcessMemoryDump::os_dump)& os_dump(
      const ::memory_instrumentation::mojom::RawProcessMemoryDumpPtr& input) {
    return input->os_dump;
  }

  static const decltype(::memory_instrumentation::mojom::RawProcessMemoryDump::chrome_dump)& chrome_dump(
      const ::memory_instrumentation::mojom::RawProcessMemoryDumpPtr& input) {
    return input->chrome_dump;
  }

  static const decltype(::memory_instrumentation::mojom::RawProcessMemoryDump::extra_processes_dumps)& extra_processes_dumps(
      const ::memory_instrumentation::mojom::RawProcessMemoryDumpPtr& input) {
    return input->extra_processes_dumps;
  }

  static bool Read(::memory_instrumentation::mojom::RawProcessMemoryDump::DataView input, ::memory_instrumentation::mojom::RawProcessMemoryDumpPtr* output);
};


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::memory_instrumentation::mojom::OSMemDump::DataView,
                                         ::memory_instrumentation::mojom::OSMemDumpPtr> {
  static bool IsNull(const ::memory_instrumentation::mojom::OSMemDumpPtr& input) { return !input; }
  static void SetToNull(::memory_instrumentation::mojom::OSMemDumpPtr* output) { output->reset(); }

  static decltype(::memory_instrumentation::mojom::OSMemDump::resident_set_kb) resident_set_kb(
      const ::memory_instrumentation::mojom::OSMemDumpPtr& input) {
    return input->resident_set_kb;
  }

  static decltype(::memory_instrumentation::mojom::OSMemDump::private_footprint_kb) private_footprint_kb(
      const ::memory_instrumentation::mojom::OSMemDumpPtr& input) {
    return input->private_footprint_kb;
  }

  static bool Read(::memory_instrumentation::mojom::OSMemDump::DataView input, ::memory_instrumentation::mojom::OSMemDumpPtr* output);
};


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::memory_instrumentation::mojom::ProcessMemoryDump::DataView,
                                         ::memory_instrumentation::mojom::ProcessMemoryDumpPtr> {
  static bool IsNull(const ::memory_instrumentation::mojom::ProcessMemoryDumpPtr& input) { return !input; }
  static void SetToNull(::memory_instrumentation::mojom::ProcessMemoryDumpPtr* output) { output->reset(); }

  static decltype(::memory_instrumentation::mojom::ProcessMemoryDump::process_type) process_type(
      const ::memory_instrumentation::mojom::ProcessMemoryDumpPtr& input) {
    return input->process_type;
  }

  static const decltype(::memory_instrumentation::mojom::ProcessMemoryDump::os_dump)& os_dump(
      const ::memory_instrumentation::mojom::ProcessMemoryDumpPtr& input) {
    return input->os_dump;
  }

  static const decltype(::memory_instrumentation::mojom::ProcessMemoryDump::chrome_dump)& chrome_dump(
      const ::memory_instrumentation::mojom::ProcessMemoryDumpPtr& input) {
    return input->chrome_dump;
  }

  static bool Read(::memory_instrumentation::mojom::ProcessMemoryDump::DataView input, ::memory_instrumentation::mojom::ProcessMemoryDumpPtr* output);
};


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::memory_instrumentation::mojom::GlobalMemoryDump::DataView,
                                         ::memory_instrumentation::mojom::GlobalMemoryDumpPtr> {
  static bool IsNull(const ::memory_instrumentation::mojom::GlobalMemoryDumpPtr& input) { return !input; }
  static void SetToNull(::memory_instrumentation::mojom::GlobalMemoryDumpPtr* output) { output->reset(); }

  static const decltype(::memory_instrumentation::mojom::GlobalMemoryDump::process_dumps)& process_dumps(
      const ::memory_instrumentation::mojom::GlobalMemoryDumpPtr& input) {
    return input->process_dumps;
  }

  static bool Read(::memory_instrumentation::mojom::GlobalMemoryDump::DataView input, ::memory_instrumentation::mojom::GlobalMemoryDumpPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_MEMORY_INSTRUMENTATION_MEMORY_INSTRUMENTATION_MOJOM_H_