// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "services/resource_coordinator/public/interfaces/tracing/tracing.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace tracing {
namespace mojom {

namespace internal {


// static
AgentRegistry_RegisterAgent_Params_Data* AgentRegistry_RegisterAgent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(AgentRegistry_RegisterAgent_Params_Data))) AgentRegistry_RegisterAgent_Params_Data();
}

// static
bool AgentRegistry_RegisterAgent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const AgentRegistry_RegisterAgent_Params_Data* object = static_cast<const AgentRegistry_RegisterAgent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->agent,
          "invalid agent field in AgentRegistry_RegisterAgent_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->agent,
                                                 validation_context)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->label, "null label field in AgentRegistry_RegisterAgent_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams label_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->label, validation_context,
                                         &label_validate_params)) {
    return false;
  }


  if (!::tracing::mojom::internal::TraceDataType_Data
        ::Validate(object->type, validation_context))
    return false;

  return true;
}

AgentRegistry_RegisterAgent_Params_Data::AgentRegistry_RegisterAgent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Agent_StartTracing_Params_Data* Agent_StartTracing_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Agent_StartTracing_Params_Data))) Agent_StartTracing_Params_Data();
}

// static
bool Agent_StartTracing_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Agent_StartTracing_Params_Data* object = static_cast<const Agent_StartTracing_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->config, "null config field in Agent_StartTracing_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams config_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->config, validation_context,
                                         &config_validate_params)) {
    return false;
  }

  return true;
}

Agent_StartTracing_Params_Data::Agent_StartTracing_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Agent_StartTracing_ResponseParams_Data* Agent_StartTracing_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Agent_StartTracing_ResponseParams_Data))) Agent_StartTracing_ResponseParams_Data();
}

// static
bool Agent_StartTracing_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Agent_StartTracing_ResponseParams_Data* object = static_cast<const Agent_StartTracing_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Agent_StartTracing_ResponseParams_Data::Agent_StartTracing_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
Agent_StopAndFlush_Params_Data* Agent_StopAndFlush_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Agent_StopAndFlush_Params_Data))) Agent_StopAndFlush_Params_Data();
}

// static
bool Agent_StopAndFlush_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Agent_StopAndFlush_Params_Data* object = static_cast<const Agent_StopAndFlush_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->recorder,
          "invalid recorder field in Agent_StopAndFlush_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->recorder,
                                                 validation_context)) {
    return false;
  }

  return true;
}

Agent_StopAndFlush_Params_Data::Agent_StopAndFlush_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Agent_RequestClockSyncMarker_Params_Data* Agent_RequestClockSyncMarker_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Agent_RequestClockSyncMarker_Params_Data))) Agent_RequestClockSyncMarker_Params_Data();
}

// static
bool Agent_RequestClockSyncMarker_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Agent_RequestClockSyncMarker_Params_Data* object = static_cast<const Agent_RequestClockSyncMarker_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->sync_id, "null sync_id field in Agent_RequestClockSyncMarker_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams sync_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->sync_id, validation_context,
                                         &sync_id_validate_params)) {
    return false;
  }

  return true;
}

Agent_RequestClockSyncMarker_Params_Data::Agent_RequestClockSyncMarker_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Agent_RequestClockSyncMarker_ResponseParams_Data* Agent_RequestClockSyncMarker_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Agent_RequestClockSyncMarker_ResponseParams_Data))) Agent_RequestClockSyncMarker_ResponseParams_Data();
}

// static
bool Agent_RequestClockSyncMarker_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Agent_RequestClockSyncMarker_ResponseParams_Data* object = static_cast<const Agent_RequestClockSyncMarker_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->issue_ts, "null issue_ts field in Agent_RequestClockSyncMarker_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->issue_ts, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->issue_end_ts, "null issue_end_ts field in Agent_RequestClockSyncMarker_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->issue_end_ts, validation_context))
    return false;

  return true;
}

Agent_RequestClockSyncMarker_ResponseParams_Data::Agent_RequestClockSyncMarker_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
Agent_RequestBufferStatus_Params_Data* Agent_RequestBufferStatus_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Agent_RequestBufferStatus_Params_Data))) Agent_RequestBufferStatus_Params_Data();
}

// static
bool Agent_RequestBufferStatus_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Agent_RequestBufferStatus_Params_Data* object = static_cast<const Agent_RequestBufferStatus_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Agent_RequestBufferStatus_Params_Data::Agent_RequestBufferStatus_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Agent_RequestBufferStatus_ResponseParams_Data* Agent_RequestBufferStatus_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Agent_RequestBufferStatus_ResponseParams_Data))) Agent_RequestBufferStatus_ResponseParams_Data();
}

// static
bool Agent_RequestBufferStatus_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Agent_RequestBufferStatus_ResponseParams_Data* object = static_cast<const Agent_RequestBufferStatus_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Agent_RequestBufferStatus_ResponseParams_Data::Agent_RequestBufferStatus_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
Agent_GetCategories_Params_Data* Agent_GetCategories_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Agent_GetCategories_Params_Data))) Agent_GetCategories_Params_Data();
}

// static
bool Agent_GetCategories_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Agent_GetCategories_Params_Data* object = static_cast<const Agent_GetCategories_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Agent_GetCategories_Params_Data::Agent_GetCategories_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Agent_GetCategories_ResponseParams_Data* Agent_GetCategories_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Agent_GetCategories_ResponseParams_Data))) Agent_GetCategories_ResponseParams_Data();
}

// static
bool Agent_GetCategories_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Agent_GetCategories_ResponseParams_Data* object = static_cast<const Agent_GetCategories_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->categories, "null categories field in Agent_GetCategories_ResponseParams",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams categories_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->categories, validation_context,
                                         &categories_validate_params)) {
    return false;
  }

  return true;
}

Agent_GetCategories_ResponseParams_Data::Agent_GetCategories_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
Recorder_AddChunk_Params_Data* Recorder_AddChunk_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Recorder_AddChunk_Params_Data))) Recorder_AddChunk_Params_Data();
}

// static
bool Recorder_AddChunk_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Recorder_AddChunk_Params_Data* object = static_cast<const Recorder_AddChunk_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->chunk, "null chunk field in Recorder_AddChunk_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams chunk_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->chunk, validation_context,
                                         &chunk_validate_params)) {
    return false;
  }

  return true;
}

Recorder_AddChunk_Params_Data::Recorder_AddChunk_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Recorder_AddMetadata_Params_Data* Recorder_AddMetadata_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Recorder_AddMetadata_Params_Data))) Recorder_AddMetadata_Params_Data();
}

// static
bool Recorder_AddMetadata_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Recorder_AddMetadata_Params_Data* object = static_cast<const Recorder_AddMetadata_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->metadata, "null metadata field in Recorder_AddMetadata_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->metadata, validation_context))
    return false;

  return true;
}

Recorder_AddMetadata_Params_Data::Recorder_AddMetadata_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_StartTracing_Params_Data* Coordinator_StartTracing_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_StartTracing_Params_Data))) Coordinator_StartTracing_Params_Data();
}

// static
bool Coordinator_StartTracing_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_StartTracing_Params_Data* object = static_cast<const Coordinator_StartTracing_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->config, "null config field in Coordinator_StartTracing_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams config_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->config, validation_context,
                                         &config_validate_params)) {
    return false;
  }

  return true;
}

Coordinator_StartTracing_Params_Data::Coordinator_StartTracing_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_StartTracing_ResponseParams_Data* Coordinator_StartTracing_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_StartTracing_ResponseParams_Data))) Coordinator_StartTracing_ResponseParams_Data();
}

// static
bool Coordinator_StartTracing_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_StartTracing_ResponseParams_Data* object = static_cast<const Coordinator_StartTracing_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Coordinator_StartTracing_ResponseParams_Data::Coordinator_StartTracing_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_StopAndFlush_Params_Data* Coordinator_StopAndFlush_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_StopAndFlush_Params_Data))) Coordinator_StopAndFlush_Params_Data();
}

// static
bool Coordinator_StopAndFlush_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_StopAndFlush_Params_Data* object = static_cast<const Coordinator_StopAndFlush_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->stream,
          "invalid stream field in Coordinator_StopAndFlush_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->stream,
                                                 validation_context)) {
    return false;
  }

  return true;
}

Coordinator_StopAndFlush_Params_Data::Coordinator_StopAndFlush_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_IsTracing_Params_Data* Coordinator_IsTracing_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_IsTracing_Params_Data))) Coordinator_IsTracing_Params_Data();
}

// static
bool Coordinator_IsTracing_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_IsTracing_Params_Data* object = static_cast<const Coordinator_IsTracing_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Coordinator_IsTracing_Params_Data::Coordinator_IsTracing_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_IsTracing_ResponseParams_Data* Coordinator_IsTracing_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_IsTracing_ResponseParams_Data))) Coordinator_IsTracing_ResponseParams_Data();
}

// static
bool Coordinator_IsTracing_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_IsTracing_ResponseParams_Data* object = static_cast<const Coordinator_IsTracing_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Coordinator_IsTracing_ResponseParams_Data::Coordinator_IsTracing_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_RequestBufferUsage_Params_Data* Coordinator_RequestBufferUsage_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_RequestBufferUsage_Params_Data))) Coordinator_RequestBufferUsage_Params_Data();
}

// static
bool Coordinator_RequestBufferUsage_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_RequestBufferUsage_Params_Data* object = static_cast<const Coordinator_RequestBufferUsage_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Coordinator_RequestBufferUsage_Params_Data::Coordinator_RequestBufferUsage_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_RequestBufferUsage_ResponseParams_Data* Coordinator_RequestBufferUsage_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_RequestBufferUsage_ResponseParams_Data))) Coordinator_RequestBufferUsage_ResponseParams_Data();
}

// static
bool Coordinator_RequestBufferUsage_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_RequestBufferUsage_ResponseParams_Data* object = static_cast<const Coordinator_RequestBufferUsage_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Coordinator_RequestBufferUsage_ResponseParams_Data::Coordinator_RequestBufferUsage_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_GetCategories_Params_Data* Coordinator_GetCategories_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_GetCategories_Params_Data))) Coordinator_GetCategories_Params_Data();
}

// static
bool Coordinator_GetCategories_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_GetCategories_Params_Data* object = static_cast<const Coordinator_GetCategories_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

Coordinator_GetCategories_Params_Data::Coordinator_GetCategories_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_GetCategories_ResponseParams_Data* Coordinator_GetCategories_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_GetCategories_ResponseParams_Data))) Coordinator_GetCategories_ResponseParams_Data();
}

// static
bool Coordinator_GetCategories_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_GetCategories_ResponseParams_Data* object = static_cast<const Coordinator_GetCategories_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->categories, "null categories field in Coordinator_GetCategories_ResponseParams",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams categories_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->categories, validation_context,
                                         &categories_validate_params)) {
    return false;
  }

  return true;
}

Coordinator_GetCategories_ResponseParams_Data::Coordinator_GetCategories_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace tracing

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
