// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_TRACING_TRACING_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_TRACING_TRACING_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/interfaces/tracing/tracing.mojom-shared.h"
#include "mojo/common/time.mojom.h"
#include "mojo/common/values.mojom.h"
#include <string>
#include <vector>
#include "services/resource_coordinator/public/cpp/resource_coordinator_export.h"


namespace tracing {
namespace mojom {
class AgentRegistry;
using AgentRegistryPtr = mojo::InterfacePtr<AgentRegistry>;
using AgentRegistryPtrInfo = mojo::InterfacePtrInfo<AgentRegistry>;
using ThreadSafeAgentRegistryPtr =
    mojo::ThreadSafeInterfacePtr<AgentRegistry>;
using AgentRegistryRequest = mojo::InterfaceRequest<AgentRegistry>;
using AgentRegistryAssociatedPtr =
    mojo::AssociatedInterfacePtr<AgentRegistry>;
using ThreadSafeAgentRegistryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<AgentRegistry>;
using AgentRegistryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<AgentRegistry>;
using AgentRegistryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<AgentRegistry>;

class Agent;
using AgentPtr = mojo::InterfacePtr<Agent>;
using AgentPtrInfo = mojo::InterfacePtrInfo<Agent>;
using ThreadSafeAgentPtr =
    mojo::ThreadSafeInterfacePtr<Agent>;
using AgentRequest = mojo::InterfaceRequest<Agent>;
using AgentAssociatedPtr =
    mojo::AssociatedInterfacePtr<Agent>;
using ThreadSafeAgentAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Agent>;
using AgentAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Agent>;
using AgentAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Agent>;

class Recorder;
using RecorderPtr = mojo::InterfacePtr<Recorder>;
using RecorderPtrInfo = mojo::InterfacePtrInfo<Recorder>;
using ThreadSafeRecorderPtr =
    mojo::ThreadSafeInterfacePtr<Recorder>;
using RecorderRequest = mojo::InterfaceRequest<Recorder>;
using RecorderAssociatedPtr =
    mojo::AssociatedInterfacePtr<Recorder>;
using ThreadSafeRecorderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Recorder>;
using RecorderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Recorder>;
using RecorderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Recorder>;

class Coordinator;
using CoordinatorPtr = mojo::InterfacePtr<Coordinator>;
using CoordinatorPtrInfo = mojo::InterfacePtrInfo<Coordinator>;
using ThreadSafeCoordinatorPtr =
    mojo::ThreadSafeInterfacePtr<Coordinator>;
using CoordinatorRequest = mojo::InterfaceRequest<Coordinator>;
using CoordinatorAssociatedPtr =
    mojo::AssociatedInterfacePtr<Coordinator>;
using ThreadSafeCoordinatorAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Coordinator>;
using CoordinatorAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Coordinator>;
using CoordinatorAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Coordinator>;


class AgentRegistryProxy;

template <typename ImplRefTraits>
class AgentRegistryStub;

class AgentRegistryRequestValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT AgentRegistry
    : public AgentRegistryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AgentRegistryProxy;

  template <typename ImplRefTraits>
  using Stub_ = AgentRegistryStub<ImplRefTraits>;

  using RequestValidator_ = AgentRegistryRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kRegisterAgentMinVersion = 0,
  };
  virtual ~AgentRegistry() {}

  virtual void RegisterAgent(AgentPtr agent, const std::string& label, TraceDataType type, bool supports_explicit_clock_sync_) = 0;
};

class AgentProxy;

template <typename ImplRefTraits>
class AgentStub;

class AgentRequestValidator;
class AgentResponseValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT Agent
    : public AgentInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AgentProxy;

  template <typename ImplRefTraits>
  using Stub_ = AgentStub<ImplRefTraits>;

  using RequestValidator_ = AgentRequestValidator;
  using ResponseValidator_ = AgentResponseValidator;
  enum MethodMinVersions : uint32_t {
    kStartTracingMinVersion = 0,
    kStopAndFlushMinVersion = 0,
    kRequestClockSyncMarkerMinVersion = 0,
    kRequestBufferStatusMinVersion = 0,
    kGetCategoriesMinVersion = 0,
  };
  virtual ~Agent() {}


  using StartTracingCallback = base::Callback<void()>;
  virtual void StartTracing(const std::string& config, const StartTracingCallback& callback) = 0;

  virtual void StopAndFlush(RecorderPtr recorder) = 0;


  using RequestClockSyncMarkerCallback = base::Callback<void(base::TimeTicks, base::TimeTicks)>;
  virtual void RequestClockSyncMarker(const std::string& sync_id, const RequestClockSyncMarkerCallback& callback) = 0;


  using RequestBufferStatusCallback = base::Callback<void(uint32_t, uint32_t)>;
  virtual void RequestBufferStatus(const RequestBufferStatusCallback& callback) = 0;


  using GetCategoriesCallback = base::Callback<void(const std::string&)>;
  virtual void GetCategories(const GetCategoriesCallback& callback) = 0;
};

class RecorderProxy;

template <typename ImplRefTraits>
class RecorderStub;

class RecorderRequestValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT Recorder
    : public RecorderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = RecorderProxy;

  template <typename ImplRefTraits>
  using Stub_ = RecorderStub<ImplRefTraits>;

  using RequestValidator_ = RecorderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kAddChunkMinVersion = 0,
    kAddMetadataMinVersion = 0,
  };
  virtual ~Recorder() {}

  virtual void AddChunk(const std::string& chunk) = 0;

  virtual void AddMetadata(std::unique_ptr<base::DictionaryValue> metadata) = 0;
};

class CoordinatorProxy;

template <typename ImplRefTraits>
class CoordinatorStub;

class CoordinatorRequestValidator;
class CoordinatorResponseValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT Coordinator
    : public CoordinatorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CoordinatorProxy;

  template <typename ImplRefTraits>
  using Stub_ = CoordinatorStub<ImplRefTraits>;

  using RequestValidator_ = CoordinatorRequestValidator;
  using ResponseValidator_ = CoordinatorResponseValidator;
  enum MethodMinVersions : uint32_t {
    kStartTracingMinVersion = 0,
    kStopAndFlushMinVersion = 0,
    kIsTracingMinVersion = 0,
    kRequestBufferUsageMinVersion = 0,
    kGetCategoriesMinVersion = 0,
  };
  virtual ~Coordinator() {}


  using StartTracingCallback = base::Callback<void(bool)>;
  virtual void StartTracing(const std::string& config, const StartTracingCallback& callback) = 0;

  virtual void StopAndFlush(mojo::ScopedDataPipeProducerHandle stream) = 0;


  using IsTracingCallback = base::Callback<void(bool)>;
  virtual void IsTracing(const IsTracingCallback& callback) = 0;


  using RequestBufferUsageCallback = base::Callback<void(bool, float, uint32_t)>;
  virtual void RequestBufferUsage(const RequestBufferUsageCallback& callback) = 0;


  using GetCategoriesCallback = base::Callback<void(bool, const std::string&)>;
  virtual void GetCategories(const GetCategoriesCallback& callback) = 0;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT AgentRegistryProxy
    : public AgentRegistry {
 public:
  explicit AgentRegistryProxy(mojo::MessageReceiverWithResponder* receiver);
  void RegisterAgent(AgentPtr agent, const std::string& label, TraceDataType type, bool supports_explicit_clock_sync_) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT AgentProxy
    : public Agent {
 public:
  explicit AgentProxy(mojo::MessageReceiverWithResponder* receiver);
  void StartTracing(const std::string& config, const StartTracingCallback& callback) override;
  void StopAndFlush(RecorderPtr recorder) override;
  void RequestClockSyncMarker(const std::string& sync_id, const RequestClockSyncMarkerCallback& callback) override;
  void RequestBufferStatus(const RequestBufferStatusCallback& callback) override;
  void GetCategories(const GetCategoriesCallback& callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT RecorderProxy
    : public Recorder {
 public:
  explicit RecorderProxy(mojo::MessageReceiverWithResponder* receiver);
  void AddChunk(const std::string& chunk) override;
  void AddMetadata(std::unique_ptr<base::DictionaryValue> metadata) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinatorProxy
    : public Coordinator {
 public:
  explicit CoordinatorProxy(mojo::MessageReceiverWithResponder* receiver);
  void StartTracing(const std::string& config, const StartTracingCallback& callback) override;
  void StopAndFlush(mojo::ScopedDataPipeProducerHandle stream) override;
  void IsTracing(const IsTracingCallback& callback) override;
  void RequestBufferUsage(const RequestBufferUsageCallback& callback) override;
  void GetCategories(const GetCategoriesCallback& callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT AgentRegistryStubDispatch {
 public:
  static bool Accept(AgentRegistry* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AgentRegistry* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AgentRegistry>>
class AgentRegistryStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AgentRegistryStub() {}
  ~AgentRegistryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentRegistryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentRegistryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT AgentStubDispatch {
 public:
  static bool Accept(Agent* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Agent* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Agent>>
class AgentStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AgentStub() {}
  ~AgentStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT RecorderStubDispatch {
 public:
  static bool Accept(Recorder* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Recorder* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Recorder>>
class RecorderStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RecorderStub() {}
  ~RecorderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RecorderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RecorderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinatorStubDispatch {
 public:
  static bool Accept(Coordinator* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Coordinator* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Coordinator>>
class CoordinatorStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CoordinatorStub() {}
  ~CoordinatorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinatorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinatorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT AgentRegistryRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT AgentRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT RecorderRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinatorRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT AgentResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinatorResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace tracing

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_TRACING_TRACING_MOJOM_H_