// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_GPU_INTERFACES_GPU_MAIN_MOJOM_H_
#define SERVICES_UI_GPU_INTERFACES_GPU_MAIN_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/ui/gpu/interfaces/gpu_main.mojom-shared.h"
#include "cc/ipc/frame_sink_manager.mojom.h"
#include "gpu/ipc/common/gpu_preferences.mojom.h"
#include "services/ui/gpu/interfaces/gpu_host.mojom.h"
#include "services/ui/gpu/interfaces/gpu_service.mojom.h"
#include <string>
#include <vector>


namespace ui {
namespace mojom {
class GpuMain;
using GpuMainPtr = mojo::InterfacePtr<GpuMain>;
using GpuMainPtrInfo = mojo::InterfacePtrInfo<GpuMain>;
using ThreadSafeGpuMainPtr =
    mojo::ThreadSafeInterfacePtr<GpuMain>;
using GpuMainRequest = mojo::InterfaceRequest<GpuMain>;
using GpuMainAssociatedPtr =
    mojo::AssociatedInterfacePtr<GpuMain>;
using ThreadSafeGpuMainAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<GpuMain>;
using GpuMainAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<GpuMain>;
using GpuMainAssociatedRequest =
    mojo::AssociatedInterfaceRequest<GpuMain>;


class GpuMainProxy;

template <typename ImplRefTraits>
class GpuMainStub;

class GpuMainRequestValidator;

class  GpuMain
    : public GpuMainInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = GpuMainProxy;

  template <typename ImplRefTraits>
  using Stub_ = GpuMainStub<ImplRefTraits>;

  using RequestValidator_ = GpuMainRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateFrameSinkManagerMinVersion = 0,
    kCreateGpuServiceMinVersion = 0,
  };
  virtual ~GpuMain() {}

  virtual void CreateFrameSinkManager(::cc::mojom::FrameSinkManagerRequest frame_sink_manager, ::cc::mojom::FrameSinkManagerClientPtr frame_sink_manager_client) = 0;

  virtual void CreateGpuService(::ui::mojom::GpuServiceRequest gpu_service, ::ui::mojom::GpuHostPtr gpu_host, const gpu::GpuPreferences& preferences, mojo::ScopedSharedBufferHandle activity_flags) = 0;
};

class  GpuMainProxy
    : public GpuMain {
 public:
  explicit GpuMainProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateFrameSinkManager(::cc::mojom::FrameSinkManagerRequest frame_sink_manager, ::cc::mojom::FrameSinkManagerClientPtr frame_sink_manager_client) override;
  void CreateGpuService(::ui::mojom::GpuServiceRequest gpu_service, ::ui::mojom::GpuHostPtr gpu_host, const gpu::GpuPreferences& preferences, mojo::ScopedSharedBufferHandle activity_flags) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  GpuMainStubDispatch {
 public:
  static bool Accept(GpuMain* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      GpuMain* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<GpuMain>>
class GpuMainStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  GpuMainStub() {}
  ~GpuMainStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return GpuMainStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return GpuMainStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  GpuMainRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace ui

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_UI_GPU_INTERFACES_GPU_MAIN_MOJOM_H_