// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_GPU_MOJOM_H_
#define SERVICES_UI_PUBLIC_INTERFACES_GPU_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/ui/public/interfaces/gpu.mojom-shared.h"
#include "gpu/ipc/common/gpu_info.mojom.h"
#include "gpu/ipc/common/sync_token.mojom.h"
#include "media/gpu/mojo/jpeg_decoder.mojom.h"
#include "media/mojo/interfaces/video_encode_accelerator.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/buffer_types.mojom.h"
#include <string>
#include <vector>


namespace ui {
namespace mojom {
class Gpu;
using GpuPtr = mojo::InterfacePtr<Gpu>;
using GpuPtrInfo = mojo::InterfacePtrInfo<Gpu>;
using ThreadSafeGpuPtr =
    mojo::ThreadSafeInterfacePtr<Gpu>;
using GpuRequest = mojo::InterfaceRequest<Gpu>;
using GpuAssociatedPtr =
    mojo::AssociatedInterfacePtr<Gpu>;
using ThreadSafeGpuAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Gpu>;
using GpuAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Gpu>;
using GpuAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Gpu>;


class GpuProxy;

template <typename ImplRefTraits>
class GpuStub;

class GpuRequestValidator;
class GpuResponseValidator;

class  Gpu
    : public GpuInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = GpuProxy;

  template <typename ImplRefTraits>
  using Stub_ = GpuStub<ImplRefTraits>;

  using RequestValidator_ = GpuRequestValidator;
  using ResponseValidator_ = GpuResponseValidator;
  enum MethodMinVersions : uint32_t {
    kEstablishGpuChannelMinVersion = 0,
    kCreateJpegDecodeAcceleratorMinVersion = 0,
    kCreateVideoEncodeAcceleratorMinVersion = 0,
    kCreateGpuMemoryBufferMinVersion = 0,
    kDestroyGpuMemoryBufferMinVersion = 0,
  };
  virtual ~Gpu() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool EstablishGpuChannel(int32_t* client_id, mojo::ScopedMessagePipeHandle* channel_handle, gpu::GPUInfo* gpu_info);

  using EstablishGpuChannelCallback = base::Callback<void(int32_t, mojo::ScopedMessagePipeHandle, const gpu::GPUInfo&)>;
  virtual void EstablishGpuChannel(const EstablishGpuChannelCallback& callback) = 0;

  virtual void CreateJpegDecodeAccelerator(::media::mojom::GpuJpegDecodeAcceleratorRequest jda) = 0;

  virtual void CreateVideoEncodeAccelerator(::media::mojom::VideoEncodeAcceleratorRequest vea) = 0;


  using CreateGpuMemoryBufferCallback = base::Callback<void(const gfx::GpuMemoryBufferHandle&)>;
  virtual void CreateGpuMemoryBuffer(gfx::GpuMemoryBufferId id, const gfx::Size& size, gfx::BufferFormat format, gfx::BufferUsage usage, const CreateGpuMemoryBufferCallback& callback) = 0;

  virtual void DestroyGpuMemoryBuffer(gfx::GpuMemoryBufferId id, const ::gpu::SyncToken& sync_token) = 0;
};

class  GpuProxy
    : public Gpu {
 public:
  explicit GpuProxy(mojo::MessageReceiverWithResponder* receiver);
  bool EstablishGpuChannel(int32_t* client_id, mojo::ScopedMessagePipeHandle* channel_handle, gpu::GPUInfo* gpu_info) override;
  void EstablishGpuChannel(const EstablishGpuChannelCallback& callback) override;
  void CreateJpegDecodeAccelerator(::media::mojom::GpuJpegDecodeAcceleratorRequest jda) override;
  void CreateVideoEncodeAccelerator(::media::mojom::VideoEncodeAcceleratorRequest vea) override;
  void CreateGpuMemoryBuffer(gfx::GpuMemoryBufferId id, const gfx::Size& size, gfx::BufferFormat format, gfx::BufferUsage usage, const CreateGpuMemoryBufferCallback& callback) override;
  void DestroyGpuMemoryBuffer(gfx::GpuMemoryBufferId id, const ::gpu::SyncToken& sync_token) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  GpuStubDispatch {
 public:
  static bool Accept(Gpu* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Gpu* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Gpu>>
class GpuStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  GpuStub() {}
  ~GpuStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return GpuStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return GpuStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  GpuRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  GpuResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace ui

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_UI_PUBLIC_INTERFACES_GPU_MOJOM_H_