// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_IME_IME_MOJOM_SHARED_H_
#define SERVICES_UI_PUBLIC_INTERFACES_IME_IME_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/ui/public/interfaces/ime/ime.mojom-shared-internal.h"
#include "mojo/common/text_direction.mojom-shared.h"
#include "ui/events/mojo/event.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"
#include "ui/gfx/range/mojo/range.mojom-shared.h"


namespace ui {
namespace mojom {
class CandidateWindowPropertiesDataView;

class CandidateWindowEntryDataView;

class CompositionUnderlineDataView;

class CompositionTextDataView;

class StartSessionDetailsDataView;



}  // namespace mojom
}  // namespace ui

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::ui::mojom::CandidateWindowPropertiesDataView> {
  using Data = ::ui::mojom::internal::CandidateWindowProperties_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::ui::mojom::CandidateWindowEntryDataView> {
  using Data = ::ui::mojom::internal::CandidateWindowEntry_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::ui::mojom::CompositionUnderlineDataView> {
  using Data = ::ui::mojom::internal::CompositionUnderline_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::ui::mojom::CompositionTextDataView> {
  using Data = ::ui::mojom::internal::CompositionText_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::ui::mojom::StartSessionDetailsDataView> {
  using Data = ::ui::mojom::internal::StartSessionDetails_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace ui {
namespace mojom {

enum class CandidateWindowPosition : int32_t {
  kCursor,
  kComposition,
};

inline std::ostream& operator<<(std::ostream& os, CandidateWindowPosition value) {
  switch(value) {
    case CandidateWindowPosition::kCursor:
      return os << "CandidateWindowPosition::kCursor";
    case CandidateWindowPosition::kComposition:
      return os << "CandidateWindowPosition::kComposition";
    default:
      return os << "Unknown CandidateWindowPosition value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(CandidateWindowPosition value) {
  return internal::CandidateWindowPosition_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class TextInputType : int32_t {
  kNone,
  kText,
  kPassword,
  kSearch,
  kEmail,
  kNumber,
  kTelephone,
  kUrl,
  kDate,
  kTime,
  kDateTime,
  kDateTimeLocal,
  kMonth,
  kWeek,
  kTextArea,
  kContentEditable,
  kDateTimeField,
};

inline std::ostream& operator<<(std::ostream& os, TextInputType value) {
  switch(value) {
    case TextInputType::kNone:
      return os << "TextInputType::kNone";
    case TextInputType::kText:
      return os << "TextInputType::kText";
    case TextInputType::kPassword:
      return os << "TextInputType::kPassword";
    case TextInputType::kSearch:
      return os << "TextInputType::kSearch";
    case TextInputType::kEmail:
      return os << "TextInputType::kEmail";
    case TextInputType::kNumber:
      return os << "TextInputType::kNumber";
    case TextInputType::kTelephone:
      return os << "TextInputType::kTelephone";
    case TextInputType::kUrl:
      return os << "TextInputType::kUrl";
    case TextInputType::kDate:
      return os << "TextInputType::kDate";
    case TextInputType::kTime:
      return os << "TextInputType::kTime";
    case TextInputType::kDateTime:
      return os << "TextInputType::kDateTime";
    case TextInputType::kDateTimeLocal:
      return os << "TextInputType::kDateTimeLocal";
    case TextInputType::kMonth:
      return os << "TextInputType::kMonth";
    case TextInputType::kWeek:
      return os << "TextInputType::kWeek";
    case TextInputType::kTextArea:
      return os << "TextInputType::kTextArea";
    case TextInputType::kContentEditable:
      return os << "TextInputType::kContentEditable";
    case TextInputType::kDateTimeField:
      return os << "TextInputType::kDateTimeField";
    default:
      return os << "Unknown TextInputType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(TextInputType value) {
  return internal::TextInputType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class TextInputMode : int32_t {
  kDefault,
  kVerbatim,
  kLatin,
  kLatinName,
  kLatinProse,
  kFullWidthLatin,
  kKana,
  kKanaName,
  kKatakana,
  kNumeric,
  kTel,
  kEmail,
  kUrl,
};

inline std::ostream& operator<<(std::ostream& os, TextInputMode value) {
  switch(value) {
    case TextInputMode::kDefault:
      return os << "TextInputMode::kDefault";
    case TextInputMode::kVerbatim:
      return os << "TextInputMode::kVerbatim";
    case TextInputMode::kLatin:
      return os << "TextInputMode::kLatin";
    case TextInputMode::kLatinName:
      return os << "TextInputMode::kLatinName";
    case TextInputMode::kLatinProse:
      return os << "TextInputMode::kLatinProse";
    case TextInputMode::kFullWidthLatin:
      return os << "TextInputMode::kFullWidthLatin";
    case TextInputMode::kKana:
      return os << "TextInputMode::kKana";
    case TextInputMode::kKanaName:
      return os << "TextInputMode::kKanaName";
    case TextInputMode::kKatakana:
      return os << "TextInputMode::kKatakana";
    case TextInputMode::kNumeric:
      return os << "TextInputMode::kNumeric";
    case TextInputMode::kTel:
      return os << "TextInputMode::kTel";
    case TextInputMode::kEmail:
      return os << "TextInputMode::kEmail";
    case TextInputMode::kUrl:
      return os << "TextInputMode::kUrl";
    default:
      return os << "Unknown TextInputMode value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(TextInputMode value) {
  return internal::TextInputMode_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class IMEDriverInterfaceBase {};

using IMEDriverPtrDataView =
    mojo::InterfacePtrDataView<IMEDriverInterfaceBase>;
using IMEDriverRequestDataView =
    mojo::InterfaceRequestDataView<IMEDriverInterfaceBase>;
using IMEDriverAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<IMEDriverInterfaceBase>;
using IMEDriverAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<IMEDriverInterfaceBase>;
class IMERegistrarInterfaceBase {};

using IMERegistrarPtrDataView =
    mojo::InterfacePtrDataView<IMERegistrarInterfaceBase>;
using IMERegistrarRequestDataView =
    mojo::InterfaceRequestDataView<IMERegistrarInterfaceBase>;
using IMERegistrarAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<IMERegistrarInterfaceBase>;
using IMERegistrarAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<IMERegistrarInterfaceBase>;
class InputMethodInterfaceBase {};

using InputMethodPtrDataView =
    mojo::InterfacePtrDataView<InputMethodInterfaceBase>;
using InputMethodRequestDataView =
    mojo::InterfaceRequestDataView<InputMethodInterfaceBase>;
using InputMethodAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<InputMethodInterfaceBase>;
using InputMethodAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<InputMethodInterfaceBase>;
class TextInputClientInterfaceBase {};

using TextInputClientPtrDataView =
    mojo::InterfacePtrDataView<TextInputClientInterfaceBase>;
using TextInputClientRequestDataView =
    mojo::InterfaceRequestDataView<TextInputClientInterfaceBase>;
using TextInputClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<TextInputClientInterfaceBase>;
using TextInputClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<TextInputClientInterfaceBase>;
class CandidateWindowPropertiesDataView {
 public:
  CandidateWindowPropertiesDataView() {}

  CandidateWindowPropertiesDataView(
      internal::CandidateWindowProperties_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t page_size() const {
    return data_->page_size;
  }
  bool vertical() const {
    return data_->vertical;
  }
  inline void GetAuxiliaryTextDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAuxiliaryText(UserType* output) {
    auto* pointer = data_->auxiliary_text.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  bool auxiliary_text_visible() const {
    return data_->auxiliary_text_visible;
  }
  int32_t cursor_position() const {
    return data_->cursor_position;
  }
  bool cursor_visible() const {
    return data_->cursor_visible;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadWindowPosition(UserType* output) const {
    auto data_value = data_->window_position;
    return mojo::internal::Deserialize<::ui::mojom::CandidateWindowPosition>(
        data_value, output);
  }

  CandidateWindowPosition window_position() const {
    return static_cast<CandidateWindowPosition>(data_->window_position);
  }
 private:
  internal::CandidateWindowProperties_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CandidateWindowEntryDataView {
 public:
  CandidateWindowEntryDataView() {}

  CandidateWindowEntryDataView(
      internal::CandidateWindowEntry_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetValueDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadValue(UserType* output) {
    auto* pointer = data_->value.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetLabelDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLabel(UserType* output) {
    auto* pointer = data_->label.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetAnnotationDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAnnotation(UserType* output) {
    auto* pointer = data_->annotation.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetDescriptionTitleDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDescriptionTitle(UserType* output) {
    auto* pointer = data_->description_title.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetDescriptionBodyDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDescriptionBody(UserType* output) {
    auto* pointer = data_->description_body.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::CandidateWindowEntry_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CompositionUnderlineDataView {
 public:
  CompositionUnderlineDataView() {}

  CompositionUnderlineDataView(
      internal::CompositionUnderline_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint32_t start_offset() const {
    return data_->start_offset;
  }
  uint32_t end_offset() const {
    return data_->end_offset;
  }
  bool thick() const {
    return data_->thick;
  }
  uint32_t color() const {
    return data_->color;
  }
  uint32_t background_color() const {
    return data_->background_color;
  }
 private:
  internal::CompositionUnderline_Data* data_ = nullptr;
};

class CompositionTextDataView {
 public:
  CompositionTextDataView() {}

  CompositionTextDataView(
      internal::CompositionText_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTextDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadText(UserType* output) {
    auto* pointer = data_->text.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetUnderlinesDataView(
      mojo::ArrayDataView<CompositionUnderlineDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUnderlines(UserType* output) {
    auto* pointer = data_->underlines.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::ui::mojom::CompositionUnderlineDataView>>(
        pointer, output, context_);
  }
  inline void GetSelectionDataView(
      ::gfx::mojom::RangeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSelection(UserType* output) {
    auto* pointer = data_->selection.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RangeDataView>(
        pointer, output, context_);
  }
 private:
  internal::CompositionText_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class StartSessionDetailsDataView {
 public:
  StartSessionDetailsDataView() {}

  StartSessionDetailsDataView(
      internal::StartSessionDetails_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::ui::mojom::TextInputClientPtrDataView>(
            &data_->client, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeInputMethodRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::ui::mojom::InputMethodRequestDataView>(
            &data_->input_method_request, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTextInputType(UserType* output) const {
    auto data_value = data_->text_input_type;
    return mojo::internal::Deserialize<::ui::mojom::TextInputType>(
        data_value, output);
  }

  TextInputType text_input_type() const {
    return static_cast<TextInputType>(data_->text_input_type);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTextInputMode(UserType* output) const {
    auto data_value = data_->text_input_mode;
    return mojo::internal::Deserialize<::ui::mojom::TextInputMode>(
        data_value, output);
  }

  TextInputMode text_input_mode() const {
    return static_cast<TextInputMode>(data_->text_input_mode);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTextDirection(UserType* output) const {
    auto data_value = data_->text_direction;
    return mojo::internal::Deserialize<::mojo::common::mojom::TextDirection>(
        data_value, output);
  }

  ::mojo::common::mojom::TextDirection text_direction() const {
    return static_cast<::mojo::common::mojom::TextDirection>(data_->text_direction);
  }
  int32_t text_input_flags() const {
    return data_->text_input_flags;
  }
  inline void GetCaretBoundsDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCaretBounds(UserType* output) {
    auto* pointer = data_->caret_bounds.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
 private:
  internal::StartSessionDetails_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class IMEDriver_StartSession_ParamsDataView {
 public:
  IMEDriver_StartSession_ParamsDataView() {}

  IMEDriver_StartSession_ParamsDataView(
      internal::IMEDriver_StartSession_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDetailsDataView(
      StartSessionDetailsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDetails(UserType* output) {
    auto* pointer = data_->details.Get();
    return mojo::internal::Deserialize<::ui::mojom::StartSessionDetailsDataView>(
        pointer, output, context_);
  }
 private:
  internal::IMEDriver_StartSession_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class IMERegistrar_RegisterDriver_ParamsDataView {
 public:
  IMERegistrar_RegisterDriver_ParamsDataView() {}

  IMERegistrar_RegisterDriver_ParamsDataView(
      internal::IMERegistrar_RegisterDriver_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeDriver() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::ui::mojom::IMEDriverPtrDataView>(
            &data_->driver, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::IMERegistrar_RegisterDriver_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class InputMethod_OnTextInputTypeChanged_ParamsDataView {
 public:
  InputMethod_OnTextInputTypeChanged_ParamsDataView() {}

  InputMethod_OnTextInputTypeChanged_ParamsDataView(
      internal::InputMethod_OnTextInputTypeChanged_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTextInputType(UserType* output) const {
    auto data_value = data_->text_input_type;
    return mojo::internal::Deserialize<::ui::mojom::TextInputType>(
        data_value, output);
  }

  TextInputType text_input_type() const {
    return static_cast<TextInputType>(data_->text_input_type);
  }
 private:
  internal::InputMethod_OnTextInputTypeChanged_Params_Data* data_ = nullptr;
};

class InputMethod_OnCaretBoundsChanged_ParamsDataView {
 public:
  InputMethod_OnCaretBoundsChanged_ParamsDataView() {}

  InputMethod_OnCaretBoundsChanged_ParamsDataView(
      internal::InputMethod_OnCaretBoundsChanged_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCaretBoundsDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCaretBounds(UserType* output) {
    auto* pointer = data_->caret_bounds.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
 private:
  internal::InputMethod_OnCaretBoundsChanged_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class InputMethod_ProcessKeyEvent_ParamsDataView {
 public:
  InputMethod_ProcessKeyEvent_ParamsDataView() {}

  InputMethod_ProcessKeyEvent_ParamsDataView(
      internal::InputMethod_ProcessKeyEvent_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetKeyEventDataView(
      ::ui::mojom::EventDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadKeyEvent(UserType* output) {
    auto* pointer = data_->key_event.Get();
    return mojo::internal::Deserialize<::ui::mojom::EventDataView>(
        pointer, output, context_);
  }
 private:
  internal::InputMethod_ProcessKeyEvent_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class InputMethod_ProcessKeyEvent_ResponseParamsDataView {
 public:
  InputMethod_ProcessKeyEvent_ResponseParamsDataView() {}

  InputMethod_ProcessKeyEvent_ResponseParamsDataView(
      internal::InputMethod_ProcessKeyEvent_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool handled() const {
    return data_->handled;
  }
 private:
  internal::InputMethod_ProcessKeyEvent_ResponseParams_Data* data_ = nullptr;
};

class InputMethod_CancelComposition_ParamsDataView {
 public:
  InputMethod_CancelComposition_ParamsDataView() {}

  InputMethod_CancelComposition_ParamsDataView(
      internal::InputMethod_CancelComposition_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::InputMethod_CancelComposition_Params_Data* data_ = nullptr;
};

class TextInputClient_SetCompositionText_ParamsDataView {
 public:
  TextInputClient_SetCompositionText_ParamsDataView() {}

  TextInputClient_SetCompositionText_ParamsDataView(
      internal::TextInputClient_SetCompositionText_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCompositionDataView(
      CompositionTextDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadComposition(UserType* output) {
    auto* pointer = data_->composition.Get();
    return mojo::internal::Deserialize<::ui::mojom::CompositionTextDataView>(
        pointer, output, context_);
  }
 private:
  internal::TextInputClient_SetCompositionText_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TextInputClient_ConfirmCompositionText_ParamsDataView {
 public:
  TextInputClient_ConfirmCompositionText_ParamsDataView() {}

  TextInputClient_ConfirmCompositionText_ParamsDataView(
      internal::TextInputClient_ConfirmCompositionText_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::TextInputClient_ConfirmCompositionText_Params_Data* data_ = nullptr;
};

class TextInputClient_ClearCompositionText_ParamsDataView {
 public:
  TextInputClient_ClearCompositionText_ParamsDataView() {}

  TextInputClient_ClearCompositionText_ParamsDataView(
      internal::TextInputClient_ClearCompositionText_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::TextInputClient_ClearCompositionText_Params_Data* data_ = nullptr;
};

class TextInputClient_InsertText_ParamsDataView {
 public:
  TextInputClient_InsertText_ParamsDataView() {}

  TextInputClient_InsertText_ParamsDataView(
      internal::TextInputClient_InsertText_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTextDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadText(UserType* output) {
    auto* pointer = data_->text.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::TextInputClient_InsertText_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TextInputClient_InsertChar_ParamsDataView {
 public:
  TextInputClient_InsertChar_ParamsDataView() {}

  TextInputClient_InsertChar_ParamsDataView(
      internal::TextInputClient_InsertChar_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetEventDataView(
      ::ui::mojom::EventDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEvent(UserType* output) {
    auto* pointer = data_->event.Get();
    return mojo::internal::Deserialize<::ui::mojom::EventDataView>(
        pointer, output, context_);
  }
 private:
  internal::TextInputClient_InsertChar_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace ui

namespace std {

template <>
struct hash<::ui::mojom::CandidateWindowPosition>
    : public mojo::internal::EnumHashImpl<::ui::mojom::CandidateWindowPosition> {};

template <>
struct hash<::ui::mojom::TextInputType>
    : public mojo::internal::EnumHashImpl<::ui::mojom::TextInputType> {};

template <>
struct hash<::ui::mojom::TextInputMode>
    : public mojo::internal::EnumHashImpl<::ui::mojom::TextInputMode> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::ui::mojom::CandidateWindowPosition, ::ui::mojom::CandidateWindowPosition> {
  static ::ui::mojom::CandidateWindowPosition ToMojom(::ui::mojom::CandidateWindowPosition input) { return input; }
  static bool FromMojom(::ui::mojom::CandidateWindowPosition input, ::ui::mojom::CandidateWindowPosition* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::CandidateWindowPosition, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::ui::mojom::CandidateWindowPosition, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::ui::mojom::CandidateWindowPosition>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::ui::mojom::TextInputType, ::ui::mojom::TextInputType> {
  static ::ui::mojom::TextInputType ToMojom(::ui::mojom::TextInputType input) { return input; }
  static bool FromMojom(::ui::mojom::TextInputType input, ::ui::mojom::TextInputType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::TextInputType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::ui::mojom::TextInputType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::ui::mojom::TextInputType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::ui::mojom::TextInputMode, ::ui::mojom::TextInputMode> {
  static ::ui::mojom::TextInputMode ToMojom(::ui::mojom::TextInputMode input) { return input; }
  static bool FromMojom(::ui::mojom::TextInputMode input, ::ui::mojom::TextInputMode* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::TextInputMode, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::ui::mojom::TextInputMode, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::ui::mojom::TextInputMode>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::CandidateWindowPropertiesDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::CandidateWindowPropertiesDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::ui::mojom::internal::CandidateWindowProperties_Data);
    decltype(CallWithContext(Traits::auxiliary_text, input, custom_context)) in_auxiliary_text = CallWithContext(Traits::auxiliary_text, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_auxiliary_text, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::CandidateWindowProperties_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::ui::mojom::internal::CandidateWindowProperties_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->page_size = CallWithContext(Traits::page_size, input, custom_context);
    result->vertical = CallWithContext(Traits::vertical, input, custom_context);
    decltype(CallWithContext(Traits::auxiliary_text, input, custom_context)) in_auxiliary_text = CallWithContext(Traits::auxiliary_text, input, custom_context);
    typename decltype(result->auxiliary_text)::BaseType* auxiliary_text_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_auxiliary_text, buffer, &auxiliary_text_ptr, context);
    result->auxiliary_text.Set(auxiliary_text_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->auxiliary_text.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null auxiliary_text in CandidateWindowProperties struct");
    result->auxiliary_text_visible = CallWithContext(Traits::auxiliary_text_visible, input, custom_context);
    result->cursor_position = CallWithContext(Traits::cursor_position, input, custom_context);
    result->cursor_visible = CallWithContext(Traits::cursor_visible, input, custom_context);
    mojo::internal::Serialize<::ui::mojom::CandidateWindowPosition>(
        CallWithContext(Traits::window_position, input, custom_context), &result->window_position);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::CandidateWindowProperties_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::CandidateWindowPropertiesDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::CandidateWindowEntryDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::CandidateWindowEntryDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::ui::mojom::internal::CandidateWindowEntry_Data);
    decltype(CallWithContext(Traits::value, input, custom_context)) in_value = CallWithContext(Traits::value, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_value, context);
    decltype(CallWithContext(Traits::label, input, custom_context)) in_label = CallWithContext(Traits::label, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_label, context);
    decltype(CallWithContext(Traits::annotation, input, custom_context)) in_annotation = CallWithContext(Traits::annotation, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_annotation, context);
    decltype(CallWithContext(Traits::description_title, input, custom_context)) in_description_title = CallWithContext(Traits::description_title, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_description_title, context);
    decltype(CallWithContext(Traits::description_body, input, custom_context)) in_description_body = CallWithContext(Traits::description_body, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_description_body, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::CandidateWindowEntry_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::ui::mojom::internal::CandidateWindowEntry_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::value, input, custom_context)) in_value = CallWithContext(Traits::value, input, custom_context);
    typename decltype(result->value)::BaseType* value_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_value, buffer, &value_ptr, context);
    result->value.Set(value_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->value.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null value in CandidateWindowEntry struct");
    decltype(CallWithContext(Traits::label, input, custom_context)) in_label = CallWithContext(Traits::label, input, custom_context);
    typename decltype(result->label)::BaseType* label_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_label, buffer, &label_ptr, context);
    result->label.Set(label_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->label.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null label in CandidateWindowEntry struct");
    decltype(CallWithContext(Traits::annotation, input, custom_context)) in_annotation = CallWithContext(Traits::annotation, input, custom_context);
    typename decltype(result->annotation)::BaseType* annotation_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_annotation, buffer, &annotation_ptr, context);
    result->annotation.Set(annotation_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->annotation.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null annotation in CandidateWindowEntry struct");
    decltype(CallWithContext(Traits::description_title, input, custom_context)) in_description_title = CallWithContext(Traits::description_title, input, custom_context);
    typename decltype(result->description_title)::BaseType* description_title_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_description_title, buffer, &description_title_ptr, context);
    result->description_title.Set(description_title_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->description_title.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null description_title in CandidateWindowEntry struct");
    decltype(CallWithContext(Traits::description_body, input, custom_context)) in_description_body = CallWithContext(Traits::description_body, input, custom_context);
    typename decltype(result->description_body)::BaseType* description_body_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_description_body, buffer, &description_body_ptr, context);
    result->description_body.Set(description_body_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->description_body.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null description_body in CandidateWindowEntry struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::CandidateWindowEntry_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::CandidateWindowEntryDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::CompositionUnderlineDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::CompositionUnderlineDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::ui::mojom::internal::CompositionUnderline_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::CompositionUnderline_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::ui::mojom::internal::CompositionUnderline_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->start_offset = CallWithContext(Traits::start_offset, input, custom_context);
    result->end_offset = CallWithContext(Traits::end_offset, input, custom_context);
    result->thick = CallWithContext(Traits::thick, input, custom_context);
    result->color = CallWithContext(Traits::color, input, custom_context);
    result->background_color = CallWithContext(Traits::background_color, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::CompositionUnderline_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::CompositionUnderlineDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::CompositionTextDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::CompositionTextDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::ui::mojom::internal::CompositionText_Data);
    decltype(CallWithContext(Traits::text, input, custom_context)) in_text = CallWithContext(Traits::text, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_text, context);
    decltype(CallWithContext(Traits::underlines, input, custom_context)) in_underlines = CallWithContext(Traits::underlines, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::ui::mojom::CompositionUnderlineDataView>>(
        in_underlines, context);
    decltype(CallWithContext(Traits::selection, input, custom_context)) in_selection = CallWithContext(Traits::selection, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::RangeDataView>(
        in_selection, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::CompositionText_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::ui::mojom::internal::CompositionText_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::text, input, custom_context)) in_text = CallWithContext(Traits::text, input, custom_context);
    typename decltype(result->text)::BaseType* text_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_text, buffer, &text_ptr, context);
    result->text.Set(text_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->text.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null text in CompositionText struct");
    decltype(CallWithContext(Traits::underlines, input, custom_context)) in_underlines = CallWithContext(Traits::underlines, input, custom_context);
    typename decltype(result->underlines)::BaseType* underlines_ptr;
    const mojo::internal::ContainerValidateParams underlines_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::ui::mojom::CompositionUnderlineDataView>>(
        in_underlines, buffer, &underlines_ptr, &underlines_validate_params,
        context);
    result->underlines.Set(underlines_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->underlines.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null underlines in CompositionText struct");
    decltype(CallWithContext(Traits::selection, input, custom_context)) in_selection = CallWithContext(Traits::selection, input, custom_context);
    typename decltype(result->selection)::BaseType* selection_ptr;
    mojo::internal::Serialize<::gfx::mojom::RangeDataView>(
        in_selection, buffer, &selection_ptr, context);
    result->selection.Set(selection_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->selection.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null selection in CompositionText struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::CompositionText_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::CompositionTextDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::StartSessionDetailsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::StartSessionDetailsDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::ui::mojom::internal::StartSessionDetails_Data);
    decltype(CallWithContext(Traits::client, input, custom_context)) in_client = CallWithContext(Traits::client, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::ui::mojom::TextInputClientPtrDataView>(
        in_client, context);
    decltype(CallWithContext(Traits::input_method_request, input, custom_context)) in_input_method_request = CallWithContext(Traits::input_method_request, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::ui::mojom::InputMethodRequestDataView>(
        in_input_method_request, context);
    decltype(CallWithContext(Traits::caret_bounds, input, custom_context)) in_caret_bounds = CallWithContext(Traits::caret_bounds, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectDataView>(
        in_caret_bounds, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::StartSessionDetails_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::ui::mojom::internal::StartSessionDetails_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::client, input, custom_context))>::type
        in_client;
    mojo::internal::Serialize<::ui::mojom::TextInputClientPtrDataView>(
        in_client, &result->client, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->client),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid client in StartSessionDetails struct");
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::input_method_request, input, custom_context))>::type
        in_input_method_request;
    mojo::internal::Serialize<::ui::mojom::InputMethodRequestDataView>(
        in_input_method_request, &result->input_method_request, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->input_method_request),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid input_method_request in StartSessionDetails struct");
    mojo::internal::Serialize<::ui::mojom::TextInputType>(
        CallWithContext(Traits::text_input_type, input, custom_context), &result->text_input_type);
    mojo::internal::Serialize<::ui::mojom::TextInputMode>(
        CallWithContext(Traits::text_input_mode, input, custom_context), &result->text_input_mode);
    mojo::internal::Serialize<::mojo::common::mojom::TextDirection>(
        CallWithContext(Traits::text_direction, input, custom_context), &result->text_direction);
    result->text_input_flags = CallWithContext(Traits::text_input_flags, input, custom_context);
    decltype(CallWithContext(Traits::caret_bounds, input, custom_context)) in_caret_bounds = CallWithContext(Traits::caret_bounds, input, custom_context);
    typename decltype(result->caret_bounds)::BaseType* caret_bounds_ptr;
    mojo::internal::Serialize<::gfx::mojom::RectDataView>(
        in_caret_bounds, buffer, &caret_bounds_ptr, context);
    result->caret_bounds.Set(caret_bounds_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->caret_bounds.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null caret_bounds in StartSessionDetails struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::StartSessionDetails_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::StartSessionDetailsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace ui {
namespace mojom {

inline void CandidateWindowPropertiesDataView::GetAuxiliaryTextDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->auxiliary_text.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void CandidateWindowEntryDataView::GetValueDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->value.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void CandidateWindowEntryDataView::GetLabelDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->label.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void CandidateWindowEntryDataView::GetAnnotationDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->annotation.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void CandidateWindowEntryDataView::GetDescriptionTitleDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->description_title.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void CandidateWindowEntryDataView::GetDescriptionBodyDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->description_body.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void CompositionTextDataView::GetTextDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->text.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void CompositionTextDataView::GetUnderlinesDataView(
    mojo::ArrayDataView<CompositionUnderlineDataView>* output) {
  auto pointer = data_->underlines.Get();
  *output = mojo::ArrayDataView<CompositionUnderlineDataView>(pointer, context_);
}
inline void CompositionTextDataView::GetSelectionDataView(
    ::gfx::mojom::RangeDataView* output) {
  auto pointer = data_->selection.Get();
  *output = ::gfx::mojom::RangeDataView(pointer, context_);
}


inline void StartSessionDetailsDataView::GetCaretBoundsDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->caret_bounds.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}


inline void IMEDriver_StartSession_ParamsDataView::GetDetailsDataView(
    StartSessionDetailsDataView* output) {
  auto pointer = data_->details.Get();
  *output = StartSessionDetailsDataView(pointer, context_);
}






inline void InputMethod_OnCaretBoundsChanged_ParamsDataView::GetCaretBoundsDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->caret_bounds.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}


inline void InputMethod_ProcessKeyEvent_ParamsDataView::GetKeyEventDataView(
    ::ui::mojom::EventDataView* output) {
  auto pointer = data_->key_event.Get();
  *output = ::ui::mojom::EventDataView(pointer, context_);
}






inline void TextInputClient_SetCompositionText_ParamsDataView::GetCompositionDataView(
    CompositionTextDataView* output) {
  auto pointer = data_->composition.Get();
  *output = CompositionTextDataView(pointer, context_);
}






inline void TextInputClient_InsertText_ParamsDataView::GetTextDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->text.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void TextInputClient_InsertChar_ParamsDataView::GetEventDataView(
    ::ui::mojom::EventDataView* output) {
  auto pointer = data_->event.Get();
  *output = ::ui::mojom::EventDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace ui

#endif  // SERVICES_UI_PUBLIC_INTERFACES_IME_IME_MOJOM_SHARED_H_
