// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/video_capture/public/interfaces/device_factory.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "media/base/ipc/media_param_traits_macros.h"
#include "media/capture/ipc/capture_param_traits.h"
#include "media/capture/ipc/capture_param_traits_macros.h"
#include "media/capture/mojo/video_capture_types_typemap_traits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace video_capture {
namespace mojom {
const char DeviceFactory::Name_[] = "video_capture::mojom::DeviceFactory";

class DeviceFactory_GetDeviceInfos_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  DeviceFactory_GetDeviceInfos_ForwardToCallback(
      DeviceFactory::GetDeviceInfosCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  DeviceFactory::GetDeviceInfosCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(DeviceFactory_GetDeviceInfos_ForwardToCallback);
};

class DeviceFactory_CreateDevice_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  DeviceFactory_CreateDevice_ForwardToCallback(
      DeviceFactory::CreateDeviceCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  DeviceFactory::CreateDeviceCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(DeviceFactory_CreateDevice_ForwardToCallback);
};

DeviceFactoryProxy::DeviceFactoryProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void DeviceFactoryProxy::GetDeviceInfos(
    GetDeviceInfosCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::DeviceFactory_GetDeviceInfos_Params_Data);
  serialization_context.PrepareMessage(
      internal::kDeviceFactory_GetDeviceInfos_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::DeviceFactory_GetDeviceInfos_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new DeviceFactory_GetDeviceInfos_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void DeviceFactoryProxy::CreateDevice(
    const std::string& in_device_id, ::video_capture::mojom::DeviceRequest in_device_request, CreateDeviceCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::DeviceFactory_CreateDevice_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_device_id, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::video_capture::mojom::DeviceRequestDataView>(
      in_device_request, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kDeviceFactory_CreateDevice_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::DeviceFactory_CreateDevice_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->device_id)::BaseType* device_id_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_device_id, buffer, &device_id_ptr, &serialization_context);
  params->device_id.Set(device_id_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->device_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null device_id in DeviceFactory.CreateDevice request");
  mojo::internal::Serialize<::video_capture::mojom::DeviceRequestDataView>(
      in_device_request, &params->device_request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->device_request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid device_request in DeviceFactory.CreateDevice request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new DeviceFactory_CreateDevice_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class DeviceFactory_GetDeviceInfos_ProxyToResponder {
 public:
  static DeviceFactory::GetDeviceInfosCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<DeviceFactory_GetDeviceInfos_ProxyToResponder> proxy(
        new DeviceFactory_GetDeviceInfos_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&DeviceFactory_GetDeviceInfos_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~DeviceFactory_GetDeviceInfos_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "DeviceFactory::GetDeviceInfos() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  DeviceFactory_GetDeviceInfos_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      const std::vector<media::VideoCaptureDeviceInfo>& in_device_infos);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(DeviceFactory_GetDeviceInfos_ProxyToResponder);
};

bool DeviceFactory_GetDeviceInfos_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::DeviceFactory_GetDeviceInfos_ResponseParams_Data* params =
      reinterpret_cast<
          internal::DeviceFactory_GetDeviceInfos_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<media::VideoCaptureDeviceInfo> p_device_infos{};
  DeviceFactory_GetDeviceInfos_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadDeviceInfos(&p_device_infos))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "DeviceFactory::GetDeviceInfos response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_device_infos));
  return true;
}

void DeviceFactory_GetDeviceInfos_ProxyToResponder::Run(
    const std::vector<media::VideoCaptureDeviceInfo>& in_device_infos) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::DeviceFactory_GetDeviceInfos_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::media::mojom::VideoCaptureDeviceInfoDataView>>(
      in_device_infos, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kDeviceFactory_GetDeviceInfos_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::DeviceFactory_GetDeviceInfos_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->device_infos)::BaseType* device_infos_ptr;
  const mojo::internal::ContainerValidateParams device_infos_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::media::mojom::VideoCaptureDeviceInfoDataView>>(
      in_device_infos, buffer, &device_infos_ptr, &device_infos_validate_params,
      &serialization_context);
  params->device_infos.Set(device_infos_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->device_infos.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null device_infos in ");

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class DeviceFactory_CreateDevice_ProxyToResponder {
 public:
  static DeviceFactory::CreateDeviceCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<DeviceFactory_CreateDevice_ProxyToResponder> proxy(
        new DeviceFactory_CreateDevice_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&DeviceFactory_CreateDevice_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~DeviceFactory_CreateDevice_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "DeviceFactory::CreateDevice() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  DeviceFactory_CreateDevice_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      DeviceAccessResultCode in_result_code);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(DeviceFactory_CreateDevice_ProxyToResponder);
};

bool DeviceFactory_CreateDevice_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::DeviceFactory_CreateDevice_ResponseParams_Data* params =
      reinterpret_cast<
          internal::DeviceFactory_CreateDevice_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  DeviceAccessResultCode p_result_code{};
  DeviceFactory_CreateDevice_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadResultCode(&p_result_code))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "DeviceFactory::CreateDevice response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result_code));
  return true;
}

void DeviceFactory_CreateDevice_ProxyToResponder::Run(
    DeviceAccessResultCode in_result_code) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::DeviceFactory_CreateDevice_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kDeviceFactory_CreateDevice_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::DeviceFactory_CreateDevice_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::video_capture::mojom::DeviceAccessResultCode>(
      in_result_code, &params->result_code);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool DeviceFactoryStubDispatch::Accept(
    DeviceFactory* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kDeviceFactory_GetDeviceInfos_Name: {
      break;
    }
    case internal::kDeviceFactory_CreateDevice_Name: {
      break;
    }
  }
  return false;
}

// static
bool DeviceFactoryStubDispatch::AcceptWithResponder(
    DeviceFactory* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kDeviceFactory_GetDeviceInfos_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::DeviceFactory_GetDeviceInfos_Params_Data* params =
          reinterpret_cast<
              internal::DeviceFactory_GetDeviceInfos_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      DeviceFactory_GetDeviceInfos_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "DeviceFactory::GetDeviceInfos deserializer");
        return false;
      }
      DeviceFactory::GetDeviceInfosCallback callback =
          DeviceFactory_GetDeviceInfos_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetDeviceInfos(std::move(callback));
      return true;
    }
    case internal::kDeviceFactory_CreateDevice_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::DeviceFactory_CreateDevice_Params_Data* params =
          reinterpret_cast<
              internal::DeviceFactory_CreateDevice_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_device_id{};
      ::video_capture::mojom::DeviceRequest p_device_request{};
      DeviceFactory_CreateDevice_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadDeviceId(&p_device_id))
        success = false;
      p_device_request =
          input_data_view.TakeDeviceRequest<decltype(p_device_request)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "DeviceFactory::CreateDevice deserializer");
        return false;
      }
      DeviceFactory::CreateDeviceCallback callback =
          DeviceFactory_CreateDevice_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateDevice(
std::move(p_device_id), 
std::move(p_device_request), std::move(callback));
      return true;
    }
  }
  return false;
}

bool DeviceFactoryRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "DeviceFactory RequestValidator");

  switch (message->header()->name) {
    case internal::kDeviceFactory_GetDeviceInfos_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::DeviceFactory_GetDeviceInfos_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kDeviceFactory_CreateDevice_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::DeviceFactory_CreateDevice_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool DeviceFactoryResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "DeviceFactory ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kDeviceFactory_GetDeviceInfos_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::DeviceFactory_GetDeviceInfos_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kDeviceFactory_CreateDevice_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::DeviceFactory_CreateDevice_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
}  // namespace mojom
}  // namespace video_capture

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif