// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/video_capture/public/interfaces/receiver.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "ipc/ipc_message_utils.h"
#include "media/base/ipc/media_param_traits_macros.h"
#include "media/capture/ipc/capture_param_traits.h"
#include "media/capture/ipc/capture_param_traits_macros.h"
#include "media/capture/mojo/video_capture_types_typemap_traits.h"
#include "mojo/common/common_custom_types_struct_traits.h"
#include "mojo/common/values_struct_traits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace video_capture {
namespace mojom {
const char ScopedAccessPermission::Name_[] = "video_capture::mojom::ScopedAccessPermission";

ScopedAccessPermissionProxy::ScopedAccessPermissionProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

// static
bool ScopedAccessPermissionStubDispatch::Accept(
    ScopedAccessPermission* impl,
    mojo::Message* message) {
  return false;
}

// static
bool ScopedAccessPermissionStubDispatch::AcceptWithResponder(
    ScopedAccessPermission* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  return false;
}

bool ScopedAccessPermissionRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ScopedAccessPermission RequestValidator");

  switch (message->header()->name) {
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

const char Receiver::Name_[] = "video_capture::mojom::Receiver";

ReceiverProxy::ReceiverProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void ReceiverProxy::OnNewBufferHandle(
    int32_t in_buffer_id, mojo::ScopedSharedBufferHandle in_buffer_handle) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::Receiver_OnNewBufferHandle_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ScopedSharedBufferHandle>(
      in_buffer_handle, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kReceiver_OnNewBufferHandle_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::Receiver_OnNewBufferHandle_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->buffer_id = in_buffer_id;
  mojo::internal::Serialize<mojo::ScopedSharedBufferHandle>(
      in_buffer_handle, &params->buffer_handle, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->buffer_handle),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid buffer_handle in Receiver.OnNewBufferHandle request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void ReceiverProxy::OnFrameReadyInBuffer(
    int32_t in_buffer_id, int32_t in_frame_feedback_id, ScopedAccessPermissionPtr in_access_permission, ::media::mojom::VideoFrameInfoPtr in_frame_info) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::Receiver_OnFrameReadyInBuffer_Params_Data);
  size += mojo::internal::PrepareToSerialize<::video_capture::mojom::ScopedAccessPermissionPtrDataView>(
      in_access_permission, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::media::mojom::VideoFrameInfoDataView>(
      in_frame_info, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kReceiver_OnFrameReadyInBuffer_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::Receiver_OnFrameReadyInBuffer_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->buffer_id = in_buffer_id;
  params->frame_feedback_id = in_frame_feedback_id;
  mojo::internal::Serialize<::video_capture::mojom::ScopedAccessPermissionPtrDataView>(
      in_access_permission, &params->access_permission, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->access_permission),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid access_permission in Receiver.OnFrameReadyInBuffer request");
  typename decltype(params->frame_info)::BaseType* frame_info_ptr;
  mojo::internal::Serialize<::media::mojom::VideoFrameInfoDataView>(
      in_frame_info, buffer, &frame_info_ptr, &serialization_context);
  params->frame_info.Set(frame_info_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->frame_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null frame_info in Receiver.OnFrameReadyInBuffer request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void ReceiverProxy::OnBufferRetired(
    int32_t in_buffer_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::Receiver_OnBufferRetired_Params_Data);
  serialization_context.PrepareMessage(
      internal::kReceiver_OnBufferRetired_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::Receiver_OnBufferRetired_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->buffer_id = in_buffer_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void ReceiverProxy::OnError(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::Receiver_OnError_Params_Data);
  serialization_context.PrepareMessage(
      internal::kReceiver_OnError_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::Receiver_OnError_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void ReceiverProxy::OnLog(
    const std::string& in_message) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::Receiver_OnLog_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_message, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kReceiver_OnLog_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::Receiver_OnLog_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->message)::BaseType* message_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_message, buffer, &message_ptr, &serialization_context);
  params->message.Set(message_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->message.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null message in Receiver.OnLog request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void ReceiverProxy::OnStarted(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::Receiver_OnStarted_Params_Data);
  serialization_context.PrepareMessage(
      internal::kReceiver_OnStarted_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::Receiver_OnStarted_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void ReceiverProxy::OnStartedUsingGpuDecode(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::video_capture::mojom::internal::Receiver_OnStartedUsingGpuDecode_Params_Data);
  serialization_context.PrepareMessage(
      internal::kReceiver_OnStartedUsingGpuDecode_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::video_capture::mojom::internal::Receiver_OnStartedUsingGpuDecode_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool ReceiverStubDispatch::Accept(
    Receiver* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kReceiver_OnNewBufferHandle_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Receiver_OnNewBufferHandle_Params_Data* params =
          reinterpret_cast<internal::Receiver_OnNewBufferHandle_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_buffer_id{};
      mojo::ScopedSharedBufferHandle p_buffer_handle{};
      Receiver_OnNewBufferHandle_ParamsDataView input_data_view(params, &serialization_context);
      
      p_buffer_id = input_data_view.buffer_id();
      p_buffer_handle = input_data_view.TakeBufferHandle();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Receiver::OnNewBufferHandle deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnNewBufferHandle(
std::move(p_buffer_id), 
std::move(p_buffer_handle));
      return true;
    }
    case internal::kReceiver_OnFrameReadyInBuffer_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Receiver_OnFrameReadyInBuffer_Params_Data* params =
          reinterpret_cast<internal::Receiver_OnFrameReadyInBuffer_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_buffer_id{};
      int32_t p_frame_feedback_id{};
      ScopedAccessPermissionPtr p_access_permission{};
      ::media::mojom::VideoFrameInfoPtr p_frame_info{};
      Receiver_OnFrameReadyInBuffer_ParamsDataView input_data_view(params, &serialization_context);
      
      p_buffer_id = input_data_view.buffer_id();
      p_frame_feedback_id = input_data_view.frame_feedback_id();
      p_access_permission =
          input_data_view.TakeAccessPermission<decltype(p_access_permission)>();
      if (!input_data_view.ReadFrameInfo(&p_frame_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Receiver::OnFrameReadyInBuffer deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnFrameReadyInBuffer(
std::move(p_buffer_id), 
std::move(p_frame_feedback_id), 
std::move(p_access_permission), 
std::move(p_frame_info));
      return true;
    }
    case internal::kReceiver_OnBufferRetired_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Receiver_OnBufferRetired_Params_Data* params =
          reinterpret_cast<internal::Receiver_OnBufferRetired_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_buffer_id{};
      Receiver_OnBufferRetired_ParamsDataView input_data_view(params, &serialization_context);
      
      p_buffer_id = input_data_view.buffer_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Receiver::OnBufferRetired deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnBufferRetired(
std::move(p_buffer_id));
      return true;
    }
    case internal::kReceiver_OnError_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Receiver_OnError_Params_Data* params =
          reinterpret_cast<internal::Receiver_OnError_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      Receiver_OnError_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Receiver::OnError deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnError();
      return true;
    }
    case internal::kReceiver_OnLog_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Receiver_OnLog_Params_Data* params =
          reinterpret_cast<internal::Receiver_OnLog_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_message{};
      Receiver_OnLog_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadMessage(&p_message))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Receiver::OnLog deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnLog(
std::move(p_message));
      return true;
    }
    case internal::kReceiver_OnStarted_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Receiver_OnStarted_Params_Data* params =
          reinterpret_cast<internal::Receiver_OnStarted_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      Receiver_OnStarted_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Receiver::OnStarted deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnStarted();
      return true;
    }
    case internal::kReceiver_OnStartedUsingGpuDecode_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Receiver_OnStartedUsingGpuDecode_Params_Data* params =
          reinterpret_cast<internal::Receiver_OnStartedUsingGpuDecode_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      Receiver_OnStartedUsingGpuDecode_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Receiver::OnStartedUsingGpuDecode deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnStartedUsingGpuDecode();
      return true;
    }
  }
  return false;
}

// static
bool ReceiverStubDispatch::AcceptWithResponder(
    Receiver* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kReceiver_OnNewBufferHandle_Name: {
      break;
    }
    case internal::kReceiver_OnFrameReadyInBuffer_Name: {
      break;
    }
    case internal::kReceiver_OnBufferRetired_Name: {
      break;
    }
    case internal::kReceiver_OnError_Name: {
      break;
    }
    case internal::kReceiver_OnLog_Name: {
      break;
    }
    case internal::kReceiver_OnStarted_Name: {
      break;
    }
    case internal::kReceiver_OnStartedUsingGpuDecode_Name: {
      break;
    }
  }
  return false;
}

bool ReceiverRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "Receiver RequestValidator");

  switch (message->header()->name) {
    case internal::kReceiver_OnNewBufferHandle_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Receiver_OnNewBufferHandle_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kReceiver_OnFrameReadyInBuffer_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Receiver_OnFrameReadyInBuffer_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kReceiver_OnBufferRetired_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Receiver_OnBufferRetired_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kReceiver_OnError_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Receiver_OnError_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kReceiver_OnLog_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Receiver_OnLog_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kReceiver_OnStarted_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Receiver_OnStarted_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kReceiver_OnStartedUsingGpuDecode_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Receiver_OnStartedUsingGpuDecode_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace video_capture

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif