// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_VIZ_HIT_TEST_PUBLIC_INTERFACES_HIT_TEST_REGION_LIST_MOJOM_H_
#define SERVICES_VIZ_HIT_TEST_PUBLIC_INTERFACES_HIT_TEST_REGION_LIST_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/viz/hit_test/public/interfaces/hit_test_region_list.mojom-shared.h"
#include "cc/ipc/surface_id.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/transform.mojom.h"
#include <string>
#include <vector>


namespace viz {
namespace mojom {
constexpr uint32_t kHitTestMine = 0x01U;
constexpr uint32_t kHitTestIgnore = 0x02U;
constexpr uint32_t kHitTestChildSurface = 0x04U;
constexpr uint32_t kHitTestAsk = 0x08U;
constexpr uint32_t kHitTestTouchEventHandler = 0x10U;
class HitTestRegion;
using HitTestRegionPtr = mojo::StructPtr<HitTestRegion>;

class HitTestRegionList;
using HitTestRegionListPtr = mojo::StructPtr<HitTestRegionList>;







class  HitTestRegion {
 public:
  using DataView = HitTestRegionDataView;
  using Data_ = internal::HitTestRegion_Data;

  template <typename... Args>
  static HitTestRegionPtr New(Args&&... args) {
    return HitTestRegionPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static HitTestRegionPtr From(const U& u) {
    return mojo::TypeConverter<HitTestRegionPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, HitTestRegion>::Convert(*this);
  }


  HitTestRegion();

  HitTestRegion(
      uint32_t flags,
      const viz::SurfaceId& surface_id,
      const gfx::Rect& rect,
      const gfx::Transform& transform);

  ~HitTestRegion();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = HitTestRegionPtr>
  HitTestRegionPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, HitTestRegion>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        HitTestRegion::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        HitTestRegion::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint32_t flags;
  viz::SurfaceId surface_id;
  gfx::Rect rect;
  gfx::Transform transform;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class  HitTestRegionList {
 public:
  using DataView = HitTestRegionListDataView;
  using Data_ = internal::HitTestRegionList_Data;

  template <typename... Args>
  static HitTestRegionListPtr New(Args&&... args) {
    return HitTestRegionListPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static HitTestRegionListPtr From(const U& u) {
    return mojo::TypeConverter<HitTestRegionListPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, HitTestRegionList>::Convert(*this);
  }


  HitTestRegionList();

  HitTestRegionList(
      const viz::SurfaceId& surface_id,
      uint32_t flags,
      const gfx::Rect& bounds,
      const gfx::Transform& transform,
      std::vector<HitTestRegionPtr> regions);

  ~HitTestRegionList();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = HitTestRegionListPtr>
  HitTestRegionListPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, HitTestRegionList>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        HitTestRegionList::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        HitTestRegionList::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  viz::SurfaceId surface_id;
  uint32_t flags;
  gfx::Rect bounds;
  gfx::Transform transform;
  std::vector<HitTestRegionPtr> regions;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(HitTestRegionList);
};


template <typename StructPtrType>
HitTestRegionPtr HitTestRegion::Clone() const {
  return New(
      mojo::Clone(flags),
      mojo::Clone(surface_id),
      mojo::Clone(rect),
      mojo::Clone(transform)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, HitTestRegion>::value>::type*>
bool HitTestRegion::Equals(const T& other) const {
  if (!mojo::Equals(this->flags, other.flags))
    return false;
  if (!mojo::Equals(this->surface_id, other.surface_id))
    return false;
  if (!mojo::Equals(this->rect, other.rect))
    return false;
  if (!mojo::Equals(this->transform, other.transform))
    return false;
  return true;
}
template <typename StructPtrType>
HitTestRegionListPtr HitTestRegionList::Clone() const {
  return New(
      mojo::Clone(surface_id),
      mojo::Clone(flags),
      mojo::Clone(bounds),
      mojo::Clone(transform),
      mojo::Clone(regions)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, HitTestRegionList>::value>::type*>
bool HitTestRegionList::Equals(const T& other) const {
  if (!mojo::Equals(this->surface_id, other.surface_id))
    return false;
  if (!mojo::Equals(this->flags, other.flags))
    return false;
  if (!mojo::Equals(this->bounds, other.bounds))
    return false;
  if (!mojo::Equals(this->transform, other.transform))
    return false;
  if (!mojo::Equals(this->regions, other.regions))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace viz

namespace mojo {


template <>
struct  StructTraits<::viz::mojom::HitTestRegion::DataView,
                                         ::viz::mojom::HitTestRegionPtr> {
  static bool IsNull(const ::viz::mojom::HitTestRegionPtr& input) { return !input; }
  static void SetToNull(::viz::mojom::HitTestRegionPtr* output) { output->reset(); }

  static decltype(::viz::mojom::HitTestRegion::flags) flags(
      const ::viz::mojom::HitTestRegionPtr& input) {
    return input->flags;
  }

  static const decltype(::viz::mojom::HitTestRegion::surface_id)& surface_id(
      const ::viz::mojom::HitTestRegionPtr& input) {
    return input->surface_id;
  }

  static const decltype(::viz::mojom::HitTestRegion::rect)& rect(
      const ::viz::mojom::HitTestRegionPtr& input) {
    return input->rect;
  }

  static const decltype(::viz::mojom::HitTestRegion::transform)& transform(
      const ::viz::mojom::HitTestRegionPtr& input) {
    return input->transform;
  }

  static bool Read(::viz::mojom::HitTestRegion::DataView input, ::viz::mojom::HitTestRegionPtr* output);
};


template <>
struct  StructTraits<::viz::mojom::HitTestRegionList::DataView,
                                         ::viz::mojom::HitTestRegionListPtr> {
  static bool IsNull(const ::viz::mojom::HitTestRegionListPtr& input) { return !input; }
  static void SetToNull(::viz::mojom::HitTestRegionListPtr* output) { output->reset(); }

  static const decltype(::viz::mojom::HitTestRegionList::surface_id)& surface_id(
      const ::viz::mojom::HitTestRegionListPtr& input) {
    return input->surface_id;
  }

  static decltype(::viz::mojom::HitTestRegionList::flags) flags(
      const ::viz::mojom::HitTestRegionListPtr& input) {
    return input->flags;
  }

  static const decltype(::viz::mojom::HitTestRegionList::bounds)& bounds(
      const ::viz::mojom::HitTestRegionListPtr& input) {
    return input->bounds;
  }

  static const decltype(::viz::mojom::HitTestRegionList::transform)& transform(
      const ::viz::mojom::HitTestRegionListPtr& input) {
    return input->transform;
  }

  static const decltype(::viz::mojom::HitTestRegionList::regions)& regions(
      const ::viz::mojom::HitTestRegionListPtr& input) {
    return input->regions;
  }

  static bool Read(::viz::mojom::HitTestRegionList::DataView input, ::viz::mojom::HitTestRegionListPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_VIZ_HIT_TEST_PUBLIC_INTERFACES_HIT_TEST_REGION_LIST_MOJOM_H_