// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef STORAGE_PUBLIC_INTERFACES_BLOBS_MOJOM_SHARED_INTERNAL_H_
#define STORAGE_PUBLIC_INTERFACES_BLOBS_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "mojo/common/file.mojom-shared-internal.h"
#include "mojo/common/file_path.mojom-shared-internal.h"
#include "mojo/common/time.mojom-shared-internal.h"
#include "url/mojo/url.mojom-shared-internal.h"
#include "storage/public/interfaces/interfaces_shared_export.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace storage {
namespace mojom {
namespace internal {
class DataElementBytes_Data;
class DataElementFile_Data;
class DataElementFilesystemURL_Data;
class DataElementBlob_Data;
class DataElement_Data;

#pragma pack(push, 1)


class MOJOM_SHARED_STORAGE_EXPORT DataElement_Data {
 public:
  // Used to identify Mojom Union Data Classes.
  typedef void MojomUnionDataType;

  DataElement_Data() {}
  // Do nothing in the destructor since it won't be called when it is a
  // non-inlined union.
  ~DataElement_Data() {}

  static DataElement_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(DataElement_Data))) DataElement_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context,
                       bool inlined);

  bool is_null() const { return size == 0; }

  void set_null() {
    size = 0U;
    tag = static_cast<DataElement_Tag>(0);
    data.unknown = 0U;
  }

  enum class DataElement_Tag : uint32_t {

    BYTES,
    FILE,
    FILE_FILESYSTEM,
    BLOB,
  };

  // A note on layout:
  // "Each non-static data member is allocated as if it were the sole member of
  // a struct." - Section 9.5.2 ISO/IEC 14882:2011 (The C++ Spec)
  union MOJO_ALIGNAS(8) Union_ {
    mojo::internal::Pointer<internal::DataElementBytes_Data> f_bytes;
    mojo::internal::Pointer<internal::DataElementFile_Data> f_file;
    mojo::internal::Pointer<internal::DataElementFilesystemURL_Data> f_file_filesystem;
    mojo::internal::Pointer<internal::DataElementBlob_Data> f_blob;
    uint64_t unknown;
  };

  uint32_t size;
  DataElement_Tag tag;
  Union_ data;
};
static_assert(sizeof(DataElement_Data) == mojo::internal::kUnionDataSize,
              "Bad sizeof(DataElement_Data)");
class MOJOM_SHARED_STORAGE_EXPORT DataElementBytes_Data {
 public:
  static DataElementBytes_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint64_t length;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> embedded_data;
  mojo::internal::Interface_Data data;

 private:
  DataElementBytes_Data();
  ~DataElementBytes_Data() = delete;
};
static_assert(sizeof(DataElementBytes_Data) == 32,
              "Bad sizeof(DataElementBytes_Data)");
class MOJOM_SHARED_STORAGE_EXPORT DataElementFile_Data {
 public:
  static DataElementFile_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo::common::mojom::internal::FilePath_Data> path;
  uint64_t offset;
  uint64_t length;
  mojo::internal::Pointer<::mojo::common::mojom::internal::Time_Data> expected_modification_time;

 private:
  DataElementFile_Data();
  ~DataElementFile_Data() = delete;
};
static_assert(sizeof(DataElementFile_Data) == 40,
              "Bad sizeof(DataElementFile_Data)");
class MOJOM_SHARED_STORAGE_EXPORT DataElementFilesystemURL_Data {
 public:
  static DataElementFilesystemURL_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> url;
  uint64_t offset;
  uint64_t length;
  mojo::internal::Pointer<::mojo::common::mojom::internal::Time_Data> expected_modification_time;

 private:
  DataElementFilesystemURL_Data();
  ~DataElementFilesystemURL_Data() = delete;
};
static_assert(sizeof(DataElementFilesystemURL_Data) == 40,
              "Bad sizeof(DataElementFilesystemURL_Data)");
class MOJOM_SHARED_STORAGE_EXPORT DataElementBlob_Data {
 public:
  static DataElementBlob_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Interface_Data blob;
  uint64_t offset;
  uint64_t length;

 private:
  DataElementBlob_Data();
  ~DataElementBlob_Data() = delete;
};
static_assert(sizeof(DataElementBlob_Data) == 32,
              "Bad sizeof(DataElementBlob_Data)");
constexpr uint32_t kBlob_Clone_Name = 0;
class MOJOM_SHARED_STORAGE_EXPORT Blob_Clone_Params_Data {
 public:
  static Blob_Clone_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data blob;
  uint8_t padfinal_[4];

 private:
  Blob_Clone_Params_Data();
  ~Blob_Clone_Params_Data() = delete;
};
static_assert(sizeof(Blob_Clone_Params_Data) == 16,
              "Bad sizeof(Blob_Clone_Params_Data)");
constexpr uint32_t kBlob_GetInternalUUID_Name = 1;
class MOJOM_SHARED_STORAGE_EXPORT Blob_GetInternalUUID_Params_Data {
 public:
  static Blob_GetInternalUUID_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  Blob_GetInternalUUID_Params_Data();
  ~Blob_GetInternalUUID_Params_Data() = delete;
};
static_assert(sizeof(Blob_GetInternalUUID_Params_Data) == 8,
              "Bad sizeof(Blob_GetInternalUUID_Params_Data)");
class MOJOM_SHARED_STORAGE_EXPORT Blob_GetInternalUUID_ResponseParams_Data {
 public:
  static Blob_GetInternalUUID_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> uuid;

 private:
  Blob_GetInternalUUID_ResponseParams_Data();
  ~Blob_GetInternalUUID_ResponseParams_Data() = delete;
};
static_assert(sizeof(Blob_GetInternalUUID_ResponseParams_Data) == 16,
              "Bad sizeof(Blob_GetInternalUUID_ResponseParams_Data)");
constexpr uint32_t kBlobRegistry_Register_Name = 0;
class MOJOM_SHARED_STORAGE_EXPORT BlobRegistry_Register_Params_Data {
 public:
  static BlobRegistry_Register_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data blob;
  uint8_t pad0_[4];
  mojo::internal::Pointer<mojo::internal::String_Data> uuid;
  mojo::internal::Pointer<mojo::internal::String_Data> content_type;
  mojo::internal::Pointer<mojo::internal::String_Data> content_disposition;
  mojo::internal::Pointer<mojo::internal::Array_Data<internal::DataElement_Data>> elements;

 private:
  BlobRegistry_Register_Params_Data();
  ~BlobRegistry_Register_Params_Data() = delete;
};
static_assert(sizeof(BlobRegistry_Register_Params_Data) == 48,
              "Bad sizeof(BlobRegistry_Register_Params_Data)");
class MOJOM_SHARED_STORAGE_EXPORT BlobRegistry_Register_ResponseParams_Data {
 public:
  static BlobRegistry_Register_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  BlobRegistry_Register_ResponseParams_Data();
  ~BlobRegistry_Register_ResponseParams_Data() = delete;
};
static_assert(sizeof(BlobRegistry_Register_ResponseParams_Data) == 8,
              "Bad sizeof(BlobRegistry_Register_ResponseParams_Data)");
constexpr uint32_t kBlobRegistry_GetBlobFromUUID_Name = 1;
class MOJOM_SHARED_STORAGE_EXPORT BlobRegistry_GetBlobFromUUID_Params_Data {
 public:
  static BlobRegistry_GetBlobFromUUID_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data blob;
  uint8_t pad0_[4];
  mojo::internal::Pointer<mojo::internal::String_Data> uuid;

 private:
  BlobRegistry_GetBlobFromUUID_Params_Data();
  ~BlobRegistry_GetBlobFromUUID_Params_Data() = delete;
};
static_assert(sizeof(BlobRegistry_GetBlobFromUUID_Params_Data) == 24,
              "Bad sizeof(BlobRegistry_GetBlobFromUUID_Params_Data)");
constexpr uint32_t kBytesProvider_RequestAsReply_Name = 0;
class MOJOM_SHARED_STORAGE_EXPORT BytesProvider_RequestAsReply_Params_Data {
 public:
  static BytesProvider_RequestAsReply_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  BytesProvider_RequestAsReply_Params_Data();
  ~BytesProvider_RequestAsReply_Params_Data() = delete;
};
static_assert(sizeof(BytesProvider_RequestAsReply_Params_Data) == 8,
              "Bad sizeof(BytesProvider_RequestAsReply_Params_Data)");
class MOJOM_SHARED_STORAGE_EXPORT BytesProvider_RequestAsReply_ResponseParams_Data {
 public:
  static BytesProvider_RequestAsReply_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> data;

 private:
  BytesProvider_RequestAsReply_ResponseParams_Data();
  ~BytesProvider_RequestAsReply_ResponseParams_Data() = delete;
};
static_assert(sizeof(BytesProvider_RequestAsReply_ResponseParams_Data) == 16,
              "Bad sizeof(BytesProvider_RequestAsReply_ResponseParams_Data)");
constexpr uint32_t kBytesProvider_RequestAsStream_Name = 1;
class MOJOM_SHARED_STORAGE_EXPORT BytesProvider_RequestAsStream_Params_Data {
 public:
  static BytesProvider_RequestAsStream_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data pipe;
  uint8_t padfinal_[4];

 private:
  BytesProvider_RequestAsStream_Params_Data();
  ~BytesProvider_RequestAsStream_Params_Data() = delete;
};
static_assert(sizeof(BytesProvider_RequestAsStream_Params_Data) == 16,
              "Bad sizeof(BytesProvider_RequestAsStream_Params_Data)");
constexpr uint32_t kBytesProvider_RequestAsFile_Name = 2;
class MOJOM_SHARED_STORAGE_EXPORT BytesProvider_RequestAsFile_Params_Data {
 public:
  static BytesProvider_RequestAsFile_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint64_t source_offset;
  uint64_t source_size;
  mojo::internal::Pointer<::mojo::common::mojom::internal::File_Data> file;
  uint64_t file_offset;

 private:
  BytesProvider_RequestAsFile_Params_Data();
  ~BytesProvider_RequestAsFile_Params_Data() = delete;
};
static_assert(sizeof(BytesProvider_RequestAsFile_Params_Data) == 40,
              "Bad sizeof(BytesProvider_RequestAsFile_Params_Data)");
class MOJOM_SHARED_STORAGE_EXPORT BytesProvider_RequestAsFile_ResponseParams_Data {
 public:
  static BytesProvider_RequestAsFile_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo::common::mojom::internal::Time_Data> time_file_modified;

 private:
  BytesProvider_RequestAsFile_ResponseParams_Data();
  ~BytesProvider_RequestAsFile_ResponseParams_Data() = delete;
};
static_assert(sizeof(BytesProvider_RequestAsFile_ResponseParams_Data) == 16,
              "Bad sizeof(BytesProvider_RequestAsFile_ResponseParams_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace storage

#endif  // STORAGE_PUBLIC_INTERFACES_BLOBS_MOJOM_SHARED_INTERNAL_H_