// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "third_party/WebKit/public/platform/modules/background_fetch/background_fetch.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
#include "third_party/WebKit/Source/platform/mojo/FetchAPIRequestStructTraits.h"
#include "third_party/WebKit/Source/platform/mojo/KURLStructTraits.h"
#include "third_party/WebKit/Source/platform/mojo/ReferrerStructTraits.h"
#include "third_party/WebKit/Source/platform/mojo/SecurityOriginStructTraits.h"
#include "third_party/WebKit/public/platform/ReferrerPolicyEnumTraits.h"
namespace blink {
namespace mojom {
namespace blink {
IconDefinition::IconDefinition()
    : src(),
      sizes(),
      type() {}

IconDefinition::IconDefinition(
    const WTF::String& src_in,
    const WTF::String& sizes_in,
    const WTF::String& type_in)
    : src(std::move(src_in)),
      sizes(std::move(sizes_in)),
      type(std::move(type_in)) {}

IconDefinition::~IconDefinition() = default;
size_t IconDefinition::Hash(size_t seed) const {
  seed = mojo::internal::WTFHash(seed, this->src);
  seed = mojo::internal::WTFHash(seed, this->sizes);
  seed = mojo::internal::WTFHash(seed, this->type);
  return seed;
}

bool IconDefinition::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
BackgroundFetchOptions::BackgroundFetchOptions()
    : icons(),
      total_download_size(),
      title() {}

BackgroundFetchOptions::BackgroundFetchOptions(
    WTF::Vector<IconDefinitionPtr> icons_in,
    int64_t total_download_size_in,
    const WTF::String& title_in)
    : icons(std::move(icons_in)),
      total_download_size(std::move(total_download_size_in)),
      title(std::move(title_in)) {}

BackgroundFetchOptions::~BackgroundFetchOptions() = default;

bool BackgroundFetchOptions::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
BackgroundFetchRegistration::BackgroundFetchRegistration()
    : tag(),
      icons(),
      total_download_size(),
      title() {}

BackgroundFetchRegistration::BackgroundFetchRegistration(
    const WTF::String& tag_in,
    WTF::Vector<IconDefinitionPtr> icons_in,
    int64_t total_download_size_in,
    const WTF::String& title_in)
    : tag(std::move(tag_in)),
      icons(std::move(icons_in)),
      total_download_size(std::move(total_download_size_in)),
      title(std::move(title_in)) {}

BackgroundFetchRegistration::~BackgroundFetchRegistration() = default;

bool BackgroundFetchRegistration::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char BackgroundFetchService::Name_[] = "blink::mojom::BackgroundFetchService";

class BackgroundFetchService_Fetch_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  BackgroundFetchService_Fetch_ForwardToCallback(
      BackgroundFetchService::FetchCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  BackgroundFetchService::FetchCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_Fetch_ForwardToCallback);
};

class BackgroundFetchService_UpdateUI_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  BackgroundFetchService_UpdateUI_ForwardToCallback(
      BackgroundFetchService::UpdateUICallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  BackgroundFetchService::UpdateUICallback callback_;
  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_UpdateUI_ForwardToCallback);
};

class BackgroundFetchService_Abort_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  BackgroundFetchService_Abort_ForwardToCallback(
      BackgroundFetchService::AbortCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  BackgroundFetchService::AbortCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_Abort_ForwardToCallback);
};

class BackgroundFetchService_GetRegistration_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  BackgroundFetchService_GetRegistration_ForwardToCallback(
      BackgroundFetchService::GetRegistrationCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  BackgroundFetchService::GetRegistrationCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_GetRegistration_ForwardToCallback);
};

class BackgroundFetchService_GetTags_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  BackgroundFetchService_GetTags_ForwardToCallback(
      BackgroundFetchService::GetTagsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  BackgroundFetchService::GetTagsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_GetTags_ForwardToCallback);
};

BackgroundFetchServiceProxy::BackgroundFetchServiceProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void BackgroundFetchServiceProxy::Fetch(
    int64_t in_service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& in_origin, const WTF::String& in_tag, const WTF::Vector<::blink::WebServiceWorkerRequest>& in_requests, BackgroundFetchOptionsPtr in_options, FetchCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_Fetch_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::OriginDataView>(
      in_origin, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_tag, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::blink::mojom::FetchAPIRequestDataView>>(
      in_requests, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::BackgroundFetchOptionsDataView>(
      in_options, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_Fetch_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_Fetch_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->service_worker_registration_id = in_service_worker_registration_id;
  typename decltype(params->origin)::BaseType* origin_ptr;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      in_origin, buffer, &origin_ptr, &serialization_context);
  params->origin.Set(origin_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in BackgroundFetchService.Fetch request");
  typename decltype(params->tag)::BaseType* tag_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_tag, buffer, &tag_ptr, &serialization_context);
  params->tag.Set(tag_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->tag.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null tag in BackgroundFetchService.Fetch request");
  typename decltype(params->requests)::BaseType* requests_ptr;
  const mojo::internal::ContainerValidateParams requests_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::blink::mojom::FetchAPIRequestDataView>>(
      in_requests, buffer, &requests_ptr, &requests_validate_params,
      &serialization_context);
  params->requests.Set(requests_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->requests.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null requests in BackgroundFetchService.Fetch request");
  typename decltype(params->options)::BaseType* options_ptr;
  mojo::internal::Serialize<::blink::mojom::BackgroundFetchOptionsDataView>(
      in_options, buffer, &options_ptr, &serialization_context);
  params->options.Set(options_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->options.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null options in BackgroundFetchService.Fetch request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new BackgroundFetchService_Fetch_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void BackgroundFetchServiceProxy::UpdateUI(
    int64_t in_service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& in_origin, const WTF::String& in_tag, const WTF::String& in_title, UpdateUICallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_UpdateUI_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::OriginDataView>(
      in_origin, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_tag, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_title, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_UpdateUI_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_UpdateUI_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->service_worker_registration_id = in_service_worker_registration_id;
  typename decltype(params->origin)::BaseType* origin_ptr;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      in_origin, buffer, &origin_ptr, &serialization_context);
  params->origin.Set(origin_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in BackgroundFetchService.UpdateUI request");
  typename decltype(params->tag)::BaseType* tag_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_tag, buffer, &tag_ptr, &serialization_context);
  params->tag.Set(tag_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->tag.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null tag in BackgroundFetchService.UpdateUI request");
  typename decltype(params->title)::BaseType* title_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_title, buffer, &title_ptr, &serialization_context);
  params->title.Set(title_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->title.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null title in BackgroundFetchService.UpdateUI request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new BackgroundFetchService_UpdateUI_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void BackgroundFetchServiceProxy::Abort(
    int64_t in_service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& in_origin, const WTF::String& in_tag, AbortCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_Abort_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::OriginDataView>(
      in_origin, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_tag, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_Abort_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_Abort_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->service_worker_registration_id = in_service_worker_registration_id;
  typename decltype(params->origin)::BaseType* origin_ptr;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      in_origin, buffer, &origin_ptr, &serialization_context);
  params->origin.Set(origin_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in BackgroundFetchService.Abort request");
  typename decltype(params->tag)::BaseType* tag_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_tag, buffer, &tag_ptr, &serialization_context);
  params->tag.Set(tag_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->tag.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null tag in BackgroundFetchService.Abort request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new BackgroundFetchService_Abort_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void BackgroundFetchServiceProxy::GetRegistration(
    int64_t in_service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& in_origin, const WTF::String& in_tag, GetRegistrationCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_GetRegistration_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::OriginDataView>(
      in_origin, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_tag, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_GetRegistration_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_GetRegistration_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->service_worker_registration_id = in_service_worker_registration_id;
  typename decltype(params->origin)::BaseType* origin_ptr;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      in_origin, buffer, &origin_ptr, &serialization_context);
  params->origin.Set(origin_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in BackgroundFetchService.GetRegistration request");
  typename decltype(params->tag)::BaseType* tag_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_tag, buffer, &tag_ptr, &serialization_context);
  params->tag.Set(tag_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->tag.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null tag in BackgroundFetchService.GetRegistration request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new BackgroundFetchService_GetRegistration_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void BackgroundFetchServiceProxy::GetTags(
    int64_t in_service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& in_origin, GetTagsCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_GetTags_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::OriginDataView>(
      in_origin, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_GetTags_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_GetTags_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->service_worker_registration_id = in_service_worker_registration_id;
  typename decltype(params->origin)::BaseType* origin_ptr;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      in_origin, buffer, &origin_ptr, &serialization_context);
  params->origin.Set(origin_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in BackgroundFetchService.GetTags request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new BackgroundFetchService_GetTags_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class BackgroundFetchService_Fetch_ProxyToResponder {
 public:
  static BackgroundFetchService::FetchCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<BackgroundFetchService_Fetch_ProxyToResponder> proxy(
        new BackgroundFetchService_Fetch_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&BackgroundFetchService_Fetch_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~BackgroundFetchService_Fetch_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "BackgroundFetchService::Fetch() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  BackgroundFetchService_Fetch_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      BackgroundFetchError in_error, BackgroundFetchRegistrationPtr in_registration);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_Fetch_ProxyToResponder);
};

bool BackgroundFetchService_Fetch_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::BackgroundFetchService_Fetch_ResponseParams_Data* params =
      reinterpret_cast<
          internal::BackgroundFetchService_Fetch_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  BackgroundFetchError p_error{};
  BackgroundFetchRegistrationPtr p_registration{};
  BackgroundFetchService_Fetch_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadError(&p_error))
    success = false;
  if (!input_data_view.ReadRegistration(&p_registration))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "BackgroundFetchService::Fetch response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_error), 
std::move(p_registration));
  return true;
}

void BackgroundFetchService_Fetch_ProxyToResponder::Run(
    BackgroundFetchError in_error, BackgroundFetchRegistrationPtr in_registration) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_Fetch_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::BackgroundFetchRegistrationDataView>(
      in_registration, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_Fetch_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_Fetch_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::blink::mojom::BackgroundFetchError>(
      in_error, &params->error);
  typename decltype(params->registration)::BaseType* registration_ptr;
  mojo::internal::Serialize<::blink::mojom::BackgroundFetchRegistrationDataView>(
      in_registration, buffer, &registration_ptr, &serialization_context);
  params->registration.Set(registration_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class BackgroundFetchService_UpdateUI_ProxyToResponder {
 public:
  static BackgroundFetchService::UpdateUICallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<BackgroundFetchService_UpdateUI_ProxyToResponder> proxy(
        new BackgroundFetchService_UpdateUI_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&BackgroundFetchService_UpdateUI_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~BackgroundFetchService_UpdateUI_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "BackgroundFetchService::UpdateUI() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  BackgroundFetchService_UpdateUI_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      BackgroundFetchError in_error);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_UpdateUI_ProxyToResponder);
};

bool BackgroundFetchService_UpdateUI_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::BackgroundFetchService_UpdateUI_ResponseParams_Data* params =
      reinterpret_cast<
          internal::BackgroundFetchService_UpdateUI_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  BackgroundFetchError p_error{};
  BackgroundFetchService_UpdateUI_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadError(&p_error))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "BackgroundFetchService::UpdateUI response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_error));
  return true;
}

void BackgroundFetchService_UpdateUI_ProxyToResponder::Run(
    BackgroundFetchError in_error) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_UpdateUI_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_UpdateUI_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_UpdateUI_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::blink::mojom::BackgroundFetchError>(
      in_error, &params->error);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class BackgroundFetchService_Abort_ProxyToResponder {
 public:
  static BackgroundFetchService::AbortCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<BackgroundFetchService_Abort_ProxyToResponder> proxy(
        new BackgroundFetchService_Abort_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&BackgroundFetchService_Abort_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~BackgroundFetchService_Abort_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "BackgroundFetchService::Abort() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  BackgroundFetchService_Abort_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      BackgroundFetchError in_error);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_Abort_ProxyToResponder);
};

bool BackgroundFetchService_Abort_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::BackgroundFetchService_Abort_ResponseParams_Data* params =
      reinterpret_cast<
          internal::BackgroundFetchService_Abort_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  BackgroundFetchError p_error{};
  BackgroundFetchService_Abort_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadError(&p_error))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "BackgroundFetchService::Abort response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_error));
  return true;
}

void BackgroundFetchService_Abort_ProxyToResponder::Run(
    BackgroundFetchError in_error) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_Abort_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_Abort_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_Abort_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::blink::mojom::BackgroundFetchError>(
      in_error, &params->error);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class BackgroundFetchService_GetRegistration_ProxyToResponder {
 public:
  static BackgroundFetchService::GetRegistrationCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<BackgroundFetchService_GetRegistration_ProxyToResponder> proxy(
        new BackgroundFetchService_GetRegistration_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&BackgroundFetchService_GetRegistration_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~BackgroundFetchService_GetRegistration_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "BackgroundFetchService::GetRegistration() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  BackgroundFetchService_GetRegistration_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      BackgroundFetchError in_error, BackgroundFetchRegistrationPtr in_registration);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_GetRegistration_ProxyToResponder);
};

bool BackgroundFetchService_GetRegistration_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::BackgroundFetchService_GetRegistration_ResponseParams_Data* params =
      reinterpret_cast<
          internal::BackgroundFetchService_GetRegistration_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  BackgroundFetchError p_error{};
  BackgroundFetchRegistrationPtr p_registration{};
  BackgroundFetchService_GetRegistration_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadError(&p_error))
    success = false;
  if (!input_data_view.ReadRegistration(&p_registration))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "BackgroundFetchService::GetRegistration response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_error), 
std::move(p_registration));
  return true;
}

void BackgroundFetchService_GetRegistration_ProxyToResponder::Run(
    BackgroundFetchError in_error, BackgroundFetchRegistrationPtr in_registration) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_GetRegistration_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::BackgroundFetchRegistrationDataView>(
      in_registration, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_GetRegistration_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_GetRegistration_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::blink::mojom::BackgroundFetchError>(
      in_error, &params->error);
  typename decltype(params->registration)::BaseType* registration_ptr;
  mojo::internal::Serialize<::blink::mojom::BackgroundFetchRegistrationDataView>(
      in_registration, buffer, &registration_ptr, &serialization_context);
  params->registration.Set(registration_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class BackgroundFetchService_GetTags_ProxyToResponder {
 public:
  static BackgroundFetchService::GetTagsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<BackgroundFetchService_GetTags_ProxyToResponder> proxy(
        new BackgroundFetchService_GetTags_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&BackgroundFetchService_GetTags_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~BackgroundFetchService_GetTags_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "BackgroundFetchService::GetTags() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  BackgroundFetchService_GetTags_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      BackgroundFetchError in_error, const WTF::Vector<WTF::String>& in_tags);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchService_GetTags_ProxyToResponder);
};

bool BackgroundFetchService_GetTags_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::BackgroundFetchService_GetTags_ResponseParams_Data* params =
      reinterpret_cast<
          internal::BackgroundFetchService_GetTags_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  BackgroundFetchError p_error{};
  WTF::Vector<WTF::String> p_tags{};
  BackgroundFetchService_GetTags_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadError(&p_error))
    success = false;
  if (!input_data_view.ReadTags(&p_tags))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "BackgroundFetchService::GetTags response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_error), 
std::move(p_tags));
  return true;
}

void BackgroundFetchService_GetTags_ProxyToResponder::Run(
    BackgroundFetchError in_error, const WTF::Vector<WTF::String>& in_tags) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::BackgroundFetchService_GetTags_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<mojo::StringDataView>>(
      in_tags, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBackgroundFetchService_GetTags_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::BackgroundFetchService_GetTags_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::blink::mojom::BackgroundFetchError>(
      in_error, &params->error);
  typename decltype(params->tags)::BaseType* tags_ptr;
  const mojo::internal::ContainerValidateParams tags_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      in_tags, buffer, &tags_ptr, &tags_validate_params,
      &serialization_context);
  params->tags.Set(tags_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->tags.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null tags in ");

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool BackgroundFetchServiceStubDispatch::Accept(
    BackgroundFetchService* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kBackgroundFetchService_Fetch_Name: {
      break;
    }
    case internal::kBackgroundFetchService_UpdateUI_Name: {
      break;
    }
    case internal::kBackgroundFetchService_Abort_Name: {
      break;
    }
    case internal::kBackgroundFetchService_GetRegistration_Name: {
      break;
    }
    case internal::kBackgroundFetchService_GetTags_Name: {
      break;
    }
  }
  return false;
}

// static
bool BackgroundFetchServiceStubDispatch::AcceptWithResponder(
    BackgroundFetchService* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kBackgroundFetchService_Fetch_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::BackgroundFetchService_Fetch_Params_Data* params =
          reinterpret_cast<
              internal::BackgroundFetchService_Fetch_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int64_t p_service_worker_registration_id{};
      RefPtr<::blink::SecurityOrigin> p_origin{};
      WTF::String p_tag{};
      WTF::Vector<::blink::WebServiceWorkerRequest> p_requests{};
      BackgroundFetchOptionsPtr p_options{};
      BackgroundFetchService_Fetch_ParamsDataView input_data_view(params, &serialization_context);
      
      p_service_worker_registration_id = input_data_view.service_worker_registration_id();
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      if (!input_data_view.ReadTag(&p_tag))
        success = false;
      if (!input_data_view.ReadRequests(&p_requests))
        success = false;
      if (!input_data_view.ReadOptions(&p_options))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BackgroundFetchService::Fetch deserializer");
        return false;
      }
      BackgroundFetchService::FetchCallback callback =
          BackgroundFetchService_Fetch_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Fetch(
std::move(p_service_worker_registration_id), 
std::move(p_origin), 
std::move(p_tag), 
std::move(p_requests), 
std::move(p_options), std::move(callback));
      return true;
    }
    case internal::kBackgroundFetchService_UpdateUI_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::BackgroundFetchService_UpdateUI_Params_Data* params =
          reinterpret_cast<
              internal::BackgroundFetchService_UpdateUI_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int64_t p_service_worker_registration_id{};
      RefPtr<::blink::SecurityOrigin> p_origin{};
      WTF::String p_tag{};
      WTF::String p_title{};
      BackgroundFetchService_UpdateUI_ParamsDataView input_data_view(params, &serialization_context);
      
      p_service_worker_registration_id = input_data_view.service_worker_registration_id();
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      if (!input_data_view.ReadTag(&p_tag))
        success = false;
      if (!input_data_view.ReadTitle(&p_title))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BackgroundFetchService::UpdateUI deserializer");
        return false;
      }
      BackgroundFetchService::UpdateUICallback callback =
          BackgroundFetchService_UpdateUI_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->UpdateUI(
std::move(p_service_worker_registration_id), 
std::move(p_origin), 
std::move(p_tag), 
std::move(p_title), std::move(callback));
      return true;
    }
    case internal::kBackgroundFetchService_Abort_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::BackgroundFetchService_Abort_Params_Data* params =
          reinterpret_cast<
              internal::BackgroundFetchService_Abort_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int64_t p_service_worker_registration_id{};
      RefPtr<::blink::SecurityOrigin> p_origin{};
      WTF::String p_tag{};
      BackgroundFetchService_Abort_ParamsDataView input_data_view(params, &serialization_context);
      
      p_service_worker_registration_id = input_data_view.service_worker_registration_id();
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      if (!input_data_view.ReadTag(&p_tag))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BackgroundFetchService::Abort deserializer");
        return false;
      }
      BackgroundFetchService::AbortCallback callback =
          BackgroundFetchService_Abort_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Abort(
std::move(p_service_worker_registration_id), 
std::move(p_origin), 
std::move(p_tag), std::move(callback));
      return true;
    }
    case internal::kBackgroundFetchService_GetRegistration_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::BackgroundFetchService_GetRegistration_Params_Data* params =
          reinterpret_cast<
              internal::BackgroundFetchService_GetRegistration_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int64_t p_service_worker_registration_id{};
      RefPtr<::blink::SecurityOrigin> p_origin{};
      WTF::String p_tag{};
      BackgroundFetchService_GetRegistration_ParamsDataView input_data_view(params, &serialization_context);
      
      p_service_worker_registration_id = input_data_view.service_worker_registration_id();
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      if (!input_data_view.ReadTag(&p_tag))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BackgroundFetchService::GetRegistration deserializer");
        return false;
      }
      BackgroundFetchService::GetRegistrationCallback callback =
          BackgroundFetchService_GetRegistration_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetRegistration(
std::move(p_service_worker_registration_id), 
std::move(p_origin), 
std::move(p_tag), std::move(callback));
      return true;
    }
    case internal::kBackgroundFetchService_GetTags_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::BackgroundFetchService_GetTags_Params_Data* params =
          reinterpret_cast<
              internal::BackgroundFetchService_GetTags_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int64_t p_service_worker_registration_id{};
      RefPtr<::blink::SecurityOrigin> p_origin{};
      BackgroundFetchService_GetTags_ParamsDataView input_data_view(params, &serialization_context);
      
      p_service_worker_registration_id = input_data_view.service_worker_registration_id();
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BackgroundFetchService::GetTags deserializer");
        return false;
      }
      BackgroundFetchService::GetTagsCallback callback =
          BackgroundFetchService_GetTags_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetTags(
std::move(p_service_worker_registration_id), 
std::move(p_origin), std::move(callback));
      return true;
    }
  }
  return false;
}

bool BackgroundFetchServiceRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "BackgroundFetchService RequestValidator");

  switch (message->header()->name) {
    case internal::kBackgroundFetchService_Fetch_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_Fetch_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBackgroundFetchService_UpdateUI_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_UpdateUI_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBackgroundFetchService_Abort_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_Abort_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBackgroundFetchService_GetRegistration_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_GetRegistration_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBackgroundFetchService_GetTags_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_GetTags_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool BackgroundFetchServiceResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "BackgroundFetchService ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kBackgroundFetchService_Fetch_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_Fetch_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBackgroundFetchService_UpdateUI_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_UpdateUI_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBackgroundFetchService_Abort_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_Abort_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBackgroundFetchService_GetRegistration_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_GetRegistration_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBackgroundFetchService_GetTags_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::BackgroundFetchService_GetTags_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {


// static
bool StructTraits<::blink::mojom::blink::IconDefinition::DataView, ::blink::mojom::blink::IconDefinitionPtr>::Read(
    ::blink::mojom::blink::IconDefinition::DataView input,
    ::blink::mojom::blink::IconDefinitionPtr* output) {
  bool success = true;
  ::blink::mojom::blink::IconDefinitionPtr result(::blink::mojom::blink::IconDefinition::New());
  
      if (!input.ReadSrc(&result->src))
        success = false;
      if (!input.ReadSizes(&result->sizes))
        success = false;
      if (!input.ReadType(&result->type))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::blink::mojom::blink::BackgroundFetchOptions::DataView, ::blink::mojom::blink::BackgroundFetchOptionsPtr>::Read(
    ::blink::mojom::blink::BackgroundFetchOptions::DataView input,
    ::blink::mojom::blink::BackgroundFetchOptionsPtr* output) {
  bool success = true;
  ::blink::mojom::blink::BackgroundFetchOptionsPtr result(::blink::mojom::blink::BackgroundFetchOptions::New());
  
      if (!input.ReadIcons(&result->icons))
        success = false;
      result->total_download_size = input.total_download_size();
      if (!input.ReadTitle(&result->title))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::blink::mojom::blink::BackgroundFetchRegistration::DataView, ::blink::mojom::blink::BackgroundFetchRegistrationPtr>::Read(
    ::blink::mojom::blink::BackgroundFetchRegistration::DataView input,
    ::blink::mojom::blink::BackgroundFetchRegistrationPtr* output) {
  bool success = true;
  ::blink::mojom::blink::BackgroundFetchRegistrationPtr result(::blink::mojom::blink::BackgroundFetchRegistration::New());
  
      if (!input.ReadTag(&result->tag))
        success = false;
      if (!input.ReadIcons(&result->icons))
        success = false;
      result->total_download_size = input.total_download_size();
      if (!input.ReadTitle(&result->title))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif