// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "third_party/WebKit/public/platform/modules/payments/payment_app.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace payments {
namespace mojom {

namespace internal {


// static
ImageObject_Data* ImageObject_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ImageObject_Data))) ImageObject_Data();
}

// static
bool ImageObject_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ImageObject_Data* object = static_cast<const ImageObject_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->src, "null src field in ImageObject",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->src, validation_context))
    return false;

  return true;
}

ImageObject_Data::ImageObject_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentInstrument_Data* PaymentInstrument_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentInstrument_Data))) PaymentInstrument_Data();
}

// static
bool PaymentInstrument_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentInstrument_Data* object = static_cast<const PaymentInstrument_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->name, "null name field in PaymentInstrument",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams name_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->name, validation_context,
                                         &name_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->icons, "null icons field in PaymentInstrument",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams icons_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->icons, validation_context,
                                         &icons_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->enabled_methods, "null enabled_methods field in PaymentInstrument",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams enabled_methods_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  if (!mojo::internal::ValidateContainer(object->enabled_methods, validation_context,
                                         &enabled_methods_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->stringified_capabilities, "null stringified_capabilities field in PaymentInstrument",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams stringified_capabilities_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->stringified_capabilities, validation_context,
                                         &stringified_capabilities_validate_params)) {
    return false;
  }

  return true;
}

PaymentInstrument_Data::PaymentInstrument_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentRequestEventData_Data* PaymentRequestEventData_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentRequestEventData_Data))) PaymentRequestEventData_Data();
}

// static
bool PaymentRequestEventData_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentRequestEventData_Data* object = static_cast<const PaymentRequestEventData_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 64 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->top_level_origin, "null top_level_origin field in PaymentRequestEventData",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->top_level_origin, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->payment_request_origin, "null payment_request_origin field in PaymentRequestEventData",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->payment_request_origin, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->payment_request_id, "null payment_request_id field in PaymentRequestEventData",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams payment_request_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->payment_request_id, validation_context,
                                         &payment_request_id_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->method_data, "null method_data field in PaymentRequestEventData",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams method_data_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->method_data, validation_context,
                                         &method_data_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->total, "null total field in PaymentRequestEventData",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->total, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->modifiers, "null modifiers field in PaymentRequestEventData",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams modifiers_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->modifiers, validation_context,
                                         &modifiers_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->instrument_key, "null instrument_key field in PaymentRequestEventData",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams instrument_key_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->instrument_key, validation_context,
                                         &instrument_key_validate_params)) {
    return false;
  }

  return true;
}

PaymentRequestEventData_Data::PaymentRequestEventData_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentHandlerResponse_Data* PaymentHandlerResponse_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentHandlerResponse_Data))) PaymentHandlerResponse_Data();
}

// static
bool PaymentHandlerResponse_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentHandlerResponse_Data* object = static_cast<const PaymentHandlerResponse_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->method_name, "null method_name field in PaymentHandlerResponse",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams method_name_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->method_name, validation_context,
                                         &method_name_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->stringified_details, "null stringified_details field in PaymentHandlerResponse",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams stringified_details_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->stringified_details, validation_context,
                                         &stringified_details_validate_params)) {
    return false;
  }

  return true;
}

PaymentHandlerResponse_Data::PaymentHandlerResponse_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_Init_Params_Data* PaymentManager_Init_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_Init_Params_Data))) PaymentManager_Init_Params_Data();
}

// static
bool PaymentManager_Init_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_Init_Params_Data* object = static_cast<const PaymentManager_Init_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->service_worker_script_url, "null service_worker_script_url field in PaymentManager_Init_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams service_worker_script_url_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->service_worker_script_url, validation_context,
                                         &service_worker_script_url_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->service_worker_scope, "null service_worker_scope field in PaymentManager_Init_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams service_worker_scope_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->service_worker_scope, validation_context,
                                         &service_worker_scope_validate_params)) {
    return false;
  }

  return true;
}

PaymentManager_Init_Params_Data::PaymentManager_Init_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_DeletePaymentInstrument_Params_Data* PaymentManager_DeletePaymentInstrument_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_DeletePaymentInstrument_Params_Data))) PaymentManager_DeletePaymentInstrument_Params_Data();
}

// static
bool PaymentManager_DeletePaymentInstrument_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_DeletePaymentInstrument_Params_Data* object = static_cast<const PaymentManager_DeletePaymentInstrument_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->instrument_key, "null instrument_key field in PaymentManager_DeletePaymentInstrument_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams instrument_key_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->instrument_key, validation_context,
                                         &instrument_key_validate_params)) {
    return false;
  }

  return true;
}

PaymentManager_DeletePaymentInstrument_Params_Data::PaymentManager_DeletePaymentInstrument_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_DeletePaymentInstrument_ResponseParams_Data* PaymentManager_DeletePaymentInstrument_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_DeletePaymentInstrument_ResponseParams_Data))) PaymentManager_DeletePaymentInstrument_ResponseParams_Data();
}

// static
bool PaymentManager_DeletePaymentInstrument_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_DeletePaymentInstrument_ResponseParams_Data* object = static_cast<const PaymentManager_DeletePaymentInstrument_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::payments::mojom::internal::PaymentHandlerStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  return true;
}

PaymentManager_DeletePaymentInstrument_ResponseParams_Data::PaymentManager_DeletePaymentInstrument_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_GetPaymentInstrument_Params_Data* PaymentManager_GetPaymentInstrument_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_GetPaymentInstrument_Params_Data))) PaymentManager_GetPaymentInstrument_Params_Data();
}

// static
bool PaymentManager_GetPaymentInstrument_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_GetPaymentInstrument_Params_Data* object = static_cast<const PaymentManager_GetPaymentInstrument_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->instrument_key, "null instrument_key field in PaymentManager_GetPaymentInstrument_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams instrument_key_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->instrument_key, validation_context,
                                         &instrument_key_validate_params)) {
    return false;
  }

  return true;
}

PaymentManager_GetPaymentInstrument_Params_Data::PaymentManager_GetPaymentInstrument_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_GetPaymentInstrument_ResponseParams_Data* PaymentManager_GetPaymentInstrument_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_GetPaymentInstrument_ResponseParams_Data))) PaymentManager_GetPaymentInstrument_ResponseParams_Data();
}

// static
bool PaymentManager_GetPaymentInstrument_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_GetPaymentInstrument_ResponseParams_Data* object = static_cast<const PaymentManager_GetPaymentInstrument_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->instrument, "null instrument field in PaymentManager_GetPaymentInstrument_ResponseParams",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->instrument, validation_context))
    return false;


  if (!::payments::mojom::internal::PaymentHandlerStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  return true;
}

PaymentManager_GetPaymentInstrument_ResponseParams_Data::PaymentManager_GetPaymentInstrument_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_KeysOfPaymentInstruments_Params_Data* PaymentManager_KeysOfPaymentInstruments_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_KeysOfPaymentInstruments_Params_Data))) PaymentManager_KeysOfPaymentInstruments_Params_Data();
}

// static
bool PaymentManager_KeysOfPaymentInstruments_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_KeysOfPaymentInstruments_Params_Data* object = static_cast<const PaymentManager_KeysOfPaymentInstruments_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

PaymentManager_KeysOfPaymentInstruments_Params_Data::PaymentManager_KeysOfPaymentInstruments_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data* PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data))) PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data();
}

// static
bool PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data* object = static_cast<const PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->keys, "null keys field in PaymentManager_KeysOfPaymentInstruments_ResponseParams",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams keys_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  if (!mojo::internal::ValidateContainer(object->keys, validation_context,
                                         &keys_validate_params)) {
    return false;
  }


  if (!::payments::mojom::internal::PaymentHandlerStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  return true;
}

PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data::PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_HasPaymentInstrument_Params_Data* PaymentManager_HasPaymentInstrument_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_HasPaymentInstrument_Params_Data))) PaymentManager_HasPaymentInstrument_Params_Data();
}

// static
bool PaymentManager_HasPaymentInstrument_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_HasPaymentInstrument_Params_Data* object = static_cast<const PaymentManager_HasPaymentInstrument_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->instrument_key, "null instrument_key field in PaymentManager_HasPaymentInstrument_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams instrument_key_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->instrument_key, validation_context,
                                         &instrument_key_validate_params)) {
    return false;
  }

  return true;
}

PaymentManager_HasPaymentInstrument_Params_Data::PaymentManager_HasPaymentInstrument_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_HasPaymentInstrument_ResponseParams_Data* PaymentManager_HasPaymentInstrument_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_HasPaymentInstrument_ResponseParams_Data))) PaymentManager_HasPaymentInstrument_ResponseParams_Data();
}

// static
bool PaymentManager_HasPaymentInstrument_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_HasPaymentInstrument_ResponseParams_Data* object = static_cast<const PaymentManager_HasPaymentInstrument_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::payments::mojom::internal::PaymentHandlerStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  return true;
}

PaymentManager_HasPaymentInstrument_ResponseParams_Data::PaymentManager_HasPaymentInstrument_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_SetPaymentInstrument_Params_Data* PaymentManager_SetPaymentInstrument_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_SetPaymentInstrument_Params_Data))) PaymentManager_SetPaymentInstrument_Params_Data();
}

// static
bool PaymentManager_SetPaymentInstrument_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_SetPaymentInstrument_Params_Data* object = static_cast<const PaymentManager_SetPaymentInstrument_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->instrument_key, "null instrument_key field in PaymentManager_SetPaymentInstrument_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams instrument_key_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->instrument_key, validation_context,
                                         &instrument_key_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->instrument, "null instrument field in PaymentManager_SetPaymentInstrument_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->instrument, validation_context))
    return false;

  return true;
}

PaymentManager_SetPaymentInstrument_Params_Data::PaymentManager_SetPaymentInstrument_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_SetPaymentInstrument_ResponseParams_Data* PaymentManager_SetPaymentInstrument_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_SetPaymentInstrument_ResponseParams_Data))) PaymentManager_SetPaymentInstrument_ResponseParams_Data();
}

// static
bool PaymentManager_SetPaymentInstrument_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_SetPaymentInstrument_ResponseParams_Data* object = static_cast<const PaymentManager_SetPaymentInstrument_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::payments::mojom::internal::PaymentHandlerStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  return true;
}

PaymentManager_SetPaymentInstrument_ResponseParams_Data::PaymentManager_SetPaymentInstrument_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_ClearPaymentInstruments_Params_Data* PaymentManager_ClearPaymentInstruments_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_ClearPaymentInstruments_Params_Data))) PaymentManager_ClearPaymentInstruments_Params_Data();
}

// static
bool PaymentManager_ClearPaymentInstruments_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_ClearPaymentInstruments_Params_Data* object = static_cast<const PaymentManager_ClearPaymentInstruments_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

PaymentManager_ClearPaymentInstruments_Params_Data::PaymentManager_ClearPaymentInstruments_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentManager_ClearPaymentInstruments_ResponseParams_Data* PaymentManager_ClearPaymentInstruments_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentManager_ClearPaymentInstruments_ResponseParams_Data))) PaymentManager_ClearPaymentInstruments_ResponseParams_Data();
}

// static
bool PaymentManager_ClearPaymentInstruments_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentManager_ClearPaymentInstruments_ResponseParams_Data* object = static_cast<const PaymentManager_ClearPaymentInstruments_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::payments::mojom::internal::PaymentHandlerStatus_Data
        ::Validate(object->status, validation_context))
    return false;

  return true;
}

PaymentManager_ClearPaymentInstruments_ResponseParams_Data::PaymentManager_ClearPaymentInstruments_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data* PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data))) PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data();
}

// static
bool PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data* object = static_cast<const PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->response, "null response field in PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->response, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dispatch_event_time, "null dispatch_event_time field in PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->dispatch_event_time, validation_context))
    return false;

  return true;
}

PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data::PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace payments

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
