// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PAYMENTS_PAYMENT_APP_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PAYMENTS_PAYMENT_APP_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/payments/payment_app.mojom-shared.h"
#include "mojo/common/time.mojom.h"
#include "third_party/WebKit/public/platform/modules/payments/payment_request.mojom.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace payments {
namespace mojom {
class PaymentManager;
using PaymentManagerPtr = mojo::InterfacePtr<PaymentManager>;
using PaymentManagerPtrInfo = mojo::InterfacePtrInfo<PaymentManager>;
using ThreadSafePaymentManagerPtr =
    mojo::ThreadSafeInterfacePtr<PaymentManager>;
using PaymentManagerRequest = mojo::InterfaceRequest<PaymentManager>;
using PaymentManagerAssociatedPtr =
    mojo::AssociatedInterfacePtr<PaymentManager>;
using ThreadSafePaymentManagerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PaymentManager>;
using PaymentManagerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PaymentManager>;
using PaymentManagerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PaymentManager>;

class PaymentHandlerResponseCallback;
using PaymentHandlerResponseCallbackPtr = mojo::InterfacePtr<PaymentHandlerResponseCallback>;
using PaymentHandlerResponseCallbackPtrInfo = mojo::InterfacePtrInfo<PaymentHandlerResponseCallback>;
using ThreadSafePaymentHandlerResponseCallbackPtr =
    mojo::ThreadSafeInterfacePtr<PaymentHandlerResponseCallback>;
using PaymentHandlerResponseCallbackRequest = mojo::InterfaceRequest<PaymentHandlerResponseCallback>;
using PaymentHandlerResponseCallbackAssociatedPtr =
    mojo::AssociatedInterfacePtr<PaymentHandlerResponseCallback>;
using ThreadSafePaymentHandlerResponseCallbackAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PaymentHandlerResponseCallback>;
using PaymentHandlerResponseCallbackAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PaymentHandlerResponseCallback>;
using PaymentHandlerResponseCallbackAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PaymentHandlerResponseCallback>;

class ImageObject;
using ImageObjectPtr = mojo::StructPtr<ImageObject>;

class PaymentInstrument;
using PaymentInstrumentPtr = mojo::StructPtr<PaymentInstrument>;

class PaymentRequestEventData;
using PaymentRequestEventDataPtr = mojo::StructPtr<PaymentRequestEventData>;

class PaymentHandlerResponse;
using PaymentHandlerResponsePtr = mojo::InlinedStructPtr<PaymentHandlerResponse>;


class PaymentManagerProxy;

template <typename ImplRefTraits>
class PaymentManagerStub;

class PaymentManagerRequestValidator;
class PaymentManagerResponseValidator;

class CONTENT_EXPORT PaymentManager
    : public PaymentManagerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PaymentManagerProxy;

  template <typename ImplRefTraits>
  using Stub_ = PaymentManagerStub<ImplRefTraits>;

  using RequestValidator_ = PaymentManagerRequestValidator;
  using ResponseValidator_ = PaymentManagerResponseValidator;
  enum MethodMinVersions : uint32_t {
    kInitMinVersion = 0,
    kDeletePaymentInstrumentMinVersion = 0,
    kGetPaymentInstrumentMinVersion = 0,
    kKeysOfPaymentInstrumentsMinVersion = 0,
    kHasPaymentInstrumentMinVersion = 0,
    kSetPaymentInstrumentMinVersion = 0,
    kClearPaymentInstrumentsMinVersion = 0,
  };
  virtual ~PaymentManager() {}

  virtual void Init(const std::string& service_worker_script_url, const std::string& service_worker_scope) = 0;


  using DeletePaymentInstrumentCallback = base::OnceCallback<void(PaymentHandlerStatus)>;
  virtual void DeletePaymentInstrument(const std::string& instrument_key, DeletePaymentInstrumentCallback callback) = 0;


  using GetPaymentInstrumentCallback = base::OnceCallback<void(PaymentInstrumentPtr, PaymentHandlerStatus)>;
  virtual void GetPaymentInstrument(const std::string& instrument_key, GetPaymentInstrumentCallback callback) = 0;


  using KeysOfPaymentInstrumentsCallback = base::OnceCallback<void(const std::vector<std::string>&, PaymentHandlerStatus)>;
  virtual void KeysOfPaymentInstruments(KeysOfPaymentInstrumentsCallback callback) = 0;


  using HasPaymentInstrumentCallback = base::OnceCallback<void(PaymentHandlerStatus)>;
  virtual void HasPaymentInstrument(const std::string& instrument_key, HasPaymentInstrumentCallback callback) = 0;


  using SetPaymentInstrumentCallback = base::OnceCallback<void(PaymentHandlerStatus)>;
  virtual void SetPaymentInstrument(const std::string& instrument_key, PaymentInstrumentPtr instrument, SetPaymentInstrumentCallback callback) = 0;


  using ClearPaymentInstrumentsCallback = base::OnceCallback<void(PaymentHandlerStatus)>;
  virtual void ClearPaymentInstruments(ClearPaymentInstrumentsCallback callback) = 0;
};

class PaymentHandlerResponseCallbackProxy;

template <typename ImplRefTraits>
class PaymentHandlerResponseCallbackStub;

class PaymentHandlerResponseCallbackRequestValidator;

class CONTENT_EXPORT PaymentHandlerResponseCallback
    : public PaymentHandlerResponseCallbackInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PaymentHandlerResponseCallbackProxy;

  template <typename ImplRefTraits>
  using Stub_ = PaymentHandlerResponseCallbackStub<ImplRefTraits>;

  using RequestValidator_ = PaymentHandlerResponseCallbackRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnPaymentHandlerResponseMinVersion = 0,
  };
  virtual ~PaymentHandlerResponseCallback() {}

  virtual void OnPaymentHandlerResponse(PaymentHandlerResponsePtr response, base::Time dispatch_event_time) = 0;
};

class CONTENT_EXPORT PaymentManagerProxy
    : public PaymentManager {
 public:
  explicit PaymentManagerProxy(mojo::MessageReceiverWithResponder* receiver);
  void Init(const std::string& service_worker_script_url, const std::string& service_worker_scope) override;
  void DeletePaymentInstrument(const std::string& instrument_key, DeletePaymentInstrumentCallback callback) override;
  void GetPaymentInstrument(const std::string& instrument_key, GetPaymentInstrumentCallback callback) override;
  void KeysOfPaymentInstruments(KeysOfPaymentInstrumentsCallback callback) override;
  void HasPaymentInstrument(const std::string& instrument_key, HasPaymentInstrumentCallback callback) override;
  void SetPaymentInstrument(const std::string& instrument_key, PaymentInstrumentPtr instrument, SetPaymentInstrumentCallback callback) override;
  void ClearPaymentInstruments(ClearPaymentInstrumentsCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT PaymentHandlerResponseCallbackProxy
    : public PaymentHandlerResponseCallback {
 public:
  explicit PaymentHandlerResponseCallbackProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnPaymentHandlerResponse(PaymentHandlerResponsePtr response, base::Time dispatch_event_time) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT PaymentManagerStubDispatch {
 public:
  static bool Accept(PaymentManager* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PaymentManager* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PaymentManager>>
class PaymentManagerStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PaymentManagerStub() {}
  ~PaymentManagerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PaymentManagerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PaymentManagerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT PaymentHandlerResponseCallbackStubDispatch {
 public:
  static bool Accept(PaymentHandlerResponseCallback* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PaymentHandlerResponseCallback* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PaymentHandlerResponseCallback>>
class PaymentHandlerResponseCallbackStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PaymentHandlerResponseCallbackStub() {}
  ~PaymentHandlerResponseCallbackStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PaymentHandlerResponseCallbackStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PaymentHandlerResponseCallbackStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT PaymentManagerRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT PaymentHandlerResponseCallbackRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT PaymentManagerResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





class CONTENT_EXPORT PaymentHandlerResponse {
 public:
  using DataView = PaymentHandlerResponseDataView;
  using Data_ = internal::PaymentHandlerResponse_Data;

  template <typename... Args>
  static PaymentHandlerResponsePtr New(Args&&... args) {
    return PaymentHandlerResponsePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PaymentHandlerResponsePtr From(const U& u) {
    return mojo::TypeConverter<PaymentHandlerResponsePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PaymentHandlerResponse>::Convert(*this);
  }


  PaymentHandlerResponse();

  PaymentHandlerResponse(
      const std::string& method_name,
      const std::string& stringified_details);

  ~PaymentHandlerResponse();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PaymentHandlerResponsePtr>
  PaymentHandlerResponsePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PaymentHandlerResponse>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PaymentHandlerResponse::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PaymentHandlerResponse::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::string method_name;
  std::string stringified_details;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class CONTENT_EXPORT ImageObject {
 public:
  using DataView = ImageObjectDataView;
  using Data_ = internal::ImageObject_Data;

  template <typename... Args>
  static ImageObjectPtr New(Args&&... args) {
    return ImageObjectPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ImageObjectPtr From(const U& u) {
    return mojo::TypeConverter<ImageObjectPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ImageObject>::Convert(*this);
  }


  ImageObject();

  explicit ImageObject(
      const GURL& src);

  ~ImageObject();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ImageObjectPtr>
  ImageObjectPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ImageObject>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ImageObject::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ImageObject::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  GURL src;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class CONTENT_EXPORT PaymentInstrument {
 public:
  using DataView = PaymentInstrumentDataView;
  using Data_ = internal::PaymentInstrument_Data;

  template <typename... Args>
  static PaymentInstrumentPtr New(Args&&... args) {
    return PaymentInstrumentPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PaymentInstrumentPtr From(const U& u) {
    return mojo::TypeConverter<PaymentInstrumentPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PaymentInstrument>::Convert(*this);
  }


  PaymentInstrument();

  PaymentInstrument(
      const std::string& name,
      std::vector<ImageObjectPtr> icons,
      const std::vector<std::string>& enabled_methods,
      const std::string& stringified_capabilities);

  ~PaymentInstrument();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PaymentInstrumentPtr>
  PaymentInstrumentPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PaymentInstrument>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PaymentInstrument::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PaymentInstrument::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::string name;
  std::vector<ImageObjectPtr> icons;
  std::vector<std::string> enabled_methods;
  std::string stringified_capabilities;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(PaymentInstrument);
};



class CONTENT_EXPORT PaymentRequestEventData {
 public:
  using DataView = PaymentRequestEventDataDataView;
  using Data_ = internal::PaymentRequestEventData_Data;

  template <typename... Args>
  static PaymentRequestEventDataPtr New(Args&&... args) {
    return PaymentRequestEventDataPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PaymentRequestEventDataPtr From(const U& u) {
    return mojo::TypeConverter<PaymentRequestEventDataPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PaymentRequestEventData>::Convert(*this);
  }


  PaymentRequestEventData();

  PaymentRequestEventData(
      const GURL& top_level_origin,
      const GURL& payment_request_origin,
      const std::string& payment_request_id,
      std::vector<::payments::mojom::PaymentMethodDataPtr> method_data,
      ::payments::mojom::PaymentCurrencyAmountPtr total,
      std::vector<::payments::mojom::PaymentDetailsModifierPtr> modifiers,
      const std::string& instrument_key);

  ~PaymentRequestEventData();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PaymentRequestEventDataPtr>
  PaymentRequestEventDataPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PaymentRequestEventData>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PaymentRequestEventData::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PaymentRequestEventData::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  GURL top_level_origin;
  GURL payment_request_origin;
  std::string payment_request_id;
  std::vector<::payments::mojom::PaymentMethodDataPtr> method_data;
  ::payments::mojom::PaymentCurrencyAmountPtr total;
  std::vector<::payments::mojom::PaymentDetailsModifierPtr> modifiers;
  std::string instrument_key;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(PaymentRequestEventData);
};



template <typename StructPtrType>
ImageObjectPtr ImageObject::Clone() const {
  return New(
      mojo::Clone(src)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ImageObject>::value>::type*>
bool ImageObject::Equals(const T& other) const {
  if (!mojo::Equals(this->src, other.src))
    return false;
  return true;
}
template <typename StructPtrType>
PaymentInstrumentPtr PaymentInstrument::Clone() const {
  return New(
      mojo::Clone(name),
      mojo::Clone(icons),
      mojo::Clone(enabled_methods),
      mojo::Clone(stringified_capabilities)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PaymentInstrument>::value>::type*>
bool PaymentInstrument::Equals(const T& other) const {
  if (!mojo::Equals(this->name, other.name))
    return false;
  if (!mojo::Equals(this->icons, other.icons))
    return false;
  if (!mojo::Equals(this->enabled_methods, other.enabled_methods))
    return false;
  if (!mojo::Equals(this->stringified_capabilities, other.stringified_capabilities))
    return false;
  return true;
}
template <typename StructPtrType>
PaymentRequestEventDataPtr PaymentRequestEventData::Clone() const {
  return New(
      mojo::Clone(top_level_origin),
      mojo::Clone(payment_request_origin),
      mojo::Clone(payment_request_id),
      mojo::Clone(method_data),
      mojo::Clone(total),
      mojo::Clone(modifiers),
      mojo::Clone(instrument_key)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PaymentRequestEventData>::value>::type*>
bool PaymentRequestEventData::Equals(const T& other) const {
  if (!mojo::Equals(this->top_level_origin, other.top_level_origin))
    return false;
  if (!mojo::Equals(this->payment_request_origin, other.payment_request_origin))
    return false;
  if (!mojo::Equals(this->payment_request_id, other.payment_request_id))
    return false;
  if (!mojo::Equals(this->method_data, other.method_data))
    return false;
  if (!mojo::Equals(this->total, other.total))
    return false;
  if (!mojo::Equals(this->modifiers, other.modifiers))
    return false;
  if (!mojo::Equals(this->instrument_key, other.instrument_key))
    return false;
  return true;
}
template <typename StructPtrType>
PaymentHandlerResponsePtr PaymentHandlerResponse::Clone() const {
  return New(
      mojo::Clone(method_name),
      mojo::Clone(stringified_details)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PaymentHandlerResponse>::value>::type*>
bool PaymentHandlerResponse::Equals(const T& other) const {
  if (!mojo::Equals(this->method_name, other.method_name))
    return false;
  if (!mojo::Equals(this->stringified_details, other.stringified_details))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace payments

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::payments::mojom::ImageObject::DataView,
                                         ::payments::mojom::ImageObjectPtr> {
  static bool IsNull(const ::payments::mojom::ImageObjectPtr& input) { return !input; }
  static void SetToNull(::payments::mojom::ImageObjectPtr* output) { output->reset(); }

  static const decltype(::payments::mojom::ImageObject::src)& src(
      const ::payments::mojom::ImageObjectPtr& input) {
    return input->src;
  }

  static bool Read(::payments::mojom::ImageObject::DataView input, ::payments::mojom::ImageObjectPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::payments::mojom::PaymentInstrument::DataView,
                                         ::payments::mojom::PaymentInstrumentPtr> {
  static bool IsNull(const ::payments::mojom::PaymentInstrumentPtr& input) { return !input; }
  static void SetToNull(::payments::mojom::PaymentInstrumentPtr* output) { output->reset(); }

  static const decltype(::payments::mojom::PaymentInstrument::name)& name(
      const ::payments::mojom::PaymentInstrumentPtr& input) {
    return input->name;
  }

  static const decltype(::payments::mojom::PaymentInstrument::icons)& icons(
      const ::payments::mojom::PaymentInstrumentPtr& input) {
    return input->icons;
  }

  static const decltype(::payments::mojom::PaymentInstrument::enabled_methods)& enabled_methods(
      const ::payments::mojom::PaymentInstrumentPtr& input) {
    return input->enabled_methods;
  }

  static const decltype(::payments::mojom::PaymentInstrument::stringified_capabilities)& stringified_capabilities(
      const ::payments::mojom::PaymentInstrumentPtr& input) {
    return input->stringified_capabilities;
  }

  static bool Read(::payments::mojom::PaymentInstrument::DataView input, ::payments::mojom::PaymentInstrumentPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::payments::mojom::PaymentRequestEventData::DataView,
                                         ::payments::mojom::PaymentRequestEventDataPtr> {
  static bool IsNull(const ::payments::mojom::PaymentRequestEventDataPtr& input) { return !input; }
  static void SetToNull(::payments::mojom::PaymentRequestEventDataPtr* output) { output->reset(); }

  static const decltype(::payments::mojom::PaymentRequestEventData::top_level_origin)& top_level_origin(
      const ::payments::mojom::PaymentRequestEventDataPtr& input) {
    return input->top_level_origin;
  }

  static const decltype(::payments::mojom::PaymentRequestEventData::payment_request_origin)& payment_request_origin(
      const ::payments::mojom::PaymentRequestEventDataPtr& input) {
    return input->payment_request_origin;
  }

  static const decltype(::payments::mojom::PaymentRequestEventData::payment_request_id)& payment_request_id(
      const ::payments::mojom::PaymentRequestEventDataPtr& input) {
    return input->payment_request_id;
  }

  static const decltype(::payments::mojom::PaymentRequestEventData::method_data)& method_data(
      const ::payments::mojom::PaymentRequestEventDataPtr& input) {
    return input->method_data;
  }

  static const decltype(::payments::mojom::PaymentRequestEventData::total)& total(
      const ::payments::mojom::PaymentRequestEventDataPtr& input) {
    return input->total;
  }

  static const decltype(::payments::mojom::PaymentRequestEventData::modifiers)& modifiers(
      const ::payments::mojom::PaymentRequestEventDataPtr& input) {
    return input->modifiers;
  }

  static const decltype(::payments::mojom::PaymentRequestEventData::instrument_key)& instrument_key(
      const ::payments::mojom::PaymentRequestEventDataPtr& input) {
    return input->instrument_key;
  }

  static bool Read(::payments::mojom::PaymentRequestEventData::DataView input, ::payments::mojom::PaymentRequestEventDataPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::payments::mojom::PaymentHandlerResponse::DataView,
                                         ::payments::mojom::PaymentHandlerResponsePtr> {
  static bool IsNull(const ::payments::mojom::PaymentHandlerResponsePtr& input) { return !input; }
  static void SetToNull(::payments::mojom::PaymentHandlerResponsePtr* output) { output->reset(); }

  static const decltype(::payments::mojom::PaymentHandlerResponse::method_name)& method_name(
      const ::payments::mojom::PaymentHandlerResponsePtr& input) {
    return input->method_name;
  }

  static const decltype(::payments::mojom::PaymentHandlerResponse::stringified_details)& stringified_details(
      const ::payments::mojom::PaymentHandlerResponsePtr& input) {
    return input->stringified_details;
  }

  static bool Read(::payments::mojom::PaymentHandlerResponse::DataView input, ::payments::mojom::PaymentHandlerResponsePtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PAYMENTS_PAYMENT_APP_MOJOM_H_