// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PRESENTATION_PRESENTATION_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PRESENTATION_PRESENTATION_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/presentation/presentation.mojom-shared.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>
#include "content/public/common/presentation_connection_message.h"
#include "content/public/common/presentation_info.h"
#include "content/common/content_export.h"


namespace blink {
namespace mojom {
class PresentationConnection;
using PresentationConnectionPtr = mojo::InterfacePtr<PresentationConnection>;
using PresentationConnectionPtrInfo = mojo::InterfacePtrInfo<PresentationConnection>;
using ThreadSafePresentationConnectionPtr =
    mojo::ThreadSafeInterfacePtr<PresentationConnection>;
using PresentationConnectionRequest = mojo::InterfaceRequest<PresentationConnection>;
using PresentationConnectionAssociatedPtr =
    mojo::AssociatedInterfacePtr<PresentationConnection>;
using ThreadSafePresentationConnectionAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PresentationConnection>;
using PresentationConnectionAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PresentationConnection>;
using PresentationConnectionAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PresentationConnection>;

class PresentationService;
using PresentationServicePtr = mojo::InterfacePtr<PresentationService>;
using PresentationServicePtrInfo = mojo::InterfacePtrInfo<PresentationService>;
using ThreadSafePresentationServicePtr =
    mojo::ThreadSafeInterfacePtr<PresentationService>;
using PresentationServiceRequest = mojo::InterfaceRequest<PresentationService>;
using PresentationServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<PresentationService>;
using ThreadSafePresentationServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PresentationService>;
using PresentationServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PresentationService>;
using PresentationServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PresentationService>;

class PresentationServiceClient;
using PresentationServiceClientPtr = mojo::InterfacePtr<PresentationServiceClient>;
using PresentationServiceClientPtrInfo = mojo::InterfacePtrInfo<PresentationServiceClient>;
using ThreadSafePresentationServiceClientPtr =
    mojo::ThreadSafeInterfacePtr<PresentationServiceClient>;
using PresentationServiceClientRequest = mojo::InterfaceRequest<PresentationServiceClient>;
using PresentationServiceClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<PresentationServiceClient>;
using ThreadSafePresentationServiceClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PresentationServiceClient>;
using PresentationServiceClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PresentationServiceClient>;
using PresentationServiceClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PresentationServiceClient>;

class PresentationInfo;
using PresentationInfoPtr = mojo::StructPtr<PresentationInfo>;

class PresentationError;
using PresentationErrorPtr = mojo::InlinedStructPtr<PresentationError>;

class PresentationConnectionMessage;

typedef mojo::StructPtr<PresentationConnectionMessage> PresentationConnectionMessagePtr;


class PresentationConnectionProxy;

template <typename ImplRefTraits>
class PresentationConnectionStub;

class PresentationConnectionRequestValidator;
class PresentationConnectionResponseValidator;

class CONTENT_EXPORT PresentationConnection
    : public PresentationConnectionInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PresentationConnectionProxy;

  template <typename ImplRefTraits>
  using Stub_ = PresentationConnectionStub<ImplRefTraits>;

  using RequestValidator_ = PresentationConnectionRequestValidator;
  using ResponseValidator_ = PresentationConnectionResponseValidator;
  enum MethodMinVersions : uint32_t {
    kOnMessageMinVersion = 0,
    kDidChangeStateMinVersion = 0,
    kOnCloseMinVersion = 0,
  };
  virtual ~PresentationConnection() {}


  using OnMessageCallback = base::OnceCallback<void(bool)>;
  virtual void OnMessage(content::PresentationConnectionMessage message, OnMessageCallback callback) = 0;

  virtual void DidChangeState(content::PresentationConnectionState state) = 0;

  virtual void OnClose() = 0;
};

class PresentationServiceProxy;

template <typename ImplRefTraits>
class PresentationServiceStub;

class PresentationServiceRequestValidator;
class PresentationServiceResponseValidator;

class CONTENT_EXPORT PresentationService
    : public PresentationServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PresentationServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = PresentationServiceStub<ImplRefTraits>;

  using RequestValidator_ = PresentationServiceRequestValidator;
  using ResponseValidator_ = PresentationServiceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kSetClientMinVersion = 0,
    kSetDefaultPresentationUrlsMinVersion = 0,
    kListenForScreenAvailabilityMinVersion = 0,
    kStopListeningForScreenAvailabilityMinVersion = 0,
    kStartPresentationMinVersion = 0,
    kReconnectPresentationMinVersion = 0,
    kSetPresentationConnectionMinVersion = 0,
    kCloseConnectionMinVersion = 0,
    kTerminateMinVersion = 0,
  };
  virtual ~PresentationService() {}

  virtual void SetClient(PresentationServiceClientPtr client) = 0;

  virtual void SetDefaultPresentationUrls(const std::vector<GURL>& presentation_urls) = 0;

  virtual void ListenForScreenAvailability(const GURL& availability_url) = 0;

  virtual void StopListeningForScreenAvailability(const GURL& availability_url) = 0;


  using StartPresentationCallback = base::OnceCallback<void(const base::Optional<content::PresentationInfo>&, const base::Optional<content::PresentationError>&)>;
  virtual void StartPresentation(const std::vector<GURL>& presentation_urls, StartPresentationCallback callback) = 0;


  using ReconnectPresentationCallback = base::OnceCallback<void(const base::Optional<content::PresentationInfo>&, const base::Optional<content::PresentationError>&)>;
  virtual void ReconnectPresentation(const std::vector<GURL>& presentation_urls, const base::Optional<std::string>& presentation_id, ReconnectPresentationCallback callback) = 0;

  virtual void SetPresentationConnection(const content::PresentationInfo& presentation_info, PresentationConnectionPtr controller_connection_ptr, PresentationConnectionRequest receiver_connection_request) = 0;

  virtual void CloseConnection(const GURL& presentation_url, const std::string& presentation_id) = 0;

  virtual void Terminate(const GURL& presentation_url, const std::string& presentation_id) = 0;
};

class PresentationServiceClientProxy;

template <typename ImplRefTraits>
class PresentationServiceClientStub;

class PresentationServiceClientRequestValidator;

class CONTENT_EXPORT PresentationServiceClient
    : public PresentationServiceClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PresentationServiceClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = PresentationServiceClientStub<ImplRefTraits>;

  using RequestValidator_ = PresentationServiceClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnScreenAvailabilityUpdatedMinVersion = 0,
    kOnDefaultPresentationStartedMinVersion = 0,
    kOnReceiverConnectionAvailableMinVersion = 0,
    kOnConnectionStateChangedMinVersion = 0,
    kOnConnectionClosedMinVersion = 0,
  };
  virtual ~PresentationServiceClient() {}

  virtual void OnScreenAvailabilityUpdated(const GURL& url, ScreenAvailability availability) = 0;

  virtual void OnDefaultPresentationStarted(const content::PresentationInfo& presentation_info) = 0;

  virtual void OnReceiverConnectionAvailable(const content::PresentationInfo& presentation_info, PresentationConnectionPtr controller_connection_ptr, PresentationConnectionRequest receiver_connection_request) = 0;

  virtual void OnConnectionStateChanged(const content::PresentationInfo& presentation_info, content::PresentationConnectionState newState) = 0;

  virtual void OnConnectionClosed(const content::PresentationInfo& presentation_info, content::PresentationConnectionCloseReason reason, const std::string& message) = 0;
};

class CONTENT_EXPORT PresentationConnectionProxy
    : public PresentationConnection {
 public:
  explicit PresentationConnectionProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnMessage(content::PresentationConnectionMessage message, OnMessageCallback callback) override;
  void DidChangeState(content::PresentationConnectionState state) override;
  void OnClose() override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT PresentationServiceProxy
    : public PresentationService {
 public:
  explicit PresentationServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetClient(PresentationServiceClientPtr client) override;
  void SetDefaultPresentationUrls(const std::vector<GURL>& presentation_urls) override;
  void ListenForScreenAvailability(const GURL& availability_url) override;
  void StopListeningForScreenAvailability(const GURL& availability_url) override;
  void StartPresentation(const std::vector<GURL>& presentation_urls, StartPresentationCallback callback) override;
  void ReconnectPresentation(const std::vector<GURL>& presentation_urls, const base::Optional<std::string>& presentation_id, ReconnectPresentationCallback callback) override;
  void SetPresentationConnection(const content::PresentationInfo& presentation_info, PresentationConnectionPtr controller_connection_ptr, PresentationConnectionRequest receiver_connection_request) override;
  void CloseConnection(const GURL& presentation_url, const std::string& presentation_id) override;
  void Terminate(const GURL& presentation_url, const std::string& presentation_id) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT PresentationServiceClientProxy
    : public PresentationServiceClient {
 public:
  explicit PresentationServiceClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnScreenAvailabilityUpdated(const GURL& url, ScreenAvailability availability) override;
  void OnDefaultPresentationStarted(const content::PresentationInfo& presentation_info) override;
  void OnReceiverConnectionAvailable(const content::PresentationInfo& presentation_info, PresentationConnectionPtr controller_connection_ptr, PresentationConnectionRequest receiver_connection_request) override;
  void OnConnectionStateChanged(const content::PresentationInfo& presentation_info, content::PresentationConnectionState newState) override;
  void OnConnectionClosed(const content::PresentationInfo& presentation_info, content::PresentationConnectionCloseReason reason, const std::string& message) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT PresentationConnectionStubDispatch {
 public:
  static bool Accept(PresentationConnection* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PresentationConnection* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PresentationConnection>>
class PresentationConnectionStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PresentationConnectionStub() {}
  ~PresentationConnectionStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationConnectionStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationConnectionStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT PresentationServiceStubDispatch {
 public:
  static bool Accept(PresentationService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PresentationService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PresentationService>>
class PresentationServiceStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PresentationServiceStub() {}
  ~PresentationServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT PresentationServiceClientStubDispatch {
 public:
  static bool Accept(PresentationServiceClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PresentationServiceClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PresentationServiceClient>>
class PresentationServiceClientStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PresentationServiceClientStub() {}
  ~PresentationServiceClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationServiceClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationServiceClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT PresentationConnectionRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT PresentationServiceRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT PresentationServiceClientRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT PresentationConnectionResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT PresentationServiceResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};



class CONTENT_EXPORT PresentationError {
 public:
  using DataView = PresentationErrorDataView;
  using Data_ = internal::PresentationError_Data;

  template <typename... Args>
  static PresentationErrorPtr New(Args&&... args) {
    return PresentationErrorPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PresentationErrorPtr From(const U& u) {
    return mojo::TypeConverter<PresentationErrorPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PresentationError>::Convert(*this);
  }


  PresentationError();

  PresentationError(
      content::PresentationErrorType error_type,
      const std::string& message);

  ~PresentationError();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PresentationErrorPtr>
  PresentationErrorPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PresentationError>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PresentationError::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PresentationError::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  content::PresentationErrorType error_type;
  std::string message;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class CONTENT_EXPORT PresentationConnectionMessage {
 public:
  using DataView = PresentationConnectionMessageDataView;
  using Data_ = internal::PresentationConnectionMessage_Data;
  using Tag = Data_::PresentationConnectionMessage_Tag;

  static PresentationConnectionMessagePtr New() {
    return PresentationConnectionMessagePtr(base::in_place);
  }
  // Construct an instance holding |message|.
  static PresentationConnectionMessagePtr
  NewMessage(
      const std::string& message) {
    auto result = PresentationConnectionMessagePtr(base::in_place);
    result->set_message(std::move(message));
    return result;
  }
  // Construct an instance holding |data|.
  static PresentationConnectionMessagePtr
  NewData(
      const std::vector<uint8_t>& data) {
    auto result = PresentationConnectionMessagePtr(base::in_place);
    result->set_data(std::move(data));
    return result;
  }

  template <typename U>
  static PresentationConnectionMessagePtr From(const U& u) {
    return mojo::TypeConverter<PresentationConnectionMessagePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PresentationConnectionMessage>::Convert(*this);
  }

  PresentationConnectionMessage();
  ~PresentationConnectionMessage();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename UnionPtrType = PresentationConnectionMessagePtr>
  PresentationConnectionMessagePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PresentationConnectionMessage>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  Tag which() const {
    return tag_;
  }


  bool is_message() const { return tag_ == Tag::MESSAGE; }

  std::string& get_message() const {
    DCHECK(tag_ == Tag::MESSAGE);
    return *(data_.message);
  }

  void set_message(const std::string& message);
  bool is_data() const { return tag_ == Tag::DATA; }

  std::vector<uint8_t>& get_data() const {
    DCHECK(tag_ == Tag::DATA);
    return *(data_.data);
  }

  void set_data(const std::vector<uint8_t>& data);

 private:
  union Union_ {
    Union_() {}
    ~Union_() {}
    std::string* message;
    std::vector<uint8_t>* data;
  };
  void DestroyActive();
  Tag tag_;
  Union_ data_;
};


class CONTENT_EXPORT PresentationInfo {
 public:
  using DataView = PresentationInfoDataView;
  using Data_ = internal::PresentationInfo_Data;

  template <typename... Args>
  static PresentationInfoPtr New(Args&&... args) {
    return PresentationInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PresentationInfoPtr From(const U& u) {
    return mojo::TypeConverter<PresentationInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PresentationInfo>::Convert(*this);
  }


  PresentationInfo();

  PresentationInfo(
      const GURL& url,
      const std::string& id);

  ~PresentationInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PresentationInfoPtr>
  PresentationInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PresentationInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PresentationInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PresentationInfo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  GURL url;
  std::string id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



template <typename UnionPtrType>
PresentationConnectionMessagePtr PresentationConnectionMessage::Clone() const {
  // Use UnionPtrType to prevent the compiler from trying to compile this
  // without being asked.
  UnionPtrType rv(New());
  switch (tag_) {
    case Tag::MESSAGE:
      rv->set_message(mojo::Clone(*data_.message));
      break;
    case Tag::DATA:
      rv->set_data(mojo::Clone(*data_.data));
      break;
  };
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PresentationConnectionMessage>::value>::type*>
bool PresentationConnectionMessage::Equals(const T& other) const {
  if (tag_ != other.which())
    return false;

  switch (tag_) {
    case Tag::MESSAGE:
      return mojo::Equals(*(data_.message), *(other.data_.message));
    case Tag::DATA:
      return mojo::Equals(*(data_.data), *(other.data_.data));
  };

  return false;
}
template <typename StructPtrType>
PresentationInfoPtr PresentationInfo::Clone() const {
  return New(
      mojo::Clone(url),
      mojo::Clone(id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PresentationInfo>::value>::type*>
bool PresentationInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->url, other.url))
    return false;
  if (!mojo::Equals(this->id, other.id))
    return false;
  return true;
}
template <typename StructPtrType>
PresentationErrorPtr PresentationError::Clone() const {
  return New(
      mojo::Clone(error_type),
      mojo::Clone(message)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PresentationError>::value>::type*>
bool PresentationError::Equals(const T& other) const {
  if (!mojo::Equals(this->error_type, other.error_type))
    return false;
  if (!mojo::Equals(this->message, other.message))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::PresentationInfo::DataView,
                                         ::blink::mojom::PresentationInfoPtr> {
  static bool IsNull(const ::blink::mojom::PresentationInfoPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::PresentationInfoPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::PresentationInfo::url)& url(
      const ::blink::mojom::PresentationInfoPtr& input) {
    return input->url;
  }

  static const decltype(::blink::mojom::PresentationInfo::id)& id(
      const ::blink::mojom::PresentationInfoPtr& input) {
    return input->id;
  }

  static bool Read(::blink::mojom::PresentationInfo::DataView input, ::blink::mojom::PresentationInfoPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::PresentationError::DataView,
                                         ::blink::mojom::PresentationErrorPtr> {
  static bool IsNull(const ::blink::mojom::PresentationErrorPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::PresentationErrorPtr* output) { output->reset(); }

  static decltype(::blink::mojom::PresentationError::error_type) error_type(
      const ::blink::mojom::PresentationErrorPtr& input) {
    return input->error_type;
  }

  static const decltype(::blink::mojom::PresentationError::message)& message(
      const ::blink::mojom::PresentationErrorPtr& input) {
    return input->message;
  }

  static bool Read(::blink::mojom::PresentationError::DataView input, ::blink::mojom::PresentationErrorPtr* output);
};


template <>
struct CONTENT_EXPORT UnionTraits<::blink::mojom::PresentationConnectionMessage::DataView,
                                        ::blink::mojom::PresentationConnectionMessagePtr> {
  static bool IsNull(const ::blink::mojom::PresentationConnectionMessagePtr& input) { return !input; }
  static void SetToNull(::blink::mojom::PresentationConnectionMessagePtr* output) { output->reset(); }

  static ::blink::mojom::PresentationConnectionMessage::Tag GetTag(const ::blink::mojom::PresentationConnectionMessagePtr& input) {
    return input->which();
  }

  static const std::string& message(const ::blink::mojom::PresentationConnectionMessagePtr& input) {
    return input->get_message();
  }

  static const std::vector<uint8_t>& data(const ::blink::mojom::PresentationConnectionMessagePtr& input) {
    return input->get_data();
  }

  static bool Read(::blink::mojom::PresentationConnectionMessage::DataView input, ::blink::mojom::PresentationConnectionMessagePtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PRESENTATION_PRESENTATION_MOJOM_H_