// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "third_party/WebKit/public/platform/modules/webauth/authenticator.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
#include "ipc/ipc_message_utils.h"
#include "mojo/common/common_custom_types_struct_traits.h"
#include "third_party/WebKit/Source/platform/mojo/KURLStructTraits.h"
namespace webauth {
namespace mojom {
namespace blink {
PublicKeyCredentialInfo::PublicKeyCredentialInfo()
    : id(),
      raw_id(),
      client_data_json(),
      response() {}

PublicKeyCredentialInfo::PublicKeyCredentialInfo(
    const WTF::String& id_in,
    const WTF::Vector<uint8_t>& raw_id_in,
    const WTF::Vector<uint8_t>& client_data_json_in,
    AuthenticatorResponsePtr response_in)
    : id(std::move(id_in)),
      raw_id(std::move(raw_id_in)),
      client_data_json(std::move(client_data_json_in)),
      response(std::move(response_in)) {}

PublicKeyCredentialInfo::~PublicKeyCredentialInfo() = default;

bool PublicKeyCredentialInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
AuthenticatorResponse::AuthenticatorResponse()
    : attestation_object(),
      authenticator_data(),
      signature() {}

AuthenticatorResponse::AuthenticatorResponse(
    const WTF::Vector<uint8_t>& attestation_object_in,
    const WTF::Vector<uint8_t>& authenticator_data_in,
    const WTF::Vector<uint8_t>& signature_in)
    : attestation_object(std::move(attestation_object_in)),
      authenticator_data(std::move(authenticator_data_in)),
      signature(std::move(signature_in)) {}

AuthenticatorResponse::~AuthenticatorResponse() = default;

bool AuthenticatorResponse::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
PublicKeyCredentialEntity::PublicKeyCredentialEntity()
    : id(),
      name(),
      icon(),
      display_name() {}

PublicKeyCredentialEntity::PublicKeyCredentialEntity(
    const WTF::String& id_in,
    const WTF::String& name_in,
    const WTF::Optional<::blink::KURL>& icon_in,
    const WTF::String& display_name_in)
    : id(std::move(id_in)),
      name(std::move(name_in)),
      icon(std::move(icon_in)),
      display_name(std::move(display_name_in)) {}

PublicKeyCredentialEntity::~PublicKeyCredentialEntity() = default;

bool PublicKeyCredentialEntity::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
PublicKeyCredentialParameters::PublicKeyCredentialParameters()
    : type() {}

PublicKeyCredentialParameters::PublicKeyCredentialParameters(
    PublicKeyCredentialType type_in)
    : type(std::move(type_in)) {}

PublicKeyCredentialParameters::~PublicKeyCredentialParameters() = default;
size_t PublicKeyCredentialParameters::Hash(size_t seed) const {
  seed = mojo::internal::WTFHash(seed, this->type);
  return seed;
}

bool PublicKeyCredentialParameters::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
MakeCredentialOptions::MakeCredentialOptions()
    : relying_party(),
      user(),
      challenge(),
      crypto_parameters(),
      adjusted_timeout(),
      exclude_credentials() {}

MakeCredentialOptions::MakeCredentialOptions(
    PublicKeyCredentialEntityPtr relying_party_in,
    PublicKeyCredentialEntityPtr user_in,
    const WTF::Vector<uint8_t>& challenge_in,
    WTF::Vector<PublicKeyCredentialParametersPtr> crypto_parameters_in,
    WTF::TimeDelta adjusted_timeout_in,
    WTF::Vector<PublicKeyCredentialDescriptorPtr> exclude_credentials_in)
    : relying_party(std::move(relying_party_in)),
      user(std::move(user_in)),
      challenge(std::move(challenge_in)),
      crypto_parameters(std::move(crypto_parameters_in)),
      adjusted_timeout(std::move(adjusted_timeout_in)),
      exclude_credentials(std::move(exclude_credentials_in)) {}

MakeCredentialOptions::~MakeCredentialOptions() = default;

bool MakeCredentialOptions::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
PublicKeyCredentialDescriptor::PublicKeyCredentialDescriptor()
    : type(),
      id(),
      transports() {}

PublicKeyCredentialDescriptor::PublicKeyCredentialDescriptor(
    PublicKeyCredentialType type_in,
    const WTF::Vector<uint8_t>& id_in,
    const WTF::Vector<AuthenticatorTransport>& transports_in)
    : type(std::move(type_in)),
      id(std::move(id_in)),
      transports(std::move(transports_in)) {}

PublicKeyCredentialDescriptor::~PublicKeyCredentialDescriptor() = default;

bool PublicKeyCredentialDescriptor::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char Authenticator::Name_[] = "webauth::mojom::Authenticator";

class Authenticator_MakeCredential_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  Authenticator_MakeCredential_ForwardToCallback(
      Authenticator::MakeCredentialCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  Authenticator::MakeCredentialCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(Authenticator_MakeCredential_ForwardToCallback);
};

AuthenticatorProxy::AuthenticatorProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void AuthenticatorProxy::MakeCredential(
    MakeCredentialOptionsPtr in_options, MakeCredentialCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::webauth::mojom::internal::Authenticator_MakeCredential_Params_Data);
  size += mojo::internal::PrepareToSerialize<::webauth::mojom::MakeCredentialOptionsDataView>(
      in_options, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kAuthenticator_MakeCredential_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::webauth::mojom::internal::Authenticator_MakeCredential_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->options)::BaseType* options_ptr;
  mojo::internal::Serialize<::webauth::mojom::MakeCredentialOptionsDataView>(
      in_options, buffer, &options_ptr, &serialization_context);
  params->options.Set(options_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->options.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null options in Authenticator.MakeCredential request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new Authenticator_MakeCredential_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class Authenticator_MakeCredential_ProxyToResponder {
 public:
  static Authenticator::MakeCredentialCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<Authenticator_MakeCredential_ProxyToResponder> proxy(
        new Authenticator_MakeCredential_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&Authenticator_MakeCredential_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~Authenticator_MakeCredential_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "Authenticator::MakeCredential() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  Authenticator_MakeCredential_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      AuthenticatorStatus in_status, PublicKeyCredentialInfoPtr in_credential);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(Authenticator_MakeCredential_ProxyToResponder);
};

bool Authenticator_MakeCredential_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::Authenticator_MakeCredential_ResponseParams_Data* params =
      reinterpret_cast<
          internal::Authenticator_MakeCredential_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  AuthenticatorStatus p_status{};
  PublicKeyCredentialInfoPtr p_credential{};
  Authenticator_MakeCredential_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!input_data_view.ReadCredential(&p_credential))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "Authenticator::MakeCredential response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_status), 
std::move(p_credential));
  return true;
}

void Authenticator_MakeCredential_ProxyToResponder::Run(
    AuthenticatorStatus in_status, PublicKeyCredentialInfoPtr in_credential) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::webauth::mojom::internal::Authenticator_MakeCredential_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::webauth::mojom::PublicKeyCredentialInfoDataView>(
      in_credential, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kAuthenticator_MakeCredential_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::webauth::mojom::internal::Authenticator_MakeCredential_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::webauth::mojom::AuthenticatorStatus>(
      in_status, &params->status);
  typename decltype(params->credential)::BaseType* credential_ptr;
  mojo::internal::Serialize<::webauth::mojom::PublicKeyCredentialInfoDataView>(
      in_credential, buffer, &credential_ptr, &serialization_context);
  params->credential.Set(credential_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool AuthenticatorStubDispatch::Accept(
    Authenticator* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kAuthenticator_MakeCredential_Name: {
      break;
    }
  }
  return false;
}

// static
bool AuthenticatorStubDispatch::AcceptWithResponder(
    Authenticator* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kAuthenticator_MakeCredential_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::Authenticator_MakeCredential_Params_Data* params =
          reinterpret_cast<
              internal::Authenticator_MakeCredential_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      MakeCredentialOptionsPtr p_options{};
      Authenticator_MakeCredential_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadOptions(&p_options))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Authenticator::MakeCredential deserializer");
        return false;
      }
      Authenticator::MakeCredentialCallback callback =
          Authenticator_MakeCredential_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->MakeCredential(
std::move(p_options), std::move(callback));
      return true;
    }
  }
  return false;
}

bool AuthenticatorRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "Authenticator RequestValidator");

  switch (message->header()->name) {
    case internal::kAuthenticator_MakeCredential_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Authenticator_MakeCredential_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool AuthenticatorResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "Authenticator ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kAuthenticator_MakeCredential_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::Authenticator_MakeCredential_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
}  // namespace blink
}  // namespace mojom
}  // namespace webauth

namespace mojo {


// static
bool StructTraits<::webauth::mojom::blink::PublicKeyCredentialInfo::DataView, ::webauth::mojom::blink::PublicKeyCredentialInfoPtr>::Read(
    ::webauth::mojom::blink::PublicKeyCredentialInfo::DataView input,
    ::webauth::mojom::blink::PublicKeyCredentialInfoPtr* output) {
  bool success = true;
  ::webauth::mojom::blink::PublicKeyCredentialInfoPtr result(::webauth::mojom::blink::PublicKeyCredentialInfo::New());
  
      if (!input.ReadId(&result->id))
        success = false;
      if (!input.ReadRawId(&result->raw_id))
        success = false;
      if (!input.ReadClientDataJson(&result->client_data_json))
        success = false;
      if (!input.ReadResponse(&result->response))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::webauth::mojom::blink::AuthenticatorResponse::DataView, ::webauth::mojom::blink::AuthenticatorResponsePtr>::Read(
    ::webauth::mojom::blink::AuthenticatorResponse::DataView input,
    ::webauth::mojom::blink::AuthenticatorResponsePtr* output) {
  bool success = true;
  ::webauth::mojom::blink::AuthenticatorResponsePtr result(::webauth::mojom::blink::AuthenticatorResponse::New());
  
      if (!input.ReadAttestationObject(&result->attestation_object))
        success = false;
      if (!input.ReadAuthenticatorData(&result->authenticator_data))
        success = false;
      if (!input.ReadSignature(&result->signature))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::webauth::mojom::blink::PublicKeyCredentialEntity::DataView, ::webauth::mojom::blink::PublicKeyCredentialEntityPtr>::Read(
    ::webauth::mojom::blink::PublicKeyCredentialEntity::DataView input,
    ::webauth::mojom::blink::PublicKeyCredentialEntityPtr* output) {
  bool success = true;
  ::webauth::mojom::blink::PublicKeyCredentialEntityPtr result(::webauth::mojom::blink::PublicKeyCredentialEntity::New());
  
      if (!input.ReadId(&result->id))
        success = false;
      if (!input.ReadName(&result->name))
        success = false;
      if (!input.ReadIcon(&result->icon))
        success = false;
      if (!input.ReadDisplayName(&result->display_name))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::webauth::mojom::blink::PublicKeyCredentialParameters::DataView, ::webauth::mojom::blink::PublicKeyCredentialParametersPtr>::Read(
    ::webauth::mojom::blink::PublicKeyCredentialParameters::DataView input,
    ::webauth::mojom::blink::PublicKeyCredentialParametersPtr* output) {
  bool success = true;
  ::webauth::mojom::blink::PublicKeyCredentialParametersPtr result(::webauth::mojom::blink::PublicKeyCredentialParameters::New());
  
      if (!input.ReadType(&result->type))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::webauth::mojom::blink::MakeCredentialOptions::DataView, ::webauth::mojom::blink::MakeCredentialOptionsPtr>::Read(
    ::webauth::mojom::blink::MakeCredentialOptions::DataView input,
    ::webauth::mojom::blink::MakeCredentialOptionsPtr* output) {
  bool success = true;
  ::webauth::mojom::blink::MakeCredentialOptionsPtr result(::webauth::mojom::blink::MakeCredentialOptions::New());
  
      if (!input.ReadRelyingParty(&result->relying_party))
        success = false;
      if (!input.ReadUser(&result->user))
        success = false;
      if (!input.ReadChallenge(&result->challenge))
        success = false;
      if (!input.ReadCryptoParameters(&result->crypto_parameters))
        success = false;
      if (!input.ReadAdjustedTimeout(&result->adjusted_timeout))
        success = false;
      if (!input.ReadExcludeCredentials(&result->exclude_credentials))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::webauth::mojom::blink::PublicKeyCredentialDescriptor::DataView, ::webauth::mojom::blink::PublicKeyCredentialDescriptorPtr>::Read(
    ::webauth::mojom::blink::PublicKeyCredentialDescriptor::DataView input,
    ::webauth::mojom::blink::PublicKeyCredentialDescriptorPtr* output) {
  bool success = true;
  ::webauth::mojom::blink::PublicKeyCredentialDescriptorPtr result(::webauth::mojom::blink::PublicKeyCredentialDescriptor::New());
  
      if (!input.ReadType(&result->type))
        success = false;
      if (!input.ReadId(&result->id))
        success = false;
      if (!input.ReadTransports(&result->transports))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif