// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_WEBAUTH_AUTHENTICATOR_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_WEBAUTH_AUTHENTICATOR_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/webauth/authenticator.mojom-shared.h"
#include "mojo/common/time.mojom.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace webauth {
namespace mojom {
class Authenticator;
using AuthenticatorPtr = mojo::InterfacePtr<Authenticator>;
using AuthenticatorPtrInfo = mojo::InterfacePtrInfo<Authenticator>;
using ThreadSafeAuthenticatorPtr =
    mojo::ThreadSafeInterfacePtr<Authenticator>;
using AuthenticatorRequest = mojo::InterfaceRequest<Authenticator>;
using AuthenticatorAssociatedPtr =
    mojo::AssociatedInterfacePtr<Authenticator>;
using ThreadSafeAuthenticatorAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Authenticator>;
using AuthenticatorAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Authenticator>;
using AuthenticatorAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Authenticator>;

class PublicKeyCredentialInfo;
using PublicKeyCredentialInfoPtr = mojo::StructPtr<PublicKeyCredentialInfo>;

class AuthenticatorResponse;
using AuthenticatorResponsePtr = mojo::StructPtr<AuthenticatorResponse>;

class PublicKeyCredentialEntity;
using PublicKeyCredentialEntityPtr = mojo::StructPtr<PublicKeyCredentialEntity>;

class PublicKeyCredentialParameters;
using PublicKeyCredentialParametersPtr = mojo::InlinedStructPtr<PublicKeyCredentialParameters>;

class MakeCredentialOptions;
using MakeCredentialOptionsPtr = mojo::StructPtr<MakeCredentialOptions>;

class PublicKeyCredentialDescriptor;
using PublicKeyCredentialDescriptorPtr = mojo::StructPtr<PublicKeyCredentialDescriptor>;


class AuthenticatorProxy;

template <typename ImplRefTraits>
class AuthenticatorStub;

class AuthenticatorRequestValidator;
class AuthenticatorResponseValidator;

class CONTENT_EXPORT Authenticator
    : public AuthenticatorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AuthenticatorProxy;

  template <typename ImplRefTraits>
  using Stub_ = AuthenticatorStub<ImplRefTraits>;

  using RequestValidator_ = AuthenticatorRequestValidator;
  using ResponseValidator_ = AuthenticatorResponseValidator;
  enum MethodMinVersions : uint32_t {
    kMakeCredentialMinVersion = 0,
  };
  virtual ~Authenticator() {}


  using MakeCredentialCallback = base::OnceCallback<void(AuthenticatorStatus, PublicKeyCredentialInfoPtr)>;
  virtual void MakeCredential(MakeCredentialOptionsPtr options, MakeCredentialCallback callback) = 0;
};

class CONTENT_EXPORT AuthenticatorProxy
    : public Authenticator {
 public:
  explicit AuthenticatorProxy(mojo::MessageReceiverWithResponder* receiver);
  void MakeCredential(MakeCredentialOptionsPtr options, MakeCredentialCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT AuthenticatorStubDispatch {
 public:
  static bool Accept(Authenticator* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Authenticator* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Authenticator>>
class AuthenticatorStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AuthenticatorStub() {}
  ~AuthenticatorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AuthenticatorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AuthenticatorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT AuthenticatorRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT AuthenticatorResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





class CONTENT_EXPORT PublicKeyCredentialParameters {
 public:
  using DataView = PublicKeyCredentialParametersDataView;
  using Data_ = internal::PublicKeyCredentialParameters_Data;

  template <typename... Args>
  static PublicKeyCredentialParametersPtr New(Args&&... args) {
    return PublicKeyCredentialParametersPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PublicKeyCredentialParametersPtr From(const U& u) {
    return mojo::TypeConverter<PublicKeyCredentialParametersPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PublicKeyCredentialParameters>::Convert(*this);
  }


  PublicKeyCredentialParameters();

  explicit PublicKeyCredentialParameters(
      PublicKeyCredentialType type);

  ~PublicKeyCredentialParameters();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PublicKeyCredentialParametersPtr>
  PublicKeyCredentialParametersPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PublicKeyCredentialParameters>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PublicKeyCredentialParameters::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PublicKeyCredentialParameters::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  PublicKeyCredentialType type;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};







class CONTENT_EXPORT PublicKeyCredentialInfo {
 public:
  using DataView = PublicKeyCredentialInfoDataView;
  using Data_ = internal::PublicKeyCredentialInfo_Data;

  template <typename... Args>
  static PublicKeyCredentialInfoPtr New(Args&&... args) {
    return PublicKeyCredentialInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PublicKeyCredentialInfoPtr From(const U& u) {
    return mojo::TypeConverter<PublicKeyCredentialInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PublicKeyCredentialInfo>::Convert(*this);
  }


  PublicKeyCredentialInfo();

  PublicKeyCredentialInfo(
      const std::string& id,
      const std::vector<uint8_t>& raw_id,
      const std::vector<uint8_t>& client_data_json,
      AuthenticatorResponsePtr response);

  ~PublicKeyCredentialInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PublicKeyCredentialInfoPtr>
  PublicKeyCredentialInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PublicKeyCredentialInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PublicKeyCredentialInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PublicKeyCredentialInfo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::string id;
  std::vector<uint8_t> raw_id;
  std::vector<uint8_t> client_data_json;
  AuthenticatorResponsePtr response;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(PublicKeyCredentialInfo);
};



class CONTENT_EXPORT AuthenticatorResponse {
 public:
  using DataView = AuthenticatorResponseDataView;
  using Data_ = internal::AuthenticatorResponse_Data;

  template <typename... Args>
  static AuthenticatorResponsePtr New(Args&&... args) {
    return AuthenticatorResponsePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static AuthenticatorResponsePtr From(const U& u) {
    return mojo::TypeConverter<AuthenticatorResponsePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AuthenticatorResponse>::Convert(*this);
  }


  AuthenticatorResponse();

  AuthenticatorResponse(
      const std::vector<uint8_t>& attestation_object,
      const std::vector<uint8_t>& authenticator_data,
      const std::vector<uint8_t>& signature);

  ~AuthenticatorResponse();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AuthenticatorResponsePtr>
  AuthenticatorResponsePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AuthenticatorResponse>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        AuthenticatorResponse::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        AuthenticatorResponse::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::vector<uint8_t> attestation_object;
  std::vector<uint8_t> authenticator_data;
  std::vector<uint8_t> signature;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class CONTENT_EXPORT PublicKeyCredentialEntity {
 public:
  using DataView = PublicKeyCredentialEntityDataView;
  using Data_ = internal::PublicKeyCredentialEntity_Data;

  template <typename... Args>
  static PublicKeyCredentialEntityPtr New(Args&&... args) {
    return PublicKeyCredentialEntityPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PublicKeyCredentialEntityPtr From(const U& u) {
    return mojo::TypeConverter<PublicKeyCredentialEntityPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PublicKeyCredentialEntity>::Convert(*this);
  }


  PublicKeyCredentialEntity();

  PublicKeyCredentialEntity(
      const std::string& id,
      const std::string& name,
      const base::Optional<GURL>& icon,
      const base::Optional<std::string>& display_name);

  ~PublicKeyCredentialEntity();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PublicKeyCredentialEntityPtr>
  PublicKeyCredentialEntityPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PublicKeyCredentialEntity>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PublicKeyCredentialEntity::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PublicKeyCredentialEntity::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::string id;
  std::string name;
  base::Optional<GURL> icon;
  base::Optional<std::string> display_name;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




class CONTENT_EXPORT MakeCredentialOptions {
 public:
  using DataView = MakeCredentialOptionsDataView;
  using Data_ = internal::MakeCredentialOptions_Data;

  template <typename... Args>
  static MakeCredentialOptionsPtr New(Args&&... args) {
    return MakeCredentialOptionsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static MakeCredentialOptionsPtr From(const U& u) {
    return mojo::TypeConverter<MakeCredentialOptionsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, MakeCredentialOptions>::Convert(*this);
  }


  MakeCredentialOptions();

  MakeCredentialOptions(
      PublicKeyCredentialEntityPtr relying_party,
      PublicKeyCredentialEntityPtr user,
      const std::vector<uint8_t>& challenge,
      std::vector<PublicKeyCredentialParametersPtr> crypto_parameters,
      base::TimeDelta adjusted_timeout,
      std::vector<PublicKeyCredentialDescriptorPtr> exclude_credentials);

  ~MakeCredentialOptions();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = MakeCredentialOptionsPtr>
  MakeCredentialOptionsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, MakeCredentialOptions>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        MakeCredentialOptions::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        MakeCredentialOptions::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  PublicKeyCredentialEntityPtr relying_party;
  PublicKeyCredentialEntityPtr user;
  std::vector<uint8_t> challenge;
  std::vector<PublicKeyCredentialParametersPtr> crypto_parameters;
  base::TimeDelta adjusted_timeout;
  std::vector<PublicKeyCredentialDescriptorPtr> exclude_credentials;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(MakeCredentialOptions);
};



class CONTENT_EXPORT PublicKeyCredentialDescriptor {
 public:
  using DataView = PublicKeyCredentialDescriptorDataView;
  using Data_ = internal::PublicKeyCredentialDescriptor_Data;

  template <typename... Args>
  static PublicKeyCredentialDescriptorPtr New(Args&&... args) {
    return PublicKeyCredentialDescriptorPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static PublicKeyCredentialDescriptorPtr From(const U& u) {
    return mojo::TypeConverter<PublicKeyCredentialDescriptorPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PublicKeyCredentialDescriptor>::Convert(*this);
  }


  PublicKeyCredentialDescriptor();

  PublicKeyCredentialDescriptor(
      PublicKeyCredentialType type,
      const std::vector<uint8_t>& id,
      const std::vector<AuthenticatorTransport>& transports);

  ~PublicKeyCredentialDescriptor();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PublicKeyCredentialDescriptorPtr>
  PublicKeyCredentialDescriptorPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PublicKeyCredentialDescriptor>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PublicKeyCredentialDescriptor::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PublicKeyCredentialDescriptor::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  PublicKeyCredentialType type;
  std::vector<uint8_t> id;
  std::vector<AuthenticatorTransport> transports;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
PublicKeyCredentialInfoPtr PublicKeyCredentialInfo::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(raw_id),
      mojo::Clone(client_data_json),
      mojo::Clone(response)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PublicKeyCredentialInfo>::value>::type*>
bool PublicKeyCredentialInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->id, other.id))
    return false;
  if (!mojo::Equals(this->raw_id, other.raw_id))
    return false;
  if (!mojo::Equals(this->client_data_json, other.client_data_json))
    return false;
  if (!mojo::Equals(this->response, other.response))
    return false;
  return true;
}
template <typename StructPtrType>
AuthenticatorResponsePtr AuthenticatorResponse::Clone() const {
  return New(
      mojo::Clone(attestation_object),
      mojo::Clone(authenticator_data),
      mojo::Clone(signature)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AuthenticatorResponse>::value>::type*>
bool AuthenticatorResponse::Equals(const T& other) const {
  if (!mojo::Equals(this->attestation_object, other.attestation_object))
    return false;
  if (!mojo::Equals(this->authenticator_data, other.authenticator_data))
    return false;
  if (!mojo::Equals(this->signature, other.signature))
    return false;
  return true;
}
template <typename StructPtrType>
PublicKeyCredentialEntityPtr PublicKeyCredentialEntity::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(name),
      mojo::Clone(icon),
      mojo::Clone(display_name)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PublicKeyCredentialEntity>::value>::type*>
bool PublicKeyCredentialEntity::Equals(const T& other) const {
  if (!mojo::Equals(this->id, other.id))
    return false;
  if (!mojo::Equals(this->name, other.name))
    return false;
  if (!mojo::Equals(this->icon, other.icon))
    return false;
  if (!mojo::Equals(this->display_name, other.display_name))
    return false;
  return true;
}
template <typename StructPtrType>
PublicKeyCredentialParametersPtr PublicKeyCredentialParameters::Clone() const {
  return New(
      mojo::Clone(type)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PublicKeyCredentialParameters>::value>::type*>
bool PublicKeyCredentialParameters::Equals(const T& other) const {
  if (!mojo::Equals(this->type, other.type))
    return false;
  return true;
}
template <typename StructPtrType>
MakeCredentialOptionsPtr MakeCredentialOptions::Clone() const {
  return New(
      mojo::Clone(relying_party),
      mojo::Clone(user),
      mojo::Clone(challenge),
      mojo::Clone(crypto_parameters),
      mojo::Clone(adjusted_timeout),
      mojo::Clone(exclude_credentials)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, MakeCredentialOptions>::value>::type*>
bool MakeCredentialOptions::Equals(const T& other) const {
  if (!mojo::Equals(this->relying_party, other.relying_party))
    return false;
  if (!mojo::Equals(this->user, other.user))
    return false;
  if (!mojo::Equals(this->challenge, other.challenge))
    return false;
  if (!mojo::Equals(this->crypto_parameters, other.crypto_parameters))
    return false;
  if (!mojo::Equals(this->adjusted_timeout, other.adjusted_timeout))
    return false;
  if (!mojo::Equals(this->exclude_credentials, other.exclude_credentials))
    return false;
  return true;
}
template <typename StructPtrType>
PublicKeyCredentialDescriptorPtr PublicKeyCredentialDescriptor::Clone() const {
  return New(
      mojo::Clone(type),
      mojo::Clone(id),
      mojo::Clone(transports)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PublicKeyCredentialDescriptor>::value>::type*>
bool PublicKeyCredentialDescriptor::Equals(const T& other) const {
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->id, other.id))
    return false;
  if (!mojo::Equals(this->transports, other.transports))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace webauth

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::webauth::mojom::PublicKeyCredentialInfo::DataView,
                                         ::webauth::mojom::PublicKeyCredentialInfoPtr> {
  static bool IsNull(const ::webauth::mojom::PublicKeyCredentialInfoPtr& input) { return !input; }
  static void SetToNull(::webauth::mojom::PublicKeyCredentialInfoPtr* output) { output->reset(); }

  static const decltype(::webauth::mojom::PublicKeyCredentialInfo::id)& id(
      const ::webauth::mojom::PublicKeyCredentialInfoPtr& input) {
    return input->id;
  }

  static const decltype(::webauth::mojom::PublicKeyCredentialInfo::raw_id)& raw_id(
      const ::webauth::mojom::PublicKeyCredentialInfoPtr& input) {
    return input->raw_id;
  }

  static const decltype(::webauth::mojom::PublicKeyCredentialInfo::client_data_json)& client_data_json(
      const ::webauth::mojom::PublicKeyCredentialInfoPtr& input) {
    return input->client_data_json;
  }

  static const decltype(::webauth::mojom::PublicKeyCredentialInfo::response)& response(
      const ::webauth::mojom::PublicKeyCredentialInfoPtr& input) {
    return input->response;
  }

  static bool Read(::webauth::mojom::PublicKeyCredentialInfo::DataView input, ::webauth::mojom::PublicKeyCredentialInfoPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::webauth::mojom::AuthenticatorResponse::DataView,
                                         ::webauth::mojom::AuthenticatorResponsePtr> {
  static bool IsNull(const ::webauth::mojom::AuthenticatorResponsePtr& input) { return !input; }
  static void SetToNull(::webauth::mojom::AuthenticatorResponsePtr* output) { output->reset(); }

  static const decltype(::webauth::mojom::AuthenticatorResponse::attestation_object)& attestation_object(
      const ::webauth::mojom::AuthenticatorResponsePtr& input) {
    return input->attestation_object;
  }

  static const decltype(::webauth::mojom::AuthenticatorResponse::authenticator_data)& authenticator_data(
      const ::webauth::mojom::AuthenticatorResponsePtr& input) {
    return input->authenticator_data;
  }

  static const decltype(::webauth::mojom::AuthenticatorResponse::signature)& signature(
      const ::webauth::mojom::AuthenticatorResponsePtr& input) {
    return input->signature;
  }

  static bool Read(::webauth::mojom::AuthenticatorResponse::DataView input, ::webauth::mojom::AuthenticatorResponsePtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::webauth::mojom::PublicKeyCredentialEntity::DataView,
                                         ::webauth::mojom::PublicKeyCredentialEntityPtr> {
  static bool IsNull(const ::webauth::mojom::PublicKeyCredentialEntityPtr& input) { return !input; }
  static void SetToNull(::webauth::mojom::PublicKeyCredentialEntityPtr* output) { output->reset(); }

  static const decltype(::webauth::mojom::PublicKeyCredentialEntity::id)& id(
      const ::webauth::mojom::PublicKeyCredentialEntityPtr& input) {
    return input->id;
  }

  static const decltype(::webauth::mojom::PublicKeyCredentialEntity::name)& name(
      const ::webauth::mojom::PublicKeyCredentialEntityPtr& input) {
    return input->name;
  }

  static const decltype(::webauth::mojom::PublicKeyCredentialEntity::icon)& icon(
      const ::webauth::mojom::PublicKeyCredentialEntityPtr& input) {
    return input->icon;
  }

  static const decltype(::webauth::mojom::PublicKeyCredentialEntity::display_name)& display_name(
      const ::webauth::mojom::PublicKeyCredentialEntityPtr& input) {
    return input->display_name;
  }

  static bool Read(::webauth::mojom::PublicKeyCredentialEntity::DataView input, ::webauth::mojom::PublicKeyCredentialEntityPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::webauth::mojom::PublicKeyCredentialParameters::DataView,
                                         ::webauth::mojom::PublicKeyCredentialParametersPtr> {
  static bool IsNull(const ::webauth::mojom::PublicKeyCredentialParametersPtr& input) { return !input; }
  static void SetToNull(::webauth::mojom::PublicKeyCredentialParametersPtr* output) { output->reset(); }

  static decltype(::webauth::mojom::PublicKeyCredentialParameters::type) type(
      const ::webauth::mojom::PublicKeyCredentialParametersPtr& input) {
    return input->type;
  }

  static bool Read(::webauth::mojom::PublicKeyCredentialParameters::DataView input, ::webauth::mojom::PublicKeyCredentialParametersPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::webauth::mojom::MakeCredentialOptions::DataView,
                                         ::webauth::mojom::MakeCredentialOptionsPtr> {
  static bool IsNull(const ::webauth::mojom::MakeCredentialOptionsPtr& input) { return !input; }
  static void SetToNull(::webauth::mojom::MakeCredentialOptionsPtr* output) { output->reset(); }

  static const decltype(::webauth::mojom::MakeCredentialOptions::relying_party)& relying_party(
      const ::webauth::mojom::MakeCredentialOptionsPtr& input) {
    return input->relying_party;
  }

  static const decltype(::webauth::mojom::MakeCredentialOptions::user)& user(
      const ::webauth::mojom::MakeCredentialOptionsPtr& input) {
    return input->user;
  }

  static const decltype(::webauth::mojom::MakeCredentialOptions::challenge)& challenge(
      const ::webauth::mojom::MakeCredentialOptionsPtr& input) {
    return input->challenge;
  }

  static const decltype(::webauth::mojom::MakeCredentialOptions::crypto_parameters)& crypto_parameters(
      const ::webauth::mojom::MakeCredentialOptionsPtr& input) {
    return input->crypto_parameters;
  }

  static const decltype(::webauth::mojom::MakeCredentialOptions::adjusted_timeout)& adjusted_timeout(
      const ::webauth::mojom::MakeCredentialOptionsPtr& input) {
    return input->adjusted_timeout;
  }

  static const decltype(::webauth::mojom::MakeCredentialOptions::exclude_credentials)& exclude_credentials(
      const ::webauth::mojom::MakeCredentialOptionsPtr& input) {
    return input->exclude_credentials;
  }

  static bool Read(::webauth::mojom::MakeCredentialOptions::DataView input, ::webauth::mojom::MakeCredentialOptionsPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::webauth::mojom::PublicKeyCredentialDescriptor::DataView,
                                         ::webauth::mojom::PublicKeyCredentialDescriptorPtr> {
  static bool IsNull(const ::webauth::mojom::PublicKeyCredentialDescriptorPtr& input) { return !input; }
  static void SetToNull(::webauth::mojom::PublicKeyCredentialDescriptorPtr* output) { output->reset(); }

  static decltype(::webauth::mojom::PublicKeyCredentialDescriptor::type) type(
      const ::webauth::mojom::PublicKeyCredentialDescriptorPtr& input) {
    return input->type;
  }

  static const decltype(::webauth::mojom::PublicKeyCredentialDescriptor::id)& id(
      const ::webauth::mojom::PublicKeyCredentialDescriptorPtr& input) {
    return input->id;
  }

  static const decltype(::webauth::mojom::PublicKeyCredentialDescriptor::transports)& transports(
      const ::webauth::mojom::PublicKeyCredentialDescriptorPtr& input) {
    return input->transports;
  }

  static bool Read(::webauth::mojom::PublicKeyCredentialDescriptor::DataView input, ::webauth::mojom::PublicKeyCredentialDescriptorPtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_WEBAUTH_AUTHENTICATOR_MOJOM_H_