// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_DISPLAY_MOJO_DISPLAY_LAYOUT_MOJOM_H_
#define UI_DISPLAY_MOJO_DISPLAY_LAYOUT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/display/mojo/display_layout.mojom-shared.h"
#include <string>
#include <vector>
#include "ui/display/display_layout.h"


namespace display {
namespace mojom {
class DisplayPlacement;
using DisplayPlacementPtr = mojo::StructPtr<DisplayPlacement>;

class DisplayLayout;
using DisplayLayoutPtr = mojo::StructPtr<DisplayLayout>;







class  DisplayPlacement {
 public:
  using DataView = DisplayPlacementDataView;
  using Data_ = internal::DisplayPlacement_Data;

  template <typename... Args>
  static DisplayPlacementPtr New(Args&&... args) {
    return DisplayPlacementPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DisplayPlacementPtr From(const U& u) {
    return mojo::TypeConverter<DisplayPlacementPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DisplayPlacement>::Convert(*this);
  }


  DisplayPlacement();

  DisplayPlacement(
      int64_t display_id,
      int64_t parent_display_id,
      display::DisplayPlacement::Position position,
      int32_t offset,
      display::DisplayPlacement::OffsetReference offset_reference);

  ~DisplayPlacement();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DisplayPlacementPtr>
  DisplayPlacementPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DisplayPlacement>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        DisplayPlacement::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        DisplayPlacement::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  int64_t display_id;
  int64_t parent_display_id;
  display::DisplayPlacement::Position position;
  int32_t offset;
  display::DisplayPlacement::OffsetReference offset_reference;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class  DisplayLayout {
 public:
  using DataView = DisplayLayoutDataView;
  using Data_ = internal::DisplayLayout_Data;

  template <typename... Args>
  static DisplayLayoutPtr New(Args&&... args) {
    return DisplayLayoutPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DisplayLayoutPtr From(const U& u) {
    return mojo::TypeConverter<DisplayLayoutPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DisplayLayout>::Convert(*this);
  }


  DisplayLayout();

  DisplayLayout(
      bool mirrored,
      bool default_unified,
      int64_t primary_display_id,
      const std::vector<display::DisplayPlacement>& placement_list);

  ~DisplayLayout();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DisplayLayoutPtr>
  DisplayLayoutPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DisplayLayout>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        DisplayLayout::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        DisplayLayout::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  bool mirrored;
  bool default_unified;
  int64_t primary_display_id;
  std::vector<display::DisplayPlacement> placement_list;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
DisplayPlacementPtr DisplayPlacement::Clone() const {
  return New(
      mojo::Clone(display_id),
      mojo::Clone(parent_display_id),
      mojo::Clone(position),
      mojo::Clone(offset),
      mojo::Clone(offset_reference)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DisplayPlacement>::value>::type*>
bool DisplayPlacement::Equals(const T& other) const {
  if (!mojo::Equals(this->display_id, other.display_id))
    return false;
  if (!mojo::Equals(this->parent_display_id, other.parent_display_id))
    return false;
  if (!mojo::Equals(this->position, other.position))
    return false;
  if (!mojo::Equals(this->offset, other.offset))
    return false;
  if (!mojo::Equals(this->offset_reference, other.offset_reference))
    return false;
  return true;
}
template <typename StructPtrType>
DisplayLayoutPtr DisplayLayout::Clone() const {
  return New(
      mojo::Clone(mirrored),
      mojo::Clone(default_unified),
      mojo::Clone(primary_display_id),
      mojo::Clone(placement_list)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DisplayLayout>::value>::type*>
bool DisplayLayout::Equals(const T& other) const {
  if (!mojo::Equals(this->mirrored, other.mirrored))
    return false;
  if (!mojo::Equals(this->default_unified, other.default_unified))
    return false;
  if (!mojo::Equals(this->primary_display_id, other.primary_display_id))
    return false;
  if (!mojo::Equals(this->placement_list, other.placement_list))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace display

namespace mojo {


template <>
struct  StructTraits<::display::mojom::DisplayPlacement::DataView,
                                         ::display::mojom::DisplayPlacementPtr> {
  static bool IsNull(const ::display::mojom::DisplayPlacementPtr& input) { return !input; }
  static void SetToNull(::display::mojom::DisplayPlacementPtr* output) { output->reset(); }

  static decltype(::display::mojom::DisplayPlacement::display_id) display_id(
      const ::display::mojom::DisplayPlacementPtr& input) {
    return input->display_id;
  }

  static decltype(::display::mojom::DisplayPlacement::parent_display_id) parent_display_id(
      const ::display::mojom::DisplayPlacementPtr& input) {
    return input->parent_display_id;
  }

  static decltype(::display::mojom::DisplayPlacement::position) position(
      const ::display::mojom::DisplayPlacementPtr& input) {
    return input->position;
  }

  static decltype(::display::mojom::DisplayPlacement::offset) offset(
      const ::display::mojom::DisplayPlacementPtr& input) {
    return input->offset;
  }

  static decltype(::display::mojom::DisplayPlacement::offset_reference) offset_reference(
      const ::display::mojom::DisplayPlacementPtr& input) {
    return input->offset_reference;
  }

  static bool Read(::display::mojom::DisplayPlacement::DataView input, ::display::mojom::DisplayPlacementPtr* output);
};


template <>
struct  StructTraits<::display::mojom::DisplayLayout::DataView,
                                         ::display::mojom::DisplayLayoutPtr> {
  static bool IsNull(const ::display::mojom::DisplayLayoutPtr& input) { return !input; }
  static void SetToNull(::display::mojom::DisplayLayoutPtr* output) { output->reset(); }

  static decltype(::display::mojom::DisplayLayout::mirrored) mirrored(
      const ::display::mojom::DisplayLayoutPtr& input) {
    return input->mirrored;
  }

  static decltype(::display::mojom::DisplayLayout::default_unified) default_unified(
      const ::display::mojom::DisplayLayoutPtr& input) {
    return input->default_unified;
  }

  static decltype(::display::mojom::DisplayLayout::primary_display_id) primary_display_id(
      const ::display::mojom::DisplayLayoutPtr& input) {
    return input->primary_display_id;
  }

  static const decltype(::display::mojom::DisplayLayout::placement_list)& placement_list(
      const ::display::mojom::DisplayLayoutPtr& input) {
    return input->placement_list;
  }

  static bool Read(::display::mojom::DisplayLayout::DataView input, ::display::mojom::DisplayLayoutPtr* output);
};

}  // namespace mojo

#endif  // UI_DISPLAY_MOJO_DISPLAY_LAYOUT_MOJOM_H_