// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_DISPLAY_MOJO_DISPLAY_SNAPSHOT_MOJO_MOJOM_SHARED_H_
#define UI_DISPLAY_MOJO_DISPLAY_SNAPSHOT_MOJO_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "ui/display/mojo/display_snapshot_mojo.mojom-shared-internal.h"
#include "mojo/common/file_path.mojom-shared.h"
#include "ui/display/mojo/display_constants.mojom-shared.h"
#include "ui/display/mojo/display_mode.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"


namespace display {
namespace mojom {
class DisplaySnapshotMojoDataView;



}  // namespace mojom
}  // namespace display

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::display::mojom::DisplaySnapshotMojoDataView> {
  using Data = ::display::mojom::internal::DisplaySnapshotMojo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace display {
namespace mojom {
class DisplaySnapshotMojoDataView {
 public:
  DisplaySnapshotMojoDataView() {}

  DisplaySnapshotMojoDataView(
      internal::DisplaySnapshotMojo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int64_t display_id() const {
    return data_->display_id;
  }
  inline void GetOriginDataView(
      ::gfx::mojom::PointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrigin(UserType* output) {
    auto* pointer = data_->origin.Get();
    return mojo::internal::Deserialize<::gfx::mojom::PointDataView>(
        pointer, output, context_);
  }
  inline void GetPhysicalSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPhysicalSize(UserType* output) {
    auto* pointer = data_->physical_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::display::mojom::DisplayConnectionType>(
        data_value, output);
  }

  ::display::mojom::DisplayConnectionType type() const {
    return static_cast<::display::mojom::DisplayConnectionType>(data_->type);
  }
  bool is_aspect_preserving_scaling() const {
    return data_->is_aspect_preserving_scaling;
  }
  bool has_overscan() const {
    return data_->has_overscan;
  }
  bool has_color_correction_matrix() const {
    return data_->has_color_correction_matrix;
  }
  inline void GetDisplayNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDisplayName(UserType* output) {
    auto* pointer = data_->display_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetSysPathDataView(
      ::mojo::common::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSysPath(UserType* output) {
    auto* pointer = data_->sys_path.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::FilePathDataView>(
        pointer, output, context_);
  }
  inline void GetModesDataView(
      mojo::ArrayDataView<::display::mojom::DisplayModeDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadModes(UserType* output) {
    auto* pointer = data_->modes.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::display::mojom::DisplayModeDataView>>(
        pointer, output, context_);
  }
  inline void GetEdidDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEdid(UserType* output) {
    auto* pointer = data_->edid.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
  uint64_t current_mode_index() const {
    return data_->current_mode_index;
  }
  bool has_current_mode() const {
    return data_->has_current_mode;
  }
  uint64_t native_mode_index() const {
    return data_->native_mode_index;
  }
  bool has_native_mode() const {
    return data_->has_native_mode;
  }
  int64_t product_id() const {
    return data_->product_id;
  }
  inline void GetMaximumCursorSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMaximumCursorSize(UserType* output) {
    auto* pointer = data_->maximum_cursor_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
 private:
  internal::DisplaySnapshotMojo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace display

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::display::mojom::DisplaySnapshotMojoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::display::mojom::DisplaySnapshotMojoDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::display::mojom::internal::DisplaySnapshotMojo_Data);
    decltype(CallWithContext(Traits::origin, input, custom_context)) in_origin = CallWithContext(Traits::origin, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::PointDataView>(
        in_origin, context);
    decltype(CallWithContext(Traits::physical_size, input, custom_context)) in_physical_size = CallWithContext(Traits::physical_size, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::SizeDataView>(
        in_physical_size, context);
    decltype(CallWithContext(Traits::display_name, input, custom_context)) in_display_name = CallWithContext(Traits::display_name, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_display_name, context);
    decltype(CallWithContext(Traits::sys_path, input, custom_context)) in_sys_path = CallWithContext(Traits::sys_path, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::FilePathDataView>(
        in_sys_path, context);
    decltype(CallWithContext(Traits::modes, input, custom_context)) in_modes = CallWithContext(Traits::modes, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::display::mojom::DisplayModeDataView>>(
        in_modes, context);
    decltype(CallWithContext(Traits::edid, input, custom_context)) in_edid = CallWithContext(Traits::edid, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
        in_edid, context);
    decltype(CallWithContext(Traits::maximum_cursor_size, input, custom_context)) in_maximum_cursor_size = CallWithContext(Traits::maximum_cursor_size, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::SizeDataView>(
        in_maximum_cursor_size, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::display::mojom::internal::DisplaySnapshotMojo_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::display::mojom::internal::DisplaySnapshotMojo_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->display_id = CallWithContext(Traits::display_id, input, custom_context);
    decltype(CallWithContext(Traits::origin, input, custom_context)) in_origin = CallWithContext(Traits::origin, input, custom_context);
    typename decltype(result->origin)::BaseType* origin_ptr;
    mojo::internal::Serialize<::gfx::mojom::PointDataView>(
        in_origin, buffer, &origin_ptr, context);
    result->origin.Set(origin_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->origin.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null origin in DisplaySnapshotMojo struct");
    decltype(CallWithContext(Traits::physical_size, input, custom_context)) in_physical_size = CallWithContext(Traits::physical_size, input, custom_context);
    typename decltype(result->physical_size)::BaseType* physical_size_ptr;
    mojo::internal::Serialize<::gfx::mojom::SizeDataView>(
        in_physical_size, buffer, &physical_size_ptr, context);
    result->physical_size.Set(physical_size_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->physical_size.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null physical_size in DisplaySnapshotMojo struct");
    mojo::internal::Serialize<::display::mojom::DisplayConnectionType>(
        CallWithContext(Traits::type, input, custom_context), &result->type);
    result->is_aspect_preserving_scaling = CallWithContext(Traits::is_aspect_preserving_scaling, input, custom_context);
    result->has_overscan = CallWithContext(Traits::has_overscan, input, custom_context);
    result->has_color_correction_matrix = CallWithContext(Traits::has_color_correction_matrix, input, custom_context);
    decltype(CallWithContext(Traits::display_name, input, custom_context)) in_display_name = CallWithContext(Traits::display_name, input, custom_context);
    typename decltype(result->display_name)::BaseType* display_name_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_display_name, buffer, &display_name_ptr, context);
    result->display_name.Set(display_name_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->display_name.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null display_name in DisplaySnapshotMojo struct");
    decltype(CallWithContext(Traits::sys_path, input, custom_context)) in_sys_path = CallWithContext(Traits::sys_path, input, custom_context);
    typename decltype(result->sys_path)::BaseType* sys_path_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::FilePathDataView>(
        in_sys_path, buffer, &sys_path_ptr, context);
    result->sys_path.Set(sys_path_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->sys_path.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null sys_path in DisplaySnapshotMojo struct");
    decltype(CallWithContext(Traits::modes, input, custom_context)) in_modes = CallWithContext(Traits::modes, input, custom_context);
    typename decltype(result->modes)::BaseType* modes_ptr;
    const mojo::internal::ContainerValidateParams modes_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::display::mojom::DisplayModeDataView>>(
        in_modes, buffer, &modes_ptr, &modes_validate_params,
        context);
    result->modes.Set(modes_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->modes.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null modes in DisplaySnapshotMojo struct");
    decltype(CallWithContext(Traits::edid, input, custom_context)) in_edid = CallWithContext(Traits::edid, input, custom_context);
    typename decltype(result->edid)::BaseType* edid_ptr;
    const mojo::internal::ContainerValidateParams edid_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_edid, buffer, &edid_ptr, &edid_validate_params,
        context);
    result->edid.Set(edid_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->edid.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null edid in DisplaySnapshotMojo struct");
    result->current_mode_index = CallWithContext(Traits::current_mode_index, input, custom_context);
    result->has_current_mode = CallWithContext(Traits::has_current_mode, input, custom_context);
    result->native_mode_index = CallWithContext(Traits::native_mode_index, input, custom_context);
    result->has_native_mode = CallWithContext(Traits::has_native_mode, input, custom_context);
    result->product_id = CallWithContext(Traits::product_id, input, custom_context);
    decltype(CallWithContext(Traits::maximum_cursor_size, input, custom_context)) in_maximum_cursor_size = CallWithContext(Traits::maximum_cursor_size, input, custom_context);
    typename decltype(result->maximum_cursor_size)::BaseType* maximum_cursor_size_ptr;
    mojo::internal::Serialize<::gfx::mojom::SizeDataView>(
        in_maximum_cursor_size, buffer, &maximum_cursor_size_ptr, context);
    result->maximum_cursor_size.Set(maximum_cursor_size_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->maximum_cursor_size.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null maximum_cursor_size in DisplaySnapshotMojo struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::display::mojom::internal::DisplaySnapshotMojo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::display::mojom::DisplaySnapshotMojoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace display {
namespace mojom {

inline void DisplaySnapshotMojoDataView::GetOriginDataView(
    ::gfx::mojom::PointDataView* output) {
  auto pointer = data_->origin.Get();
  *output = ::gfx::mojom::PointDataView(pointer, context_);
}
inline void DisplaySnapshotMojoDataView::GetPhysicalSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->physical_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}
inline void DisplaySnapshotMojoDataView::GetDisplayNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->display_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void DisplaySnapshotMojoDataView::GetSysPathDataView(
    ::mojo::common::mojom::FilePathDataView* output) {
  auto pointer = data_->sys_path.Get();
  *output = ::mojo::common::mojom::FilePathDataView(pointer, context_);
}
inline void DisplaySnapshotMojoDataView::GetModesDataView(
    mojo::ArrayDataView<::display::mojom::DisplayModeDataView>* output) {
  auto pointer = data_->modes.Get();
  *output = mojo::ArrayDataView<::display::mojom::DisplayModeDataView>(pointer, context_);
}
inline void DisplaySnapshotMojoDataView::GetEdidDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->edid.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}
inline void DisplaySnapshotMojoDataView::GetMaximumCursorSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->maximum_cursor_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace display

#endif  // UI_DISPLAY_MOJO_DISPLAY_SNAPSHOT_MOJO_MOJOM_SHARED_H_
