// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_DISPLAY_MOJO_DISPLAY_SNAPSHOT_MOJO_MOJOM_H_
#define UI_DISPLAY_MOJO_DISPLAY_SNAPSHOT_MOJO_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/display/mojo/display_snapshot_mojo.mojom-shared.h"
#include "mojo/common/file_path.mojom.h"
#include "ui/display/mojo/display_constants.mojom.h"
#include "ui/display/mojo/display_mode.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>
#include "ui/display/types/display_snapshot_mojo.h"


namespace display {
namespace mojom {
class DisplaySnapshotMojo;
using DisplaySnapshotMojoPtr = mojo::StructPtr<DisplaySnapshotMojo>;






class  DisplaySnapshotMojo {
 public:
  using DataView = DisplaySnapshotMojoDataView;
  using Data_ = internal::DisplaySnapshotMojo_Data;

  template <typename... Args>
  static DisplaySnapshotMojoPtr New(Args&&... args) {
    return DisplaySnapshotMojoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DisplaySnapshotMojoPtr From(const U& u) {
    return mojo::TypeConverter<DisplaySnapshotMojoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DisplaySnapshotMojo>::Convert(*this);
  }


  DisplaySnapshotMojo();

  DisplaySnapshotMojo(
      int64_t display_id,
      const gfx::Point& origin,
      const gfx::Size& physical_size,
      display::DisplayConnectionType type,
      bool is_aspect_preserving_scaling,
      bool has_overscan,
      bool has_color_correction_matrix,
      const std::string& display_name,
      const base::FilePath& sys_path,
      std::vector<std::unique_ptr<display::DisplayMode>> modes,
      const std::vector<uint8_t>& edid,
      uint64_t current_mode_index,
      bool has_current_mode,
      uint64_t native_mode_index,
      bool has_native_mode,
      int64_t product_id,
      const gfx::Size& maximum_cursor_size);

  ~DisplaySnapshotMojo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DisplaySnapshotMojoPtr>
  DisplaySnapshotMojoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DisplaySnapshotMojo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        DisplaySnapshotMojo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        DisplaySnapshotMojo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  int64_t display_id;
  gfx::Point origin;
  gfx::Size physical_size;
  display::DisplayConnectionType type;
  bool is_aspect_preserving_scaling;
  bool has_overscan;
  bool has_color_correction_matrix;
  std::string display_name;
  base::FilePath sys_path;
  std::vector<std::unique_ptr<display::DisplayMode>> modes;
  std::vector<uint8_t> edid;
  uint64_t current_mode_index;
  bool has_current_mode;
  uint64_t native_mode_index;
  bool has_native_mode;
  int64_t product_id;
  gfx::Size maximum_cursor_size;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(DisplaySnapshotMojo);
};


template <typename StructPtrType>
DisplaySnapshotMojoPtr DisplaySnapshotMojo::Clone() const {
  return New(
      mojo::Clone(display_id),
      mojo::Clone(origin),
      mojo::Clone(physical_size),
      mojo::Clone(type),
      mojo::Clone(is_aspect_preserving_scaling),
      mojo::Clone(has_overscan),
      mojo::Clone(has_color_correction_matrix),
      mojo::Clone(display_name),
      mojo::Clone(sys_path),
      mojo::Clone(modes),
      mojo::Clone(edid),
      mojo::Clone(current_mode_index),
      mojo::Clone(has_current_mode),
      mojo::Clone(native_mode_index),
      mojo::Clone(has_native_mode),
      mojo::Clone(product_id),
      mojo::Clone(maximum_cursor_size)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DisplaySnapshotMojo>::value>::type*>
bool DisplaySnapshotMojo::Equals(const T& other) const {
  if (!mojo::Equals(this->display_id, other.display_id))
    return false;
  if (!mojo::Equals(this->origin, other.origin))
    return false;
  if (!mojo::Equals(this->physical_size, other.physical_size))
    return false;
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->is_aspect_preserving_scaling, other.is_aspect_preserving_scaling))
    return false;
  if (!mojo::Equals(this->has_overscan, other.has_overscan))
    return false;
  if (!mojo::Equals(this->has_color_correction_matrix, other.has_color_correction_matrix))
    return false;
  if (!mojo::Equals(this->display_name, other.display_name))
    return false;
  if (!mojo::Equals(this->sys_path, other.sys_path))
    return false;
  if (!mojo::Equals(this->modes, other.modes))
    return false;
  if (!mojo::Equals(this->edid, other.edid))
    return false;
  if (!mojo::Equals(this->current_mode_index, other.current_mode_index))
    return false;
  if (!mojo::Equals(this->has_current_mode, other.has_current_mode))
    return false;
  if (!mojo::Equals(this->native_mode_index, other.native_mode_index))
    return false;
  if (!mojo::Equals(this->has_native_mode, other.has_native_mode))
    return false;
  if (!mojo::Equals(this->product_id, other.product_id))
    return false;
  if (!mojo::Equals(this->maximum_cursor_size, other.maximum_cursor_size))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace display

namespace mojo {


template <>
struct  StructTraits<::display::mojom::DisplaySnapshotMojo::DataView,
                                         ::display::mojom::DisplaySnapshotMojoPtr> {
  static bool IsNull(const ::display::mojom::DisplaySnapshotMojoPtr& input) { return !input; }
  static void SetToNull(::display::mojom::DisplaySnapshotMojoPtr* output) { output->reset(); }

  static decltype(::display::mojom::DisplaySnapshotMojo::display_id) display_id(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->display_id;
  }

  static const decltype(::display::mojom::DisplaySnapshotMojo::origin)& origin(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->origin;
  }

  static const decltype(::display::mojom::DisplaySnapshotMojo::physical_size)& physical_size(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->physical_size;
  }

  static decltype(::display::mojom::DisplaySnapshotMojo::type) type(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->type;
  }

  static decltype(::display::mojom::DisplaySnapshotMojo::is_aspect_preserving_scaling) is_aspect_preserving_scaling(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->is_aspect_preserving_scaling;
  }

  static decltype(::display::mojom::DisplaySnapshotMojo::has_overscan) has_overscan(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->has_overscan;
  }

  static decltype(::display::mojom::DisplaySnapshotMojo::has_color_correction_matrix) has_color_correction_matrix(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->has_color_correction_matrix;
  }

  static const decltype(::display::mojom::DisplaySnapshotMojo::display_name)& display_name(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->display_name;
  }

  static const decltype(::display::mojom::DisplaySnapshotMojo::sys_path)& sys_path(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->sys_path;
  }

  static const decltype(::display::mojom::DisplaySnapshotMojo::modes)& modes(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->modes;
  }

  static const decltype(::display::mojom::DisplaySnapshotMojo::edid)& edid(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->edid;
  }

  static decltype(::display::mojom::DisplaySnapshotMojo::current_mode_index) current_mode_index(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->current_mode_index;
  }

  static decltype(::display::mojom::DisplaySnapshotMojo::has_current_mode) has_current_mode(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->has_current_mode;
  }

  static decltype(::display::mojom::DisplaySnapshotMojo::native_mode_index) native_mode_index(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->native_mode_index;
  }

  static decltype(::display::mojom::DisplaySnapshotMojo::has_native_mode) has_native_mode(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->has_native_mode;
  }

  static decltype(::display::mojom::DisplaySnapshotMojo::product_id) product_id(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->product_id;
  }

  static const decltype(::display::mojom::DisplaySnapshotMojo::maximum_cursor_size)& maximum_cursor_size(
      const ::display::mojom::DisplaySnapshotMojoPtr& input) {
    return input->maximum_cursor_size;
  }

  static bool Read(::display::mojom::DisplaySnapshotMojo::DataView input, ::display::mojom::DisplaySnapshotMojoPtr* output);
};

}  // namespace mojo

#endif  // UI_DISPLAY_MOJO_DISPLAY_SNAPSHOT_MOJO_MOJOM_H_