// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_LATENCY_MOJO_LATENCY_INFO_MOJOM_SHARED_INTERNAL_H_
#define UI_LATENCY_MOJO_LATENCY_INFO_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "mojo/common/time.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared-internal.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace ui {
namespace mojom {
namespace internal {
class LatencyComponentId_Data;
class LatencyComponent_Data;
class LatencyComponentPair_Data;
class LatencyInfo_Data;

struct LatencyComponentType_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 6:
      case 7:
      case 8:
      case 9:
      case 10:
      case 11:
      case 12:
      case 13:
      case 14:
      case 15:
      case 16:
      case 17:
      case 18:
      case 19:
      case 20:
      case 21:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct SourceEventType_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class  LatencyComponentId_Data {
 public:
  static LatencyComponentId_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t type;
  uint8_t pad0_[4];
  int64_t id;

 private:
  LatencyComponentId_Data();
  ~LatencyComponentId_Data() = delete;
};
static_assert(sizeof(LatencyComponentId_Data) == 24,
              "Bad sizeof(LatencyComponentId_Data)");
class  LatencyComponent_Data {
 public:
  static LatencyComponent_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t sequence_number;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeTicks_Data> event_time;
  uint32_t event_count;
  uint8_t pad2_[4];
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeTicks_Data> first_event_time;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeTicks_Data> last_event_time;

 private:
  LatencyComponent_Data();
  ~LatencyComponent_Data() = delete;
};
static_assert(sizeof(LatencyComponent_Data) == 48,
              "Bad sizeof(LatencyComponent_Data)");
class  LatencyComponentPair_Data {
 public:
  static LatencyComponentPair_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::LatencyComponentId_Data> key;
  mojo::internal::Pointer<internal::LatencyComponent_Data> value;

 private:
  LatencyComponentPair_Data();
  ~LatencyComponentPair_Data() = delete;
};
static_assert(sizeof(LatencyComponentPair_Data) == 24,
              "Bad sizeof(LatencyComponentPair_Data)");
class  LatencyInfo_Data {
 public:
  static LatencyInfo_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> trace_name;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::LatencyComponentPair_Data>>> latency_components;
  int64_t trace_id;
  uint8_t coalesced : 1;
  uint8_t began : 1;
  uint8_t terminated : 1;
  uint8_t pad5_[3];
  int32_t source_event_type;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> expected_queueing_time_on_dispatch;

 private:
  LatencyInfo_Data();
  ~LatencyInfo_Data() = delete;
};
static_assert(sizeof(LatencyInfo_Data) == 48,
              "Bad sizeof(LatencyInfo_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace ui

#endif  // UI_LATENCY_MOJO_LATENCY_INFO_MOJOM_SHARED_INTERNAL_H_